@@ -18,7 +18,7 @@
 #
 
 BASEDIR=`dirname $0`
-BASE_BRANCH=cassandra-4.1
+BASE_BRANCH=cassandra-5.0
 set -e
 
 die ()
@@ -33,7 +33,9 @@ print_help()
   echo "Usage: $0 [-f|-p|-a|-e|-i|-b|-s]"
   echo "   -a Generate the config.yml, config.yml.FREE and config.yml.PAID expanded configuration"
   echo "      files from the main config_template.yml reusable configuration file."
-  echo "      Use this for permanent changes in config that will be committed to the main repo."
+  echo "      Use this for permanent changes in config.yml that will be committed to the main repo."
+  echo "   -d Minimal development checks only. Sanity check during your dev before sending it to review for speed and cost reductions."
+  echo "      Submitting cleaning pre-commit clean CI run is still a requirement when the patch is ready for review"
   echo "   -f Generate config.yml for tests compatible with the CircleCI free tier resources"
   echo "   -p Generate config.yml for tests compatible with the CircleCI paid tier resources"
   echo "   -b Specify the base git branch for comparison when determining changed tests to"
@@ -80,21 +82,22 @@ print_help()
 all=false
 free=false
 paid=false
+dev_min=false
 env_vars=""
 has_env_vars=false
 check_env_vars=true
 detect_changed_tests=true
-while getopts "e:afpib:s" opt; do
+while getopts "e:afpdib:s" opt; do
   case $opt in
       a ) all=true
           detect_changed_tests=false
           ;;
+      d ) dev_min=true
+          ;;
       f ) free=true
           ;;
       p ) paid=true
           ;;
-      b ) BASE_BRANCH="$OPTARG"
-          ;;
       e ) if (! ($has_env_vars)); then
             env_vars="$OPTARG"
           else
@@ -102,6 +105,8 @@ while getopts "e:afpib:s" opt; do
           fi
           has_env_vars=true
           ;;
+      b ) BASE_BRANCH="$OPTARG"
+          ;;
       i ) check_env_vars=false
           ;;
       s ) detect_changed_tests=false
@@ -152,6 +157,10 @@ if $has_env_vars && $check_env_vars; then
   done
 fi
 
+if $dev_min && $detect_changed_tests; then
+  die "-d doesn't support repeated tests. Use -s to skip it."
+fi
+
 if $free; then
   ($all || $paid) && die "Cannot use option -f with options -a or -p"
   echo "Generating new config.yml file for free tier from config_template.yml"
@@ -201,6 +210,7 @@ if $detect_changed_tests; then
     echo    "(e.g. origin/${BASE_BRANCH}, apache/${BASE_BRANCH}, etc.)."
     exit 2
   fi
+
   add_diff_tests ()
   {
     dir="${BASEDIR}/../${2}"
@@ -252,7 +262,7 @@ if $has_env_vars; then
 fi
 
 # Define function to remove unneeded jobs.
-# The first argument is the file name, and the second arguemnt is the job name.
+# The first argument is the file name, and the second argument is the job name.
 delete_job()
 {
   delete_yaml_block()
@@ -270,66 +280,147 @@ delete_job()
 delete_repeated_jobs()
 {
   if (! (echo "$env_vars" | grep -q "REPEATED_UTESTS=" )); then
-    delete_job "$1" "j8_unit_tests_repeat"
     delete_job "$1" "j11_unit_tests_repeat"
-    delete_job "$1" "j8_utests_cdc_repeat"
+    delete_job "$1" "j17_unit_tests_repeat"
     delete_job "$1" "j11_utests_cdc_repeat"
-    delete_job "$1" "j8_utests_compression_repeat"
+    delete_job "$1" "j17_utests_cdc_repeat"
     delete_job "$1" "j11_utests_compression_repeat"
-    delete_job "$1" "j8_utests_system_keyspace_directory_repeat"
+    delete_job "$1" "j17_utests_compression_repeat"
+    delete_job "$1" "j11_utests_latest_repeat"
+    delete_job "$1" "j17_utests_latest_repeat"
+    delete_job "$1" "j11_utests_oa_repeat"
+    delete_job "$1" "j17_utests_oa_repeat"
     delete_job "$1" "j11_utests_system_keyspace_directory_repeat"
+    delete_job "$1" "j17_utests_system_keyspace_directory_repeat"
   fi
   if (! (echo "$env_vars" | grep -q "REPEATED_UTESTS_LONG=")); then
-    delete_job "$1" "j8_utests_long_repeat"
     delete_job "$1" "j11_utests_long_repeat"
+    delete_job "$1" "j17_utests_long_repeat"
   fi
   if (! (echo "$env_vars" | grep -q "REPEATED_UTESTS_STRESS=")); then
-    delete_job "$1" "j8_utests_stress_repeat"
     delete_job "$1" "j11_utests_stress_repeat"
+    delete_job "$1" "j17_utests_stress_repeat"
   fi
   if (! (echo "$env_vars" | grep -q "REPEATED_UTESTS_FQLTOOL=")); then
-    delete_job "$1" "j8_utests_fqltool_repeat"
     delete_job "$1" "j11_utests_fqltool_repeat"
+    delete_job "$1" "j17_utests_fqltool_repeat"
   fi
   if (! (echo "$env_vars" | grep -q "REPEATED_SIMULATOR_DTESTS=")); then
-    delete_job "$1" "j8_simulator_dtests_repeat"
     delete_job "$1" "j11_simulator_dtests_repeat"
   fi
   if (! (echo "$env_vars" | grep -q "REPEATED_JVM_DTESTS=")); then
-    delete_job "$1" "j8_jvm_dtests_repeat"
-    delete_job "$1" "j8_jvm_dtests_vnode_repeat"
     delete_job "$1" "j11_jvm_dtests_repeat"
-    delete_job "$1" "j11_jvm_dtests_vnode_repeat"
+    delete_job "$1" "j11_jvm_dtests_latest_vnode_repeat"
+    delete_job "$1" "j17_jvm_dtests_repeat"
+    delete_job "$1" "j17_jvm_dtests_latest_vnode_repeat"
   fi
   if (! (echo "$env_vars" | grep -q "REPEATED_JVM_UPGRADE_DTESTS=")); then
     delete_job "$1" "start_jvm_upgrade_dtests_repeat"
-    delete_job "$1" "j8_jvm_upgrade_dtests_repeat"
+    delete_job "$1" "j11_jvm_upgrade_dtests_repeat"
   fi
   if (! (echo "$env_vars" | grep -q "REPEATED_DTESTS=")); then
-    delete_job "$1" "j8_dtests_repeat"
-    delete_job "$1" "j8_dtests_vnode_repeat"
-    delete_job "$1" "j8_dtests_offheap_repeat"
     delete_job "$1" "j11_dtests_repeat"
     delete_job "$1" "j11_dtests_vnode_repeat"
-    delete_job "$1" "j11_dtests_offheap_repeat"
+    delete_job "$1" "j11_dtests_latest_repeat"
+    delete_job "$1" "j17_dtests_repeat"
+    delete_job "$1" "j17_dtests_vnode_repeat"
+    delete_job "$1" "j17_dtests_latest_repeat"
   fi
   if (! (echo "$env_vars" | grep -q "REPEATED_LARGE_DTESTS=")); then
-    delete_job "$1" "j8_dtests_large_repeat"
-    delete_job "$1" "j8_dtests_large_vnode_repeat"
     delete_job "$1" "j11_dtests_large_repeat"
     delete_job "$1" "j11_dtests_large_vnode_repeat"
+    delete_job "$1" "j17_dtests_large_repeat"
+    delete_job "$1" "j17_dtests_large_vnode_repeat"
   fi
   if (! (echo "$env_vars" | grep -q "REPEATED_UPGRADE_DTESTS=")); then
-    delete_job "$1" "j8_upgrade_dtests_repeat"
+    delete_job "$1" "j11_upgrade_dtests_repeat"
   fi
   if (! (echo "$env_vars" | grep -q "REPEATED_ANT_TEST_CLASS=")); then
-    delete_job "$1" "j8_repeated_ant_test"
     delete_job "$1" "j11_repeated_ant_test"
+    delete_job "$1" "j17_repeated_ant_test"
   fi
 }
 
+# Update the workflow names
+rename_workflow()
+{
+  file="$BASEDIR/$1"
+  echo "Updating workflow names in the configuration $2 -> $3"
+
+  sed -Ei.bak "s/$2/$3/g" "$file"
+}
+
+# Define function to leave only a single config run for each test group.
+# This builds a minimal sanity check config for dev only for time and cost purposes.
+# The first and only argument is the file name.
+build_dev_min_jobs()
+{
+  delete_job "$1" "j11_cqlsh_dtests_py311_latest"
+  delete_job "$1" "j11_cqlsh_dtests_py38_latest"
+  delete_job "$1" "j17_cqlsh_dtests_py311_latest"
+  delete_job "$1" "j17_cqlsh_dtests_py38_latest"
+  delete_job "$1" "j11_cqlsh_dtests_py311_vnode"
+  delete_job "$1" "j11_cqlsh_dtests_py38_vnode"
+  delete_job "$1" "j11_cqlsh_dtests_py38"
+  delete_job "$1" "j11_cqlshlib_cython_tests"
+  delete_job "$1" "j17_cqlsh_dtests_py311_vnode"
+  delete_job "$1" "j17_cqlsh_dtests_py311"
+  delete_job "$1" "j17_cqlsh_dtests_py38_vnode"
+  delete_job "$1" "j17_cqlsh_dtests_py38"
+  delete_job "$1" "j17_cqlshlib_tests"
+  delete_job "$1" "j17_cqlshlib_cython_tests"
+  delete_job "$1" "j11_dtests_vnode"
+  delete_job "$1" "j11_dtests_large_vnode"
+  delete_job "$1" "j11_dtests_latest"
+  delete_job "$1" "j17_dtests_vnode"
+  delete_job "$1" "j17_dtests_large"
+  delete_job "$1" "j17_dtests_large_vnode"
+  delete_job "$1" "j17_dtests_latest"
+  delete_job "$1" "j17_dtests"
+  delete_job "$1" "j11_jvm_dtests_latest_vnode"
+  delete_job "$1" "j17_jvm_dtests_vnode"
+  delete_job "$1" "j17_jvm_dtests_latest_vnode"
+  delete_job "$1" "j17_jvm_dtests_vnode"
+  delete_job "$1" "j17_jvm_dtests"
+  delete_job "$1" "j11_utests_oa"
+  delete_job "$1" "j11_utests_cdc"
+  delete_job "$1" "j11_utests_compression"
+  delete_job "$1" "j11_utests_fqltool"
+  delete_job "$1" "j11_utests_long"
+  delete_job "$1" "j11_utests_stress"
+  delete_job "$1" "j11_utests_system_keyspace_directory"
+  delete_job "$1" "j17_unit_tests"
+  delete_job "$1" "j17_utests_oa"
+  delete_job "$1" "j17_utests_cdc"
+  delete_job "$1" "j17_utests_compression"
+  delete_job "$1" "j17_utests_fqltool"
+  delete_job "$1" "j17_utests_long"
+  delete_job "$1" "j17_utests_stress"
+  delete_job "$1" "j11_utests_latest"
+  delete_job "$1" "j17_utests_latest"
+  delete_job "$1" "j17_utests_system_keyspace_directory"
+  delete_job "$1" "start_utests_system_keyspace_directory"
+  delete_job "$1" "start_utests_stress"
+  delete_job "$1" "start_utests_long"
+  delete_job "$1" "start_utests_fqltool"
+  delete_job "$1" "start_utests_compression"
+  delete_job "$1" "start_utests_cdc"
+  delete_job "$1" "start_j17_cqlsh-dtests-latest"
+  delete_job "$1" "start_j11_cqlsh_dtests_latest"
+  delete_job "$1" "start_j17_cqlsh_tests"
+  delete_job "$1" "start_j17_cqlsh_tests_latest"
+  delete_job "$1" "start_j11_cqlsh_tests_latest"
+
+  rename_workflow "$1" "java11_pre-commit_tests" "java11_dev_tests"
+  rename_workflow "$1" "java17_pre-commit_tests" "java17_dev_tests"
+}
+
 delete_repeated_jobs "config.yml"
 if $all; then
   delete_repeated_jobs "config.yml.FREE"
   delete_repeated_jobs "config.yml.PAID"
 fi
+
+if $dev_min; then
+  build_dev_min_jobs "config.yml"
+fi
