@@ -17,8 +17,11 @@
  */
 package org.apache.cassandra.auth;
 
+import java.util.Set;
 import java.util.concurrent.TimeUnit;
 
+import com.google.common.collect.ImmutableSet;
+
 import org.apache.cassandra.config.CassandraRelevantProperties;
 import org.apache.cassandra.cql3.statements.schema.CreateTableStatement;
 import org.apache.cassandra.config.DatabaseDescriptor;
@@ -30,6 +33,7 @@ import org.apache.cassandra.schema.KeyspaceParams;
 import org.apache.cassandra.schema.Tables;
 
 import static java.lang.String.format;
+import static org.apache.cassandra.config.CassandraRelevantProperties.SUPERUSER_SETUP_DELAY_MS;
 
 public final class AuthKeyspace
 {
@@ -54,8 +58,15 @@ public final class AuthKeyspace
     public static final String ROLE_PERMISSIONS = "role_permissions";
     public static final String RESOURCE_ROLE_INDEX = "resource_role_permissons_index";
     public static final String NETWORK_PERMISSIONS = "network_permissions";
+    public static final String CIDR_PERMISSIONS = "cidr_permissions";
+    public static final String CIDR_GROUPS = "cidr_groups";
+    public static final String IDENTITY_TO_ROLES = "identity_to_role";
+    public static final Set<String> TABLE_NAMES = ImmutableSet.of(ROLES, ROLE_MEMBERS, ROLE_PERMISSIONS,
+                                                                  RESOURCE_ROLE_INDEX, NETWORK_PERMISSIONS,
+                                                                  CIDR_PERMISSIONS, CIDR_GROUPS,
+                                                                  IDENTITY_TO_ROLES);
 
-    public static final long SUPERUSER_SETUP_DELAY = Long.getLong("cassandra.superuser_setup_delay_ms", 10000);
+    public static final long SUPERUSER_SETUP_DELAY = SUPERUSER_SETUP_DELAY_MS.getLong();
 
     private static final TableMetadata Roles =
         parse(ROLES,
@@ -68,6 +79,15 @@ public final class AuthKeyspace
               + "member_of set<text>,"
               + "PRIMARY KEY(role))");
 
+    private static final TableMetadata IdentityToRoles =
+        parse(IDENTITY_TO_ROLES,
+              "mtls authorized identities lookup table",
+              "CREATE TABLE %s ("
+              + "identity text," // opaque identity string for use by role authenticators
+              + "role text,"
+              + "PRIMARY KEY(identity))"
+          );
+
     private static final TableMetadata RoleMembers =
         parse(ROLE_MEMBERS,
               "role memberships lookup table",
@@ -101,6 +121,28 @@ public final class AuthKeyspace
               + "dcs frozen<set<text>>, "
               + "PRIMARY KEY(role))");
 
+    public static final String CIDR_PERMISSIONS_TBL_ROLE_COL_NAME = "role";
+    public static final String CIDR_PERMISSIONS_TBL_CIDR_GROUPS_COL_NAME = "cidr_groups";
+    private static final TableMetadata CIDRPermissions =
+    parse(CIDR_PERMISSIONS,
+          "user cidr permissions",
+          "CREATE TABLE %s ("
+          + CIDR_PERMISSIONS_TBL_ROLE_COL_NAME + " text, "
+          + CIDR_PERMISSIONS_TBL_CIDR_GROUPS_COL_NAME + " frozen<set<text>>, "
+          + "PRIMARY KEY(" + CIDR_PERMISSIONS_TBL_ROLE_COL_NAME + "))"
+    );
+
+    public static final String CIDR_GROUPS_TBL_CIDR_GROUP_COL_NAME = "cidr_group";
+    public static final String CIDR_GROUPS_TBL_CIDRS_COL_NAME = "cidrs";
+    private static final TableMetadata CIDRGroups =
+    parse(CIDR_GROUPS,
+          "cidr groups to cidrs mapping",
+          "CREATE TABLE %s ("
+          + CIDR_GROUPS_TBL_CIDR_GROUP_COL_NAME + " text, "
+          + CIDR_GROUPS_TBL_CIDRS_COL_NAME + " frozen<set<tuple<inet, smallint>>>, "
+          + "PRIMARY KEY(" + CIDR_GROUPS_TBL_CIDR_GROUP_COL_NAME + "))"
+    );
+
     private static TableMetadata parse(String name, String description, String cql)
     {
         return CreateTableStatement.parse(format(cql, name), SchemaConstants.AUTH_KEYSPACE_NAME)
@@ -114,6 +156,9 @@ public final class AuthKeyspace
     {
         return KeyspaceMetadata.create(SchemaConstants.AUTH_KEYSPACE_NAME,
                                        KeyspaceParams.simple(Math.max(DEFAULT_RF, DatabaseDescriptor.getDefaultKeyspaceRF())),
-                                       Tables.of(Roles, RoleMembers, RolePermissions, ResourceRoleIndex, NetworkPermissions));
+                                       Tables.of(Roles, RoleMembers, RolePermissions,
+                                                 ResourceRoleIndex, NetworkPermissions,
+                                                 CIDRPermissions, CIDRGroups,
+                                                 IdentityToRoles));
     }
 }
