@@ -17,6 +17,7 @@
  */
 package org.apache.cassandra.auth;
 
+import java.nio.ByteBuffer;
 import java.util.*;
 import java.util.concurrent.Callable;
 import java.util.concurrent.TimeUnit;
@@ -34,7 +35,6 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import org.apache.cassandra.concurrent.ScheduledExecutors;
-import org.apache.cassandra.config.Config;
 import org.apache.cassandra.config.DatabaseDescriptor;
 import org.apache.cassandra.schema.SchemaConstants;
 import org.apache.cassandra.cql3.*;
@@ -43,12 +43,15 @@ import org.apache.cassandra.db.ConsistencyLevel;
 import org.apache.cassandra.db.marshal.UTF8Type;
 import org.apache.cassandra.exceptions.*;
 import org.apache.cassandra.service.ClientState;
+import org.apache.cassandra.service.StorageProxy;
 import org.apache.cassandra.service.StorageService;
 import org.apache.cassandra.transport.Dispatcher;
 import org.apache.cassandra.transport.messages.ResultMessage;
 import org.apache.cassandra.utils.ByteBufferUtil;
+import org.apache.cassandra.utils.NoSpamLogger;
 import org.mindrot.jbcrypt.BCrypt;
 
+import static org.apache.cassandra.config.CassandraRelevantProperties.AUTH_BCRYPT_GENSALT_LOG2_ROUNDS;
 import static org.apache.cassandra.service.QueryState.forInternalCalls;
 
 /**
@@ -79,9 +82,10 @@ import static org.apache.cassandra.service.QueryState.forInternalCalls;
 public class CassandraRoleManager implements IRoleManager
 {
     private static final Logger logger = LoggerFactory.getLogger(CassandraRoleManager.class);
+    private static final NoSpamLogger nospamLogger = NoSpamLogger.getLogger(logger, 1L, TimeUnit.MINUTES);
 
     public static final String DEFAULT_SUPERUSER_NAME = "cassandra";
-    static final String DEFAULT_SUPERUSER_PASSWORD = "cassandra";
+    public static final String DEFAULT_SUPERUSER_PASSWORD = "cassandra";
 
     /**
      * We need to treat the default superuser as a special case since during initial node startup, we may end up with
@@ -112,22 +116,19 @@ public class CassandraRoleManager implements IRoleManager
         }
     };
 
-    // 2 ** GENSALT_LOG2_ROUNDS rounds of hashing will be performed.
-    @VisibleForTesting
-    public static final String GENSALT_LOG2_ROUNDS_PROPERTY = Config.PROPERTY_PREFIX + "auth_bcrypt_gensalt_log2_rounds";
     private static final int GENSALT_LOG2_ROUNDS = getGensaltLogRounds();
 
     static int getGensaltLogRounds()
     {
-         int rounds = Integer.getInteger(GENSALT_LOG2_ROUNDS_PROPERTY, 10);
-         if (rounds < 4 || rounds > 30)
-         throw new ConfigurationException(String.format("Bad value for system property -D%s." +
-                                                        "Please use a value between 4 and 30 inclusively",
-                                                        GENSALT_LOG2_ROUNDS_PROPERTY));
-         return rounds;
+        int rounds = AUTH_BCRYPT_GENSALT_LOG2_ROUNDS.getInt(10);
+        if (rounds < 4 || rounds > 30)
+            throw new ConfigurationException(String.format("Bad value for system property %s." +
+                                                           "Please use a value between 4 and 30 inclusively", AUTH_BCRYPT_GENSALT_LOG2_ROUNDS.getKey()));
+        return rounds;
     }
 
     private SelectStatement loadRoleStatement;
+    private SelectStatement loadIdentityStatement;
 
     private final Set<Option> supportedOptions;
     private final Set<Option> alterableOptions;
@@ -146,12 +147,72 @@ public class CassandraRoleManager implements IRoleManager
     public void setup()
     {
         loadRoleStatement();
+        loadIdentityStatement();
         scheduleSetupTask(() -> {
             setupDefaultRole();
             return null;
         });
     }
 
+    @Override
+    public String roleForIdentity(String identity)
+    {
+        QueryOptions options = QueryOptions.forInternalCalls(CassandraAuthorizer.authReadConsistencyLevel(),
+                                                             Collections.singletonList(byteBuf(identity)));
+        ResultMessage.Rows rows = select(loadIdentityStatement, options);
+        if (rows.result.isEmpty())
+        {
+            nospamLogger.warn("No such identity {} in the identity_to_roles table", identity);
+            return null;
+        }
+        return UntypedResultSet.create(rows.result).one().getString("role");
+    }
+
+    @Override
+    public Map<String, String> authorizedIdentities()
+    {
+        Map<String, String> validIdentities = new HashMap<>();
+        String query = String.format("SELECT identity, role from %s.%s",
+                                     SchemaConstants.AUTH_KEYSPACE_NAME,
+                                     AuthKeyspace.IDENTITY_TO_ROLES);
+        UntypedResultSet rows = process(query, CassandraAuthorizer.authReadConsistencyLevel());
+        rows.forEach(row -> validIdentities.put(row.getString("identity"), row.getString("role")));
+        return validIdentities;
+    }
+
+    @Override
+    public void addIdentity(String identity, String role)
+    {
+        if (isExistingIdentity(identity))
+        {
+            throw new IllegalStateException("Identity is already associated with another role, cannot associate it with role " + role);
+        }
+
+        String query = String.format("INSERT INTO %s.%s (identity, role) VALUES (?, ?)",
+                                     SchemaConstants.AUTH_KEYSPACE_NAME,
+                                     AuthKeyspace.IDENTITY_TO_ROLES);
+        process(query, CassandraAuthorizer.authWriteConsistencyLevel(), byteBuf(identity), byteBuf(role));
+    }
+
+    @Override
+    public boolean isExistingIdentity(String identity)
+    {
+        String query = String.format("SELECT identity from %s.%s where identity=?",
+                                     SchemaConstants.AUTH_KEYSPACE_NAME,
+                                     AuthKeyspace.IDENTITY_TO_ROLES);
+        UntypedResultSet rows = process(query, CassandraAuthorizer.authReadConsistencyLevel(), byteBuf(identity));
+        return !rows.isEmpty();
+    }
+
+    @Override
+    public void dropIdentity(String identity)
+    {
+        String query = String.format("DELETE FROM %s.%s WHERE identity = ?",
+                                     SchemaConstants.AUTH_KEYSPACE_NAME,
+                                     AuthKeyspace.IDENTITY_TO_ROLES);
+        process(query, CassandraAuthorizer.authWriteConsistencyLevel(), byteBuf(identity));
+    }
+
     protected final void loadRoleStatement()
     {
         loadRoleStatement = (SelectStatement) prepare("SELECT * from %s.%s WHERE role = ?",
@@ -159,6 +220,14 @@ public class CassandraRoleManager implements IRoleManager
                                                       AuthKeyspace.ROLES);
     }
 
+
+    protected void loadIdentityStatement()
+    {
+        loadIdentityStatement = (SelectStatement) prepare("SELECT role from %s.%s where identity=?",
+                                                          SchemaConstants.AUTH_KEYSPACE_NAME,
+                                                          AuthKeyspace.IDENTITY_TO_ROLES);
+    }
+
     public Set<Option> supportedOptions()
     {
         return supportedOptions;
@@ -172,6 +241,11 @@ public class CassandraRoleManager implements IRoleManager
     public void createRole(AuthenticatedUser performer, RoleResource role, RoleOptions options)
     throws RequestValidationException, RequestExecutionException
     {
+        List<String> identitiesOfRole = identitiesForRole(role.getRoleName());
+        if (!identitiesOfRole.isEmpty())
+        {
+            throw new IllegalStateException(String.format("Cannot create a role '%s' when identities already exists for it", role.getRoleName()));
+        }
         String insertCql = options.getPassword().isPresent() || options.getHashedPassword().isPresent()
                          ? String.format("INSERT INTO %s.%s (role, is_superuser, can_login, salted_hash) VALUES ('%s', %s, %s, '%s')",
                                          SchemaConstants.AUTH_KEYSPACE_NAME,
@@ -197,6 +271,7 @@ public class CassandraRoleManager implements IRoleManager
                               escape(role.getRoleName())),
                 consistencyForRoleWrite(role.getRoleName()));
         removeAllMembers(role.getRoleName());
+        removeAllIdentitiesOfRole(role.getRoleName());
     }
 
     public void alterRole(AuthenticatedUser performer, RoleResource role, RoleOptions options)
@@ -386,6 +461,12 @@ public class CassandraRoleManager implements IRoleManager
     {
         // The delay is to give the node a chance to see its peers before attempting the operation
         ScheduledExecutors.optionalTasks.scheduleSelfRecurring(() -> {
+            if (!StorageProxy.isSafeToPerformRead())
+            {
+                logger.trace("Setup task may not run due to it not being safe to perform reads... rescheduling");
+                scheduleSetupTask(setupTask);
+                return;
+            }
             try
             {
                 setupTask.call();
@@ -467,6 +548,31 @@ public class CassandraRoleManager implements IRoleManager
                 consistencyForRoleWrite(grantee));
     }
 
+    private List<String> identitiesForRole(String role)
+    {
+        // Get all identities associated with a given role
+        String query = String.format("SELECT identity FROM %s.%s WHERE role = ? ALLOW FILTERING",
+                                     SchemaConstants.AUTH_KEYSPACE_NAME,
+                                     AuthKeyspace.IDENTITY_TO_ROLES);
+        UntypedResultSet rows = process(query, consistencyForRoleRead(role), byteBuf(role));
+        List<String> identities = new ArrayList<>();
+        rows.forEach(row -> identities.add(row.getString("identity")));
+        return identities;
+    }
+
+    private void removeAllIdentitiesOfRole(String role)
+    {
+        List<String> identities = identitiesForRole(role);
+        String query = String.format("DELETE FROM %s.%s WHERE identity = ?",
+                                     SchemaConstants.AUTH_KEYSPACE_NAME,
+                                     AuthKeyspace.IDENTITY_TO_ROLES);
+        // Remove all the identities associated with the role from the table
+        for (String identity : identities)
+        {
+            process(query, consistencyForRoleWrite(role), byteBuf(identity));
+        }
+    }
+
     /*
      * Clear the membership list of the given role
      */
@@ -531,6 +637,11 @@ public class CassandraRoleManager implements IRoleManager
         return StringUtils.replace(name, "'", "''");
     }
 
+    private static ByteBuffer byteBuf(String str)
+    {
+        return UTF8Type.instance.decompose(str);
+    }
+
     /** Allows selective overriding of the consistency level for specific roles. */
     protected static ConsistencyLevel consistencyForRoleWrite(String role)
     {
@@ -558,6 +669,12 @@ public class CassandraRoleManager implements IRoleManager
         return QueryProcessor.process(query, consistencyLevel);
     }
 
+    UntypedResultSet process(String query, ConsistencyLevel consistencyLevel, ByteBuffer... values)
+    throws RequestValidationException, RequestExecutionException
+    {
+        return QueryProcessor.process(query, consistencyLevel, Arrays.asList(values));
+    }
+
     @VisibleForTesting
     ResultMessage.Rows select(SelectStatement statement, QueryOptions options)
     {
