@@ -21,8 +21,6 @@ import java.util.ArrayList;
 import java.util.List;
 import java.util.function.Predicate;
 
-import org.apache.cassandra.utils.Hex;
-
 public final class Resources
 {
     /**
@@ -63,7 +61,7 @@ public final class Resources
      * Creates an IResource instance from its external name.
      * Resource implementation class is inferred by matching against the known IResource
      * impls' root level resources.
-     * @param name
+     * @param name external name to create IResource from
      * @return an IResource instance created from the name
      */
     public static IResource fromName(String name)
@@ -79,24 +77,4 @@ public final class Resources
         else
             throw new IllegalArgumentException(String.format("Name %s is not valid for any resource type", name));
     }
-
-    @Deprecated
-    public final static String ROOT = "cassandra";
-    @Deprecated
-    public final static String KEYSPACES = "keyspaces";
-
-    @Deprecated
-    public static String toString(List<Object> resource)
-    {
-        StringBuilder buff = new StringBuilder();
-        for (Object component : resource)
-        {
-            buff.append("/");
-            if (component instanceof byte[])
-                buff.append(Hex.bytesToHex((byte[])component));
-            else
-                buff.append(component);
-        }
-        return buff.toString();
-    }
 }
