@@ -19,32 +19,41 @@ package org.apache.cassandra.cache;
 
 import java.nio.ByteBuffer;
 import java.util.Arrays;
+import java.util.Objects;
+
+import com.google.common.annotations.VisibleForTesting;
 
 import org.apache.cassandra.db.DecoratedKey;
+import org.apache.cassandra.schema.Schema;
+import org.apache.cassandra.schema.TableId;
+import org.apache.cassandra.schema.TableMetadata;
+import org.apache.cassandra.schema.TableMetadataRef;
 import org.apache.cassandra.utils.ByteBufferUtil;
 import org.apache.cassandra.utils.ObjectSizes;
-import org.apache.cassandra.utils.Pair;
 
 public final class RowCacheKey extends CacheKey
 {
     public final byte[] key;
 
-    private static final long EMPTY_SIZE = ObjectSizes.measure(new RowCacheKey(null, ByteBufferUtil.EMPTY_BYTE_BUFFER));
+    private static final long EMPTY_SIZE = ObjectSizes.measure(new RowCacheKey(null, null, new byte[0]));
 
-    public RowCacheKey(Pair<String, String> ksAndCFName, byte[] key)
+    public RowCacheKey(TableId tableId, String indexName, byte[] key)
     {
-        super(ksAndCFName);
+        super(tableId, indexName);
         this.key = key;
     }
 
-    public RowCacheKey(Pair<String, String> ksAndCFName, DecoratedKey key)
+    public RowCacheKey(TableMetadata metadata, DecoratedKey key)
     {
-        this(ksAndCFName, key.getKey());
+        super(metadata);
+        this.key = ByteBufferUtil.getArray(key.getKey());
+        assert this.key != null;
     }
 
-    public RowCacheKey(Pair<String, String> ksAndCFName, ByteBuffer key)
+    @VisibleForTesting
+    public RowCacheKey(TableId tableId, String indexName, ByteBuffer key)
     {
-        super(ksAndCFName);
+        super(tableId, indexName);
         this.key = ByteBufferUtil.getArray(key);
         assert this.key != null;
     }
@@ -62,13 +71,16 @@ public final class RowCacheKey extends CacheKey
 
         RowCacheKey that = (RowCacheKey) o;
 
-        return ksAndCFName.equals(that.ksAndCFName) && Arrays.equals(key, that.key);
+        return tableId.equals(that.tableId)
+               && Objects.equals(indexName, that.indexName)
+               && Arrays.equals(key, that.key);
     }
 
     @Override
     public int hashCode()
     {
-        int result = ksAndCFName.hashCode();
+        int result = tableId.hashCode();
+        result = 31 * result + Objects.hashCode(indexName);
         result = 31 * result + (key != null ? Arrays.hashCode(key) : 0);
         return result;
     }
@@ -76,6 +88,7 @@ public final class RowCacheKey extends CacheKey
     @Override
     public String toString()
     {
-        return String.format("RowCacheKey(ksAndCFName:%s, key:%s)", ksAndCFName, Arrays.toString(key));
+        TableMetadataRef tableRef = Schema.instance.getTableMetadataRef(tableId);
+        return String.format("RowCacheKey(%s, %s, key:%s)", tableRef, indexName, Arrays.toString(key));
     }
 }
