@@ -46,7 +46,7 @@ public abstract class AbstractFunction implements Function
     {
         this.name = name;
         this.argTypes = argTypes;
-        this.returnType = returnType;
+        this.returnType = returnType != null ? returnType.unwrap() : null;
     }
 
     public FunctionName name()
@@ -117,6 +117,12 @@ public abstract class AbstractFunction implements Function
         return AssignmentTestable.TestResult.NOT_ASSIGNABLE;
     }
 
+    @Override
+    public AbstractType<?> getCompatibleTypeIfKnown(String keyspace)
+    {
+        return returnType();
+    }
+
     @Override
     public String toString()
     {
@@ -165,4 +171,34 @@ public abstract class AbstractFunction implements Function
                                                 .append(')')
                                                 .toString();
     }
+
+    /*
+     * We need to compare the CQL3 representation of the type because comparing
+     * the AbstractType will fail for example if a UDT has been changed.
+     * Reason is that UserType.equals() takes the field names and types into account.
+     * Example CQL sequence that would fail when comparing AbstractType:
+     *    CREATE TYPE foo ...
+     *    CREATE FUNCTION bar ( par foo ) RETURNS foo ...
+     *    ALTER TYPE foo ADD ...
+     * or
+     *    ALTER TYPE foo ALTER ...
+     * or
+     *    ALTER TYPE foo RENAME ...
+     */
+    public boolean typesMatch(List<AbstractType<?>> types)
+    {
+        if (argTypes().size() != types.size())
+            return false;
+
+        for (int i = 0; i < argTypes().size(); i++)
+            if (!typesMatch(argTypes().get(i), types.get(i)))
+                return false;
+
+        return true;
+    }
+
+    private static boolean typesMatch(AbstractType<?> t1, AbstractType<?> t2)
+    {
+        return t1.freeze().asCQL3Type().toString().equals(t2.freeze().asCQL3Type().toString());
+    }
 }
