@@ -21,11 +21,8 @@ import java.math.BigDecimal;
 import java.math.BigInteger;
 import java.math.RoundingMode;
 import java.nio.ByteBuffer;
-import java.util.ArrayList;
-import java.util.Collection;
 import java.util.List;
 
-import org.apache.cassandra.cql3.CQL3Type;
 import org.apache.cassandra.db.marshal.*;
 import org.apache.cassandra.exceptions.InvalidRequestException;
 import org.apache.cassandra.transport.ProtocolVersion;
@@ -35,10 +32,8 @@ import org.apache.cassandra.transport.ProtocolVersion;
  */
 public abstract class AggregateFcts
 {
-    public static Collection<AggregateFunction> all()
+    public static void addFunctionsTo(NativeFunctions functions)
     {
-        Collection<AggregateFunction> functions = new ArrayList<>();
-
         functions.add(countRowsFunction);
 
         // sum for primitives
@@ -63,69 +58,87 @@ public abstract class AggregateFcts
         functions.add(avgFunctionForVarint);
         functions.add(avgFunctionForCounter);
 
-        // count, max, and min for all standard types
-        for (CQL3Type type : CQL3Type.Native.values())
-        {
-            if (type != CQL3Type.Native.VARCHAR) // varchar and text both mapping to UTF8Type
-            {
-                functions.add(AggregateFcts.makeCountFunction(type.getType()));
-                if (type != CQL3Type.Native.COUNTER)
-                {
-                    functions.add(AggregateFcts.makeMaxFunction(type.getType()));
-                    functions.add(AggregateFcts.makeMinFunction(type.getType()));
-                }
-                else
-                {
-                    functions.add(AggregateFcts.maxFunctionForCounter);
-                    functions.add(AggregateFcts.minFunctionForCounter);
-                }
-            }
-        }
+        // count for all types
+        functions.add(makeCountFunction(BytesType.instance));
 
-        return functions;
+        // max for all types
+        functions.add(new FunctionFactory("max", FunctionParameter.anyType(true))
+        {
+            @Override
+            protected NativeFunction doGetOrCreateFunction(List<AbstractType<?>> argTypes, AbstractType<?> receiverType)
+            {
+                AbstractType<?> type = argTypes.get(0);
+                return type.isCounter() ? maxFunctionForCounter : makeMaxFunction(type);
+            }
+        });
+
+        // min for all types
+        functions.add(new FunctionFactory("min", FunctionParameter.anyType(true))
+        {
+            @Override
+            protected NativeFunction doGetOrCreateFunction(List<AbstractType<?>> argTypes, AbstractType<?> receiverType)
+            {
+                AbstractType<?> type = argTypes.get(0);
+                return type.isCounter() ? minFunctionForCounter : makeMinFunction(type);
+            }
+        });
     }
 
     /**
      * The function used to count the number of rows of a result set. This function is called when COUNT(*) or COUNT(1)
      * is specified.
      */
-    public static final AggregateFunction countRowsFunction =
-            new NativeAggregateFunction("countRows", LongType.instance)
+    public static final CountRowsFunction countRowsFunction = new CountRowsFunction(false);
+
+    public static class CountRowsFunction extends NativeAggregateFunction
+    {
+        private CountRowsFunction(boolean useLegacyName)
+        {
+            super(useLegacyName ? "countRows" : "count_rows", LongType.instance);
+        }
+
+        @Override
+        public Aggregate newAggregate()
+        {
+            return new Aggregate()
             {
-                public Aggregate newAggregate()
+                private long count;
+
+                public void reset()
                 {
-                    return new Aggregate()
-                    {
-                        private long count;
+                    count = 0;
+                }
 
-                        public void reset()
-                        {
-                            count = 0;
-                        }
-
-                        public ByteBuffer compute(ProtocolVersion protocolVersion)
-                        {
-                            return LongType.instance.decompose(count);
-                        }
-
-                        public void addInput(ProtocolVersion protocolVersion, List<ByteBuffer> values)
-                        {
-                            count++;
-                        }
-                    };
+                public ByteBuffer compute(ProtocolVersion protocolVersion)
+                {
+                    return LongType.instance.decompose(count);
                 }
 
                 @Override
-                public String columnName(List<String> columnNames)
+                public void addInput(Arguments arguments)
                 {
-                    return "count";
+                    count++;
                 }
             };
+        }
+
+        @Override
+        public String columnName(List<String> columnNames)
+        {
+            return "count";
+        }
+
+        @Override
+        public NativeFunction withLegacyName()
+        {
+            return new CountRowsFunction(true);
+        }
+    }
 
     /**
      * The SUM function for decimal values.
      */
-    public static final AggregateFunction sumFunctionForDecimal =
+    public static final NativeAggregateFunction sumFunctionForDecimal =
             new NativeAggregateFunction("sum", DecimalType.instance, DecimalType.instance)
             {
                 @Override
@@ -145,14 +158,14 @@ public abstract class AggregateFcts
                             return ((DecimalType) returnType()).decompose(sum);
                         }
 
-                        public void addInput(ProtocolVersion protocolVersion, List<ByteBuffer> values)
+                        @Override
+                        public void addInput(Arguments arguments)
                         {
-                            ByteBuffer value = values.get(0);
+                            BigDecimal number = arguments.get(0);
 
-                            if (value == null)
+                            if (number == null)
                                 return;
 
-                            BigDecimal number = DecimalType.instance.compose(value);
                             sum = sum.add(number);
                         }
                     };
@@ -161,8 +174,10 @@ public abstract class AggregateFcts
 
     /**
      * The AVG function for decimal values.
+     * </p>
+     * The average of an empty value set returns zero.
      */
-    public static final AggregateFunction avgFunctionForDecimal =
+    public static final NativeAggregateFunction avgFunctionForDecimal =
             new NativeAggregateFunction("avg", DecimalType.instance, DecimalType.instance)
             {
                 public Aggregate newAggregate()
@@ -184,15 +199,15 @@ public abstract class AggregateFcts
                             return DecimalType.instance.decompose(avg);
                         }
 
-                        public void addInput(ProtocolVersion protocolVersion, List<ByteBuffer> values)
+                        @Override
+                        public void addInput(Arguments arguments)
                         {
-                            ByteBuffer value = values.get(0);
+                            BigDecimal number = arguments.get(0);
 
-                            if (value == null)
+                            if (number == null)
                                 return;
 
                             count++;
-                            BigDecimal number = DecimalType.instance.compose(value);
 
                             // avg = avg + (value - sum) / count.
                             avg = avg.add(number.subtract(avg).divide(BigDecimal.valueOf(count), RoundingMode.HALF_EVEN));
@@ -205,7 +220,7 @@ public abstract class AggregateFcts
     /**
      * The SUM function for varint values.
      */
-    public static final AggregateFunction sumFunctionForVarint =
+    public static final NativeAggregateFunction sumFunctionForVarint =
             new NativeAggregateFunction("sum", IntegerType.instance, IntegerType.instance)
             {
                 public Aggregate newAggregate()
@@ -224,14 +239,14 @@ public abstract class AggregateFcts
                             return ((IntegerType) returnType()).decompose(sum);
                         }
 
-                        public void addInput(ProtocolVersion protocolVersion, List<ByteBuffer> values)
+                        @Override
+                        public void addInput(Arguments arguments)
                         {
-                            ByteBuffer value = values.get(0);
+                            BigInteger number = arguments.get(0);
 
-                            if (value == null)
+                            if (number == null)
                                 return;
 
-                            BigInteger number = IntegerType.instance.compose(value);
                             sum = sum.add(number);
                         }
                     };
@@ -240,8 +255,11 @@ public abstract class AggregateFcts
 
     /**
      * The AVG function for varint values.
+     * </p>
+     * The average of an empty value set returns zero. The returned value is of the same type as the input values, 
+     * so the returned average won't have a decimal part.
      */
-    public static final AggregateFunction avgFunctionForVarint =
+    public static final NativeAggregateFunction avgFunctionForVarint =
             new NativeAggregateFunction("avg", IntegerType.instance, IntegerType.instance)
             {
                 public Aggregate newAggregate()
@@ -266,15 +284,15 @@ public abstract class AggregateFcts
                             return IntegerType.instance.decompose(sum.divide(BigInteger.valueOf(count)));
                         }
 
-                        public void addInput(ProtocolVersion protocolVersion, List<ByteBuffer> values)
+                        @Override
+                        public void addInput(Arguments arguments)
                         {
-                            ByteBuffer value = values.get(0);
+                            BigInteger number = arguments.get(0);
 
-                            if (value == null)
+                            if (number == null)
                                 return;
 
                             count++;
-                            BigInteger number = ((BigInteger) argTypes().get(0).compose(value));
                             sum = sum.add(number);
                         }
                     };
@@ -283,8 +301,11 @@ public abstract class AggregateFcts
 
     /**
      * The SUM function for byte values (tinyint).
+     * </p>
+     * The returned value is of the same type as the input values, so there is a risk of overflow if the sum of the
+     * values exceeds the maximum value that the type can represent.
      */
-    public static final AggregateFunction sumFunctionForByte =
+    public static final NativeAggregateFunction sumFunctionForByte =
             new NativeAggregateFunction("sum", ByteType.instance, ByteType.instance)
             {
                 public Aggregate newAggregate()
@@ -303,14 +324,14 @@ public abstract class AggregateFcts
                             return ((ByteType) returnType()).decompose(sum);
                         }
 
-                        public void addInput(ProtocolVersion protocolVersion, List<ByteBuffer> values)
+                        @Override
+                        public void addInput(Arguments arguments)
                         {
-                            ByteBuffer value = values.get(0);
+                            Number number = arguments.get(0);
 
-                            if (value == null)
+                            if (number == null)
                                 return;
 
-                            Number number = ((Number) argTypes().get(0).compose(value));
                             sum += number.byteValue();
                         }
                     };
@@ -319,13 +340,16 @@ public abstract class AggregateFcts
 
     /**
      * AVG function for byte values (tinyint).
+     * </p>
+     * The average of an empty value set returns zero. The returned value is of the same type as the input values, 
+     * so the returned average won't have a decimal part.
      */
-    public static final AggregateFunction avgFunctionForByte =
+    public static final NativeAggregateFunction avgFunctionForByte =
             new NativeAggregateFunction("avg", ByteType.instance, ByteType.instance)
             {
                 public Aggregate newAggregate()
                 {
-                    return new AvgAggregate(ByteType.instance)
+                    return new AvgAggregate()
                     {
                         public ByteBuffer compute(ProtocolVersion protocolVersion) throws InvalidRequestException
                         {
@@ -337,8 +361,11 @@ public abstract class AggregateFcts
 
     /**
      * The SUM function for short values (smallint).
+     * </p>
+     * The returned value is of the same type as the input values, so there is a risk of overflow if the sum of the
+     * values exceeds the maximum value that the type can represent.
      */
-    public static final AggregateFunction sumFunctionForShort =
+    public static final NativeAggregateFunction sumFunctionForShort =
             new NativeAggregateFunction("sum", ShortType.instance, ShortType.instance)
             {
                 public Aggregate newAggregate()
@@ -357,14 +384,14 @@ public abstract class AggregateFcts
                             return ((ShortType) returnType()).decompose(sum);
                         }
 
-                        public void addInput(ProtocolVersion protocolVersion, List<ByteBuffer> values)
+                        @Override
+                        public void addInput(Arguments arguments)
                         {
-                            ByteBuffer value = values.get(0);
+                            Number number = arguments.get(0);
 
-                            if (value == null)
+                            if (number == null)
                                 return;
 
-                            Number number = ((Number) argTypes().get(0).compose(value));
                             sum += number.shortValue();
                         }
                     };
@@ -373,13 +400,16 @@ public abstract class AggregateFcts
 
     /**
      * AVG function for for short values (smallint).
+     * </p>
+     * The average of an empty value set returns zero. The returned value is of the same type as the input values, 
+     * so the returned average won't have a decimal part.
      */
-    public static final AggregateFunction avgFunctionForShort =
+    public static final NativeAggregateFunction avgFunctionForShort =
             new NativeAggregateFunction("avg", ShortType.instance, ShortType.instance)
             {
                 public Aggregate newAggregate()
                 {
-                    return new AvgAggregate(ShortType.instance)
+                    return new AvgAggregate()
                     {
                         public ByteBuffer compute(ProtocolVersion protocolVersion)
                         {
@@ -391,8 +421,11 @@ public abstract class AggregateFcts
 
     /**
      * The SUM function for int32 values.
+     * </p>
+     * The returned value is of the same type as the input values, so there is a risk of overflow if the sum of the
+     * values exceeds the maximum value that the type can represent.
      */
-    public static final AggregateFunction sumFunctionForInt32 =
+    public static final NativeAggregateFunction sumFunctionForInt32 =
             new NativeAggregateFunction("sum", Int32Type.instance, Int32Type.instance)
             {
                 public Aggregate newAggregate()
@@ -411,14 +444,14 @@ public abstract class AggregateFcts
                             return ((Int32Type) returnType()).decompose(sum);
                         }
 
-                        public void addInput(ProtocolVersion protocolVersion, List<ByteBuffer> values)
+                        @Override
+                        public void addInput(Arguments arguments)
                         {
-                            ByteBuffer value = values.get(0);
+                            Number number = arguments.get(0);
 
-                            if (value == null)
+                            if (number == null)
                                 return;
 
-                            Number number = ((Number) argTypes().get(0).compose(value));
                             sum += number.intValue();
                         }
                     };
@@ -427,13 +460,16 @@ public abstract class AggregateFcts
 
     /**
      * AVG function for int32 values.
+     * </p>
+     * The average of an empty value set returns zero. The returned value is of the same type as the input values, 
+     * so the returned average won't have a decimal part.
      */
-    public static final AggregateFunction avgFunctionForInt32 =
+    public static final NativeAggregateFunction avgFunctionForInt32 =
             new NativeAggregateFunction("avg", Int32Type.instance, Int32Type.instance)
             {
                 public Aggregate newAggregate()
                 {
-                    return new AvgAggregate(Int32Type.instance)
+                    return new AvgAggregate()
                     {
                         public ByteBuffer compute(ProtocolVersion protocolVersion)
                         {
@@ -445,8 +481,11 @@ public abstract class AggregateFcts
 
     /**
      * The SUM function for long values.
+     * </p>
+     * The returned value is of the same type as the input values, so there is a risk of overflow if the sum of the
+     * values exceeds the maximum value that the type can represent.
      */
-    public static final AggregateFunction sumFunctionForLong =
+    public static final NativeAggregateFunction sumFunctionForLong =
             new NativeAggregateFunction("sum", LongType.instance, LongType.instance)
             {
                 public Aggregate newAggregate()
@@ -457,13 +496,16 @@ public abstract class AggregateFcts
 
     /**
      * AVG function for long values.
+     * </p>
+     * The average of an empty value set returns zero. The returned value is of the same type as the input values, 
+     * so the returned average won't have a decimal part.
      */
-    public static final AggregateFunction avgFunctionForLong =
+    public static final NativeAggregateFunction avgFunctionForLong =
             new NativeAggregateFunction("avg", LongType.instance, LongType.instance)
             {
                 public Aggregate newAggregate()
                 {
-                    return new AvgAggregate(LongType.instance)
+                    return new AvgAggregate()
                     {
                         public ByteBuffer compute(ProtocolVersion protocolVersion)
                         {
@@ -475,13 +517,16 @@ public abstract class AggregateFcts
 
     /**
      * The SUM function for float values.
+     * </p>
+     * The returned value is of the same type as the input values, so there is a risk of overflow if the sum of the
+     * values exceeds the maximum value that the type can represent.
      */
-    public static final AggregateFunction sumFunctionForFloat =
+    public static final NativeAggregateFunction sumFunctionForFloat =
             new NativeAggregateFunction("sum", FloatType.instance, FloatType.instance)
             {
                 public Aggregate newAggregate()
                 {
-                    return new FloatSumAggregate(FloatType.instance)
+                    return new FloatSumAggregate()
                     {
                         public ByteBuffer compute(ProtocolVersion protocolVersion) throws InvalidRequestException
                         {
@@ -493,13 +538,15 @@ public abstract class AggregateFcts
 
     /**
      * AVG function for float values.
+     * </p>
+     * The average of an empty value set returns zero.
      */
-    public static final AggregateFunction avgFunctionForFloat =
+    public static final NativeAggregateFunction avgFunctionForFloat =
             new NativeAggregateFunction("avg", FloatType.instance, FloatType.instance)
             {
                 public Aggregate newAggregate()
                 {
-                    return new FloatAvgAggregate(FloatType.instance)
+                    return new FloatAvgAggregate()
                     {
                         public ByteBuffer compute(ProtocolVersion protocolVersion) throws InvalidRequestException
                         {
@@ -511,13 +558,16 @@ public abstract class AggregateFcts
 
     /**
      * The SUM function for double values.
+     * </p>
+     * The returned value is of the same type as the input values, so there is a risk of overflow if the sum of the
+     * values exceeds the maximum value that the type can represent.
      */
-    public static final AggregateFunction sumFunctionForDouble =
+    public static final NativeAggregateFunction sumFunctionForDouble =
             new NativeAggregateFunction("sum", DoubleType.instance, DoubleType.instance)
             {
                 public Aggregate newAggregate()
                 {
-                    return new FloatSumAggregate(DoubleType.instance)
+                    return new FloatSumAggregate()
                     {
                         public ByteBuffer compute(ProtocolVersion protocolVersion) throws InvalidRequestException
                         {
@@ -537,13 +587,6 @@ public abstract class AggregateFcts
         private double compensation;
         private double simpleSum;
 
-        private final AbstractType numberType;
-
-        public FloatSumAggregate(AbstractType numberType)
-        {
-            this.numberType = numberType;
-        }
-
         public void reset()
         {
             sum = 0;
@@ -551,16 +594,18 @@ public abstract class AggregateFcts
             simpleSum = 0;
         }
 
-        public void addInput(ProtocolVersion protocolVersion, List<ByteBuffer> values)
+        @Override
+        public void addInput(Arguments arguments)
         {
-            ByteBuffer value = values.get(0);
+            Number number = arguments.get(0);
 
-            if (value == null)
+            if (number == null)
                 return;
 
-            double number = ((Number) numberType.compose(value)).doubleValue();
-            simpleSum += number;
-            double tmp = number - compensation;
+            double d = number.doubleValue();
+
+            simpleSum += d;
+            double tmp = d - compensation;
             double rounded = sum + tmp;
             compensation = (rounded - sum) - tmp;
             sum = rounded;
@@ -595,13 +640,6 @@ public abstract class AggregateFcts
         private BigDecimal bigSum = null;
         private boolean overflow = false;
 
-        private final AbstractType numberType;
-
-        public FloatAvgAggregate(AbstractType numberType)
-        {
-            this.numberType = numberType;
-        }
-
         public void reset()
         {
             sum = 0;
@@ -636,34 +674,35 @@ public abstract class AggregateFcts
             }
         }
 
-        public void addInput(ProtocolVersion protocolVersion, List<ByteBuffer> values)
+        @Override
+        public void addInput(Arguments arguments)
         {
-            ByteBuffer value = values.get(0);
+            Number number = arguments.get(0);
 
-            if (value == null)
+            if (number == null)
                 return;
 
             count++;
 
-            double number = ((Number) numberType.compose(value)).doubleValue();
+            double d = number.doubleValue();
 
             if (overflow)
             {
-                bigSum = bigSum.add(BigDecimal.valueOf(number));
+                bigSum = bigSum.add(BigDecimal.valueOf(d));
             }
             else
             {
-                simpleSum += number;
+                simpleSum += d;
                 double prev = sum;
-                double tmp = number - compensation;
+                double tmp = d - compensation;
                 double rounded = sum + tmp;
                 compensation = (rounded - sum) - tmp;
                 sum = rounded;
 
-                if (Double.isInfinite(sum) && !Double.isInfinite(number))
+                if (Double.isInfinite(sum) && !Double.isInfinite(d))
                 {
                     overflow = true;
-                    bigSum = BigDecimal.valueOf(prev).add(BigDecimal.valueOf(number));
+                    bigSum = BigDecimal.valueOf(prev).add(BigDecimal.valueOf(d));
                 }
             }
         }
@@ -671,13 +710,15 @@ public abstract class AggregateFcts
 
     /**
      * AVG function for double values.
+     * </p>
+     * The average of an empty value set returns zero.
      */
-    public static final AggregateFunction avgFunctionForDouble =
+    public static final NativeAggregateFunction avgFunctionForDouble =
             new NativeAggregateFunction("avg", DoubleType.instance, DoubleType.instance)
             {
                 public Aggregate newAggregate()
                 {
-                    return new FloatAvgAggregate(DoubleType.instance)
+                    return new FloatAvgAggregate()
                     {
                         public ByteBuffer compute(ProtocolVersion protocolVersion) throws InvalidRequestException
                         {
@@ -690,7 +731,7 @@ public abstract class AggregateFcts
     /**
      * The SUM function for counter column values.
      */
-    public static final AggregateFunction sumFunctionForCounter =
+    public static final NativeAggregateFunction sumFunctionForCounter =
     new NativeAggregateFunction("sum", CounterColumnType.instance, CounterColumnType.instance)
     {
         public Aggregate newAggregate()
@@ -702,12 +743,12 @@ public abstract class AggregateFcts
     /**
      * AVG function for counter column values.
      */
-    public static final AggregateFunction avgFunctionForCounter =
+    public static final NativeAggregateFunction avgFunctionForCounter =
     new NativeAggregateFunction("avg", CounterColumnType.instance, CounterColumnType.instance)
     {
         public Aggregate newAggregate()
         {
-            return new AvgAggregate(LongType.instance)
+            return new AvgAggregate()
             {
                 public ByteBuffer compute(ProtocolVersion protocolVersion) throws InvalidRequestException
                 {
@@ -720,7 +761,7 @@ public abstract class AggregateFcts
     /**
      * The MIN function for counter column values.
      */
-    public static final AggregateFunction minFunctionForCounter =
+    public static final NativeAggregateFunction minFunctionForCounter =
     new NativeAggregateFunction("min", CounterColumnType.instance, CounterColumnType.instance)
     {
         public Aggregate newAggregate()
@@ -739,14 +780,15 @@ public abstract class AggregateFcts
                     return min != null ? LongType.instance.decompose(min) : null;
                 }
 
-                public void addInput(ProtocolVersion protocolVersion, List<ByteBuffer> values)
+                @Override
+                public void addInput(Arguments arguments)
                 {
-                    ByteBuffer value = values.get(0);
+                    Number number = arguments.get(0);
 
-                    if (value == null)
+                    if (number == null)
                         return;
 
-                    long lval = LongType.instance.compose(value);
+                    long lval = number.longValue();
 
                     if (min == null || lval < min)
                         min = lval;
@@ -758,7 +800,7 @@ public abstract class AggregateFcts
     /**
      * MAX function for counter column values.
      */
-    public static final AggregateFunction maxFunctionForCounter =
+    public static final NativeAggregateFunction maxFunctionForCounter =
     new NativeAggregateFunction("max", CounterColumnType.instance, CounterColumnType.instance)
     {
         public Aggregate newAggregate()
@@ -777,14 +819,15 @@ public abstract class AggregateFcts
                     return max != null ? LongType.instance.decompose(max) : null;
                 }
 
-                public void addInput(ProtocolVersion protocolVersion, List<ByteBuffer> values)
+                @Override
+                public void addInput(Arguments arguments)
                 {
-                    ByteBuffer value = values.get(0);
+                    Number number = arguments.get(0);
 
-                    if (value == null)
+                    if (number == null)
                         return;
 
-                    long lval = LongType.instance.compose(value);
+                    long lval = number.longValue();
 
                     if (max == null || lval > max)
                         max = lval;
@@ -799,10 +842,17 @@ public abstract class AggregateFcts
      * @param inputType the function input and output type
      * @return a MAX function for the specified type.
      */
-    public static AggregateFunction makeMaxFunction(final AbstractType<?> inputType)
+    public static NativeAggregateFunction makeMaxFunction(final AbstractType<?> inputType)
     {
         return new NativeAggregateFunction("max", inputType, inputType)
         {
+            @Override
+            public Arguments newArguments(ProtocolVersion version)
+            {
+                return FunctionArguments.newNoopInstance(version, 1);
+            }
+
+            @Override
             public Aggregate newAggregate()
             {
                 return new Aggregate()
@@ -819,9 +869,10 @@ public abstract class AggregateFcts
                         return max;
                     }
 
-                    public void addInput(ProtocolVersion protocolVersion, List<ByteBuffer> values)
+                    @Override
+                    public void addInput(Arguments arguments)
                     {
-                        ByteBuffer value = values.get(0);
+                        ByteBuffer value = arguments.get(0);
 
                         if (value == null)
                             return;
@@ -840,10 +891,17 @@ public abstract class AggregateFcts
      * @param inputType the function input and output type
      * @return a MIN function for the specified type.
      */
-    public static AggregateFunction makeMinFunction(final AbstractType<?> inputType)
+    public static NativeAggregateFunction makeMinFunction(final AbstractType<?> inputType)
     {
         return new NativeAggregateFunction("min", inputType, inputType)
         {
+            @Override
+            public Arguments newArguments(ProtocolVersion version)
+            {
+                return FunctionArguments.newNoopInstance(version, 1);
+            }
+
+            @Override
             public Aggregate newAggregate()
             {
                 return new Aggregate()
@@ -860,9 +918,10 @@ public abstract class AggregateFcts
                         return min;
                     }
 
-                    public void addInput(ProtocolVersion protocolVersion, List<ByteBuffer> values)
+                    @Override
+                    public void addInput(Arguments arguments)
                     {
-                        ByteBuffer value = values.get(0);
+                        ByteBuffer value = arguments.get(0);
 
                         if (value == null)
                             return;
@@ -881,10 +940,17 @@ public abstract class AggregateFcts
      * @param inputType the function input type
      * @return a COUNT function for the specified type.
      */
-    public static AggregateFunction makeCountFunction(AbstractType<?> inputType)
+    public static NativeAggregateFunction makeCountFunction(AbstractType<?> inputType)
     {
         return new NativeAggregateFunction("count", LongType.instance, inputType)
         {
+            @Override
+            public Arguments newArguments(ProtocolVersion version)
+            {
+                return FunctionArguments.newNoopInstance(version, 1);
+            }
+
+            @Override
             public Aggregate newAggregate()
             {
                 return new Aggregate()
@@ -901,11 +967,10 @@ public abstract class AggregateFcts
                         return ((LongType) returnType()).decompose(count);
                     }
 
-                    public void addInput(ProtocolVersion protocolVersion, List<ByteBuffer> values)
+                    @Override
+                    public void addInput(Arguments arguments)
                     {
-                        ByteBuffer value = values.get(0);
-
-                        if (value == null)
+                        if (arguments.get(0) == null)
                             return;
 
                         count++;
@@ -929,14 +994,14 @@ public abstract class AggregateFcts
             return LongType.instance.decompose(sum);
         }
 
-        public void addInput(ProtocolVersion protocolVersion, List<ByteBuffer> values)
+        @Override
+        public void addInput(Arguments arguments)
         {
-            ByteBuffer value = values.get(0);
+            Number number = arguments.get(0);
 
-            if (value == null)
+            if (number == null)
                 return;
 
-            Number number = LongType.instance.compose(value);
             sum += number.longValue();
         }
     }
@@ -953,13 +1018,6 @@ public abstract class AggregateFcts
         private BigInteger bigSum = null;
         private boolean overflow = false;
 
-        private final AbstractType numberType;
-
-        public AvgAggregate(AbstractType type)
-        {
-            this.numberType = type;
-        }
-
         public void reset()
         {
             count = 0;
@@ -980,28 +1038,30 @@ public abstract class AggregateFcts
             }
         }
 
-        public void addInput(ProtocolVersion protocolVersion, List<ByteBuffer> values)
+        @Override
+        public void addInput(Arguments arguments)
         {
-            ByteBuffer value = values.get(0);
+            Number number = arguments.get(0);
 
-            if (value == null)
+            if (number == null)
                 return;
 
             count++;
-            long number = ((Number) numberType.compose(value)).longValue();
+            long l = number.longValue();
+
             if (overflow)
             {
-                bigSum = bigSum.add(BigInteger.valueOf(number));
+                bigSum = bigSum.add(BigInteger.valueOf(l));
             }
             else
             {
                 long prev = sum;
-                sum += number;
+                sum += l;
 
-                if (((prev ^ sum) & (number ^ sum)) < 0)
+                if (((prev ^ sum) & (l ^ sum)) < 0)
                 {
                     overflow = true;
-                    bigSum = BigInteger.valueOf(prev).add(BigInteger.valueOf(number));
+                    bigSum = BigInteger.valueOf(prev).add(BigInteger.valueOf(l));
                 }
             }
         }
