@@ -37,19 +37,16 @@ import java.util.concurrent.ThreadLocalRandom;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.regex.Pattern;
 
-import com.google.common.annotations.VisibleForTesting;
 import com.google.common.io.ByteStreams;
-import com.google.common.reflect.TypeToken;
 
+import org.apache.commons.lang3.StringUtils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import org.apache.cassandra.concurrent.NamedThreadFactory;
 import org.apache.cassandra.cql3.ColumnIdentifier;
-import org.apache.cassandra.cql3.functions.types.TypeCodec;
 import org.apache.cassandra.db.marshal.AbstractType;
 import org.apache.cassandra.exceptions.InvalidRequestException;
-import org.apache.cassandra.transport.ProtocolVersion;
 import org.apache.cassandra.utils.FBUtilities;
 import org.apache.cassandra.security.SecurityThreadGroup;
 import org.apache.cassandra.security.ThreadAwareSecurityManager;
@@ -62,14 +59,14 @@ import org.eclipse.jdt.internal.compiler.env.ICompilationUnit;
 import org.eclipse.jdt.internal.compiler.env.INameEnvironment;
 import org.eclipse.jdt.internal.compiler.env.NameEnvironmentAnswer;
 import org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
+import org.eclipse.jdt.internal.compiler.lookup.LookupEnvironment;
+import org.eclipse.jdt.internal.compiler.lookup.ModuleBinding;
 import org.eclipse.jdt.internal.compiler.problem.DefaultProblemFactory;
 
 public final class JavaBasedUDFunction extends UDFunction
 {
     private static final String BASE_PACKAGE = "org.apache.cassandra.cql3.udf.gen";
 
-    private static final Pattern JAVA_LANG_PREFIX = Pattern.compile("\\bjava\\.lang\\.");
-
     private static final Logger logger = LoggerFactory.getLogger(JavaBasedUDFunction.class);
 
     private static final AtomicInteger classSequence = new AtomicInteger();
@@ -143,9 +140,9 @@ public final class JavaBasedUDFunction extends UDFunction
         settings.put(CompilerOptions.OPTION_ReportDeprecation,
                      CompilerOptions.IGNORE);
         settings.put(CompilerOptions.OPTION_Source,
-                     CompilerOptions.VERSION_1_8);
+                     CompilerOptions.VERSION_11);
         settings.put(CompilerOptions.OPTION_TargetPlatform,
-                     CompilerOptions.VERSION_1_8);
+                     CompilerOptions.VERSION_11);
 
         compilerOptions = new CompilerOptions(settings);
         compilerOptions.parseLiteralExpressionsAsConstants = true;
@@ -192,13 +189,7 @@ public final class JavaBasedUDFunction extends UDFunction
     JavaBasedUDFunction(FunctionName name, List<ColumnIdentifier> argNames, List<AbstractType<?>> argTypes,
                         AbstractType<?> returnType, boolean calledOnNullInput, String body)
     {
-        super(name, argNames, argTypes, UDHelper.driverTypes(argTypes),
-              returnType, UDHelper.driverType(returnType), calledOnNullInput, "java", body);
-
-        // javaParamTypes is just the Java representation for argTypes resp. argDataTypes
-        TypeToken<?>[] javaParamTypes = UDHelper.typeTokens(argCodecs, calledOnNullInput);
-        // javaReturnType is just the Java representation for returnType resp. returnTypeCodec
-        TypeToken<?> javaReturnType = returnCodec.getJavaType();
+        super(name, argNames, argTypes, returnType, calledOnNullInput, "java", body);
 
         // put each UDF in a separate package to prevent cross-UDF code access
         String pkgName = BASE_PACKAGE + '.' + generateClassName(name, 'p');
@@ -228,16 +219,16 @@ public final class JavaBasedUDFunction extends UDFunction
                         s = patternJavaDriver.matcher(body).replaceAll("org.apache.cassandra.cql3.functions.types.");
                         break;
                     case "arguments":
-                        s = generateArguments(javaParamTypes, argNames, false);
+                        s = generateArguments(argumentTypes, argNames, false);
                         break;
                     case "arguments_aggregate":
-                        s = generateArguments(javaParamTypes, argNames, true);
+                        s = generateArguments(argumentTypes, argNames, true);
                         break;
                     case "argument_list":
-                        s = generateArgumentList(javaParamTypes, argNames);
+                        s = generateArgumentList(argumentTypes, argNames);
                         break;
                     case "return_type":
-                        s = javaSourceName(javaReturnType);
+                        s = resultType.getJavaTypeName();
                         break;
                     case "execute_internal_name":
                         s = executeInternalName;
@@ -338,9 +329,9 @@ public final class JavaBasedUDFunction extends UDFunction
                 if (nonSyntheticMethodCount != 3 || cls.getDeclaredConstructors().length != 1)
                     throw new InvalidRequestException("Check your source to not define additional Java methods or constructors");
                 MethodType methodType = MethodType.methodType(void.class)
-                                                  .appendParameterTypes(TypeCodec.class, TypeCodec[].class, UDFContext.class);
+                                                  .appendParameterTypes(UDFDataType.class, UDFContext.class);
                 MethodHandle ctor = MethodHandles.lookup().findConstructor(cls, methodType);
-                this.javaUDF = (JavaUDF) ctor.invokeWithArguments(returnCodec, argCodecs, udfContext);
+                this.javaUDF = (JavaUDF) ctor.invokeWithArguments(resultType, udfContext);
             }
             finally
             {
@@ -364,19 +355,22 @@ public final class JavaBasedUDFunction extends UDFunction
         }
     }
 
+    @Override
     protected ExecutorService executor()
     {
         return executor;
     }
 
-    protected ByteBuffer executeUserDefined(ProtocolVersion protocolVersion, List<ByteBuffer> params)
+    @Override
+    protected ByteBuffer executeUserDefined(Arguments arguments)
     {
-        return javaUDF.executeImpl(protocolVersion, params);
+        return javaUDF.executeImpl(arguments);
     }
 
-    protected Object executeAggregateUserDefined(ProtocolVersion protocolVersion, Object firstParam, List<ByteBuffer> params)
+    @Override
+    protected Object executeAggregateUserDefined(Object state, Arguments arguments)
     {
-        return javaUDF.executeAggregateImpl(protocolVersion, firstParam, params);
+        return javaUDF.executeAggregateImpl(state, arguments);
     }
 
     private static int countNewlines(StringBuilder javaSource)
@@ -409,22 +403,15 @@ public final class JavaBasedUDFunction extends UDFunction
         return sb.toString();
     }
 
-    @VisibleForTesting
-    public static String javaSourceName(TypeToken<?> type)
-    {
-        String n = type.toString();
-        return JAVA_LANG_PREFIX.matcher(n).replaceAll("");
-    }
-
-    private static String generateArgumentList(TypeToken<?>[] paramTypes, List<ColumnIdentifier> argNames)
+    private static String generateArgumentList(List<UDFDataType> argTypes, List<ColumnIdentifier> argNames)
     {
         // initial builder size can just be a guess (prevent temp object allocations)
-        StringBuilder code = new StringBuilder(32 * paramTypes.length);
-        for (int i = 0; i < paramTypes.length; i++)
+        StringBuilder code = new StringBuilder(32 * argTypes.size());
+        for (int i = 0; i < argTypes.size(); i++)
         {
             if (i > 0)
                 code.append(", ");
-            code.append(javaSourceName(paramTypes[i]))
+            code.append(argTypes.get(i).getJavaTypeName())
                 .append(' ')
                 .append(argNames.get(i));
         }
@@ -450,36 +437,56 @@ public final class JavaBasedUDFunction extends UDFunction
      * {@code firstParam, (double) super.compose_double(protocolVersion, 1, params.get(1))}
      * </p>
      */
-    private static String generateArguments(TypeToken<?>[] paramTypes, List<ColumnIdentifier> argNames, boolean forAggregate)
+    private static String generateArguments(List<UDFDataType> argTypes, List<ColumnIdentifier> argNames, boolean forAggregate)
     {
-        StringBuilder code = new StringBuilder(64 * paramTypes.length);
-        for (int i = 0; i < paramTypes.length; i++)
+        int size = argTypes.size();
+        StringBuilder code = new StringBuilder(64 * size);
+        for (int i = 0; i < size; i++)
         {
+            UDFDataType argType = argTypes.get(i);
             if (i > 0)
                 // add separator, if not the first argument
                 code.append(",\n");
 
             // add comment only if trace is enabled
             if (logger.isTraceEnabled())
-                code.append("            /* parameter '").append(argNames.get(i)).append("' */\n");
+                code.append("            /* argument '").append(argNames.get(i)).append("' */\n");
+
+
+            code.append("            ");
 
             // cast to Java type
-            code.append("            (").append(javaSourceName(paramTypes[i])).append(") ");
+            code.append('(').append(argType.getJavaTypeName()).append(") ");
 
             if (forAggregate && i == 0)
+            {
                 // special case for aggregations where the state variable (1st arg to state + final function and
                 // return value from state function) is not re-serialized
-                code.append("firstParam");
+                code.append("state");
+            }
             else
-                // generate object representation of input parameter (call UDFunction.compose)
-                code.append(composeMethod(paramTypes[i])).append("(protocolVersion, ").append(i).append(", params.get(").append(forAggregate ? i - 1 : i).append("))");
+            {
+                // generate object representation of input parameter
+                code.append("arguments.");
+                appendGetMethodName(code, argType).append('(').append(forAggregate ? i - 1 : i).append(')');
+            }
         }
         return code.toString();
     }
 
-    private static String composeMethod(TypeToken<?> type)
+    /**
+     * Appends the get method name for the specified type to avoid boxing.
+     * @param code the builder to append to
+     * @param type the data type
+     * @return the builder
+     */
+    private static StringBuilder appendGetMethodName(StringBuilder code, UDFDataType type)
     {
-        return (type.isPrimitive()) ? ("super.compose_" + type.getRawType().getName()) : "super.compose";
+        code.append("get");
+        if (!type.isPrimitive())
+            return code;
+
+        return code.append("As").append(StringUtils.capitalize(type.getJavaTypeName()));
     }
 
     // Java source UDFs are a very simple compilation task, which allows us to let one class implement
@@ -533,6 +540,30 @@ public final class JavaBasedUDFunction extends UDFunction
             return false;
         }
 
+        @Override
+        public ModuleBinding module(LookupEnvironment environment)
+        {
+            return environment.getModule(this.getModuleName());
+        }
+
+        @Override
+        public char[] getModuleName()
+        {
+            return null;
+        }
+
+        @Override
+        public String getDestinationPath()
+        {
+            return null;
+        }
+
+        @Override
+        public String getExternalAnnotationPath(String qualifiedTypeName)
+        {
+            return null;
+        }
+
         // ICompilerRequestor
 
         @Override
@@ -585,7 +616,6 @@ public final class JavaBasedUDFunction extends UDFunction
             return findType(result.toString());
         }
 
-        @SuppressWarnings("resource")
         private NameEnvironmentAnswer findType(String className)
         {
             if (className.equals(this.className))
@@ -696,4 +726,5 @@ public final class JavaBasedUDFunction extends UDFunction
         {
             return ThreadAwareSecurityManager.noPermissions;
         }
-    }}
+    }
+}
