@@ -26,14 +26,12 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import org.apache.cassandra.cql3.CqlBuilder;
-import org.apache.cassandra.cql3.SchemaElement;
-import org.apache.cassandra.cql3.functions.types.TypeCodec;
 import org.apache.cassandra.db.marshal.AbstractType;
 import org.apache.cassandra.db.marshal.UserType;
 import org.apache.cassandra.exceptions.ConfigurationException;
 import org.apache.cassandra.exceptions.InvalidRequestException;
 import org.apache.cassandra.schema.Difference;
-import org.apache.cassandra.schema.Functions;
+import org.apache.cassandra.schema.UserFunctions;
 import org.apache.cassandra.tracing.Tracing;
 import org.apache.cassandra.transport.ProtocolVersion;
 
@@ -44,13 +42,13 @@ import static org.apache.cassandra.utils.Clock.Global.nanoTime;
 /**
  * Base class for user-defined-aggregates.
  */
-public class UDAggregate extends AbstractFunction implements AggregateFunction, SchemaElement
+public class UDAggregate extends UserFunction implements AggregateFunction
 {
     protected static final Logger logger = LoggerFactory.getLogger(UDAggregate.class);
 
-    private final AbstractType<?> stateType;
-    private final TypeCodec stateTypeCodec;
-    private final TypeCodec returnTypeCodec;
+    private final UDFDataType stateType;
+    private final List<UDFDataType> argumentTypes;
+    private final UDFDataType resultType;
     protected final ByteBuffer initcond;
     private final ScalarFunction stateFunction;
     private final ScalarFunction finalFunction;
@@ -65,9 +63,9 @@ public class UDAggregate extends AbstractFunction implements AggregateFunction,
         super(name, argTypes, returnType);
         this.stateFunction = stateFunc;
         this.finalFunction = finalFunc;
-        this.stateType = stateFunc.returnType();
-        this.stateTypeCodec = UDHelper.codecFor(UDHelper.driverType(stateType));
-        this.returnTypeCodec = UDHelper.codecFor(UDHelper.driverType(returnType));
+        this.argumentTypes = UDFDataType.wrap(argTypes, false);
+        this.resultType = UDFDataType.wrap(returnType, false);
+        this.stateType = stateFunc != null ? UDFDataType.wrap(stateFunc.returnType(), false) : null;
         this.initcond = initcond;
     }
 
@@ -95,7 +93,7 @@ public class UDAggregate extends AbstractFunction implements AggregateFunction,
     private static UDFunction findFunction(FunctionName udaName, Collection<UDFunction> functions, FunctionName name, List<AbstractType<?>> arguments)
     {
         return functions.stream()
-                        .filter(f -> f.name().equals(name) && Functions.typesMatch(f.argTypes(), arguments))
+                        .filter(f -> f.name().equals(name) && f.typesMatch(arguments))
                         .findFirst()
                         .orElseThrow(() -> new ConfigurationException(String.format("Unable to find function %s referenced by UDA %s", name, udaName)));
     }
@@ -106,6 +104,12 @@ public class UDAggregate extends AbstractFunction implements AggregateFunction,
         return false;
     }
 
+    @Override
+    public Arguments newArguments(ProtocolVersion version)
+    {
+        return FunctionArguments.newInstanceForUdf(version, argumentTypes);
+    }
+
     public boolean hasReferenceTo(Function function)
     {
         return stateFunction == function || finalFunction == function;
@@ -116,7 +120,7 @@ public class UDAggregate extends AbstractFunction implements AggregateFunction,
     {
         return any(argTypes(), t -> t.referencesUserType(name))
             || returnType.referencesUserType(name)
-            || (null != stateType && stateType.referencesUserType(name))
+            || (null != stateType && stateType.toAbstractType().referencesUserType(name))
             || stateFunction.referencesUserType(name)
             || (null != finalFunction && finalFunction.referencesUserType(name));
     }
@@ -150,11 +154,6 @@ public class UDAggregate extends AbstractFunction implements AggregateFunction,
         return true;
     }
 
-    public boolean isNative()
-    {
-        return false;
-    }
-
     public ScalarFunction stateFunction()
     {
         return stateFunction;
@@ -172,7 +171,7 @@ public class UDAggregate extends AbstractFunction implements AggregateFunction,
 
     public AbstractType<?> stateType()
     {
-        return stateType;
+        return stateType == null ? null : stateType.toAbstractType();
     }
 
     public Aggregate newAggregate() throws InvalidRequestException
@@ -185,17 +184,18 @@ public class UDAggregate extends AbstractFunction implements AggregateFunction,
             private Object state;
             private boolean needsInit = true;
 
-            public void addInput(ProtocolVersion protocolVersion, List<ByteBuffer> values) throws InvalidRequestException
+            @Override
+            public void addInput(Arguments arguments) throws InvalidRequestException
             {
-                maybeInit(protocolVersion);
+                maybeInit(arguments.getProtocolVersion());
 
                 long startTime = nanoTime();
                 stateFunctionCount++;
                 if (stateFunction instanceof UDFunction)
                 {
                     UDFunction udf = (UDFunction)stateFunction;
-                    if (udf.isCallableWrtNullable(values))
-                        state = udf.executeForAggregate(protocolVersion, state, values);
+                    if (udf.isCallableWrtNullable(arguments))
+                        state = udf.executeForAggregate(state, arguments);
                 }
                 else
                 {
@@ -208,7 +208,7 @@ public class UDAggregate extends AbstractFunction implements AggregateFunction,
             {
                 if (needsInit)
                 {
-                    state = initcond != null ? UDHelper.deserialize(stateTypeCodec, protocolVersion, initcond.duplicate()) : null;
+                    state = initcond != null ? stateType.compose(protocolVersion, initcond.duplicate()) : null;
                     stateFunctionDuration = 0;
                     stateFunctionCount = 0;
                     needsInit = false;
@@ -222,13 +222,13 @@ public class UDAggregate extends AbstractFunction implements AggregateFunction,
                 // final function is traced in UDFunction
                 Tracing.trace("Executed UDA {}: {} call(s) to state function {} in {}\u03bcs", name(), stateFunctionCount, stateFunction.name(), stateFunctionDuration);
                 if (finalFunction == null)
-                    return UDFunction.decompose(stateTypeCodec, protocolVersion, state);
+                    return stateType.decompose(protocolVersion, state);
 
                 if (finalFunction instanceof UDFunction)
                 {
                     UDFunction udf = (UDFunction)finalFunction;
-                    Object result = udf.executeForAggregate(protocolVersion, state, Collections.emptyList());
-                    return UDFunction.decompose(returnTypeCodec, protocolVersion, result);
+                    Object result = udf.executeForAggregate(state, FunctionArguments.emptyInstance(protocolVersion));
+                    return resultType.decompose(protocolVersion, result);
                 }
                 throw new UnsupportedOperationException("UDAs only support UDFs");
             }
@@ -287,7 +287,8 @@ public class UDAggregate extends AbstractFunction implements AggregateFunction,
 
         if (null != stateType && !stateType.equals(other.stateType))
         {
-            if (stateType.asCQL3Type().toString().equals(other.stateType.asCQL3Type().toString()))
+            if (stateType.toAbstractType().asCQL3Type().toString()
+                         .equals(other.stateType.toAbstractType().asCQL3Type().toString()))
                 differsDeeply = true;
             else
                 return Optional.of(Difference.SHALLOW);
@@ -329,7 +330,7 @@ public class UDAggregate extends AbstractFunction implements AggregateFunction,
     @Override
     public int hashCode()
     {
-        return Objects.hashCode(name, Functions.typeHashCode(argTypes), Functions.typeHashCode(returnType), stateFunction, finalFunction, stateType, initcond);
+        return Objects.hashCode(name, UserFunctions.typeHashCode(argTypes), UserFunctions.typeHashCode(returnType), stateFunction, finalFunction, stateType, initcond);
     }
 
     @Override
@@ -369,7 +370,7 @@ public class UDAggregate extends AbstractFunction implements AggregateFunction,
         if (initialCondition() != null)
             builder.newLine()
                    .append("INITCOND ")
-                   .append(stateType().asCQL3Type().toCQLLiteral(initialCondition(), ProtocolVersion.CURRENT));
+                   .append(stateType().asCQL3Type().toCQLLiteral(initialCondition()));
 
         return builder.append(";")
                       .toString();
