@@ -540,6 +540,22 @@ implements SettableData<T>
         return wrapped;
     }
 
+    @Override
+    public <E> T setVector(int i, List<E> v)
+    {
+        return setValue(i, codecFor(i).serialize(v, protocolVersion));
+    }
+
+    @Override
+    public <E> T setVector(String name, List<E> v)
+    {
+        for (int i : getAllIndexesOf(name))
+        {
+            setVector(i, v);
+        }
+        return wrapped;
+    }
+
     @Override
     public T setUDTValue(int i, UDTValue v)
     {
