@@ -22,6 +22,8 @@ import java.util.ArrayList;
 import java.util.List;
 
 import org.apache.cassandra.cql3.QualifiedName;
+import org.apache.cassandra.index.Index;
+import org.apache.cassandra.index.IndexRegistry;
 import org.apache.cassandra.schema.TableMetadata;
 import org.apache.cassandra.exceptions.InvalidRequestException;
 import org.apache.commons.lang3.builder.ToStringBuilder;
@@ -63,6 +65,49 @@ public class IndexRestrictions
         return customExpressions;
     }
 
+    /**
+     * Returns whether these restrictions would need filtering if the specified index registry were used.
+     *
+     * @param indexRegistry an index registry
+     * @return {@code true} if this would need filtering if {@code indexRegistry} were used, {@code false} otherwise
+     */
+    public boolean needsFiltering(IndexRegistry indexRegistry)
+    {
+        if (isEmpty())
+            return false;
+
+        for (Index.Group group : indexRegistry.listIndexGroups())
+        {
+            if (!needsFiltering(group))
+                return false;
+        }
+
+        return true;
+    }
+
+    /**
+     * Returns whether these restrictions would need filtering if the specified index group were used.
+     *
+     * @param indexGroup an index group
+     * @return {@code true} if this would need filtering if {@code indexGroup} were used, {@code false} otherwise
+     */
+    private boolean needsFiltering(Index.Group indexGroup)
+    {
+        for (Restrictions restrictions : regularRestrictions)
+        {
+            if (restrictions.needsFiltering(indexGroup))
+                return true;
+        }
+
+        for (CustomIndexExpression restriction : customExpressions)
+        {
+            if (restriction.needsFiltering(indexGroup))
+                return true;
+        }
+
+        return false;
+    }
+
     static InvalidRequestException invalidIndex(QualifiedName indexName, TableMetadata table)
     {
         return new InvalidRequestException(String.format(INVALID_INDEX, indexName.getName(), table));
