@@ -19,6 +19,7 @@ package org.apache.cassandra.cql3.restrictions;
 
 import java.util.List;
 
+import org.apache.cassandra.index.Index;
 import org.apache.cassandra.schema.ColumnMetadata;
 import org.apache.cassandra.cql3.QueryOptions;
 import org.apache.cassandra.cql3.functions.Function;
@@ -66,7 +67,30 @@ public interface Restriction
      * @param indexRegistry the index registry
      * @return <code>true</code> if the restriction is on indexed columns, <code>false</code>
      */
-    public boolean hasSupportingIndex(IndexRegistry indexRegistry);
+    boolean hasSupportingIndex(IndexRegistry indexRegistry);
+
+    /**
+     * Find first supporting index for current restriction
+     *
+     * @param indexRegistry the index registry
+     * @return <code>index</code> if the restriction is on indexed columns, <code>null</code>
+     */
+    Index findSupportingIndex(IndexRegistry indexRegistry);
+
+    /**
+     * Find the first supporting index for the current restriction from an {@link Index.QueryPlan}.
+     * @param indexQueryPlan the index query plan
+     * @return <code>index</code> if the restriction is on indexed columns, <code>null</code>
+     */
+    Index findSupportingIndexFromQueryPlan(Index.QueryPlan indexQueryPlan);
+
+    /**
+     * Returns whether this restriction would need filtering if the specified index group were used.
+     *
+     * @param indexGroup an index group
+     * @return {@code true} if this would need filtering if {@code indexGroup} were used, {@code false} otherwise
+     */
+    boolean needsFiltering(Index.Group indexGroup);
 
     /**
      * Adds to the specified row filter the expressions corresponding to this <code>Restriction</code>.
@@ -75,7 +99,7 @@ public interface Restriction
      * @param indexRegistry the index registry
      * @param options the query options
      */
-    public void addRowFilterTo(RowFilter filter,
+    public void addToRowFilter(RowFilter filter,
                                IndexRegistry indexRegistry,
                                QueryOptions options);
 }
