@@ -17,6 +17,7 @@
  */
 package org.apache.cassandra.cql3.restrictions;
 
+import java.util.Collection;
 import java.util.Set;
 
 import org.apache.cassandra.schema.ColumnMetadata;
@@ -34,6 +35,15 @@ public interface Restrictions extends Restriction
      */
     Set<Restriction> getRestrictions(ColumnMetadata columnDef);
 
+    /**
+     * This method exists in addition to {@link #getColumnDefs()} in case implementations want to
+     * provide columns definitions that are not strictly in position order.
+     */
+    default Collection<ColumnMetadata> getColumnDefinitions()
+    {
+        return getColumnDefs();
+    }
+
     /**
      * Checks if this <code>Restrictions</code> is empty or not.
      *
