@@ -22,8 +22,6 @@ import java.util.*;
 import java.util.function.Predicate;
 import java.util.stream.Collectors;
 
-import org.apache.commons.lang3.text.StrBuilder;
-
 import org.apache.cassandra.cql3.*;
 import org.apache.cassandra.cql3.functions.*;
 import org.apache.cassandra.cql3.selection.Selector.Factory;
@@ -81,7 +79,7 @@ public interface Selectable extends AssignmentTestable
      */
     public default boolean processesSelection()
     {
-        // ColumnMetadata is the only case that returns false and override this
+        // ColumnMetadata is the only case that returns false (if the column is not masked) and overrides this
         return true;
     }
 
@@ -92,6 +90,12 @@ public interface Selectable extends AssignmentTestable
         return type == null ? TestResult.NOT_ASSIGNABLE : type.testAssignment(keyspace, receiver);
     }
 
+    @Override
+    public default AbstractType<?> getCompatibleTypeIfKnown(String keyspace)
+    {
+        return getExactTypeIfKnown(keyspace);
+    }
+
     default int addAndGetIndex(ColumnMetadata def, List<ColumnMetadata> l)
     {
         int idx = l.indexOf(def);
@@ -152,7 +156,7 @@ public interface Selectable extends AssignmentTestable
             /*
              * expectedType will be null if we have no constraint on what the type should be. For instance, if this term is a bind marker:
              *   - it will be null if we do "SELECT ? FROM foo"
-             *   - it won't be null (and be LongType) if we do "SELECT bigintAsBlob(?) FROM foo" because the function constrain it.
+             *   - it won't be null (and be LongType) if we do "SELECT bigint_as_blob(?) FROM foo" because the function constrain it.
              *
              * In the first case, we have to error out: we need to infer the type of the metadata of a SELECT at preparation time, which we can't
              * here (users will have to do "SELECT (varint)? FROM foo" for instance).
@@ -192,6 +196,12 @@ public interface Selectable extends AssignmentTestable
             return rawTerm.getExactTypeIfKnown(keyspace);
         }
 
+        @Override
+        public AbstractType<?> getCompatibleTypeIfKnown(String keyspace)
+        {
+            return rawTerm.getCompatibleTypeIfKnown(keyspace);
+        }
+
         @Override
         public boolean selectColumns(Predicate<ColumnMetadata> predicate)
         {
@@ -222,19 +232,48 @@ public interface Selectable extends AssignmentTestable
 
     public static class WritetimeOrTTL implements Selectable
     {
-        public final ColumnMetadata column;
-        public final boolean isWritetime;
+        // The order of the variants in the Kind enum matters as they are used in ser/deser
+        public enum Kind
+        {
+            TTL("ttl", Int32Type.instance),
+            WRITE_TIME("writetime", LongType.instance),
+            MAX_WRITE_TIME("maxwritetime", LongType.instance); // maxwritetime is available after Cassandra 4.1 (exclusive)
 
-        public WritetimeOrTTL(ColumnMetadata column, boolean isWritetime)
+            public final String name;
+            public final AbstractType<?> returnType;
+
+            public static Kind fromOrdinal(int ordinal)
+            {
+                return values()[ordinal];
+            }
+
+            Kind(String name, AbstractType<?> returnType)
+            {
+                this.name = name;
+                this.returnType = returnType;
+            }
+
+            public boolean aggregatesMultiCell()
+            {
+                return this == MAX_WRITE_TIME;
+            }
+        }
+
+        public final ColumnMetadata column;
+        public final Selectable selectable;
+        public final Kind kind;
+
+        public WritetimeOrTTL(ColumnMetadata column, Selectable selectable, Kind kind)
         {
             this.column = column;
-            this.isWritetime = isWritetime;
+            this.selectable = selectable;
+            this.kind = kind;
         }
 
         @Override
         public String toString()
         {
-            return (isWritetime ? "writetime" : "ttl") + "(" + column.name + ")";
+            return kind.name + "(" + selectable + ")";
         }
 
         public Selector.Factory newSelectorFactory(TableMetadata table,
@@ -245,43 +284,45 @@ public interface Selectable extends AssignmentTestable
             if (column.isPrimaryKeyColumn())
                 throw new InvalidRequestException(
                         String.format("Cannot use selection function %s on PRIMARY KEY part %s",
-                                      isWritetime ? "writeTime" : "ttl",
+                                      kind.name,
                                       column.name));
-            if (column.type.isMultiCell())
-                throw new InvalidRequestException(String.format("Cannot use selection function %s on non-frozen %s %s",
-                                                                isWritetime ? "writeTime" : "ttl",
-                                                                column.type.isCollection() ? "collection" : "UDT",
-                                                                column.name));
 
-            return WritetimeOrTTLSelector.newFactory(column, addAndGetIndex(column, defs), isWritetime);
+            Selector.Factory factory = selectable.newSelectorFactory(table, expectedType, defs, boundNames);
+            boolean isMultiCell = factory.getColumnSpecification(table).type.isMultiCell();
+
+            return WritetimeOrTTLSelector.newFactory(factory, addAndGetIndex(column, defs), kind, isMultiCell);
         }
 
+        @Override
         public AbstractType<?> getExactTypeIfKnown(String keyspace)
         {
-            return isWritetime ? LongType.instance : Int32Type.instance;
+            AbstractType<?> type = kind.returnType;
+            return column.type.isMultiCell() && !kind.aggregatesMultiCell() ? ListType.getInstance(type, false) : type;
         }
 
         @Override
         public boolean selectColumns(Predicate<ColumnMetadata> predicate)
         {
-            return predicate.test(column);
+            return selectable.selectColumns(predicate);
         }
 
         public static class Raw implements Selectable.Raw
         {
-            private final Selectable.RawIdentifier id;
-            private final boolean isWritetime;
+            private final Selectable.RawIdentifier column;
+            private final Selectable.Raw selected;
+            private final Kind kind;
 
-            public Raw(Selectable.RawIdentifier id, boolean isWritetime)
+            public Raw(Selectable.RawIdentifier column, Selectable.Raw selected, Kind kind)
             {
-                this.id = id;
-                this.isWritetime = isWritetime;
+                this.column = column;
+                this.selected = selected;
+                this.kind = kind;
             }
 
             @Override
             public WritetimeOrTTL prepare(TableMetadata table)
             {
-                return new WritetimeOrTTL(id.prepare(table), isWritetime);
+                return new WritetimeOrTTL((ColumnMetadata) column.prepare(table), selected.prepare(table), kind);
             }
         }
     }
@@ -357,17 +398,11 @@ public interface Selectable extends AssignmentTestable
                     preparedArgs.add(arg.prepare(table));
 
                 FunctionName name = functionName;
-                // We need to circumvent the normal function lookup process for toJson() because instances of the function
-                // are not pre-declared (because it can accept any type of argument). We also have to wait until we have the
-                // selector factories of the argument so we can access their final type.
-                if (functionName.equalsNativeFunction(ToJsonFct.NAME))
-                {
-                    return new WithToJSonFunction(preparedArgs);
-                }
-                // Also, COUNT(x) is equivalent to COUNT(*) for any non-null term x (since count(x) don't care about it's argument outside of check for nullness) and
-                // for backward compatibilty we want to support COUNT(1), but we actually have COUNT(x) method for every existing (simple) input types so currently COUNT(1)
-                // will throw as ambiguous (since 1 works for any type). So we have have to special case COUNT.
-                else if (functionName.equalsNativeFunction(FunctionName.nativeFunction("count"))
+                // COUNT(x) is equivalent to COUNT(*) for any non-null term x (since count(x) don't care about its
+                // argument outside of check for nullness) and for backward compatibilty we want to support COUNT(1),
+                // but we actually have COUNT(x) method for every existing (simple) input types so currently COUNT(1)
+                // will throw as ambiguous (since 1 works for any type). So we have to special case COUNT.
+                if (functionName.equalsNativeFunction(FunctionName.nativeFunction("count"))
                         && preparedArgs.size() == 1
                         && (preparedArgs.get(0) instanceof WithTerm)
                         && (((WithTerm)preparedArgs.get(0)).rawTerm instanceof Constants.Literal))
@@ -390,44 +425,6 @@ public interface Selectable extends AssignmentTestable
         }
     }
 
-    public static class WithToJSonFunction implements Selectable
-    {
-        public final List<Selectable> args;
-
-        private WithToJSonFunction(List<Selectable> args)
-        {
-            this.args = args;
-        }
-
-        @Override
-        public String toString()
-        {
-            return new StrBuilder().append(ToJsonFct.NAME)
-                                   .append("(")
-                                   .appendWithSeparators(args, ", ")
-                                   .append(")")
-                                   .toString();
-        }
-
-        public Selector.Factory newSelectorFactory(TableMetadata table, AbstractType<?> expectedType, List<ColumnMetadata> defs, VariableSpecifications boundNames)
-        {
-            SelectorFactories factories = SelectorFactories.createFactoriesAndCollectColumnDefinitions(args, null, table, defs, boundNames);
-            Function fun = ToJsonFct.getInstance(factories.getReturnTypes());
-            return AbstractFunctionSelector.newFactory(fun, factories);
-        }
-
-        public AbstractType<?> getExactTypeIfKnown(String keyspace)
-        {
-            return UTF8Type.instance;
-        }
-
-        @Override
-        public boolean selectColumns(Predicate<ColumnMetadata> predicate)
-        {
-            return Selectable.selectColumns(args, predicate);
-        }
-    }
-
     public static class WithCast implements Selectable
     {
         private final CQL3Type type;
@@ -681,6 +678,17 @@ public interface Selectable extends AssignmentTestable
             return Tuples.getExactTupleTypeIfKnown(selectables, p -> p.getExactTypeIfKnown(keyspace));
         }
 
+        @Override
+        public AbstractType<?> getCompatibleTypeIfKnown(String keyspace)
+        {
+            // If there is only one element we cannot know if it is an element between parentheses or a tuple
+            // with only one element. By consequence, we need to force the user to specify the type.
+            if (selectables.size() == 1)
+                return null;
+
+            return Tuples.getExactTupleTypeIfKnown(selectables, p -> p.getCompatibleTypeIfKnown(keyspace));
+        }
+
         @Override
         public boolean selectColumns(Predicate<ColumnMetadata> predicate)
         {
@@ -713,18 +721,101 @@ public interface Selectable extends AssignmentTestable
     /**
      * <code>Selectable</code> for literal Lists.
      */
-    public static class WithList implements Selectable
+    public static class WithArrayLiteral implements Selectable
     {
         /**
          * The list elements
          */
-        private final List<Selectable> selectables;
+        protected final List<Selectable> selectables;
 
-        public WithList(List<Selectable> selectables)
+        public WithArrayLiteral(List<Selectable> selectables)
         {
             this.selectables = selectables;
         }
 
+        private Selectable target(AbstractType<?> target)
+        {
+            // when the target isn't known, fallback to list; cases like "SELECT [1, 2]" can't be known, but used to be list type!
+            // If a vector is actually desired, then can use type cast/hints: "SELECT (vector<int, 2>) [k, v1]"
+            if (target == null || target instanceof ListType)
+                return new WithList(selectables);
+            else if (target.isVector())
+                return new WithVector(selectables);
+            throw new IllegalArgumentException("Unsupported target type: " + target.asCQL3Type());
+        }
+
+        @Override
+        public TestResult testAssignment(String keyspace, ColumnSpecification receiver)
+        {
+            return target(receiver == null ? null : receiver.type).testAssignment(keyspace, receiver);
+        }
+
+        @Override
+        public Factory newSelectorFactory(TableMetadata cfm,
+                                          AbstractType<?> expectedType,
+                                          List<ColumnMetadata> defs,
+                                          VariableSpecifications boundNames)
+        {
+            return target(expectedType).newSelectorFactory(cfm, expectedType, defs, boundNames);
+        }
+
+        @Override
+        public AbstractType<?> getExactTypeIfKnown(String keyspace)
+        {
+            // TODO try to pass in a target to this API
+            // default to list when type is being infered
+            return new WithList(selectables).getExactTypeIfKnown(keyspace);
+        }
+
+        @Override
+        public AbstractType<?> getCompatibleTypeIfKnown(String keyspace)
+        {
+            // TODO try to pass in a target to this API
+            // default to list when type is being infered
+            return new WithList(selectables).getCompatibleTypeIfKnown(keyspace);
+        }
+
+        @Override
+        public boolean selectColumns(Predicate<ColumnMetadata> predicate)
+        {
+            return Selectable.selectColumns(selectables, predicate);
+        }
+
+        @Override
+        public String toString()
+        {
+            return Lists.listToString(selectables);
+        }
+
+        public int getSize()
+        {
+            return selectables.size();
+        }
+
+        public static class Raw implements Selectable.Raw
+        {
+            private final List<Selectable.Raw> raws;
+
+            public Raw(List<Selectable.Raw> raws)
+            {
+                this.raws = raws;
+            }
+
+            @Override
+            public Selectable prepare(TableMetadata cfm)
+            {
+                return new WithArrayLiteral(raws.stream().map(p -> p.prepare(cfm)).collect(Collectors.toList()));
+            }
+        }
+    }
+
+    public static class WithList extends WithArrayLiteral
+    {
+        public WithList(List<Selectable> selectables)
+        {
+            super(selectables);
+        }
+
         @Override
         public TestResult testAssignment(String keyspace, ColumnSpecification receiver)
         {
@@ -766,6 +857,12 @@ public interface Selectable extends AssignmentTestable
             return Lists.getExactListTypeIfKnown(selectables, p -> p.getExactTypeIfKnown(keyspace));
         }
 
+        @Override
+        public AbstractType<?> getCompatibleTypeIfKnown(String keyspace)
+        {
+            return Lists.getPreferredCompatibleType(selectables, p -> p.getCompatibleTypeIfKnown(keyspace));
+        }
+
         @Override
         public boolean selectColumns(Predicate<ColumnMetadata> predicate)
         {
@@ -777,21 +874,74 @@ public interface Selectable extends AssignmentTestable
         {
             return Lists.listToString(selectables);
         }
+    }
 
-        public static class Raw implements Selectable.Raw
+    public static class WithVector extends WithArrayLiteral
+    {
+        public WithVector(List<Selectable> selectables)
         {
-            private final List<Selectable.Raw> raws;
+            super(selectables);
+        }
 
-            public Raw(List<Selectable.Raw> raws)
+        @Override
+        public TestResult testAssignment(String keyspace, ColumnSpecification receiver)
+        {
+            return Vectors.testVectorAssignment(receiver, selectables);
+        }
+
+        @Override
+        public Factory newSelectorFactory(TableMetadata cfm,
+                                          AbstractType<?> expectedType,
+                                          List<ColumnMetadata> defs,
+                                          VariableSpecifications boundNames)
+        {
+            AbstractType<?> type = getExactTypeIfKnown(cfm.keyspace);
+            if (type == null)
             {
-                this.raws = raws;
+                type = expectedType;
+                if (type == null)
+                    throw invalidRequest("Cannot infer type for term %s in selection clause (try using a cast to force a type)",
+                                         this);
             }
 
-            @Override
-            public Selectable prepare(TableMetadata cfm)
-            {
-                return new WithList(raws.stream().map(p -> p.prepare(cfm)).collect(Collectors.toList()));
-            }
+            VectorType<?> vectorType = (VectorType<?>) type;
+            if (vectorType.dimension != selectables.size())
+                throw invalidRequest("Unable to create a vector selector of type %s from %d elements", vectorType.asCQL3Type(), selectables.size());
+
+            List<AbstractType<?>> expectedTypes = new ArrayList<>(selectables.size());
+            for (int i = 0, m = selectables.size(); i < m; i++)
+                expectedTypes.add(vectorType.getElementsType());
+
+            SelectorFactories factories = createFactoriesAndCollectColumnDefinitions(selectables,
+                                                                                     expectedTypes,
+                                                                                     cfm,
+                                                                                     defs,
+                                                                                     boundNames);
+            return VectorSelector.newFactory(type, factories);
+        }
+
+        @Override
+        public AbstractType<?> getExactTypeIfKnown(String keyspace)
+        {
+            return Vectors.getExactVectorTypeIfKnown(selectables, p -> p.getExactTypeIfKnown(keyspace));
+        }
+
+        @Override
+        public AbstractType<?> getCompatibleTypeIfKnown(String keyspace)
+        {
+            return Vectors.getPreferredCompatibleType(selectables, p -> p.getCompatibleTypeIfKnown(keyspace));
+        }
+
+        @Override
+        public boolean selectColumns(Predicate<ColumnMetadata> predicate)
+        {
+            return Selectable.selectColumns(selectables, predicate);
+        }
+
+        @Override
+        public String toString()
+        {
+            return Lists.listToString(selectables);
         }
     }
 
@@ -859,6 +1009,12 @@ public interface Selectable extends AssignmentTestable
             return Sets.getExactSetTypeIfKnown(selectables, p -> p.getExactTypeIfKnown(keyspace));
         }
 
+        @Override
+        public AbstractType<?> getCompatibleTypeIfKnown(String keyspace)
+        {
+            return Sets.getPreferredCompatibleType(selectables, p -> p.getCompatibleTypeIfKnown(keyspace));
+        }
+
         @Override
         public boolean selectColumns(Predicate<ColumnMetadata> predicate)
         {
@@ -991,7 +1147,14 @@ public interface Selectable extends AssignmentTestable
         @Override
         public AbstractType<?> getExactTypeIfKnown(String keyspace)
         {
-            // Lets force the user to specify the type.
+            // Let's force the user to specify the type.
+            return null;
+        }
+
+        @Override
+        public AbstractType<?> getCompatibleTypeIfKnown(String keyspace)
+        {
+            // Let's force the user to specify the type.
             return null;
         }
 
@@ -1213,12 +1376,17 @@ public interface Selectable extends AssignmentTestable
             this.quoted = quoted;
         }
 
-        @Override
-        public ColumnMetadata prepare(TableMetadata cfm)
+        public ColumnMetadata columnMetadata(TableMetadata cfm)
         {
             return cfm.getExistingColumn(ColumnIdentifier.getInterned(text, quoted));
         }
 
+        @Override
+        public Selectable prepare(TableMetadata cfm)
+        {
+            return columnMetadata(cfm);
+        }
+
         public FieldIdentifier toFieldIdentifier()
         {
             return quoted ? FieldIdentifier.forQuoted(text)
