@@ -22,6 +22,8 @@ import java.nio.ByteBuffer;
 
 import com.google.common.base.Objects;
 
+import org.apache.cassandra.config.DatabaseDescriptor;
+import org.apache.cassandra.cql3.functions.masking.ColumnMask;
 import org.apache.cassandra.schema.ColumnMetadata;
 import org.apache.cassandra.schema.TableMetadata;
 import org.apache.cassandra.cql3.ColumnSpecification;
@@ -45,7 +47,7 @@ public final class SimpleSelector extends Selector
             ByteBuffer columnName = ByteBufferUtil.readWithVIntLength(in);
             ColumnMetadata column = metadata.getColumn(columnName);
             int idx = in.readInt();
-            return new SimpleSelector(column, idx);
+            return new SimpleSelector(column, idx, false, ProtocolVersion.CURRENT);
         }
     };
 
@@ -55,13 +57,14 @@ public final class SimpleSelector extends Selector
     public static final class SimpleSelectorFactory extends Factory
     {
         private final int idx;
-
         private final ColumnMetadata column;
+        private final boolean useForPostOrdering;
 
-        private SimpleSelectorFactory(int idx, ColumnMetadata def)
+        private SimpleSelectorFactory(int idx, ColumnMetadata def, boolean useForPostOrdering)
         {
             this.idx = idx;
             this.column = def;
+            this.useForPostOrdering = useForPostOrdering;
         }
 
         @Override
@@ -84,7 +87,7 @@ public final class SimpleSelector extends Selector
         @Override
         public Selector newInstance(QueryOptions options)
         {
-            return new SimpleSelector(column, idx);
+            return new SimpleSelector(column, idx, useForPostOrdering, options.getProtocolVersion());
         }
 
         @Override
@@ -117,12 +120,15 @@ public final class SimpleSelector extends Selector
 
     public final ColumnMetadata column;
     private final int idx;
+    private final ColumnMask.Masker masker;
     private ByteBuffer current;
+    private ColumnTimestamps writetimes;
+    private ColumnTimestamps ttls;
     private boolean isSet;
 
-    public static Factory newFactory(final ColumnMetadata def, final int idx)
+    public static Factory newFactory(final ColumnMetadata def, final int idx, boolean useForPostOrdering)
     {
-        return new SimpleSelectorFactory(idx, def);
+        return new SimpleSelectorFactory(idx, def, useForPostOrdering);
     }
 
     @Override
@@ -132,12 +138,23 @@ public final class SimpleSelector extends Selector
     }
 
     @Override
-    public void addInput(ProtocolVersion protocolVersion, InputRow input) throws InvalidRequestException
+    public void addInput(InputRow input) throws InvalidRequestException
     {
         if (!isSet)
         {
             isSet = true;
-            current = input.getValue(idx);
+            writetimes = input.getWritetimes(idx);
+            ttls = input.getTtls(idx);
+
+            /*
+            We apply the column masker of the column unless:
+            - The column doesn't have a mask
+            - The input row is for a user with UNMASK permission, indicated by input.unmask()
+            - Dynamic data masking is globally disabled
+             */
+            ByteBuffer value = input.getValue(idx);
+            current = masker == null || input.unmask() || !DatabaseDescriptor.getDynamicDataMaskingEnabled()
+                      ? value : masker.mask(value);
         }
     }
 
@@ -147,11 +164,25 @@ public final class SimpleSelector extends Selector
         return current;
     }
 
+    @Override
+    protected ColumnTimestamps getWritetimes(ProtocolVersion protocolVersion)
+    {
+        return writetimes;
+    }
+
+    @Override
+    protected ColumnTimestamps getTTLs(ProtocolVersion protocolVersion)
+    {
+        return ttls;
+    }
+
     @Override
     public void reset()
     {
         isSet = false;
         current = null;
+        writetimes = null;
+        ttls = null;
     }
 
     @Override
@@ -166,11 +197,19 @@ public final class SimpleSelector extends Selector
         return column.name.toString();
     }
 
-    private SimpleSelector(ColumnMetadata column, int idx)
+    private SimpleSelector(ColumnMetadata column, int idx, boolean useForPostOrdering, ProtocolVersion version)
     {
         super(Kind.SIMPLE_SELECTOR);
         this.column = column;
         this.idx = idx;
+        /*
+         We apply the column mask of the column unless:
+         - The column doesn't have a mask
+         - This selector is for a query with ORDER BY post-ordering
+          */
+        this.masker = useForPostOrdering || column.getMask() == null
+                      ? null
+                      : column.getMask().masker(version);
     }
 
     @Override
@@ -210,6 +249,6 @@ public final class SimpleSelector extends Selector
     protected void serialize(DataOutputPlus out, int version) throws IOException
     {
         ByteBufferUtil.writeWithVIntLength(column.name.bytes, out);
-        out.writeInt(idx);;
+        out.writeInt(idx);
     }
 }
