@@ -55,7 +55,7 @@ final class UserTypeSelector extends Selector
         protected Selector deserialize(DataInputPlus in, int version, TableMetadata metadata) throws IOException
         {
             UserType type = (UserType) readType(metadata, in);
-            int size = (int) in.readUnsignedVInt();
+            int size = in.readUnsignedVInt32();
             Map<FieldIdentifier, Selector> fields = new HashMap<>(size);
             for (int i = 0; i < size; i++)
             {
@@ -182,10 +182,10 @@ final class UserTypeSelector extends Selector
             field.addFetchedColumns(builder);
     }
 
-    public void addInput(ProtocolVersion protocolVersion, InputRow input)
+    public void addInput(InputRow input)
     {
         for (Selector field : fields.values())
-            field.addInput(protocolVersion, input);
+            field.addInput(input);
     }
 
     public ByteBuffer getOutput(ProtocolVersion protocolVersion)
@@ -272,7 +272,7 @@ final class UserTypeSelector extends Selector
     protected void serialize(DataOutputPlus out, int version) throws IOException
     {
         writeType(out, type);
-        out.writeUnsignedVInt(fields.size());
+        out.writeUnsignedVInt32(fields.size());
 
         for (Map.Entry<FieldIdentifier, Selector> field : fields.entrySet())
         {
