@@ -22,107 +22,127 @@ import java.nio.ByteBuffer;
 
 import com.google.common.base.Objects;
 
-import org.apache.cassandra.schema.ColumnMetadata;
+import org.apache.cassandra.db.marshal.ListType;
 import org.apache.cassandra.schema.TableMetadata;
 import org.apache.cassandra.cql3.QueryOptions;
 import org.apache.cassandra.cql3.ColumnSpecification;
 import org.apache.cassandra.db.TypeSizes;
 import org.apache.cassandra.db.filter.ColumnFilter;
 import org.apache.cassandra.db.marshal.AbstractType;
-import org.apache.cassandra.db.marshal.Int32Type;
-import org.apache.cassandra.db.marshal.LongType;
 import org.apache.cassandra.io.util.DataInputPlus;
 import org.apache.cassandra.io.util.DataOutputPlus;
 import org.apache.cassandra.transport.ProtocolVersion;
-import org.apache.cassandra.utils.ByteBufferUtil;
 
 final class WritetimeOrTTLSelector extends Selector
 {
-    protected static final SelectorDeserializer deserializer = new SelectorDeserializer()
+    static final SelectorDeserializer deserializer = new SelectorDeserializer()
     {
+        @Override
         protected Selector deserialize(DataInputPlus in, int version, TableMetadata metadata) throws IOException
         {
-            ByteBuffer columnName = ByteBufferUtil.readWithVIntLength(in);
-            ColumnMetadata column = metadata.getColumn(columnName);
+            Selector selected = serializer.deserialize(in, version, metadata);
             int idx = in.readInt();
-            boolean isWritetime = in.readBoolean();
-            return new WritetimeOrTTLSelector(column, idx, isWritetime);
+            int ordinal = in.readByte();
+            Selectable.WritetimeOrTTL.Kind kind = Selectable.WritetimeOrTTL.Kind.fromOrdinal(ordinal);
+            boolean isMultiCell = in.readBoolean();
+            return new WritetimeOrTTLSelector(selected, idx, kind, isMultiCell);
         }
     };
 
-    private final ColumnMetadata column;
-    private final int idx;
-    private final boolean isWritetime;
+    private final Selector selected;
+    private final int columnIndex;
+    private final Selectable.WritetimeOrTTL.Kind kind;
     private ByteBuffer current;
+    private final boolean isMultiCell;
     private boolean isSet;
 
-    public static Factory newFactory(final ColumnMetadata def, final int idx, final boolean isWritetime)
+    public static Factory newFactory(final Selector.Factory factory, final int columnIndex, final Selectable.WritetimeOrTTL.Kind kind, boolean isMultiCell)
     {
         return new Factory()
         {
+            @Override
             protected String getColumnName()
             {
-                return String.format("%s(%s)", isWritetime ? "writetime" : "ttl", def.name.toString());
+                return String.format("%s(%s)", kind.name, factory.getColumnName());
             }
 
+            @Override
             protected AbstractType<?> getReturnType()
             {
-                return isWritetime ? LongType.instance : Int32Type.instance;
+                AbstractType<?> type = kind.returnType;
+                return isMultiCell && !kind.aggregatesMultiCell() ? ListType.getInstance(type, false) : type;
             }
 
+            @Override
             protected void addColumnMapping(SelectionColumnMapping mapping, ColumnSpecification resultsColumn)
             {
-               mapping.addMapping(resultsColumn, def);
+                factory.addColumnMapping(mapping, resultsColumn);
             }
 
+            @Override
             public Selector newInstance(QueryOptions options)
             {
-                return new WritetimeOrTTLSelector(def, idx, isWritetime);
+                return new WritetimeOrTTLSelector(factory.newInstance(options), columnIndex, kind, isMultiCell);
             }
 
+            @Override
             public boolean isWritetimeSelectorFactory()
             {
-                return isWritetime;
+                return kind != Selectable.WritetimeOrTTL.Kind.TTL;
             }
 
+            @Override
             public boolean isTTLSelectorFactory()
             {
-                return !isWritetime;
+                return kind == Selectable.WritetimeOrTTL.Kind.TTL;
             }
 
+            @Override
+            public boolean isMaxWritetimeSelectorFactory()
+            {
+                return kind == Selectable.WritetimeOrTTL.Kind.MAX_WRITE_TIME;
+            }
+
+            @Override
             public boolean areAllFetchedColumnsKnown()
             {
                 return true;
             }
 
+            @Override
             public void addFetchedColumns(ColumnFilter.Builder builder)
             {
-                builder.add(def);
+                factory.addFetchedColumns(builder);
             }
         };
     }
 
     public void addFetchedColumns(ColumnFilter.Builder builder)
     {
-        builder.add(column);
+        selected.addFetchedColumns(builder);
     }
 
-    public void addInput(ProtocolVersion protocolVersion, InputRow input)
+    public void addInput(InputRow input)
     {
         if (isSet)
             return;
 
         isSet = true;
 
-        if (isWritetime)
+        selected.addInput(input);
+        ProtocolVersion protocolVersion = input.getProtocolVersion();
+
+        switch (kind)
         {
-            long ts = input.getTimestamp(idx);
-            current = ts != Long.MIN_VALUE ? ByteBufferUtil.bytes(ts) : null;
-        }
-        else
-        {
-            int ttl = input.getTtl(idx);
-            current = ttl > 0 ? ByteBufferUtil.bytes(ttl) : null;
+            case WRITE_TIME:
+                current = selected.getWritetimes(protocolVersion).toByteBuffer(protocolVersion);
+                break;
+            case MAX_WRITE_TIME:
+                current = selected.getWritetimes(protocolVersion).max().toByteBuffer(protocolVersion);
+                break;
+            case TTL:
+                current = selected.getTTLs(protocolVersion).toByteBuffer(protocolVersion);
+                break;
         }
     }
 
@@ -133,27 +153,30 @@ final class WritetimeOrTTLSelector extends Selector
 
     public void reset()
     {
+        selected.reset();
         isSet = false;
         current = null;
     }
 
     public AbstractType<?> getType()
     {
-        return isWritetime ? LongType.instance : Int32Type.instance;
+        AbstractType<?> type = kind.returnType;
+        return isMultiCell ? ListType.getInstance(type, false) : type;
     }
 
     @Override
     public String toString()
     {
-        return column.name.toString();
+        return selected.toString();
     }
 
-    private WritetimeOrTTLSelector(ColumnMetadata column, int idx, boolean isWritetime)
+    private WritetimeOrTTLSelector(Selector selected, int idx, Selectable.WritetimeOrTTL.Kind kind, boolean isMultiCell)
     {
         super(Kind.WRITETIME_OR_TTL_SELECTOR);
-        this.column = column;
-        this.idx = idx;
-        this.isWritetime = isWritetime;
+        this.selected = selected;
+        this.columnIndex = idx;
+        this.kind = kind;
+        this.isMultiCell = isMultiCell;
     }
 
     @Override
@@ -167,30 +190,30 @@ final class WritetimeOrTTLSelector extends Selector
 
         WritetimeOrTTLSelector s = (WritetimeOrTTLSelector) o;
 
-        return Objects.equal(column, s.column)
-            && Objects.equal(idx, s.idx)
-            && Objects.equal(isWritetime, s.isWritetime);
+        return Objects.equal(selected, s.selected) && kind == s.kind;
     }
 
     @Override
     public int hashCode()
     {
-        return Objects.hashCode(column, idx, isWritetime);
+        return Objects.hashCode(selected, kind);
     }
 
     @Override
     protected int serializedSize(int version)
     {
-        return ByteBufferUtil.serializedSizeWithVIntLength(column.name.bytes)
-                + TypeSizes.sizeof(idx)
-                + TypeSizes.sizeof(isWritetime);
+        return serializer.serializedSize(selected, version)
+                + TypeSizes.sizeof(columnIndex)
+                + TypeSizes.sizeofUnsignedVInt(kind.ordinal())
+                + TypeSizes.sizeof(isMultiCell);
     }
 
     @Override
     protected void serialize(DataOutputPlus out, int version) throws IOException
     {
-        ByteBufferUtil.writeWithVIntLength(column.name.bytes, out);
-        out.writeInt(idx);
-        out.writeBoolean(isWritetime);
+        serializer.serialize(selected, out, version);
+        out.writeInt(columnIndex);
+        out.writeByte(kind.ordinal());
+        out.writeBoolean(isMultiCell);
     }
 }
