@@ -25,7 +25,6 @@ import com.google.common.collect.Sets;
 
 import org.apache.cassandra.cql3.statements.PropertyDefinitions;
 import org.apache.cassandra.exceptions.ConfigurationException;
-import org.apache.cassandra.exceptions.SyntaxException;
 import org.apache.cassandra.schema.CachingParams;
 import org.apache.cassandra.schema.CompactionParams;
 import org.apache.cassandra.schema.CompressionParams;
@@ -37,6 +36,7 @@ import org.apache.cassandra.service.reads.SpeculativeRetryPolicy;
 import org.apache.cassandra.service.reads.repair.ReadRepairStrategy;
 
 import static java.lang.String.format;
+import static org.apache.cassandra.schema.TableParams.Option.*;
 
 public final class TableAttributes extends PropertyDefinitions
 {
@@ -74,7 +74,7 @@ public final class TableAttributes extends PropertyDefinitions
 
     public TableId getId() throws ConfigurationException
     {
-        String id = getSimple(ID);
+        String id = getString(ID);
         try
         {
             return id != null ? TableId.fromString(id) : null;
@@ -97,111 +97,71 @@ public final class TableAttributes extends PropertyDefinitions
 
     private TableParams build(TableParams.Builder builder)
     {
-        if (hasOption(Option.BLOOM_FILTER_FP_CHANCE))
-            builder.bloomFilterFpChance(getDouble(Option.BLOOM_FILTER_FP_CHANCE));
+        if (hasOption(ALLOW_AUTO_SNAPSHOT))
+            builder.allowAutoSnapshot(getBoolean(ALLOW_AUTO_SNAPSHOT.toString(), true));
 
-        if (hasOption(Option.CACHING))
-            builder.caching(CachingParams.fromMap(getMap(Option.CACHING)));
+        if (hasOption(BLOOM_FILTER_FP_CHANCE))
+            builder.bloomFilterFpChance(getDouble(BLOOM_FILTER_FP_CHANCE));
 
-        if (hasOption(Option.COMMENT))
-            builder.comment(getString(Option.COMMENT));
+        if (hasOption(CACHING))
+            builder.caching(CachingParams.fromMap(getMap(CACHING)));
 
-        if (hasOption(Option.COMPACTION))
-            builder.compaction(CompactionParams.fromMap(getMap(Option.COMPACTION)));
+        if (hasOption(COMMENT))
+            builder.comment(getString(COMMENT));
 
-        if (hasOption(Option.COMPRESSION))
-        {
-            //crc_check_chance was "promoted" from a compression property to a top-level-property after #9839
-            //so we temporarily accept it to be defined as a compression option, to maintain backwards compatibility
-            Map<String, String> compressionOpts = getMap(Option.COMPRESSION);
-            if (compressionOpts.containsKey(Option.CRC_CHECK_CHANCE.toString().toLowerCase()))
-            {
-                Double crcCheckChance = getDeprecatedCrcCheckChance(compressionOpts);
-                builder.crcCheckChance(crcCheckChance);
-            }
-            builder.compression(CompressionParams.fromMap(getMap(Option.COMPRESSION)));
-        }
+        if (hasOption(COMPACTION))
+            builder.compaction(CompactionParams.fromMap(getMap(COMPACTION)));
 
-        if (hasOption(Option.MEMTABLE))
-            builder.memtable(MemtableParams.get(getString(Option.MEMTABLE)));
+        if (hasOption(COMPRESSION))
+            builder.compression(CompressionParams.fromMap(getMap(COMPRESSION)));
 
-        if (hasOption(Option.DEFAULT_TIME_TO_LIVE))
-            builder.defaultTimeToLive(getInt(Option.DEFAULT_TIME_TO_LIVE));
+        if (hasOption(MEMTABLE))
+            builder.memtable(MemtableParams.get(getString(MEMTABLE)));
 
-        if (hasOption(Option.GC_GRACE_SECONDS))
-            builder.gcGraceSeconds(getInt(Option.GC_GRACE_SECONDS));
+        if (hasOption(DEFAULT_TIME_TO_LIVE))
+            builder.defaultTimeToLive(getInt(DEFAULT_TIME_TO_LIVE));
 
-        if (hasOption(Option.MAX_INDEX_INTERVAL))
-            builder.maxIndexInterval(getInt(Option.MAX_INDEX_INTERVAL));
+        if (hasOption(GC_GRACE_SECONDS))
+            builder.gcGraceSeconds(getInt(GC_GRACE_SECONDS));
+        
+        if (hasOption(INCREMENTAL_BACKUPS))
+            builder.incrementalBackups(getBoolean(INCREMENTAL_BACKUPS.toString(), true));
 
-        if (hasOption(Option.MEMTABLE_FLUSH_PERIOD_IN_MS))
-            builder.memtableFlushPeriodInMs(getInt(Option.MEMTABLE_FLUSH_PERIOD_IN_MS));
+        if (hasOption(MAX_INDEX_INTERVAL))
+            builder.maxIndexInterval(getInt(MAX_INDEX_INTERVAL));
 
-        if (hasOption(Option.MIN_INDEX_INTERVAL))
-            builder.minIndexInterval(getInt(Option.MIN_INDEX_INTERVAL));
+        if (hasOption(MEMTABLE_FLUSH_PERIOD_IN_MS))
+            builder.memtableFlushPeriodInMs(getInt(MEMTABLE_FLUSH_PERIOD_IN_MS));
 
-        if (hasOption(Option.SPECULATIVE_RETRY))
-            builder.speculativeRetry(SpeculativeRetryPolicy.fromString(getString(Option.SPECULATIVE_RETRY)));
+        if (hasOption(MIN_INDEX_INTERVAL))
+            builder.minIndexInterval(getInt(MIN_INDEX_INTERVAL));
 
-        if (hasOption(Option.ADDITIONAL_WRITE_POLICY))
-            builder.additionalWritePolicy(SpeculativeRetryPolicy.fromString(getString(Option.ADDITIONAL_WRITE_POLICY)));
+        if (hasOption(SPECULATIVE_RETRY))
+            builder.speculativeRetry(SpeculativeRetryPolicy.fromString(getString(SPECULATIVE_RETRY)));
 
-        if (hasOption(Option.CRC_CHECK_CHANCE))
-            builder.crcCheckChance(getDouble(Option.CRC_CHECK_CHANCE));
+        if (hasOption(ADDITIONAL_WRITE_POLICY))
+            builder.additionalWritePolicy(SpeculativeRetryPolicy.fromString(getString(ADDITIONAL_WRITE_POLICY)));
 
-        if (hasOption(Option.CDC))
-            builder.cdc(getBoolean(Option.CDC.toString(), false));
+        if (hasOption(CRC_CHECK_CHANCE))
+            builder.crcCheckChance(getDouble(CRC_CHECK_CHANCE));
 
-        if (hasOption(Option.READ_REPAIR))
-            builder.readRepair(ReadRepairStrategy.fromString(getString(Option.READ_REPAIR)));
+        if (hasOption(CDC))
+            builder.cdc(getBoolean(CDC));
+
+        if (hasOption(READ_REPAIR))
+            builder.readRepair(ReadRepairStrategy.fromString(getString(READ_REPAIR)));
 
         return builder.build();
     }
 
-    private Double getDeprecatedCrcCheckChance(Map<String, String> compressionOpts)
+    public boolean hasOption(Option option)
     {
-        String value = compressionOpts.get(Option.CRC_CHECK_CHANCE.toString().toLowerCase());
-        try
-        {
-            return Double.valueOf(value);
-        }
-        catch (NumberFormatException e)
-        {
-            throw new SyntaxException(String.format("Invalid double value %s for crc_check_chance.'", value));
-        }
-    }
-
-    private double getDouble(Option option)
-    {
-        String value = getString(option);
-
-        try
-        {
-            return Double.parseDouble(value);
-        }
-        catch (NumberFormatException e)
-        {
-            throw new SyntaxException(format("Invalid double value %s for '%s'", value, option));
-        }
-    }
-
-    private int getInt(Option option)
-    {
-        String value = getString(option);
-
-        try
-        {
-            return Integer.parseInt(value);
-        }
-        catch (NumberFormatException e)
-        {
-            throw new SyntaxException(String.format("Invalid integer value %s for '%s'", value, option));
-        }
+        return hasProperty(option.toString());
     }
 
     private String getString(Option option)
     {
-        String value = getSimple(option.toString());
+        String value = getString(option.toString());
         if (value == null)
             throw new IllegalStateException(format("Option '%s' is absent", option));
         return value;
@@ -215,8 +175,18 @@ public final class TableAttributes extends PropertyDefinitions
         return value;
     }
 
-    public boolean hasOption(Option option)
+    private boolean getBoolean(Option option)
     {
-        return hasProperty(option.toString());
+        return parseBoolean(option.toString(), getString(option));
+    }
+
+    private int getInt(Option option)
+    {
+        return parseInt(option.toString(), getString(option));
+    }
+
+    private double getDouble(Option option)
+    {
+        return parseDouble(option.toString(), getString(option));
     }
 }
