@@ -19,7 +19,6 @@
 package org.apache.cassandra.db;
 
 import org.apache.cassandra.db.partitions.PartitionUpdate;
-import org.apache.cassandra.index.transactions.UpdateTransaction;
 import org.apache.cassandra.tracing.Tracing;
 
 public class CassandraTableWriteHandler implements TableWriteHandler
@@ -32,11 +31,10 @@ public class CassandraTableWriteHandler implements TableWriteHandler
     }
 
     @Override
-    @SuppressWarnings("resource")
-    public void write(PartitionUpdate update, WriteContext context, UpdateTransaction updateTransaction)
+    public void write(PartitionUpdate update, WriteContext context, boolean updateIndexes)
     {
         CassandraWriteContext ctx = CassandraWriteContext.fromContext(context);
         Tracing.trace("Adding to {} memtable", update.metadata().name);
-        cfs.apply(update, updateTransaction, ctx.getGroup(), ctx.getPosition());
+        cfs.apply(update, ctx, updateIndexes);
     }
 }
