@@ -56,6 +56,18 @@ public interface Clustering<V> extends ClusteringPrefix<V>, IMeasurableMemory
         return new BufferClustering(newValues);
     }
 
+    @Override
+    default ClusteringBound<V> asStartBound()
+    {
+        return ClusteringBound.inclusiveStartOf(this);
+    }
+
+    @Override
+    default ClusteringBound<V> asEndBound()
+    {
+        return ClusteringBound.inclusiveEndOf(this);
+    }
+
     public default String toString(TableMetadata metadata)
     {
         StringBuilder sb = new StringBuilder();
@@ -178,4 +190,4 @@ public interface Clustering<V> extends ClusteringPrefix<V>, IMeasurableMemory
             }
         }
     }
-}
+}
\ No newline at end of file
