@@ -20,6 +20,7 @@ package org.apache.cassandra.db;
 import java.io.IOException;
 import java.nio.ByteBuffer;
 import java.util.*;
+import java.util.function.ToIntFunction;
 
 import org.apache.cassandra.cache.IMeasurableMemory;
 import org.apache.cassandra.config.*;
@@ -29,11 +30,14 @@ import org.apache.cassandra.db.marshal.CompositeType;
 import org.apache.cassandra.db.marshal.ValueAccessor;
 import org.apache.cassandra.db.rows.*;
 import org.apache.cassandra.db.marshal.AbstractType;
+import org.apache.cassandra.exceptions.InvalidRequestException;
 import org.apache.cassandra.io.util.DataInputPlus;
 import org.apache.cassandra.io.util.DataOutputPlus;
 import org.apache.cassandra.schema.TableMetadata;
 import org.apache.cassandra.utils.ByteArrayUtil;
-import org.apache.cassandra.utils.ByteBufferUtil;
+import org.apache.cassandra.utils.FBUtilities;
+import org.apache.cassandra.utils.bytecomparable.ByteComparable.Version;
+import org.apache.cassandra.utils.bytecomparable.ByteSource;
 
 /**
  * A clustering prefix is the unit of what a {@link ClusteringComparator} can compare.
@@ -62,14 +66,20 @@ public interface ClusteringPrefix<V> extends IMeasurableMemory, Clusterable<V>
     {
         // WARNING: the ordering of that enum matters because we use ordinal() in the serialization
 
-        EXCL_END_BOUND              (0, -1),
-        INCL_START_BOUND            (0, -1),
-        EXCL_END_INCL_START_BOUNDARY(0, -1),
-        STATIC_CLUSTERING           (1, -1),
-        CLUSTERING                  (2,  0),
-        INCL_END_EXCL_START_BOUNDARY(3,  1),
-        INCL_END_BOUND              (3,  1),
-        EXCL_START_BOUND            (3,  1);
+        // @formatter:off
+        EXCL_END_BOUND               ( 0, -1, v -> ByteSource.LT_NEXT_COMPONENT),
+        INCL_START_BOUND             ( 0, -1, v -> ByteSource.LT_NEXT_COMPONENT),
+        EXCL_END_INCL_START_BOUNDARY ( 0, -1, v -> ByteSource.LT_NEXT_COMPONENT),
+        STATIC_CLUSTERING            ( 1, -1, v -> v == Version.LEGACY ? ByteSource.LT_NEXT_COMPONENT + 1
+                                                                       : ByteSource.EXCLUDED),
+        CLUSTERING                   ( 2,  0, v -> v == Version.LEGACY ? ByteSource.NEXT_COMPONENT
+                                                                       : ByteSource.TERMINATOR),
+        INCL_END_EXCL_START_BOUNDARY ( 3,  1, v -> ByteSource.GT_NEXT_COMPONENT),
+        INCL_END_BOUND               ( 3,  1, v -> ByteSource.GT_NEXT_COMPONENT),
+        EXCL_START_BOUND             ( 3,  1, v -> ByteSource.GT_NEXT_COMPONENT),
+        SSTABLE_LOWER_BOUND          (-1, -1, v -> ByteSource.LTLT_NEXT_COMPONENT),
+        SSTABLE_UPPER_BOUND          ( 4,  1, v -> ByteSource.GTGT_NEXT_COMPONENT);
+        // @formatter:on
 
         private final int comparison;
 
@@ -79,10 +89,13 @@ public interface ClusteringPrefix<V> extends IMeasurableMemory, Clusterable<V>
          */
         public final int comparedToClustering;
 
-        Kind(int comparison, int comparedToClustering)
+        public final ToIntFunction<Version> asByteComparable;
+
+        Kind(int comparison, int comparedToClustering, ToIntFunction<Version> asByteComparable)
         {
             this.comparison = comparison;
             this.comparedToClustering = comparedToClustering;
+            this.asByteComparable = asByteComparable;
         }
 
         /**
@@ -197,6 +210,16 @@ public interface ClusteringPrefix<V> extends IMeasurableMemory, Clusterable<V>
                  ? (this == INCL_END_EXCL_START_BOUNDARY ? INCL_END_BOUND : EXCL_END_BOUND)
                  : (this == INCL_END_EXCL_START_BOUNDARY ? EXCL_START_BOUND : INCL_START_BOUND);
         }
+
+        /*
+         * Returns a terminator value for this clustering type that is suitable for byte comparison.
+         * Inclusive starts / exclusive ends need a lower value than ByteSource.NEXT_COMPONENT and the clustering byte,
+         * exclusive starts / inclusive ends -- a higher.
+         */
+        public int asByteComparableValue(Version version)
+        {
+            return asByteComparable.applyAsInt(version);
+        }
     }
 
     default boolean isBottom()
@@ -246,6 +269,24 @@ public interface ClusteringPrefix<V> extends IMeasurableMemory, Clusterable<V>
         return comparator.subtype(i).getString(get(i), accessor());
     }
 
+    default void validate()
+    {
+        ValueAccessor<V> accessor = accessor();
+        int sum = 0;
+        for (V v : getRawValues())
+        {
+            if (v != null && accessor.size(v) > FBUtilities.MAX_UNSIGNED_SHORT)
+                throw new InvalidRequestException(String.format("Key length of %d is longer than maximum of %d",
+                                                                dataSize(),
+                                                                FBUtilities.MAX_UNSIGNED_SHORT));
+            sum += v == null ? 0 : accessor.size(v);
+        }
+        if (sum > FBUtilities.MAX_UNSIGNED_SHORT)
+            throw new InvalidRequestException(String.format("Key length of %d is longer than maximum of %d",
+                                                            sum,
+                                                            FBUtilities.MAX_UNSIGNED_SHORT));
+    }
+
     default void validate(int i, ClusteringComparator comparator)
     {
         comparator.subtype(i).validate(get(i), accessor());
@@ -292,6 +333,22 @@ public interface ClusteringPrefix<V> extends IMeasurableMemory, Clusterable<V>
      */
     public String toString(TableMetadata metadata);
 
+    /**
+     * Returns this prefix as a start bound.
+     * If this prefix is a bound, just returns it asserting that it is a start bound.
+     * If this prefix is a clustering, returns an included start bound.
+     * If this prefix is a boundary, returns an open bound of it
+     */
+    ClusteringBound<V> asStartBound();
+
+    /**
+     * Returns this prefix as an end bound.
+     * If this prefix is a bound, just returns it asserting that it is an end bound.
+     * If this prefix is a clustering, returns an included end bound.
+     * In this prefix is a boundary, returns a close bound of it.
+     */
+    ClusteringBound<V> asEndBound();
+
     /*
      * TODO: we should stop using Clustering for partition keys. Maybe we can add
      * a few methods to DecoratedKey so we don't have to (note that while using a Clustering
@@ -308,6 +365,24 @@ public interface ClusteringPrefix<V> extends IMeasurableMemory, Clusterable<V>
             values[i] = accessor().toBuffer(get(i));
         return CompositeType.build(ByteBufferAccessor.instance, values);
     }
+
+    /**
+     * Produce a human-readable representation of the clustering given the list of types.
+     * Easier to access than metadata for debugging.
+     */
+    public default String clusteringString(List<AbstractType<?>> types)
+    {
+        StringBuilder sb = new StringBuilder();
+        sb.append(kind()).append('(');
+        for (int i = 0; i < size(); i++)
+        {
+            if (i > 0)
+                sb.append(", ");
+            sb.append(types.get(i).getString(get(i), accessor()));
+        }
+        return sb.append(')').toString();
+    }
+
     /**
      * The values of this prefix as an array.
      * <p>
@@ -322,10 +397,11 @@ public interface ClusteringPrefix<V> extends IMeasurableMemory, Clusterable<V>
     public ByteBuffer[] getBufferArray();
 
     /**
-     * If the prefix contains byte buffers that can be minimized (see {@link ByteBufferUtil#minimalBufferFor(ByteBuffer)}),
-     * this will return a copy of the prefix with minimized values, otherwise it returns itself.
+     * Return the key in a form that can be retained for longer-term use. This means extracting keys stored in shared
+     * memory (i.e. in memtables) to minimized on-heap versions.
+     * If the object is already in minimal form, no action will be taken.
      */
-    public ClusteringPrefix<V> minimize();
+    public ClusteringPrefix<V> retainable();
 
     public static class Serializer
     {
@@ -676,4 +752,4 @@ public interface ClusteringPrefix<V> extends IMeasurableMemory, Clusterable<V>
         return equals(prefix, (ClusteringPrefix<?>) o);
     }
 
-}
+}
\ No newline at end of file
