@@ -39,7 +39,8 @@ public interface ColumnFamilyStoreMBean
     /**
      * @return the name of the column family
      */
-    @Deprecated
+    /** @deprecated See CASSANDRA-9448 */
+    @Deprecated(since = "3.0")
     public String getColumnFamilyName();
 
     public String getTableName();
@@ -58,9 +59,11 @@ public interface ColumnFamilyStoreMBean
      * last keys of a sstable, even though the {@link Range} class is suppossed to be half-open by definition.
      *
      * @param tokenRanges The token ranges to be compacted, interpreted as closed intervals.
+     *
+     * @deprecated See CASSANDRA-17527
      */
     @BreaksJMX("This API was released in 3.10 using a parameter that takes Range of Token, which can only be done IFF client has Cassandra binaries in the classpath")
-    @Deprecated
+    @Deprecated(since = "4.1")
     public void forceCompactionForTokenRange(Collection<Range<Token>> tokenRanges) throws ExecutionException, InterruptedException;
 
     /**
@@ -161,6 +164,15 @@ public interface ColumnFamilyStoreMBean
      */
     public List<String> getSSTablesForKey(String key, boolean hexFormat);
 
+    /**
+     * Returns a list of filenames that contain the given key and which level they belong to.
+     * Requires table to be compacted with {@link org.apache.cassandra.db.compaction.LeveledCompactionStrategy}
+     * @param key
+     * @param hexFormat
+     * @return list of filenames and levels containing the key
+     */
+    public Map<Integer, Set<String>> getSSTablesForKeyWithLevel(String key, boolean hexFormat);
+
     /**
      * Load new sstables from the given directory
      *
@@ -174,14 +186,15 @@ public interface ColumnFamilyStoreMBean
      *
      * @return list of failed import directories
      */
-    @Deprecated
+    /** @deprecated See CASSANDRA-16407 */
+    @Deprecated(since = "4.0")
     public List<String> importNewSSTables(Set<String> srcPaths,
-                                           boolean resetLevel,
-                                           boolean clearRepaired,
-                                           boolean verifySSTables,
-                                           boolean verifyTokens,
-                                           boolean invalidateCaches,
-                                           boolean extendedVerify);
+                                          boolean resetLevel,
+                                          boolean clearRepaired,
+                                          boolean verifySSTables,
+                                          boolean verifyTokens,
+                                          boolean invalidateCaches,
+                                          boolean extendedVerify);
 
     /**
      * Load new sstables from the given directory
@@ -197,6 +210,8 @@ public interface ColumnFamilyStoreMBean
      *
      * @return list of failed import directories
      */
+    /** @deprecated See CASSANDRA-18714 */
+    @Deprecated(since = "5.0")
     public List<String> importNewSSTables(Set<String> srcPaths,
                                           boolean resetLevel,
                                           boolean clearRepaired,
@@ -206,7 +221,35 @@ public interface ColumnFamilyStoreMBean
                                           boolean extendedVerify,
                                           boolean copyData);
 
-    @Deprecated
+    /**
+     * Load new sstables from the given directory
+     *
+     * @param srcPaths the path to the new sstables - if it is an empty set, the data directories will be scanned
+     * @param resetLevel if the level should be reset to 0 on the new sstables
+     * @param clearRepaired if repaired info should be wiped from the new sstables
+     * @param verifySSTables if the new sstables should be verified that they are not corrupt
+     * @param verifyTokens if the tokens in the new sstables should be verified that they are owned by the current node
+     * @param invalidateCaches if row cache should be invalidated for the keys in the new sstables
+     * @param extendedVerify if we should run an extended verify checking all values in the new sstables
+     * @param copyData if we should copy data from source paths instead of moving them
+     * @param validateIndexChecksum if we should also validate checksum for SAI indexes
+     * @param failOnMissingIndex if loading should fail when SSTables do not contain built SAI indexes too
+     *
+     * @return list of failed import directories
+     */
+    public List<String> importNewSSTables(Set<String> srcPaths,
+                                          boolean resetLevel,
+                                          boolean clearRepaired,
+                                          boolean verifySSTables,
+                                          boolean verifyTokens,
+                                          boolean invalidateCaches,
+                                          boolean extendedVerify,
+                                          boolean copyData,
+                                          boolean failOnMissingIndex,
+                                          boolean validateIndexChecksum);
+
+    /** @deprecated See CASSANDRA-6719 */
+    @Deprecated(since = "4.0")
     public void loadNewSSTables();
     /**
      * @return the number of SSTables in L0.  Always return 0 if Leveled compaction is not enabled.
@@ -225,6 +268,17 @@ public interface ColumnFamilyStoreMBean
      */
     public long[] getPerLevelSizeBytes();
 
+    /**
+     * @return true if the table is using LeveledCompactionStrategy. false otherwise.
+     */
+    public boolean isLeveledCompaction();
+
+    /**
+     * @return sstable count for each bucket in TWCS. null unless time window compaction is used.
+     *         array index corresponds to bucket(int[0] is for most recent, ...).
+     */
+    public int[] getSSTableCountPerTWCSBucket();
+
     /**
      * @return sstable fanout size for level compaction strategy.
      */
@@ -279,4 +333,23 @@ public interface ColumnFamilyStoreMBean
     public Long getTopSizePartitionsLastUpdate();
     public Map<String, Long> getTopTombstonePartitions();
     public Long getTopTombstonePartitionsLastUpdate();
+
+    /**
+     * Returns the size of the biggest SSTable of this table.
+     *
+     * @return (physical) size of the biggest SSTable of this table on disk or 0 if no SSTable is present
+     */
+    public long getMaxSSTableSize();
+
+    /**
+     * Returns the longest duration of an SSTable, in milliseconds, of this table,
+     * computed as {@code maxTimestamp - minTimestamp}.
+     *
+     * It returns 0 if there are no SSTables or if {@code maxTimestamp} or {@code minTimestamp} is
+     * equal to {@code Long.MAX_VALUE}. Effectively non-zero for tables on {@code TimeWindowCompactionStrategy}.
+     *
+     * @return the biggest {@code maxTimestamp - minTimestamp} among all SSTables of this table
+     * or 0 if no SSTable is present
+     */
+    public long getMaxSSTableDuration();
 }
