@@ -18,30 +18,30 @@
 package org.apache.cassandra.db;
 
 import java.io.IOException;
+import java.nio.ByteBuffer;
 import java.util.*;
 import java.util.function.Consumer;
 import java.util.function.Predicate;
-import java.nio.ByteBuffer;
 
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.Iterators;
 
 import net.nicoulaj.compilecommand.annotations.DontInline;
-import org.apache.cassandra.exceptions.UnknownColumnException;
-import org.apache.cassandra.schema.ColumnMetadata;
-import org.apache.cassandra.schema.TableMetadata;
 import org.apache.cassandra.cql3.ColumnIdentifier;
 import org.apache.cassandra.db.marshal.SetType;
 import org.apache.cassandra.db.marshal.UTF8Type;
+import org.apache.cassandra.db.rows.ColumnData;
+import org.apache.cassandra.db.rows.Row;
 import org.apache.cassandra.io.util.DataInputPlus;
 import org.apache.cassandra.io.util.DataOutputPlus;
+import org.apache.cassandra.schema.ColumnMetadata;
+import org.apache.cassandra.schema.TableMetadata;
 import org.apache.cassandra.utils.ByteBufferUtil;
 import org.apache.cassandra.utils.ObjectSizes;
 import org.apache.cassandra.utils.SearchIterator;
 import org.apache.cassandra.utils.btree.BTree;
-import org.apache.cassandra.utils.btree.BTreeSearchIterator;
 import org.apache.cassandra.utils.btree.BTreeRemoval;
-import org.apache.cassandra.utils.btree.UpdateFunction;
+import org.apache.cassandra.utils.btree.BTreeSearchIterator;
 
 /**
  * An immutable and sorted list of (non-PK) columns for a given table.
@@ -61,7 +61,8 @@ public class Columns extends AbstractCollection<ColumnMetadata> implements Colle
                            ColumnIdentifier.getInterned(ByteBufferUtil.EMPTY_BYTE_BUFFER, UTF8Type.instance),
                            SetType.getInstance(UTF8Type.instance, true),
                            ColumnMetadata.NO_POSITION,
-                           ColumnMetadata.Kind.STATIC);
+                           ColumnMetadata.Kind.STATIC,
+                           null);
 
     public static final ColumnMetadata FIRST_COMPLEX_REGULAR =
         new ColumnMetadata("",
@@ -69,7 +70,8 @@ public class Columns extends AbstractCollection<ColumnMetadata> implements Colle
                            ColumnIdentifier.getInterned(ByteBufferUtil.EMPTY_BYTE_BUFFER, UTF8Type.instance),
                            SetType.getInstance(UTF8Type.instance, true),
                            ColumnMetadata.NO_POSITION,
-                           ColumnMetadata.Kind.REGULAR);
+                           ColumnMetadata.Kind.REGULAR,
+                           null);
 
     private final Object[] columns;
     private final int complexIdx; // Index of the first complex column
@@ -98,8 +100,32 @@ public class Columns extends AbstractCollection<ColumnMetadata> implements Colle
         return new Columns(BTree.singleton(c), c.isComplex() ? 0 : 1);
     }
 
+   /**
+    * Returns a new {@code Columns} object holing the same columns as the provided Row.
+    *
+    * @param row the row from which to create the new {@code Columns}.
+    * @return the newly created {@code Columns} containing the columns from {@code row}.
+    */
+   public static Columns from(Row row)
+   {
+       try (BTree.FastBuilder<ColumnMetadata> builder = BTree.fastBuilder())
+       {
+           for (ColumnData cd : row)
+               builder.add(cd.column());
+           Object[] tree = builder.build();
+           return new Columns(tree, findFirstComplexIdx(tree));
+       }
+   }
+
+   public static Columns from(BTree.Builder<ColumnMetadata> builder)
+   {
+       Object[] tree = builder.build();
+       return new Columns(tree, findFirstComplexIdx(tree));
+   }
+
     /**
-     * Returns a new {@code Columns} object holing the same columns than the provided set.
+    * Returns a new {@code Columns} object holding the same columns than the provided set.
+     * This method assumes nothing about the order of {@code s}.
      *
      * @param s the set from which to create the new {@code Columns}.
      * @return the newly created {@code Columns} containing the columns from {@code s}.
@@ -265,8 +291,7 @@ public class Columns extends AbstractCollection<ColumnMetadata> implements Colle
         if (this == NONE)
             return other;
 
-        Object[] tree = BTree.<ColumnMetadata>merge(this.columns, other.columns, Comparator.naturalOrder(),
-                                                    UpdateFunction.noOp());
+        Object[] tree = BTree.update(this.columns, other.columns, Comparator.naturalOrder());
         if (tree == this.columns)
             return this;
         if (tree == other.columns)
@@ -413,7 +438,7 @@ public class Columns extends AbstractCollection<ColumnMetadata> implements Colle
         if(this == NONE)
             return 0;
 
-        return EMPTY_SIZE;
+        return EMPTY_SIZE + BTree.sizeOfStructureOnHeap(columns);
     }
 
     @Override
@@ -433,7 +458,7 @@ public class Columns extends AbstractCollection<ColumnMetadata> implements Colle
     {
         public void serialize(Columns columns, DataOutputPlus out) throws IOException
         {
-            out.writeUnsignedVInt(columns.size());
+            out.writeUnsignedVInt32(columns.size());
             for (ColumnMetadata column : columns)
                 ByteBufferUtil.writeWithVIntLength(column.name.bytes, out);
         }
@@ -448,26 +473,27 @@ public class Columns extends AbstractCollection<ColumnMetadata> implements Colle
 
         public Columns deserialize(DataInputPlus in, TableMetadata metadata) throws IOException
         {
-            int length = (int)in.readUnsignedVInt();
-            BTree.Builder<ColumnMetadata> builder = BTree.builder(Comparator.naturalOrder());
-            builder.auto(false);
-            for (int i = 0; i < length; i++)
+            int length = in.readUnsignedVInt32();
+            try (BTree.FastBuilder<ColumnMetadata> builder = BTree.fastBuilder())
             {
-                ByteBuffer name = ByteBufferUtil.readWithVIntLength(in);
-                ColumnMetadata column = metadata.getColumn(name);
-                if (column == null)
+                for (int i = 0; i < length; i++)
                 {
-                    // If we don't find the definition, it could be we have data for a dropped column, and we shouldn't
-                    // fail deserialization because of that. So we grab a "fake" ColumnMetadata that ensure proper
-                    // deserialization. The column will be ignore later on anyway.
-                    column = metadata.getDroppedColumn(name);
-
+                    ByteBuffer name = ByteBufferUtil.readWithVIntLength(in);
+                    ColumnMetadata column = metadata.getColumn(name);
                     if (column == null)
-                        throw new RuntimeException("Unknown column " + UTF8Type.instance.getString(name) + " during deserialization");
+                    {
+                        // If we don't find the definition, it could be we have data for a dropped column, and we shouldn't
+                        // fail deserialization because of that. So we grab a "fake" ColumnMetadata that ensure proper
+                        // deserialization. The column will be ignore later on anyway.
+                        column = metadata.getDroppedColumn(name);
+
+                        if (column == null)
+                            throw new RuntimeException("Unknown column " + UTF8Type.instance.getString(name) + " during deserialization");
+                    }
+                    builder.add(column);
                 }
-                builder.add(column);
+                return new Columns(builder.build());
             }
-            return new Columns(builder.build());
         }
 
         /**
@@ -492,7 +518,7 @@ public class Columns extends AbstractCollection<ColumnMetadata> implements Colle
             int supersetCount = superset.size();
             if (columnCount == supersetCount)
             {
-                out.writeUnsignedVInt(0);
+                out.writeUnsignedVInt32(0);
             }
             else if (supersetCount < 64)
             {
@@ -535,21 +561,23 @@ public class Columns extends AbstractCollection<ColumnMetadata> implements Colle
             }
             else
             {
-                BTree.Builder<ColumnMetadata> builder = BTree.builder(Comparator.naturalOrder());
-                int firstComplexIdx = 0;
-                for (ColumnMetadata column : superset)
+                try (BTree.FastBuilder<ColumnMetadata> builder = BTree.fastBuilder())
                 {
-                    if ((encoded & 1) == 0)
+                    int firstComplexIdx = 0;
+                    for (ColumnMetadata column : superset)
                     {
-                        builder.add(column);
-                        if (column.isSimple())
-                            ++firstComplexIdx;
+                        if ((encoded & 1) == 0)
+                        {
+                            builder.add(column);
+                            if (column.isSimple())
+                                ++firstComplexIdx;
+                        }
+                        encoded >>>= 1;
                     }
-                    encoded >>>= 1;
+                    if (encoded != 0)
+                        throw new IOException("Invalid Columns subset bytes; too many bits set:" + Long.toBinaryString(encoded));
+                    return new Columns(builder.build(), firstComplexIdx);
                 }
-                if (encoded != 0)
-                    throw new IOException("Invalid Columns subset bytes; too many bits set:" + Long.toBinaryString(encoded));
-                return new Columns(builder.build(), firstComplexIdx);
             }
         }
 
@@ -583,7 +611,7 @@ public class Columns extends AbstractCollection<ColumnMetadata> implements Colle
         private void serializeLargeSubset(Collection<ColumnMetadata> columns, int columnCount, Columns superset, int supersetCount, DataOutputPlus out) throws IOException
         {
             // write flag indicating we're in lengthy mode
-            out.writeUnsignedVInt(supersetCount - columnCount);
+            out.writeUnsignedVInt32(supersetCount - columnCount);
             BTreeSearchIterator<ColumnMetadata, ColumnMetadata> iter = superset.iterator();
             if (columnCount < supersetCount / 2)
             {
@@ -592,7 +620,7 @@ public class Columns extends AbstractCollection<ColumnMetadata> implements Colle
                 {
                     if (iter.next(column) == null)
                         throw new IllegalStateException();
-                    out.writeUnsignedVInt(iter.indexOfCurrent());
+                    out.writeUnsignedVInt32(iter.indexOfCurrent());
                 }
             }
             else
@@ -605,10 +633,10 @@ public class Columns extends AbstractCollection<ColumnMetadata> implements Colle
                         throw new IllegalStateException();
                     int cur = iter.indexOfCurrent();
                     while (++prev != cur)
-                        out.writeUnsignedVInt(prev);
+                        out.writeUnsignedVInt32(prev);
                 }
                 while (++prev != supersetCount)
-                    out.writeUnsignedVInt(prev);
+                    out.writeUnsignedVInt32(prev);
             }
         }
 
@@ -618,37 +646,39 @@ public class Columns extends AbstractCollection<ColumnMetadata> implements Colle
             int supersetCount = superset.size();
             int columnCount = supersetCount - delta;
 
-            BTree.Builder<ColumnMetadata> builder = BTree.builder(Comparator.naturalOrder());
-            if (columnCount < supersetCount / 2)
+            try (BTree.FastBuilder<ColumnMetadata> builder = BTree.fastBuilder())
             {
-                for (int i = 0 ; i < columnCount ; i++)
+                if (columnCount < supersetCount / 2)
                 {
-                    int idx = (int) in.readUnsignedVInt();
-                    builder.add(BTree.findByIndex(superset.columns, idx));
-                }
-            }
-            else
-            {
-                Iterator<ColumnMetadata> iter = superset.iterator();
-                int idx = 0;
-                int skipped = 0;
-                while (true)
-                {
-                    int nextMissingIndex = skipped < delta ? (int)in.readUnsignedVInt() : supersetCount;
-                    while (idx < nextMissingIndex)
+                    for (int i = 0 ; i < columnCount ; i++)
                     {
-                        ColumnMetadata def = iter.next();
-                        builder.add(def);
-                        idx++;
+                        int idx = in.readUnsignedVInt32();
+                        builder.add(BTree.findByIndex(superset.columns, idx));
                     }
-                    if (idx == supersetCount)
-                        break;
-                    iter.next();
-                    idx++;
-                    skipped++;
                 }
+                else
+                {
+                    Iterator<ColumnMetadata> iter = superset.iterator();
+                    int idx = 0;
+                    int skipped = 0;
+                    while (true)
+                    {
+                        int nextMissingIndex = skipped < delta ? in.readUnsignedVInt32() : supersetCount;
+                        while (idx < nextMissingIndex)
+                        {
+                            ColumnMetadata def = iter.next();
+                            builder.add(def);
+                            idx++;
+                        }
+                        if (idx == supersetCount)
+                            break;
+                        iter.next();
+                        idx++;
+                        skipped++;
+                    }
+                }
+                return new Columns(builder.build());
             }
-            return new Columns(builder.build());
         }
 
         @DontInline
