@@ -63,7 +63,6 @@ public class PartitionRangeReadCommand extends ReadCommand implements PartitionR
 {
     protected static final SelectionDeserializer selectionDeserializer = new Deserializer();
 
-    protected final DataRange dataRange;
     protected final Slices requestedSlices;
 
     @VisibleForTesting
@@ -79,8 +78,7 @@ public class PartitionRangeReadCommand extends ReadCommand implements PartitionR
                                       Index.QueryPlan indexQueryPlan,
                                       boolean trackWarnings)
     {
-        super(Kind.PARTITION_RANGE, isDigest, digestVersion, acceptsTransient, metadata, nowInSec, columnFilter, rowFilter, limits, indexQueryPlan, trackWarnings);
-        this.dataRange = dataRange;
+        super(Kind.PARTITION_RANGE, isDigest, digestVersion, acceptsTransient, metadata, nowInSec, columnFilter, rowFilter, limits, indexQueryPlan, trackWarnings, dataRange);
         this.requestedSlices = dataRange.clusteringIndexFilter.getSlices(metadata());
 
     }
@@ -167,11 +165,6 @@ public class PartitionRangeReadCommand extends ReadCommand implements PartitionR
                       false);
     }
 
-    public DataRange dataRange()
-    {
-        return dataRange;
-    }
-
     public ClusteringIndexFilter clusteringIndexFilter(DecoratedKey key)
     {
         return dataRange.clusteringIndexFilter(key);
