@@ -24,6 +24,8 @@ import java.util.Iterator;
 
 import org.apache.cassandra.config.DatabaseDescriptor;
 import org.apache.cassandra.utils.AbstractIterator;
+import org.apache.cassandra.utils.CassandraUInt;
+
 import com.google.common.collect.Iterators;
 
 import org.apache.cassandra.cache.IMeasurableMemory;
@@ -58,19 +60,25 @@ public class RangeTombstoneList implements Iterable<RangeTombstone>, IMeasurable
     private ClusteringBound<?>[] starts;
     private ClusteringBound<?>[] ends;
     private long[] markedAts;
-    private int[] delTimes;
+    private int[] delTimesUnsignedIntegers;
 
     private long boundaryHeapSize;
     private int size;
 
-    private RangeTombstoneList(ClusteringComparator comparator, ClusteringBound<?>[] starts, ClusteringBound<?>[] ends, long[] markedAts, int[] delTimes, long boundaryHeapSize, int size)
+    private RangeTombstoneList(ClusteringComparator comparator,
+                               ClusteringBound<?>[] starts,
+                               ClusteringBound<?>[] ends,
+                               long[] markedAts,
+                               int[] delTimesUnsignedIntegers,
+                               long boundaryHeapSize,
+                               int size)
     {
-        assert starts.length == ends.length && starts.length == markedAts.length && starts.length == delTimes.length;
+        assert starts.length == ends.length && starts.length == markedAts.length && starts.length == delTimesUnsignedIntegers.length;
         this.comparator = comparator;
         this.starts = starts;
         this.ends = ends;
         this.markedAts = markedAts;
-        this.delTimes = delTimes;
+        this.delTimesUnsignedIntegers = delTimesUnsignedIntegers;
         this.size = size;
         this.boundaryHeapSize = boundaryHeapSize;
     }
@@ -101,7 +109,7 @@ public class RangeTombstoneList implements Iterable<RangeTombstone>, IMeasurable
                                       Arrays.copyOf(starts, size),
                                       Arrays.copyOf(ends, size),
                                       Arrays.copyOf(markedAts, size),
-                                      Arrays.copyOf(delTimes, size),
+                                      Arrays.copyOf(delTimesUnsignedIntegers, size),
                                       boundaryHeapSize, size);
     }
 
@@ -111,7 +119,7 @@ public class RangeTombstoneList implements Iterable<RangeTombstone>, IMeasurable
                                                           new ClusteringBound<?>[size],
                                                           new ClusteringBound<?>[size],
                                                           Arrays.copyOf(markedAts, size),
-                                                          Arrays.copyOf(delTimes, size),
+                                                          Arrays.copyOf(delTimesUnsignedIntegers, size),
                                                           boundaryHeapSize, size);
 
 
@@ -137,7 +145,7 @@ public class RangeTombstoneList implements Iterable<RangeTombstone>, IMeasurable
         add(tombstone.deletedSlice().start(),
             tombstone.deletedSlice().end(),
             tombstone.deletionTime().markedForDeleteAt(),
-            tombstone.deletionTime().localDeletionTime());
+            tombstone.deletionTime().localDeletionTimeUnsignedInteger);
     }
 
     /**
@@ -146,11 +154,11 @@ public class RangeTombstoneList implements Iterable<RangeTombstone>, IMeasurable
      * This method will be faster if the new tombstone sort after all the currently existing ones (this is a common use case),
      * but it doesn't assume it.
      */
-    public void add(ClusteringBound<?> start, ClusteringBound<?> end, long markedAt, int delTime)
+    private void add(ClusteringBound<?> start, ClusteringBound<?> end, long markedAt, int delTimeUnsignedInteger)
     {
         if (isEmpty())
         {
-            addInternal(0, start, end, markedAt, delTime);
+            addInternal(0, start, end, markedAt, delTimeUnsignedInteger);
             return;
         }
 
@@ -159,13 +167,13 @@ public class RangeTombstoneList implements Iterable<RangeTombstone>, IMeasurable
         // Fast path if we add in sorted order
         if (c <= 0)
         {
-            addInternal(size, start, end, markedAt, delTime);
+            addInternal(size, start, end, markedAt, delTimeUnsignedInteger);
         }
         else
         {
             // Note: insertFrom expect i to be the insertion point in term of interval ends
             int pos = Arrays.binarySearch(ends, 0, size, start, comparator);
-            insertFrom((pos >= 0 ? pos+1 : -pos-1), start, end, markedAt, delTime);
+            insertFrom((pos >= 0 ? pos+1 : -pos-1), start, end, markedAt, delTimeUnsignedInteger);
         }
         boundaryHeapSize += start.unsharedHeapSize() + end.unsharedHeapSize();
     }
@@ -205,7 +213,7 @@ public class RangeTombstoneList implements Iterable<RangeTombstone>, IMeasurable
         if (size > 10 * tombstones.size)
         {
             for (int i = 0; i < tombstones.size; i++)
-                add(tombstones.starts[i], tombstones.ends[i], tombstones.markedAts[i], tombstones.delTimes[i]);
+                add(tombstones.starts[i], tombstones.ends[i], tombstones.markedAts[i], tombstones.delTimesUnsignedIntegers[i]);
         }
         else
         {
@@ -215,7 +223,7 @@ public class RangeTombstoneList implements Iterable<RangeTombstone>, IMeasurable
             {
                 if (comparator.compare(tombstones.starts[j], ends[i]) < 0)
                 {
-                    insertFrom(i, tombstones.starts[j], tombstones.ends[j], tombstones.markedAts[j], tombstones.delTimes[j]);
+                    insertFrom(i, tombstones.starts[j], tombstones.ends[j], tombstones.markedAts[j], tombstones.delTimesUnsignedIntegers[j]);
                     j++;
                 }
                 else
@@ -225,7 +233,7 @@ public class RangeTombstoneList implements Iterable<RangeTombstone>, IMeasurable
             }
             // Addds the remaining ones from tombstones if any (note that addInternal will increment size if relevant).
             for (; j < tombstones.size; j++)
-                addInternal(size, tombstones.starts[j], tombstones.ends[j], tombstones.markedAts[j], tombstones.delTimes[j]);
+                addInternal(size, tombstones.starts[j], tombstones.ends[j], tombstones.markedAts[j], tombstones.delTimesUnsignedIntegers[j]);
         }
     }
 
@@ -247,7 +255,7 @@ public class RangeTombstoneList implements Iterable<RangeTombstone>, IMeasurable
     public DeletionTime searchDeletionTime(Clustering<?> name)
     {
         int idx = searchInternal(name, 0, size);
-        return idx < 0 ? null : new DeletionTime(markedAts[idx], delTimes[idx]);
+        return idx < 0 ? null : DeletionTime.buildUnsafeWithUnsignedInteger(markedAts[idx], delTimesUnsignedIntegers[idx]);
     }
 
     public RangeTombstone search(Clustering<?> name)
@@ -292,7 +300,7 @@ public class RangeTombstoneList implements Iterable<RangeTombstone>, IMeasurable
         {
             dataSize += starts[i].dataSize() + ends[i].dataSize();
             dataSize += TypeSizes.sizeof(markedAts[i]);
-            dataSize += TypeSizes.sizeof(delTimes[i]);
+            dataSize += TypeSizes.sizeof(delTimesUnsignedIntegers[i]);
         }
         return dataSize;
     }
@@ -310,7 +318,7 @@ public class RangeTombstoneList implements Iterable<RangeTombstone>, IMeasurable
         for (int i = 0; i < size; i++)
         {
             collector.updateTimestamp(markedAts[i]);
-            collector.updateLocalDeletionTime(delTimes[i]);
+            collector.updateLocalDeletionTime(CassandraUInt.toLong(delTimesUnsignedIntegers[i]));
         }
     }
 
@@ -322,22 +330,22 @@ public class RangeTombstoneList implements Iterable<RangeTombstone>, IMeasurable
 
     private RangeTombstone rangeTombstone(int idx)
     {
-        return new RangeTombstone(Slice.make(starts[idx], ends[idx]), new DeletionTime(markedAts[idx], delTimes[idx]));
+        return new RangeTombstone(Slice.make(starts[idx], ends[idx]), DeletionTime.buildUnsafeWithUnsignedInteger(markedAts[idx], delTimesUnsignedIntegers[idx]));
     }
 
     private RangeTombstone rangeTombstoneWithNewStart(int idx, ClusteringBound<?> newStart)
     {
-        return new RangeTombstone(Slice.make(newStart, ends[idx]), new DeletionTime(markedAts[idx], delTimes[idx]));
+        return new RangeTombstone(Slice.make(newStart, ends[idx]), DeletionTime.buildUnsafeWithUnsignedInteger(markedAts[idx], delTimesUnsignedIntegers[idx]));
     }
 
     private RangeTombstone rangeTombstoneWithNewEnd(int idx, ClusteringBound<?> newEnd)
     {
-        return new RangeTombstone(Slice.make(starts[idx], newEnd), new DeletionTime(markedAts[idx], delTimes[idx]));
+        return new RangeTombstone(Slice.make(starts[idx], newEnd), DeletionTime.buildUnsafeWithUnsignedInteger(markedAts[idx], delTimesUnsignedIntegers[idx]));
     }
 
     private RangeTombstone rangeTombstoneWithNewBounds(int idx, ClusteringBound<?> newStart, ClusteringBound<?> newEnd)
     {
-        return new RangeTombstone(Slice.make(newStart, newEnd), new DeletionTime(markedAts[idx], delTimes[idx]));
+        return new RangeTombstone(Slice.make(newStart, newEnd), DeletionTime.buildUnsafeWithUnsignedInteger(markedAts[idx], delTimesUnsignedIntegers[idx]));
     }
 
     public Iterator<RangeTombstone> iterator()
@@ -345,7 +353,6 @@ public class RangeTombstoneList implements Iterable<RangeTombstone>, IMeasurable
         return iterator(false);
     }
 
-    @SuppressWarnings("resource")
     public Iterator<RangeTombstone> iterator(boolean reversed)
     {
         return reversed
@@ -491,7 +498,7 @@ public class RangeTombstoneList implements Iterable<RangeTombstone>, IMeasurable
                 return false;
             if (markedAts[i] != that.markedAts[i])
                 return false;
-            if (delTimes[i] != that.delTimes[i])
+            if (delTimesUnsignedIntegers[i] != that.delTimesUnsignedIntegers[i])
                 return false;
         }
         return true;
@@ -505,7 +512,7 @@ public class RangeTombstoneList implements Iterable<RangeTombstone>, IMeasurable
         {
             result += starts[i].hashCode() + ends[i].hashCode();
             result += (int)(markedAts[i] ^ (markedAts[i] >>> 32));
-            result += delTimes[i];
+            result += delTimesUnsignedIntegers[i];
         }
         return result;
     }
@@ -516,7 +523,7 @@ public class RangeTombstoneList implements Iterable<RangeTombstone>, IMeasurable
         System.arraycopy(src.starts, 0, dst.starts, 0, src.size);
         System.arraycopy(src.ends, 0, dst.ends, 0, src.size);
         System.arraycopy(src.markedAts, 0, dst.markedAts, 0, src.size);
-        System.arraycopy(src.delTimes, 0, dst.delTimes, 0, src.size);
+        System.arraycopy(src.delTimesUnsignedIntegers, 0, dst.delTimesUnsignedIntegers, 0, src.size);
         dst.size = src.size;
         dst.boundaryHeapSize = src.boundaryHeapSize;
     }
@@ -533,7 +540,7 @@ public class RangeTombstoneList implements Iterable<RangeTombstone>, IMeasurable
      *   - e_i <= s_i+1
      * Basically, range are non overlapping and in order.
      */
-    private void insertFrom(int i, ClusteringBound<?> start, ClusteringBound<?> end, long markedAt, int delTime)
+    private void insertFrom(int i, ClusteringBound<?> start, ClusteringBound<?> end, long markedAt, int delTimeUnsignedInternal)
     {
         while (i < size)
         {
@@ -555,9 +562,9 @@ public class RangeTombstoneList implements Iterable<RangeTombstone>, IMeasurable
                     ClusteringBound<?> newEnd = start.invert();
                     if (!Slice.isEmpty(comparator, starts[i], newEnd))
                     {
-                        addInternal(i, starts[i], newEnd, markedAts[i], delTimes[i]);
+                        addInternal(i, starts[i], newEnd, markedAts[i], delTimesUnsignedIntegers[i]);
                         i++;
-                        setInternal(i, start, ends[i], markedAts[i], delTimes[i]);
+                        setInternal(i, start, ends[i], markedAts[i], delTimesUnsignedIntegers[i]);
                     }
                 }
 
@@ -569,7 +576,7 @@ public class RangeTombstoneList implements Iterable<RangeTombstone>, IMeasurable
                 {
                     // Here start <= starts[i] and end < starts[i]
                     // This means the current element is before the current one.
-                    addInternal(i, start, end, markedAt, delTime);
+                    addInternal(i, start, end, markedAt, delTimeUnsignedInternal);
                     return;
                 }
 
@@ -586,11 +593,11 @@ public class RangeTombstoneList implements Iterable<RangeTombstone>, IMeasurable
                     // we're good with the next element
                     if (i == size-1 || comparator.compare(end, starts[i+1]) <= 0)
                     {
-                        setInternal(i, start, end, markedAt, delTime);
+                        setInternal(i, start, end, markedAt, delTimeUnsignedInternal);
                         return;
                     }
 
-                    setInternal(i, start, starts[i+1].invert(), markedAt, delTime);
+                    setInternal(i, start, starts[i+1].invert(), markedAt, delTimeUnsignedInternal);
                     start = starts[i+1];
                     i++;
                 }
@@ -598,12 +605,12 @@ public class RangeTombstoneList implements Iterable<RangeTombstone>, IMeasurable
                 {
                     // We don't overwrite fully. Insert the new interval, and then update the now next
                     // one to reflect the not overwritten parts. We're then done.
-                    addInternal(i, start, end, markedAt, delTime);
+                    addInternal(i, start, end, markedAt, delTimeUnsignedInternal);
                     i++;
                     ClusteringBound<?> newStart = end.invert();
                     if (!Slice.isEmpty(comparator, newStart, ends[i]))
                     {
-                        setInternal(i, newStart, ends[i], markedAts[i], delTimes[i]);
+                        setInternal(i, newStart, ends[i], markedAts[i], delTimesUnsignedIntegers[i]);
                     }
                     return;
                 }
@@ -619,13 +626,13 @@ public class RangeTombstoneList implements Iterable<RangeTombstone>, IMeasurable
                     // otherwise insert until the beginning of the current element
                     if (comparator.compare(end, starts[i]) <= 0)
                     {
-                        addInternal(i, start, end, markedAt, delTime);
+                        addInternal(i, start, end, markedAt, delTimeUnsignedInternal);
                         return;
                     }
                     ClusteringBound<?> newEnd = starts[i].invert();
                     if (!Slice.isEmpty(comparator, start, newEnd))
                     {
-                        addInternal(i, start, newEnd, markedAt, delTime);
+                        addInternal(i, start, newEnd, markedAt, delTimeUnsignedInternal);
                         i++;
                     }
                 }
@@ -643,7 +650,7 @@ public class RangeTombstoneList implements Iterable<RangeTombstone>, IMeasurable
         }
 
         // If we got there, then just insert the remainder at the end
-        addInternal(i, start, end, markedAt, delTime);
+        addInternal(i, start, end, markedAt, delTimeUnsignedInternal);
     }
 
     private int capacity()
@@ -654,7 +661,7 @@ public class RangeTombstoneList implements Iterable<RangeTombstone>, IMeasurable
     /*
      * Adds the new tombstone at index i, growing and/or moving elements to make room for it.
      */
-    private void addInternal(int i, ClusteringBound<?> start, ClusteringBound<?> end, long markedAt, int delTime)
+    private void addInternal(int i, ClusteringBound<?> start, ClusteringBound<?> end, long markedAt, int delTimeUnsignedInteger)
     {
         assert i >= 0;
 
@@ -663,7 +670,7 @@ public class RangeTombstoneList implements Iterable<RangeTombstone>, IMeasurable
         else if (i < size)
             moveElements(i);
 
-        setInternal(i, start, end, markedAt, delTime);
+        setInternal(i, start, end, markedAt, delTimeUnsignedInteger);
         size++;
     }
 
@@ -695,7 +702,7 @@ public class RangeTombstoneList implements Iterable<RangeTombstone>, IMeasurable
         starts = grow(starts, size, newLength, i);
         ends = grow(ends, size, newLength, i);
         markedAts = grow(markedAts, size, newLength, i);
-        delTimes = grow(delTimes, size, newLength, i);
+        delTimesUnsignedIntegers = grow(delTimesUnsignedIntegers, size, newLength, i);
     }
 
     private static ClusteringBound<?>[] grow(ClusteringBound<?>[] a, int size, int newLength, int i)
@@ -742,20 +749,20 @@ public class RangeTombstoneList implements Iterable<RangeTombstone>, IMeasurable
         System.arraycopy(starts, i, starts, i+1, size - i);
         System.arraycopy(ends, i, ends, i+1, size - i);
         System.arraycopy(markedAts, i, markedAts, i+1, size - i);
-        System.arraycopy(delTimes, i, delTimes, i+1, size - i);
+        System.arraycopy(delTimesUnsignedIntegers, i, delTimesUnsignedIntegers, i+1, size - i);
         // we set starts[i] to null to indicate the position is now empty, so that we update boundaryHeapSize
         // when we set it
         starts[i] = null;
     }
 
-    private void setInternal(int i, ClusteringBound<?> start, ClusteringBound<?> end, long markedAt, int delTime)
+    private void setInternal(int i, ClusteringBound<?> start, ClusteringBound<?> end, long markedAt, int delTimeUnsignedInteger)
     {
         if (starts[i] != null)
             boundaryHeapSize -= starts[i].unsharedHeapSize() + ends[i].unsharedHeapSize();
         starts[i] = start;
         ends[i] = end;
         markedAts[i] = markedAt;
-        delTimes[i] = delTime;
+        delTimesUnsignedIntegers[i] = delTimeUnsignedInteger;
         boundaryHeapSize += start.unsharedHeapSize() + end.unsharedHeapSize();
     }
 
@@ -767,6 +774,6 @@ public class RangeTombstoneList implements Iterable<RangeTombstone>, IMeasurable
                 + ObjectSizes.sizeOfArray(starts)
                 + ObjectSizes.sizeOfArray(ends)
                 + ObjectSizes.sizeOfArray(markedAts)
-                + ObjectSizes.sizeOfArray(delTimes);
+                + ObjectSizes.sizeOfArray(delTimesUnsignedIntegers);
     }
 }
