@@ -25,21 +25,28 @@ import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
+import java.util.UUID;
 
 import com.google.common.annotations.VisibleForTesting;
-
-import org.apache.cassandra.io.util.File;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import org.apache.cassandra.db.compaction.Verifier;
 import org.apache.cassandra.db.lifecycle.SSTableSet;
+import org.apache.cassandra.index.Index;
+import org.apache.cassandra.index.sai.StorageAttachedIndexGroup;
+import org.apache.cassandra.index.sai.disk.format.IndexDescriptor;
+import org.apache.cassandra.index.sai.utils.IndexIdentifier;
 import org.apache.cassandra.io.sstable.Component;
 import org.apache.cassandra.io.sstable.Descriptor;
+import org.apache.cassandra.io.sstable.IVerifier;
 import org.apache.cassandra.io.sstable.KeyIterator;
+import org.apache.cassandra.io.sstable.SSTable;
+import org.apache.cassandra.io.sstable.format.SSTableFormat.Components;
 import org.apache.cassandra.io.sstable.format.SSTableReader;
-import org.apache.cassandra.io.sstable.format.SSTableWriter;
+import org.apache.cassandra.io.util.File;
 import org.apache.cassandra.service.ActiveRepairService;
+import org.apache.cassandra.service.StorageService;
+import org.apache.cassandra.utils.OutputHandler;
 import org.apache.cassandra.utils.Pair;
 import org.apache.cassandra.utils.concurrent.Refs;
 
@@ -69,8 +76,8 @@ public class SSTableImporter
     @VisibleForTesting
     synchronized List<String> importNewSSTables(Options options)
     {
-        logger.info("Loading new SSTables for {}/{}: {}",
-                    cfs.keyspace.getName(), cfs.getTableName(), options);
+        UUID importID = UUID.randomUUID();
+        logger.info("[{}] Loading new SSTables for {}/{}: {}", importID, cfs.getKeyspaceName(), cfs.getTableName(), options);
 
         List<Pair<Directories.SSTableLister, String>> listers = getSSTableListers(options.srcPaths);
 
@@ -80,7 +87,7 @@ public class SSTableImporter
         List<String> failedDirectories = new ArrayList<>();
 
         // verify first to avoid starting to copy sstables to the data directories and then have to abort.
-        if (options.verifySSTables || options.verifyTokens)
+        if (options.verifySSTables || options.verifyTokens || options.failOnMissingIndex)
         {
             for (Pair<Directories.SSTableLister, String> listerPair : listers)
             {
@@ -93,19 +100,52 @@ public class SSTableImporter
                     {
                         try
                         {
-                            verifySSTableForImport(descriptor, entry.getValue(), options.verifyTokens, options.verifySSTables, options.extendedVerify);
+                            abortIfDraining();
+
+                            if (options.failOnMissingIndex)
+                            {
+                                Index.Group saiIndexGroup = cfs.indexManager.getIndexGroup(StorageAttachedIndexGroup.GROUP_KEY);
+                                if (saiIndexGroup != null)
+                                {
+                                    IndexDescriptor indexDescriptor = IndexDescriptor.create(descriptor,
+                                                                                             cfs.getPartitioner(),
+                                                                                             cfs.metadata().comparator);
+
+                                    String keyspace = cfs.getKeyspaceName();
+                                    String table = cfs.getTableName();
+
+                                    if (!indexDescriptor.isPerSSTableIndexBuildComplete())
+                                        throw new IllegalStateException(String.format("Missing SAI index to import for SSTable %s on %s.%s",
+                                                                                      indexDescriptor.sstableDescriptor.toString(),
+                                                                                      keyspace,
+                                                                                      table));
+
+                                    for (Index index : saiIndexGroup.getIndexes())
+                                    {
+                                        IndexIdentifier indexIdentifier = new IndexIdentifier(keyspace, table, index.getIndexMetadata().name);
+                                        if (!indexDescriptor.isPerColumnIndexBuildComplete(indexIdentifier))
+                                            throw new IllegalStateException(String.format("Missing SAI index to import for index %s on %s.%s",
+                                                                                          index.getIndexMetadata().name,
+                                                                                          keyspace,
+                                                                                          table));
+                                    }
+                                }
+                            }
+
+                            if (options.verifySSTables || options.verifyTokens)
+                                verifySSTableForImport(descriptor, entry.getValue(), options.verifyTokens, options.verifySSTables, options.extendedVerify);
                         }
                         catch (Throwable t)
                         {
                             if (dir != null)
                             {
-                                logger.error("Failed verifying sstable {} in directory {}", descriptor, dir, t);
+                                logger.error("[{}] Failed verifying SSTable {} in directory {}", importID, descriptor, dir, t);
                                 failedDirectories.add(dir);
                             }
                             else
                             {
-                                logger.error("Failed verifying sstable {}", descriptor, t);
-                                throw new RuntimeException("Failed verifying sstable "+descriptor, t);
+                                logger.error("[{}] Failed verifying SSTable {}", importID, descriptor, t);
+                                throw new RuntimeException("Failed verifying SSTable " + descriptor, t);
                             }
                             break;
                         }
@@ -128,6 +168,7 @@ public class SSTableImporter
             {
                 try
                 {
+                    abortIfDraining();
                     Descriptor oldDescriptor = entry.getKey();
                     if (currentDescriptors.contains(oldDescriptor))
                         continue;
@@ -144,7 +185,7 @@ public class SSTableImporter
                     newSSTablesPerDirectory.forEach(s -> s.selfRef().release());
                     if (dir != null)
                     {
-                        logger.error("Failed importing sstables in directory {}", dir, t);
+                        logger.error("[{}] Failed importing sstables in directory {}", importID, dir, t);
                         failedDirectories.add(dir);
                         if (options.copyData)
                         {
@@ -160,8 +201,8 @@ public class SSTableImporter
                     }
                     else
                     {
-                        logger.error("Failed importing sstables from data directory - renamed sstables are: {}", movedSSTables);
-                        throw new RuntimeException("Failed importing sstables", t);
+                        logger.error("[{}] Failed importing sstables from data directory - renamed SSTables are: {}", importID, movedSSTables, t);
+                        throw new RuntimeException("Failed importing SSTables", t);
                     }
                 }
             }
@@ -170,27 +211,72 @@ public class SSTableImporter
 
         if (newSSTables.isEmpty())
         {
-            logger.info("No new SSTables were found for {}/{}", cfs.keyspace.getName(), cfs.getTableName());
+            logger.info("[{}] No new SSTables were found for {}/{}", importID, cfs.getKeyspaceName(), cfs.getTableName());
             return failedDirectories;
         }
 
-        logger.info("Loading new SSTables and building secondary indexes for {}/{}: {}", cfs.keyspace.getName(), cfs.getTableName(), newSSTables);
+        logger.info("[{}] Loading new SSTables and building secondary indexes for {}/{}: {}", importID, cfs.getKeyspaceName(), cfs.getTableName(), newSSTables);
+        if (logger.isTraceEnabled())
+            logLeveling(importID, newSSTables);
 
         try (Refs<SSTableReader> refs = Refs.ref(newSSTables))
         {
+            abortIfDraining();
+
+            // Validate existing SSTable-attached indexes, and then build any that are missing:
+            if (!cfs.indexManager.validateSSTableAttachedIndexes(newSSTables, false, options.validateIndexChecksum))
+                cfs.indexManager.buildSSTableAttachedIndexesBlocking(newSSTables);
+
             cfs.getTracker().addSSTables(newSSTables);
             for (SSTableReader reader : newSSTables)
             {
                 if (options.invalidateCaches && cfs.isRowCacheEnabled())
-                    invalidateCachesForSSTable(reader.descriptor);
+                    invalidateCachesForSSTable(reader);
             }
-
+        }
+        catch (Throwable t)
+        {
+            logger.error("[{}] Failed adding SSTables", importID, t);
+            throw new RuntimeException("Failed adding SSTables", t);
         }
 
-        logger.info("Done loading load new SSTables for {}/{}", cfs.keyspace.getName(), cfs.getTableName());
+        logger.info("[{}] Done loading load new SSTables for {}/{}", importID, cfs.getKeyspaceName(), cfs.getTableName());
         return failedDirectories;
     }
 
+    /**
+     * Check the state of this node and throws an {@link InterruptedException} if it is currently draining
+     *
+     * @throws InterruptedException if the node is draining
+     */
+    private static void abortIfDraining() throws InterruptedException
+    {
+        if (StorageService.instance.isDraining())
+            throw new InterruptedException("SSTables import has been aborted");
+    }
+
+    private void logLeveling(UUID importID, Set<SSTableReader> newSSTables)
+    {
+        StringBuilder sb = new StringBuilder();
+        for (SSTableReader sstable : cfs.getSSTables(SSTableSet.CANONICAL))
+            sb.append(formatMetadata(sstable));
+        logger.debug("[{}] Current sstables: {}", importID, sb);
+        sb = new StringBuilder();
+        for (SSTableReader sstable : newSSTables)
+            sb.append(formatMetadata(sstable));
+        logger.debug("[{}] New sstables: {}", importID, sb);
+    }
+
+    private static String formatMetadata(SSTableReader sstable)
+    {
+        return String.format("{[%s, %s], %d, %s, %d}",
+                             sstable.getFirst().getToken(),
+                             sstable.getLast().getToken(),
+                             sstable.getSSTableLevel(),
+                             sstable.isRepaired(),
+                             sstable.onDiskLength());
+    }
+
     /**
      * Opens the sstablereader described by descriptor and figures out the correct directory for it based
      * on the first token
@@ -208,7 +294,7 @@ public class SSTableImporter
         SSTableReader sstable = null;
         try
         {
-            sstable = SSTableReader.open(descriptor, components, cfs.metadata);
+            sstable = SSTableReader.open(cfs, descriptor, components, cfs.metadata);
             targetDirectory = cfs.getDirectories().getLocationForDisk(cfs.diskBoundaryManager.getDiskBoundaries(cfs).getCorrectDiskForSSTable(sstable));
         }
         finally
@@ -216,7 +302,7 @@ public class SSTableImporter
             if (sstable != null)
                 sstable.selfRef().release();
         }
-        return targetDirectory == null ? cfs.getDirectories().getWriteableLocationToLoadFile(new File(descriptor.baseFilename())) : targetDirectory;
+        return targetDirectory == null ? cfs.getDirectories().getWriteableLocationToLoadFile(descriptor.baseFile()) : targetDirectory;
     }
 
     /**
@@ -279,11 +365,11 @@ public class SSTableImporter
     {
         for (MovedSSTable movedSSTable : movedSSTables)
         {
-            if (new File(movedSSTable.newDescriptor.filenameFor(Component.DATA)).exists())
+            if (movedSSTable.newDescriptor.fileFor(Components.DATA).exists())
             {
-                logger.debug("Moving sstable {} back to {}", movedSSTable.newDescriptor.filenameFor(Component.DATA)
-                                                          , movedSSTable.oldDescriptor.filenameFor(Component.DATA));
-                SSTableWriter.rename(movedSSTable.newDescriptor, movedSSTable.oldDescriptor, movedSSTable.components);
+                logger.debug("Moving sstable {} back to {}", movedSSTable.newDescriptor.fileFor(Components.DATA)
+                                                          , movedSSTable.oldDescriptor.fileFor(Components.DATA));
+                SSTable.rename(movedSSTable.newDescriptor, movedSSTable.oldDescriptor, movedSSTable.components);
             }
         }
     }
@@ -299,11 +385,8 @@ public class SSTableImporter
         logger.debug("Removing copied SSTables which were left in data directories after failed SSTable import.");
         for (MovedSSTable movedSSTable : movedSSTables)
         {
-            if (new File(movedSSTable.newDescriptor.filenameFor(Component.DATA)).exists())
-            {
-                // no logging here as for moveSSTablesBack case above as logging is done in delete method
-                SSTableWriter.delete(movedSSTable.newDescriptor, movedSSTable.components);
-            }
+            // no logging here as for moveSSTablesBack case above as logging is done in delete method
+            movedSSTable.newDescriptor.getFormat().delete(movedSSTable.newDescriptor);
         }
     }
 
@@ -311,9 +394,9 @@ public class SSTableImporter
      * Iterates over all keys in the sstable index and invalidates the row cache
      */
     @VisibleForTesting
-    void invalidateCachesForSSTable(Descriptor desc)
+    void invalidateCachesForSSTable(SSTableReader reader)
     {
-        try (KeyIterator iter = new KeyIterator(desc, cfs.metadata()))
+        try (KeyIterator iter = reader.keyIterator())
         {
             while (iter.hasNext())
             {
@@ -321,6 +404,10 @@ public class SSTableImporter
                 cfs.invalidateCachedPartition(decoratedKey);
             }
         }
+        catch (IOException ex)
+        {
+            throw new RuntimeException("Failed to import sstable " + reader.getFilename(), ex);
+        }
     }
 
     /**
@@ -335,14 +422,15 @@ public class SSTableImporter
         SSTableReader reader = null;
         try
         {
-            reader = SSTableReader.open(descriptor, components, cfs.metadata);
-            Verifier.Options verifierOptions = Verifier.options()
-                                                       .extendedVerification(extendedVerify)
-                                                       .checkOwnsTokens(verifyTokens)
-                                                       .quick(!verifySSTables)
-                                                       .invokeDiskFailurePolicy(false)
-                                                       .mutateRepairStatus(false).build();
-            try (Verifier verifier = new Verifier(cfs, reader, false, verifierOptions))
+            reader = SSTableReader.open(cfs, descriptor, components, cfs.metadata);
+            IVerifier.Options verifierOptions = IVerifier.options()
+                                                         .extendedVerification(extendedVerify)
+                                                         .checkOwnsTokens(verifyTokens)
+                                                         .quick(!verifySSTables)
+                                                         .invokeDiskFailurePolicy(false)
+                                                         .mutateRepairStatus(false).build();
+
+            try (IVerifier verifier = reader.getVerifier(cfs, new OutputHandler.LogOutput(), false, verifierOptions))
             {
                 verifier.verify();
             }
@@ -364,7 +452,7 @@ public class SSTableImporter
      */
     private void maybeMutateMetadata(Descriptor descriptor, Options options) throws IOException
     {
-        if (new File(descriptor.filenameFor(Component.STATS)).exists())
+        if (descriptor.fileFor(Components.STATS).exists())
         {
             if (options.resetLevel)
             {
@@ -389,8 +477,13 @@ public class SSTableImporter
         private final boolean invalidateCaches;
         private final boolean extendedVerify;
         private final boolean copyData;
+        private final boolean failOnMissingIndex;
+        public final boolean validateIndexChecksum;
 
-        public Options(Set<String> srcPaths, boolean resetLevel, boolean clearRepaired, boolean verifySSTables, boolean verifyTokens, boolean invalidateCaches, boolean extendedVerify, boolean copyData)
+        public Options(Set<String> srcPaths, boolean resetLevel, boolean clearRepaired,
+                       boolean verifySSTables, boolean verifyTokens, boolean invalidateCaches,
+                       boolean extendedVerify, boolean copyData, boolean failOnMissingIndex,
+                       boolean validateIndexChecksum)
         {
             this.srcPaths = srcPaths;
             this.resetLevel = resetLevel;
@@ -400,6 +493,8 @@ public class SSTableImporter
             this.invalidateCaches = invalidateCaches;
             this.extendedVerify = extendedVerify;
             this.copyData = copyData;
+            this.failOnMissingIndex = failOnMissingIndex;
+            this.validateIndexChecksum = validateIndexChecksum;
         }
 
         public static Builder options(String srcDir)
@@ -429,6 +524,8 @@ public class SSTableImporter
                    ", invalidateCaches=" + invalidateCaches +
                    ", extendedVerify=" + extendedVerify +
                    ", copyData= " + copyData +
+                   ", failOnMissingIndex= " + failOnMissingIndex +
+                   ", validateIndexChecksum= " + validateIndexChecksum +
                    '}';
         }
 
@@ -442,6 +539,8 @@ public class SSTableImporter
             private boolean invalidateCaches = false;
             private boolean extendedVerify = false;
             private boolean copyData = false;
+            private boolean failOnMissingIndex = false;
+            private boolean validateIndexChecksum = true;
 
             private Builder(Set<String> srcPath)
             {
@@ -491,9 +590,24 @@ public class SSTableImporter
                 return this;
             }
 
+            public Builder failOnMissingIndex(boolean value)
+            {
+                failOnMissingIndex = value;
+                return this;
+            }
+
+            public Builder validateIndexChecksum(boolean value)
+            {
+                validateIndexChecksum = value;
+                return this;
+            }
+
             public Options build()
             {
-                return new Options(srcPaths, resetLevel, clearRepaired, verifySSTables, verifyTokens, invalidateCaches, extendedVerify, copyData);
+                return new Options(srcPaths, resetLevel, clearRepaired,
+                                   verifySSTables, verifyTokens, invalidateCaches,
+                                   extendedVerify, copyData, failOnMissingIndex,
+                                   validateIndexChecksum);
             }
         }
     }
