@@ -25,9 +25,8 @@ import com.google.common.collect.ImmutableList;
 
 import org.apache.cassandra.db.filter.ColumnFilter;
 import org.apache.cassandra.db.marshal.AbstractType;
-import org.apache.cassandra.db.marshal.TypeParser;
 import org.apache.cassandra.db.marshal.UTF8Type;
-import org.apache.cassandra.db.rows.*;
+import org.apache.cassandra.db.rows.EncodingStats;
 import org.apache.cassandra.exceptions.UnknownColumnException;
 import org.apache.cassandra.io.sstable.format.SSTableReader;
 import org.apache.cassandra.io.sstable.format.Version;
@@ -38,6 +37,7 @@ import org.apache.cassandra.io.util.DataInputPlus;
 import org.apache.cassandra.io.util.DataOutputPlus;
 import org.apache.cassandra.schema.ColumnMetadata;
 import org.apache.cassandra.schema.TableMetadata;
+import org.apache.cassandra.serializers.AbstractTypeSerializer;
 import org.apache.cassandra.utils.ByteBufferUtil;
 
 public class SerializationHeader
@@ -167,14 +167,14 @@ public class SerializationHeader
         out.writeUnsignedVInt(timestamp - stats.minTimestamp);
     }
 
-    public void writeLocalDeletionTime(int localDeletionTime, DataOutputPlus out) throws IOException
+    public void writeLocalDeletionTime(long localDeletionTime, DataOutputPlus out) throws IOException
     {
-        out.writeUnsignedVInt(localDeletionTime - stats.minLocalDeletionTime);
+        out.writeUnsignedVInt32((int) (localDeletionTime - stats.minLocalDeletionTime));
     }
 
     public void writeTTL(int ttl, DataOutputPlus out) throws IOException
     {
-        out.writeUnsignedVInt(ttl - stats.minTTL);
+        out.writeUnsignedVInt32(ttl - stats.minTTL);
     }
 
     public void writeDeletionTime(DeletionTime dt, DataOutputPlus out) throws IOException
@@ -188,21 +188,21 @@ public class SerializationHeader
         return in.readUnsignedVInt() + stats.minTimestamp;
     }
 
-    public int readLocalDeletionTime(DataInputPlus in) throws IOException
+    public long readLocalDeletionTime(DataInputPlus in) throws IOException
     {
-        return (int)in.readUnsignedVInt() + stats.minLocalDeletionTime;
+        return in.readUnsignedVInt32() + stats.minLocalDeletionTime;
     }
 
     public int readTTL(DataInputPlus in) throws IOException
     {
-        return (int)in.readUnsignedVInt() + stats.minTTL;
+        return in.readUnsignedVInt32() + stats.minTTL;
     }
 
     public DeletionTime readDeletionTime(DataInputPlus in) throws IOException
     {
         long markedAt = readTimestamp(in);
-        int localDeletionTime = readLocalDeletionTime(in);
-        return new DeletionTime(markedAt, localDeletionTime);
+        long localDeletionTime = readLocalDeletionTime(in);
+        return DeletionTime.build(markedAt, localDeletionTime);
     }
 
     public long timestampSerializedSize(long timestamp)
@@ -210,7 +210,7 @@ public class SerializationHeader
         return TypeSizes.sizeofUnsignedVInt(timestamp - stats.minTimestamp);
     }
 
-    public long localDeletionTimeSerializedSize(int localDeletionTime)
+    public long localDeletionTimeSerializedSize(long localDeletionTime)
     {
         return TypeSizes.sizeofUnsignedVInt(localDeletionTime - stats.minLocalDeletionTime);
     }
@@ -289,18 +289,6 @@ public class SerializationHeader
             this.stats = stats;
         }
 
-        /**
-         * <em>Only</em> exposed for {@link org.apache.cassandra.io.sstable.SSTableHeaderFix}.
-         */
-        public static Component buildComponentForTools(AbstractType<?> keyType,
-                                                       List<AbstractType<?>> clusteringTypes,
-                                                       Map<ByteBuffer, AbstractType<?>> staticColumns,
-                                                       Map<ByteBuffer, AbstractType<?>> regularColumns,
-                                                       EncodingStats stats)
-        {
-            return new Component(keyType, clusteringTypes, staticColumns, regularColumns, stats);
-        }
-
         public MetadataType getType()
         {
             return MetadataType.HEADER;
@@ -398,6 +386,8 @@ public class SerializationHeader
 
     public static class Serializer implements IMetadataComponentSerializer<Component>
     {
+        private final AbstractTypeSerializer typeSerializer = new AbstractTypeSerializer();
+
         public void serializeForMessaging(SerializationHeader header, ColumnFilter selection, DataOutputPlus out, boolean hasStatic) throws IOException
         {
             EncodingStats.serializer.serialize(header.stats, out);
@@ -462,10 +452,8 @@ public class SerializationHeader
         {
             EncodingStats.serializer.serialize(header.stats, out);
 
-            writeType(header.keyType, out);
-            out.writeUnsignedVInt(header.clusteringTypes.size());
-            for (AbstractType<?> type : header.clusteringTypes)
-                writeType(type, out);
+            typeSerializer.serialize(header.keyType, out);
+            typeSerializer.serializeList(header.clusteringTypes, out);
 
             writeColumnsWithTypes(header.staticColumns, out);
             writeColumnsWithTypes(header.regularColumns, out);
@@ -476,17 +464,11 @@ public class SerializationHeader
         {
             EncodingStats stats = EncodingStats.serializer.deserialize(in);
 
-            AbstractType<?> keyType = readType(in);
-            int size = (int)in.readUnsignedVInt();
-            List<AbstractType<?>> clusteringTypes = new ArrayList<>(size);
-            for (int i = 0; i < size; i++)
-                clusteringTypes.add(readType(in));
+            AbstractType<?> keyType = typeSerializer.deserialize(in);
+            List<AbstractType<?>> clusteringTypes = typeSerializer.deserializeList(in);
 
-            Map<ByteBuffer, AbstractType<?>> staticColumns = new LinkedHashMap<>();
-            Map<ByteBuffer, AbstractType<?>> regularColumns = new LinkedHashMap<>();
-
-            readColumnsWithType(in, staticColumns);
-            readColumnsWithType(in, regularColumns);
+            Map<ByteBuffer, AbstractType<?>> staticColumns = readColumnsWithType(in);
+            Map<ByteBuffer, AbstractType<?>> regularColumns = readColumnsWithType(in);
 
             return new Component(keyType, clusteringTypes, staticColumns, regularColumns, stats);
         }
@@ -496,10 +478,8 @@ public class SerializationHeader
         {
             int size = EncodingStats.serializer.serializedSize(header.stats);
 
-            size += sizeofType(header.keyType);
-            size += TypeSizes.sizeofUnsignedVInt(header.clusteringTypes.size());
-            for (AbstractType<?> type : header.clusteringTypes)
-                size += sizeofType(type);
+            size += typeSerializer.serializedSize(header.keyType);
+            size += typeSerializer.serializedListSize(header.clusteringTypes);
 
             size += sizeofColumnsWithTypes(header.staticColumns);
             size += sizeofColumnsWithTypes(header.regularColumns);
@@ -508,11 +488,11 @@ public class SerializationHeader
 
         private void writeColumnsWithTypes(Map<ByteBuffer, AbstractType<?>> columns, DataOutputPlus out) throws IOException
         {
-            out.writeUnsignedVInt(columns.size());
+            out.writeUnsignedVInt32(columns.size());
             for (Map.Entry<ByteBuffer, AbstractType<?>> entry : columns.entrySet())
             {
                 ByteBufferUtil.writeWithVIntLength(entry.getKey(), out);
-                writeType(entry.getValue(), out);
+                typeSerializer.serialize(entry.getValue(), out);
             }
         }
 
@@ -522,36 +502,21 @@ public class SerializationHeader
             for (Map.Entry<ByteBuffer, AbstractType<?>> entry : columns.entrySet())
             {
                 size += ByteBufferUtil.serializedSizeWithVIntLength(entry.getKey());
-                size += sizeofType(entry.getValue());
+                size += typeSerializer.serializedSize(entry.getValue());
             }
             return size;
         }
 
-        private void readColumnsWithType(DataInputPlus in, Map<ByteBuffer, AbstractType<?>> typeMap) throws IOException
+        private Map<ByteBuffer, AbstractType<?>> readColumnsWithType(DataInputPlus in) throws IOException
         {
-            int length = (int)in.readUnsignedVInt();
+            int length =  in.readUnsignedVInt32();
+            Map<ByteBuffer, AbstractType<?>> typeMap = new LinkedHashMap<>(length);
             for (int i = 0; i < length; i++)
             {
                 ByteBuffer name = ByteBufferUtil.readWithVIntLength(in);
-                typeMap.put(name, readType(in));
+                typeMap.put(name, typeSerializer.deserialize(in));
             }
-        }
-
-        private void writeType(AbstractType<?> type, DataOutputPlus out) throws IOException
-        {
-            // TODO: we should have a terser serializaion format. Not a big deal though
-            ByteBufferUtil.writeWithVIntLength(UTF8Type.instance.decompose(type.toString()), out);
-        }
-
-        private AbstractType<?> readType(DataInputPlus in) throws IOException
-        {
-            ByteBuffer raw = ByteBufferUtil.readWithVIntLength(in);
-            return TypeParser.parse(UTF8Type.instance.compose(raw));
-        }
-
-        private int sizeofType(AbstractType<?> type)
-        {
-            return ByteBufferUtil.serializedSizeWithVIntLength(UTF8Type.instance.decompose(type.toString()));
+            return typeMap;
         }
     }
-}
+}
\ No newline at end of file
