@@ -18,6 +18,7 @@
 package org.apache.cassandra.db.aggregation;
 
 import java.io.IOException;
+import java.util.Collections;
 import java.util.List;
 
 import org.apache.cassandra.cql3.QueryOptions;
@@ -27,6 +28,7 @@ import org.apache.cassandra.db.ClusteringComparator;
 import org.apache.cassandra.db.TypeSizes;
 import org.apache.cassandra.io.util.DataInputPlus;
 import org.apache.cassandra.io.util.DataOutputPlus;
+import org.apache.cassandra.schema.ColumnMetadata;
 import org.apache.cassandra.schema.TableMetadata;
 
 /**
@@ -61,7 +63,7 @@ public abstract class AggregationSpecification
     /**
      * The <code>AggregationSpecification</code> kinds.
      */
-    public static enum Kind
+    public enum Kind
     {
         AGGREGATE_EVERYTHING, AGGREGATE_BY_PK_PREFIX, AGGREGATE_BY_PK_PREFIX_WITH_SELECTOR
     }
@@ -115,7 +117,8 @@ public abstract class AggregationSpecification
 
     public static AggregationSpecification.Factory aggregatePkPrefixFactoryWithSelector(final ClusteringComparator comparator,
                                                                                         final int clusteringPrefixSize,
-                                                                                        final Selector.Factory factory)
+                                                                                        final Selector.Factory factory,
+                                                                                        final List<ColumnMetadata> columns)
     {
         return new Factory()
         {
@@ -131,8 +134,9 @@ public abstract class AggregationSpecification
                 Selector selector = factory.newInstance(options);
                 selector.validateForGroupBy();
                 return new  AggregateByPkPrefixWithSelector(comparator,
-                        clusteringPrefixSize,
-                        selector);
+                                                            clusteringPrefixSize,
+                                                            selector,
+                                                            columns);
             }
         };
     }
@@ -200,18 +204,25 @@ public abstract class AggregationSpecification
          */
         private final Selector selector;
 
+        /**
+         * The columns used by the selector.
+         */
+        private final List<ColumnMetadata> columns;
+
         public AggregateByPkPrefixWithSelector(ClusteringComparator comparator,
                                                int clusteringPrefixSize,
-                                               Selector selector)
+                                               Selector selector,
+                                               List<ColumnMetadata> columns)
         {
             super(Kind.AGGREGATE_BY_PK_PREFIX_WITH_SELECTOR, comparator, clusteringPrefixSize);
             this.selector = selector;
+            this.columns = columns;
         }
 
         @Override
         public GroupMaker newGroupMaker(GroupingState state)
         {
-            return GroupMaker.newSelectorGroupMaker(comparator, clusteringPrefixSize, selector, state);
+            return GroupMaker.newSelectorGroupMaker(comparator, clusteringPrefixSize, selector, columns, state);
         }
     }
 
@@ -225,12 +236,15 @@ public abstract class AggregationSpecification
                 case AGGREGATE_EVERYTHING:
                     break;
                 case AGGREGATE_BY_PK_PREFIX:
-                    out.writeUnsignedVInt(((AggregateByPkPrefix) aggregationSpec).clusteringPrefixSize);
+                    out.writeUnsignedVInt32(((AggregateByPkPrefix) aggregationSpec).clusteringPrefixSize);
                     break;
                 case AGGREGATE_BY_PK_PREFIX_WITH_SELECTOR:
                     AggregateByPkPrefixWithSelector spec = (AggregateByPkPrefixWithSelector) aggregationSpec;
-                    out.writeUnsignedVInt(spec.clusteringPrefixSize);
+                    out.writeUnsignedVInt32(spec.clusteringPrefixSize);
                     Selector.serializer.serialize(spec.selector, out, version);
+                    // Ideally we should serialize the columns but that will break backward compatibility.
+                    // So for the moment we can rebuild the list from the prefix size as we know that there will be
+                    // only one column and that its indice will be: clusteringPrefixSize - 1.
                     break;
                 default:
                     throw new AssertionError("Unknow aggregation kind: " + aggregationSpec.kind());
@@ -245,13 +259,15 @@ public abstract class AggregationSpecification
                 case AGGREGATE_EVERYTHING:
                     return AggregationSpecification.AGGREGATE_EVERYTHING;
                 case AGGREGATE_BY_PK_PREFIX:
-                    return new AggregateByPkPrefix(metadata.comparator, (int) in.readUnsignedVInt());
+                    return new AggregateByPkPrefix(metadata.comparator, in.readUnsignedVInt32());
                 case AGGREGATE_BY_PK_PREFIX_WITH_SELECTOR:
-                    int clusteringPrefixSize = (int) in.readUnsignedVInt();
+                    int clusteringPrefixSize = in.readUnsignedVInt32();
                     Selector selector = Selector.serializer.deserialize(in, version, metadata);
+                    ColumnMetadata functionArgument = metadata.clusteringColumns().get(clusteringPrefixSize - 1);
                     return new AggregateByPkPrefixWithSelector(metadata.comparator,
                                                                clusteringPrefixSize,
-                                                               selector);
+                                                               selector,
+                                                               Collections.singletonList(functionArgument));
                 default:
                     throw new AssertionError("Unknow aggregation kind: " + kind);
             }
@@ -270,9 +286,7 @@ public abstract class AggregationSpecification
                 case AGGREGATE_BY_PK_PREFIX_WITH_SELECTOR:
                     AggregateByPkPrefixWithSelector spec = (AggregateByPkPrefixWithSelector) aggregationSpec;
                     size += TypeSizes.sizeofUnsignedVInt(spec.clusteringPrefixSize);
-                    size += Selector.serializer.serializedSize(spec.selector, version
-                            
-                            );
+                    size += Selector.serializer.serializedSize(spec.selector, version);
                     break;
                 default:
                     throw new AssertionError("Unknow aggregation kind: " + aggregationSpec.kind());
