@@ -50,14 +50,14 @@ public abstract class DataLimits
     public static final DataLimits NONE = new CQLLimits(NO_LIMIT)
     {
         @Override
-        public boolean hasEnoughLiveData(CachedPartition cached, int nowInSec, boolean countPartitionsWithOnlyStaticData, boolean enforceStrictLiveness)
+        public boolean hasEnoughLiveData(CachedPartition cached, long nowInSec, boolean countPartitionsWithOnlyStaticData, boolean enforceStrictLiveness)
         {
             return false;
         }
 
         @Override
         public UnfilteredPartitionIterator filter(UnfilteredPartitionIterator iter,
-                                                  int nowInSec,
+                                                  long nowInSec,
                                                   boolean countPartitionsWithOnlyStaticData)
         {
             return iter;
@@ -65,14 +65,14 @@ public abstract class DataLimits
 
         @Override
         public UnfilteredRowIterator filter(UnfilteredRowIterator iter,
-                                            int nowInSec,
+                                            long nowInSec,
                                             boolean countPartitionsWithOnlyStaticData)
         {
             return iter;
         }
 
         @Override
-        public PartitionIterator filter(PartitionIterator iter, int nowInSec, boolean countPartitionsWithOnlyStaticData, boolean enforceStrictLiveness)
+        public PartitionIterator filter(PartitionIterator iter, long nowInSec, boolean countPartitionsWithOnlyStaticData, boolean enforceStrictLiveness)
         {
             return iter;
         }
@@ -86,8 +86,10 @@ public abstract class DataLimits
     {
         CQL_LIMIT,
         CQL_PAGING_LIMIT,
-        @Deprecated THRIFT_LIMIT, //Deprecated and unused in 4.0, stop publishing in 5.0, reclaim in 6.0
-        @Deprecated SUPER_COLUMN_COUNTING_LIMIT, //Deprecated and unused in 4.0, stop publishing in 5.0, reclaim in 6.0
+        /** @deprecated See CASSANDRA-16582 */
+        @Deprecated(since = "4.0") THRIFT_LIMIT, //Deprecated and unused in 4.0, stop publishing in 5.0, reclaim in 6.0
+        /** @deprecated See CASSANDRA-16582 */
+        @Deprecated(since = "4.0") SUPER_COLUMN_COUNTING_LIMIT, //Deprecated and unused in 4.0, stop publishing in 5.0, reclaim in 6.0
         CQL_GROUP_BY_LIMIT,
         CQL_GROUP_BY_PAGING_LIMIT,
     }
@@ -156,7 +158,7 @@ public abstract class DataLimits
     }
 
     public abstract boolean hasEnoughLiveData(CachedPartition cached,
-                                              int nowInSec,
+                                              long nowInSec,
                                               boolean countPartitionsWithOnlyStaticData,
                                               boolean enforceStrictLiveness);
 
@@ -173,7 +175,7 @@ public abstract class DataLimits
      * normally retrieved from {@link org.apache.cassandra.schema.TableMetadata#enforceStrictLiveness()}
      * @return a new {@code Counter} for this limits.
      */
-    public abstract Counter newCounter(int nowInSec,
+    public abstract Counter newCounter(long nowInSec,
                                        boolean assumeLiveData,
                                        boolean countPartitionsWithOnlyStaticData,
                                        boolean enforceStrictLiveness);
@@ -197,7 +199,7 @@ public abstract class DataLimits
     public abstract DataLimits withoutState();
 
     public UnfilteredPartitionIterator filter(UnfilteredPartitionIterator iter,
-                                              int nowInSec,
+                                              long nowInSec,
                                               boolean countPartitionsWithOnlyStaticData)
     {
         return this.newCounter(nowInSec,
@@ -208,7 +210,7 @@ public abstract class DataLimits
     }
 
     public UnfilteredRowIterator filter(UnfilteredRowIterator iter,
-                                        int nowInSec,
+                                        long nowInSec,
                                         boolean countPartitionsWithOnlyStaticData)
     {
         return this.newCounter(nowInSec,
@@ -218,7 +220,7 @@ public abstract class DataLimits
                    .applyTo(iter);
     }
 
-    public PartitionIterator filter(PartitionIterator iter, int nowInSec, boolean countPartitionsWithOnlyStaticData, boolean enforceStrictLiveness)
+    public PartitionIterator filter(PartitionIterator iter, long nowInSec, boolean countPartitionsWithOnlyStaticData, boolean enforceStrictLiveness)
     {
         return this.newCounter(nowInSec, true, countPartitionsWithOnlyStaticData, enforceStrictLiveness).applyTo(iter);
     }
@@ -230,14 +232,14 @@ public abstract class DataLimits
 
     public static abstract class Counter extends StoppingTransformation<BaseRowIterator<?>>
     {
-        protected final int nowInSec;
+        protected final long nowInSec;
         protected final boolean assumeLiveData;
         private final boolean enforceStrictLiveness;
 
         // false means we do not propagate our stop signals onto the iterator, we only count
         protected boolean enforceLimits = true;
 
-        protected Counter(int nowInSec, boolean assumeLiveData, boolean enforceStrictLiveness)
+        protected Counter(long nowInSec, boolean assumeLiveData, boolean enforceStrictLiveness)
         {
             this.nowInSec = nowInSec;
             this.assumeLiveData = assumeLiveData;
@@ -402,7 +404,7 @@ public abstract class DataLimits
             return new CQLLimits(toFetch, perPartitionLimit, isDistinct);
         }
 
-        public boolean hasEnoughLiveData(CachedPartition cached, int nowInSec, boolean countPartitionsWithOnlyStaticData, boolean enforceStrictLiveness)
+        public boolean hasEnoughLiveData(CachedPartition cached, long nowInSec, boolean countPartitionsWithOnlyStaticData, boolean enforceStrictLiveness)
         {
             // We want the number of row that are currently live. Getting that precise number forces
             // us to iterate the cached partition in general, but we can avoid that if:
@@ -428,7 +430,7 @@ public abstract class DataLimits
             }
         }
 
-        public Counter newCounter(int nowInSec,
+        public Counter newCounter(long nowInSec,
                                   boolean assumeLiveData,
                                   boolean countPartitionsWithOnlyStaticData,
                                   boolean enforceStrictLiveness)
@@ -467,7 +469,7 @@ public abstract class DataLimits
 
             protected boolean hasLiveStaticRow;
 
-            public CQLCounter(int nowInSec,
+            public CQLCounter(long nowInSec,
                               boolean assumeLiveData,
                               boolean countPartitionsWithOnlyStaticData,
                               boolean enforceStrictLiveness)
@@ -597,14 +599,14 @@ public abstract class DataLimits
         }
 
         @Override
-        public Counter newCounter(int nowInSec, boolean assumeLiveData, boolean countPartitionsWithOnlyStaticData, boolean enforceStrictLiveness)
+        public Counter newCounter(long nowInSec, boolean assumeLiveData, boolean countPartitionsWithOnlyStaticData, boolean enforceStrictLiveness)
         {
             return new PagingAwareCounter(nowInSec, assumeLiveData, countPartitionsWithOnlyStaticData, enforceStrictLiveness);
         }
 
         private class PagingAwareCounter extends CQLCounter
         {
-            private PagingAwareCounter(int nowInSec,
+            private PagingAwareCounter(long nowInSec,
                                        boolean assumeLiveData,
                                        boolean countPartitionsWithOnlyStaticData,
                                        boolean enforceStrictLiveness)
@@ -745,7 +747,7 @@ public abstract class DataLimits
         }
 
         @Override
-        public Counter newCounter(int nowInSec,
+        public Counter newCounter(long nowInSec,
                                   boolean assumeLiveData,
                                   boolean countPartitionsWithOnlyStaticData,
                                   boolean enforceStrictLiveness)
@@ -845,7 +847,7 @@ public abstract class DataLimits
 
             protected boolean hasReturnedRowsFromCurrentPartition;
 
-            private GroupByAwareCounter(int nowInSec,
+            private GroupByAwareCounter(long nowInSec,
                                         boolean assumeLiveData,
                                         boolean countPartitionsWithOnlyStaticData,
                                         boolean enforceStrictLiveness)
@@ -1096,7 +1098,7 @@ public abstract class DataLimits
         }
 
         @Override
-        public Counter newCounter(int nowInSec, boolean assumeLiveData, boolean countPartitionsWithOnlyStaticData, boolean enforceStrictLiveness)
+        public Counter newCounter(long nowInSec, boolean assumeLiveData, boolean countPartitionsWithOnlyStaticData, boolean enforceStrictLiveness)
         {
             assert state == GroupingState.EMPTY_STATE || lastReturnedKey.equals(state.partitionKey());
             return new PagingGroupByAwareCounter(nowInSec, assumeLiveData, countPartitionsWithOnlyStaticData, enforceStrictLiveness);
@@ -1110,7 +1112,7 @@ public abstract class DataLimits
 
         private class PagingGroupByAwareCounter extends GroupByAwareCounter
         {
-            private PagingGroupByAwareCounter(int nowInSec, boolean assumeLiveData, boolean countPartitionsWithOnlyStaticData, boolean enforceStrictLiveness)
+            private PagingGroupByAwareCounter(long nowInSec, boolean assumeLiveData, boolean countPartitionsWithOnlyStaticData, boolean enforceStrictLiveness)
             {
                 super(nowInSec, assumeLiveData, countPartitionsWithOnlyStaticData, enforceStrictLiveness);
             }
@@ -1144,22 +1146,22 @@ public abstract class DataLimits
                 case CQL_LIMIT:
                 case CQL_PAGING_LIMIT:
                     CQLLimits cqlLimits = (CQLLimits)limits;
-                    out.writeUnsignedVInt(cqlLimits.rowLimit);
-                    out.writeUnsignedVInt(cqlLimits.perPartitionLimit);
+                    out.writeUnsignedVInt32(cqlLimits.rowLimit);
+                    out.writeUnsignedVInt32(cqlLimits.perPartitionLimit);
                     out.writeBoolean(cqlLimits.isDistinct);
                     if (limits.kind() == Kind.CQL_PAGING_LIMIT)
                     {
                         CQLPagingLimits pagingLimits = (CQLPagingLimits)cqlLimits;
                         ByteBufferUtil.writeWithVIntLength(pagingLimits.lastReturnedKey, out);
-                        out.writeUnsignedVInt(pagingLimits.lastReturnedKeyRemaining);
+                        out.writeUnsignedVInt32(pagingLimits.lastReturnedKeyRemaining);
                     }
                     break;
                 case CQL_GROUP_BY_LIMIT:
                 case CQL_GROUP_BY_PAGING_LIMIT:
                     CQLGroupByLimits groupByLimits = (CQLGroupByLimits) limits;
-                    out.writeUnsignedVInt(groupByLimits.groupLimit);
-                    out.writeUnsignedVInt(groupByLimits.groupPerPartitionLimit);
-                    out.writeUnsignedVInt(groupByLimits.rowLimit);
+                    out.writeUnsignedVInt32(groupByLimits.groupLimit);
+                    out.writeUnsignedVInt32(groupByLimits.groupPerPartitionLimit);
+                    out.writeUnsignedVInt32(groupByLimits.rowLimit);
 
                     AggregationSpecification groupBySpec = groupByLimits.groupBySpec;
                     AggregationSpecification.serializer.serialize(groupBySpec, out, version);
@@ -1170,7 +1172,7 @@ public abstract class DataLimits
                     {
                         CQLGroupByPagingLimits pagingLimits = (CQLGroupByPagingLimits) groupByLimits;
                         ByteBufferUtil.writeWithVIntLength(pagingLimits.lastReturnedKey, out);
-                        out.writeUnsignedVInt(pagingLimits.lastReturnedKeyRemaining);
+                        out.writeUnsignedVInt32(pagingLimits.lastReturnedKeyRemaining);
                      }
                      break;
             }
@@ -1184,21 +1186,21 @@ public abstract class DataLimits
                 case CQL_LIMIT:
                 case CQL_PAGING_LIMIT:
                 {
-                    int rowLimit = (int) in.readUnsignedVInt();
-                    int perPartitionLimit = (int) in.readUnsignedVInt();
+                    int rowLimit = in.readUnsignedVInt32();
+                    int perPartitionLimit = in.readUnsignedVInt32();
                     boolean isDistinct = in.readBoolean();
                     if (kind == Kind.CQL_LIMIT)
                         return cqlLimits(rowLimit, perPartitionLimit, isDistinct);
                     ByteBuffer lastKey = ByteBufferUtil.readWithVIntLength(in);
-                    int lastRemaining = (int) in.readUnsignedVInt();
+                    int lastRemaining = in.readUnsignedVInt32();
                     return new CQLPagingLimits(rowLimit, perPartitionLimit, isDistinct, lastKey, lastRemaining);
                 }
                 case CQL_GROUP_BY_LIMIT:
                 case CQL_GROUP_BY_PAGING_LIMIT:
                 {
-                    int groupLimit = (int) in.readUnsignedVInt();
-                    int groupPerPartitionLimit = (int) in.readUnsignedVInt();
-                    int rowLimit = (int) in.readUnsignedVInt();
+                    int groupLimit = in.readUnsignedVInt32();
+                    int groupPerPartitionLimit = in.readUnsignedVInt32();
+                    int rowLimit = in.readUnsignedVInt32();
 
                     AggregationSpecification groupBySpec = AggregationSpecification.serializer.deserialize(in, version, metadata);
 
@@ -1212,7 +1214,7 @@ public abstract class DataLimits
                                                     state);
 
                     ByteBuffer lastKey = ByteBufferUtil.readWithVIntLength(in);
-                    int lastRemaining = (int) in.readUnsignedVInt();
+                    int lastRemaining = in.readUnsignedVInt32();
                     return new CQLGroupByPagingLimits(groupLimit,
                                                       groupPerPartitionLimit,
                                                       rowLimit,
