@@ -21,7 +21,16 @@
 package org.apache.cassandra.db.lifecycle;
 
 import java.nio.file.Path;
-import java.util.*;
+import java.util.Collections;
+import java.util.Comparator;
+import java.util.HashMap;
+import java.util.HashSet;
+import java.util.LinkedHashSet;
+import java.util.List;
+import java.util.Map;
+import java.util.NavigableSet;
+import java.util.Optional;
+import java.util.Set;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 import java.util.stream.Collectors;
@@ -30,17 +39,18 @@ import javax.annotation.concurrent.NotThreadSafe;
 
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.collect.Iterables;
-
-import org.apache.cassandra.io.util.File;
 import org.apache.commons.lang3.StringUtils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import org.apache.cassandra.config.DatabaseDescriptor;
 import org.apache.cassandra.db.compaction.OperationType;
 import org.apache.cassandra.db.lifecycle.LogRecord.Type;
 import org.apache.cassandra.io.sstable.SSTable;
 import org.apache.cassandra.io.sstable.format.SSTableReader;
+import org.apache.cassandra.io.sstable.format.Version;
 import org.apache.cassandra.io.sstable.format.big.BigFormat;
+import org.apache.cassandra.io.util.File;
 import org.apache.cassandra.utils.Throwables;
 import org.apache.cassandra.utils.TimeUUID;
 
@@ -64,8 +74,10 @@ final class LogFile implements AutoCloseable
 
     static String EXT = ".log";
     static char SEP = '_';
-    // cc_txn_opname_id.log (where cc is one of the sstable versions defined in BigVersion)
-    static Pattern FILE_REGEX = Pattern.compile(String.format("^(.{2})_txn_(.*)_(.*)%s$", EXT));
+    // Log file name format:
+    // legacy for BIG format: cc_txn_opname_id.log (where cc is one of the sstable versions defined in BigVersion)
+    // other formats: fmt-cc_txn_opname_id.log (where fmt is the format and name and cc is one of its versions)
+    static Pattern FILE_REGEX = Pattern.compile(String.format("^((?:[a-z]+-)?.{2}_)?txn_(.*)_(.*)%s$", EXT));
 
     // A set of physical files on disk, each file is an identical replica
     private final LogReplicaSet replicas = new LogReplicaSet();
@@ -81,6 +93,8 @@ final class LogFile implements AutoCloseable
     // The unique id of the transaction
     private final TimeUUID id;
 
+    private final Version version = DatabaseDescriptor.getSelectedSSTableFormat().getLatestVersion();
+
     static LogFile make(File logReplica)
     {
         return make(logReplica.name(), Collections.singletonList(logReplica));
@@ -525,18 +539,17 @@ final class LogFile implements AutoCloseable
 
     private String getFileName()
     {
-        return StringUtils.join(BigFormat.latestVersion,
-                                LogFile.SEP,
-                                "txn",
-                                LogFile.SEP,
-                                type.fileName,
-                                LogFile.SEP,
-                                id.toString(),
-                                LogFile.EXT);
+        // For pre-5.0 versions, only BigFormat is supported, and the file name includes only the version string.
+        // To retain the ability to downgrade to 4.x, we keep the old file naming scheme for BigFormat sstables
+        // and add format names for other formats as they are supported only in 5.0 and above.
+        return StringUtils.join(BigFormat.is(version.format) ? version.toString() : version.toFormatAndVersionString(), LogFile.SEP, // remove version and separator when downgrading to 4.x is becomes unsupported
+                                "txn", LogFile.SEP,
+                                type.fileName, LogFile.SEP,
+                                id.toString(), LogFile.EXT);
     }
 
     public boolean isEmpty()
     {
         return records.isEmpty();
     }
-}
+}
\ No newline at end of file
