@@ -206,6 +206,18 @@ public class ByteArrayAccessor implements ValueAccessor<byte[]>
         return ByteArrayUtil.getInt(value, offset);
     }
 
+    @Override
+    public float getFloat(byte[] value, int offset)
+    {
+        return ByteArrayUtil.getFloat(value, offset);
+    }
+
+    @Override
+    public double getDouble(byte[] value, int offset)
+    {
+        return ByteArrayUtil.getDouble(value, offset);
+    }
+
     @Override
     public long toLong(byte[] value)
     {
@@ -224,6 +236,19 @@ public class ByteArrayAccessor implements ValueAccessor<byte[]>
         return ByteArrayUtil.getFloat(value, 0);
     }
 
+    @Override
+    public float[] toFloatArray(byte[] value, int dimension)
+    {
+        float[] array = new float[dimension];
+        int offset = 0;
+        for (int i = 0; i < dimension; i++)
+        {
+            array[i] = getFloat(value, offset);
+            offset += Float.BYTES;
+        }
+        return array;
+    }
+
     @Override
     public double toDouble(byte[] value)
     {
@@ -276,6 +301,13 @@ public class ByteArrayAccessor implements ValueAccessor<byte[]>
         return TypeSizes.LONG_SIZE;
     }
 
+    @Override
+    public int putFloat(byte[] dst, int offset, float value)
+    {
+        ByteArrayUtil.putFloat(dst, offset, value);
+        return TypeSizes.FLOAT_SIZE;
+    }
+
     @Override
     public byte[] empty()
     {
