@@ -20,6 +20,7 @@ package org.apache.cassandra.db.marshal;
 
 import java.io.IOException;
 import java.nio.ByteBuffer;
+import java.nio.FloatBuffer;
 import java.nio.charset.CharacterCodingException;
 import java.nio.charset.Charset;
 import java.util.UUID;
@@ -104,6 +105,9 @@ public class ByteBufferAccessor implements ValueAccessor<ByteBuffer>
     @Override
     public ByteBuffer slice(ByteBuffer input, int offset, int length)
     {
+        int size = sizeFromOffset(input, offset);
+        if (size < length)
+            throw new IndexOutOfBoundsException(String.format("Attempted to read %d, but the size is %d", length, size));
         ByteBuffer copy = input.duplicate();
         copy.position(copy.position() + offset);
         copy.limit(copy.position() + length);
@@ -210,6 +214,18 @@ public class ByteBufferAccessor implements ValueAccessor<ByteBuffer>
         return value.getInt(value.position() + offset);
     }
 
+    @Override
+    public float getFloat(ByteBuffer value, int offset)
+    {
+        return value.getFloat(offset);
+    }
+
+    @Override
+    public double getDouble(ByteBuffer value, int offset)
+    {
+        return value.getDouble(offset);
+    }
+
     @Override
     public long toLong(ByteBuffer value)
     {
@@ -228,6 +244,18 @@ public class ByteBufferAccessor implements ValueAccessor<ByteBuffer>
         return ByteBufferUtil.toFloat(value);
     }
 
+    @Override
+    public float[] toFloatArray(ByteBuffer value, int dimension)
+    {
+        FloatBuffer floatBuffer = value.asFloatBuffer();
+        if (floatBuffer.remaining() != dimension)
+            throw new IllegalArgumentException(String.format("Could not convert to a float[] with different dimension. " +
+                                                             "Was expecting %d but got %d", dimension, floatBuffer.remaining()));
+        float[] floatArray = new float[floatBuffer.remaining()];
+        floatBuffer.get(floatArray);
+        return floatArray;
+    }
+
     @Override
     public double toDouble(ByteBuffer value)
     {
@@ -280,6 +308,13 @@ public class ByteBufferAccessor implements ValueAccessor<ByteBuffer>
         return TypeSizes.LONG_SIZE;
     }
 
+    @Override
+    public int putFloat(ByteBuffer dst, int offset, float value)
+    {
+        dst.putFloat(dst.position() + offset, value);
+        return TypeSizes.FLOAT_SIZE;
+    }
+
     @Override
     public ByteBuffer empty()
     {
