@@ -21,6 +21,7 @@ import java.nio.ByteBuffer;
 
 import org.apache.cassandra.cql3.CQL3Type;
 import org.apache.cassandra.cql3.Term;
+import org.apache.cassandra.cql3.functions.ArgumentDeserializer;
 import org.apache.cassandra.db.context.CounterContext;
 import org.apache.cassandra.serializers.CounterSerializer;
 import org.apache.cassandra.serializers.MarshalException;
@@ -32,8 +33,17 @@ public class CounterColumnType extends NumberType<Long>
 {
     public static final CounterColumnType instance = new CounterColumnType();
 
+    private static final ByteBuffer MASKED_VALUE = instance.decompose(0L);
+
     CounterColumnType() {super(ComparisonType.NOT_COMPARABLE);} // singleton
 
+    @Override
+    public boolean allowsEmpty()
+    {
+        return true;
+    }
+
+    @Override
     public boolean isEmptyValueMeaningless()
     {
         return true;
@@ -94,38 +104,79 @@ public class CounterColumnType extends NumberType<Long>
     }
 
     @Override
-    protected long toLong(ByteBuffer value)
+    public ArgumentDeserializer getArgumentDeserializer()
     {
-        return ByteBufferUtil.toLong(value);
+        return LongType.instance.getArgumentDeserializer();
     }
 
-    public ByteBuffer add(NumberType<?> leftType, ByteBuffer left, NumberType<?> rightType, ByteBuffer right)
+    @Override
+    public ByteBuffer add(Number left, Number right)
     {
-        return ByteBufferUtil.bytes(leftType.toLong(left) + rightType.toLong(right));
+        return ByteBufferUtil.bytes(left.longValue() + right.longValue());
     }
 
-    public ByteBuffer substract(NumberType<?> leftType, ByteBuffer left, NumberType<?> rightType, ByteBuffer right)
+    @Override
+    public ByteBuffer substract(Number left, Number right)
     {
-        return ByteBufferUtil.bytes(leftType.toLong(left) - rightType.toLong(right));
+        return ByteBufferUtil.bytes(left.longValue() - right.longValue());
     }
 
-    public ByteBuffer multiply(NumberType<?> leftType, ByteBuffer left, NumberType<?> rightType, ByteBuffer right)
+    @Override
+    public ByteBuffer multiply(Number left, Number right)
     {
-        return ByteBufferUtil.bytes(leftType.toLong(left) * rightType.toLong(right));
+        return ByteBufferUtil.bytes(left.longValue() * right.longValue());
     }
 
-    public ByteBuffer divide(NumberType<?> leftType, ByteBuffer left, NumberType<?> rightType, ByteBuffer right)
+    @Override
+    public ByteBuffer divide(Number left, Number right)
     {
-        return ByteBufferUtil.bytes(leftType.toLong(left) / rightType.toLong(right));
+        return ByteBufferUtil.bytes(left.longValue() / right.longValue());
     }
 
-    public ByteBuffer mod(NumberType<?> leftType, ByteBuffer left, NumberType<?> rightType, ByteBuffer right)
+    @Override
+    public ByteBuffer mod(Number left, Number right)
     {
-        return ByteBufferUtil.bytes(leftType.toLong(left) % rightType.toLong(right));
+        return ByteBufferUtil.bytes(left.longValue() % right.longValue());
     }
 
-    public ByteBuffer negate(ByteBuffer input)
+    public ByteBuffer negate(Number input)
     {
-        return ByteBufferUtil.bytes(-toLong(input));
+        return ByteBufferUtil.bytes(-input.longValue());
+    }
+
+    @Override
+    public ByteBuffer abs(Number input)
+    {
+        return ByteBufferUtil.bytes(Math.abs(input.longValue()));
+    }
+
+    @Override
+    public ByteBuffer exp(Number input)
+    {
+        return ByteBufferUtil.bytes((long) Math.exp(input.longValue()));
+    }
+
+    @Override
+    public ByteBuffer log(Number input)
+    {
+        return ByteBufferUtil.bytes((long) Math.log(input.longValue()));
+    }
+
+    @Override
+    public ByteBuffer log10(Number input)
+    {
+        return ByteBufferUtil.bytes((long) Math.log10(input.longValue()));
+    }
+
+    @Override
+    public ByteBuffer round(Number input)
+    {
+        return decompose(input.longValue());
+    }
+
+    @Override
+    public ByteBuffer getMaskedValue()
+    {
+        return MASKED_VALUE;
     }
 }
