@@ -23,11 +23,16 @@ import java.time.ZoneOffset;
 
 import org.apache.cassandra.cql3.Constants;
 import org.apache.cassandra.cql3.Term;
+import org.apache.cassandra.cql3.functions.ArgumentDeserializer;
 import org.apache.cassandra.serializers.TimeSerializer;
 import org.apache.cassandra.cql3.CQL3Type;
 import org.apache.cassandra.serializers.TypeSerializer;
 import org.apache.cassandra.serializers.MarshalException;
 import org.apache.cassandra.transport.ProtocolVersion;
+import org.apache.cassandra.utils.bytecomparable.ByteComparable;
+import org.apache.cassandra.utils.bytecomparable.ByteComparable.Version;
+import org.apache.cassandra.utils.bytecomparable.ByteSource;
+import org.apache.cassandra.utils.bytecomparable.ByteSourceInverse;
 
 /**
  * Nanosecond resolution time values
@@ -35,6 +40,11 @@ import org.apache.cassandra.transport.ProtocolVersion;
 public class TimeType extends TemporalType<Long>
 {
     public static final TimeType instance = new TimeType();
+
+    private static final ArgumentDeserializer ARGUMENT_DESERIALIZER = new DefaultArgumentDeserializer(instance);
+
+    private static final ByteBuffer DEFAULT_MASKED_VALUE = instance.decompose(0L);
+
     private TimeType() {super(ComparisonType.BYTE_ORDER);} // singleton
 
     public ByteBuffer fromString(String source) throws MarshalException
@@ -42,6 +52,20 @@ public class TimeType extends TemporalType<Long>
         return decompose(TimeSerializer.timeStringToLong(source));
     }
 
+    @Override
+    public <V> ByteSource asComparableBytes(ValueAccessor<V> accessor, V data, Version version)
+    {
+        // While BYTE_ORDER would still work for this type, making use of the fixed length is more efficient.
+        // This type does not allow non-present values, but we do just to avoid future complexity.
+        return ByteSource.optionalFixedLength(accessor, data);
+    }
+
+    @Override
+    public <V> V fromComparableBytes(ValueAccessor<V> accessor, ByteSource.Peekable comparableBytes, ByteComparable.Version version)
+    {
+        return ByteSourceInverse.getOptionalFixedLength(accessor, comparableBytes, 8);
+    }
+
     @Override
     public boolean isValueCompatibleWithInternal(AbstractType<?> otherType)
     {
@@ -73,14 +97,27 @@ public class TimeType extends TemporalType<Long>
         return CQL3Type.Native.TIME;
     }
 
+    @Override
     public TypeSerializer<Long> getSerializer()
     {
         return TimeSerializer.instance;
     }
 
+    @Override
+    public ArgumentDeserializer getArgumentDeserializer()
+    {
+        return ARGUMENT_DESERIALIZER;
+    }
+
     @Override
     public ByteBuffer now()
     {
         return decompose(LocalTime.now(ZoneOffset.UTC).toNanoOfDay());
     }
+
+    @Override
+    public ByteBuffer getMaskedValue()
+    {
+        return DEFAULT_MASKED_VALUE;
+    }
 }
