@@ -17,6 +17,8 @@
  */
 package org.apache.cassandra.db.marshal;
 
+import java.nio.ByteBuffer;
+
 import org.apache.cassandra.serializers.TypeSerializer;
 import org.apache.cassandra.utils.TimeUUID;
 
@@ -25,6 +27,8 @@ public class TimeUUIDType extends AbstractTimeUUIDType<TimeUUID>
 {
     public static final TimeUUIDType instance = new TimeUUIDType();
 
+    private static final ByteBuffer MASKED_VALUE = instance.decompose(TimeUUID.minAtUnixMillis(0));
+
     public TypeSerializer<TimeUUID> getSerializer()
     {
         return TimeUUID.Serializer.instance;
@@ -35,4 +39,10 @@ public class TimeUUIDType extends AbstractTimeUUIDType<TimeUUID>
     {
         return LegacyTimeUUIDType.instance;
     }
+
+    @Override
+    public ByteBuffer getMaskedValue()
+    {
+        return MASKED_VALUE;
+    }
 }
