@@ -26,6 +26,7 @@ import org.apache.cassandra.db.ColumnFamilyStore;
 import org.apache.cassandra.db.PartitionPosition;
 import org.apache.cassandra.db.RegularAndStaticColumns;
 import org.apache.cassandra.db.commitlog.CommitLogPosition;
+import org.apache.cassandra.db.lifecycle.LifecycleTransaction;
 import org.apache.cassandra.db.partitions.Partition;
 import org.apache.cassandra.db.partitions.PartitionUpdate;
 import org.apache.cassandra.db.rows.EncodingStats;
@@ -379,6 +380,10 @@ public interface Memtable extends Comparable<Memtable>, UnfilteredSource
     /** True if the memtable contains no data */
     boolean isClean();
 
+    // The following two methods provide a way of tracking ongoing flushes
+    LifecycleTransaction setFlushTransaction(LifecycleTransaction transaction);
+    LifecycleTransaction getFlushTransaction();
+
     /** Order memtables by time as reflected in the commit log position at time of construction */
     default int compareTo(Memtable that)
     {
@@ -401,14 +406,14 @@ public interface Memtable extends Comparable<Memtable>, UnfilteredSource
 
     /**
      * Called when the table's metadata is updated. The memtable's metadata reference now points to the new version.
-     * This will not be called if {@link #shouldSwitch)(SCHEMA_CHANGE) returns true, as the memtable will be swapped out
+     * This will not be called if {@link #shouldSwitch } (SCHEMA_CHANGE) returns true, as the memtable will be swapped out
      * instead.
      */
     void metadataUpdated();
 
     /**
      * Called when the known ranges have been updated and owner.localRangeSplits() may return different values.
-     * This will not be called if {@link #shouldSwitch)(OWNED_RANGES_CHANGE) returns true, as the memtable will be
+     * This will not be called if {@link #shouldSwitch } (OWNED_RANGES_CHANGE) returns true, as the memtable will be
      * swapped out instead.
      */
     void localRangesUpdated();
