@@ -21,10 +21,12 @@ import java.io.IOException;
 import java.util.*;
 import java.util.function.Function;
 
+import org.apache.cassandra.cache.IMeasurableMemory;
 import org.apache.cassandra.db.*;
 import org.apache.cassandra.db.partitions.PartitionStatisticsCollector;
 import org.apache.cassandra.io.util.DataInputPlus;
 import org.apache.cassandra.io.util.DataOutputPlus;
+import org.apache.cassandra.utils.ObjectSizes;
 
 /**
  * Stats used for the encoding of the rows and tombstones of a given source.
@@ -38,13 +40,14 @@ import org.apache.cassandra.io.util.DataOutputPlus;
  * this shouldn't have too huge an impact on performance) and in fact they will not always be
  * accurate for reasons explained in {@link SerializationHeader#make}.
  */
-public class EncodingStats
+public class EncodingStats implements IMeasurableMemory
 {
     // Default values for the timestamp, deletion time and ttl. We use this both for NO_STATS, but also to serialize
     // an EncodingStats. Basically, we encode the diff of each value of to these epoch, which give values with better vint encoding.
-    private static final long TIMESTAMP_EPOCH;
+    public static final long TIMESTAMP_EPOCH;
     private static final int DELETION_TIME_EPOCH;
     private static final int TTL_EPOCH = 0;
+
     static
     {
         // We want a fixed epoch, but that provide small values when substracted from our timestamp and deletion time.
@@ -64,15 +67,16 @@ public class EncodingStats
 
     // We should use this sparingly obviously
     public static final EncodingStats NO_STATS = new EncodingStats(TIMESTAMP_EPOCH, DELETION_TIME_EPOCH, TTL_EPOCH);
+    public static final long HEAP_SIZE = ObjectSizes.measure(NO_STATS);
 
     public static final Serializer serializer = new Serializer();
 
     public final long minTimestamp;
-    public final int minLocalDeletionTime;
+    public final long minLocalDeletionTime;
     public final int minTTL;
 
     public EncodingStats(long minTimestamp,
-                         int minLocalDeletionTime,
+                         long minLocalDeletionTime,
                          int minTTL)
     {
         // Note that the exact value of those don't impact correctness, just the efficiency of the encoding. So when we
@@ -94,16 +98,16 @@ public class EncodingStats
     public EncodingStats mergeWith(EncodingStats that)
     {
         long minTimestamp = this.minTimestamp == TIMESTAMP_EPOCH
-                          ? that.minTimestamp
-                          : (that.minTimestamp == TIMESTAMP_EPOCH ? this.minTimestamp : Math.min(this.minTimestamp, that.minTimestamp));
+                            ? that.minTimestamp
+                            : (that.minTimestamp == TIMESTAMP_EPOCH ? this.minTimestamp : Math.min(this.minTimestamp, that.minTimestamp));
 
-        int minDelTime = this.minLocalDeletionTime == DELETION_TIME_EPOCH
-                       ? that.minLocalDeletionTime
-                       : (that.minLocalDeletionTime == DELETION_TIME_EPOCH ? this.minLocalDeletionTime : Math.min(this.minLocalDeletionTime, that.minLocalDeletionTime));
+        long minDelTime = this.minLocalDeletionTime == DELETION_TIME_EPOCH
+                         ? that.minLocalDeletionTime
+                         : (that.minLocalDeletionTime == DELETION_TIME_EPOCH ? this.minLocalDeletionTime : Math.min(this.minLocalDeletionTime, that.minLocalDeletionTime));
 
         int minTTL = this.minTTL == TTL_EPOCH
-                   ? that.minTTL
-                   : (that.minTTL == TTL_EPOCH ? this.minTTL : Math.min(this.minTTL, that.minTTL));
+                     ? that.minTTL
+                     : (that.minTTL == TTL_EPOCH ? this.minTTL : Math.min(this.minTTL, that.minTTL));
 
         return new EncodingStats(minTimestamp, minDelTime, minTTL);
     }
@@ -117,15 +121,15 @@ public class EncodingStats
             return function.apply(values.get(0));
 
         Collector collector = new Collector();
-        for (int i = 0, iSize = values.size(); i < iSize; i++)
+        for (int i=0, isize=values.size(); i<isize; i++)
         {
             V v = values.get(i);
             EncodingStats stats = function.apply(v);
             if (stats.minTimestamp != TIMESTAMP_EPOCH)
                 collector.updateTimestamp(stats.minTimestamp);
-            if (stats.minLocalDeletionTime != DELETION_TIME_EPOCH)
+            if(stats.minLocalDeletionTime != DELETION_TIME_EPOCH)
                 collector.updateLocalDeletionTime(stats.minLocalDeletionTime);
-            if (stats.minTTL != TTL_EPOCH)
+            if(stats.minTTL != TTL_EPOCH)
                 collector.updateTTL(stats.minTTL);
         }
         return collector.get();
@@ -150,6 +154,13 @@ public class EncodingStats
         return Objects.hash(minTimestamp, minLocalDeletionTime, minTTL);
     }
 
+    public long unsharedHeapSize()
+    {
+        if (this == NO_STATS)
+            return 0;
+        return HEAP_SIZE;
+    }
+
     @Override
     public String toString()
     {
@@ -162,7 +173,7 @@ public class EncodingStats
         private long minTimestamp = Long.MAX_VALUE;
 
         private boolean isDelTimeSet;
-        private int minDeletionTime = Integer.MAX_VALUE;
+        private long minDeletionTime = Long.MAX_VALUE;
 
         private boolean isTTLSet;
         private int minTTL = Integer.MAX_VALUE;
@@ -181,7 +192,7 @@ public class EncodingStats
             }
         }
 
-        public void update(Cell cell)
+        public void update(Cell<?> cell)
         {
             updateTimestamp(cell.timestamp());
             if (cell.isExpiring())
@@ -204,13 +215,19 @@ public class EncodingStats
             updateLocalDeletionTime(deletionTime.localDeletionTime());
         }
 
+        @Override
+        public void updatePartitionDeletion(DeletionTime dt)
+        {
+            update(dt);
+        }
+
         public void updateTimestamp(long timestamp)
         {
             isTimestampSet = true;
             minTimestamp = Math.min(minTimestamp, timestamp);
         }
 
-        public void updateLocalDeletionTime(int deletionTime)
+        public void updateLocalDeletionTime(long deletionTime)
         {
             isDelTimeSet = true;
             minDeletionTime = Math.min(minDeletionTime, deletionTime);
@@ -255,8 +272,8 @@ public class EncodingStats
         public void serialize(EncodingStats stats, DataOutputPlus out) throws IOException
         {
             out.writeUnsignedVInt(stats.minTimestamp - TIMESTAMP_EPOCH);
-            out.writeUnsignedVInt(stats.minLocalDeletionTime - DELETION_TIME_EPOCH);
-            out.writeUnsignedVInt(stats.minTTL - TTL_EPOCH);
+            out.writeUnsignedVInt32((int)(stats.minLocalDeletionTime - DELETION_TIME_EPOCH));
+            out.writeUnsignedVInt32(stats.minTTL - TTL_EPOCH);
         }
 
         public int serializedSize(EncodingStats stats)
@@ -269,9 +286,9 @@ public class EncodingStats
         public EncodingStats deserialize(DataInputPlus in) throws IOException
         {
             long minTimestamp = in.readUnsignedVInt() + TIMESTAMP_EPOCH;
-            int minLocalDeletionTime = (int)in.readUnsignedVInt() + DELETION_TIME_EPOCH;
-            int minTTL = (int)in.readUnsignedVInt() + TTL_EPOCH;
+            long minLocalDeletionTime = in.readUnsignedVInt32() + DELETION_TIME_EPOCH;
+            int minTTL = (int)in.readUnsignedVInt32() + TTL_EPOCH;
             return new EncodingStats(minTimestamp, minLocalDeletionTime, minTTL);
         }
     }
-}
+}
\ No newline at end of file
