@@ -17,15 +17,21 @@
  */
 package org.apache.cassandra.db.rows;
 
-import java.io.IOException;
 import java.io.IOError;
+import java.io.IOException;
 import java.nio.BufferOverflowException;
 
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import org.apache.cassandra.db.*;
+import org.apache.cassandra.db.DecoratedKey;
+import org.apache.cassandra.db.DeletionTime;
+import org.apache.cassandra.db.EmptyIterators;
+import org.apache.cassandra.db.RegularAndStaticColumns;
+import org.apache.cassandra.db.SerializationHeader;
+import org.apache.cassandra.db.TypeSizes;
 import org.apache.cassandra.db.filter.ColumnFilter;
+import org.apache.cassandra.io.sstable.format.big.BigFormatPartitionWriter;
 import org.apache.cassandra.io.util.DataInputPlus;
 import org.apache.cassandra.io.util.DataOutputPlus;
 import org.apache.cassandra.schema.TableMetadata;
@@ -60,13 +66,13 @@ import org.apache.cassandra.utils.ByteBufferUtil;
  *
  * Please note that the format described above is the on-wire format. On-disk, the format is basically the
  * same, but the header is written once per sstable, not once per-partition. Further, the actual row and
- * range tombstones are not written using this class, but rather by {@link ColumnIndex}.
+ * range tombstones are not written using this class, but rather by {@link BigFormatPartitionWriter}.
  */
 public class UnfilteredRowIteratorSerializer
 {
     protected static final Logger logger = LoggerFactory.getLogger(UnfilteredRowIteratorSerializer.class);
 
-    private static final int IS_EMPTY               = 0x01;
+    public  static final int IS_EMPTY               = 0x01;
     private static final int IS_REVERSED            = 0x02;
     private static final int HAS_PARTITION_DELETION = 0x04;
     private static final int HAS_STATIC_ROW         = 0x08;
@@ -140,7 +146,7 @@ public class UnfilteredRowIteratorSerializer
             UnfilteredSerializer.serializer.serialize(staticRow, helper, out, version);
 
         if (rowEstimate >= 0)
-            out.writeUnsignedVInt(rowEstimate);
+            out.writeUnsignedVInt32(rowEstimate);
 
         while (iterator.hasNext())
             UnfilteredSerializer.serializer.serialize(iterator.next(), helper, out, version);
@@ -211,7 +217,7 @@ public class UnfilteredRowIteratorSerializer
         if (hasStatic)
             staticRow = UnfilteredSerializer.serializer.deserializeStaticRow(in, header, new DeserializationHelper(metadata, version, flag));
 
-        int rowEstimate = hasRowEstimate ? (int)in.readUnsignedVInt() : -1;
+        int rowEstimate = hasRowEstimate ? in.readUnsignedVInt32() : -1;
         return new Header(header, key, isReversed, false, partitionDeletion, staticRow, rowEstimate);
     }
 
@@ -280,4 +286,4 @@ public class UnfilteredRowIteratorSerializer
                                  sHeader, key, isReversed, isEmpty, partitionDeletion, staticRow, rowEstimate);
         }
     }
-}
+}
\ No newline at end of file
