@@ -25,17 +25,20 @@ import java.util.function.UnaryOperator;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import org.apache.cassandra.config.DatabaseDescriptor;
 import org.apache.cassandra.db.ColumnFamilyStore;
 import org.apache.cassandra.db.Directories;
 import org.apache.cassandra.db.lifecycle.LifecycleNewTracker;
+import org.apache.cassandra.io.compress.BufferType;
 import org.apache.cassandra.io.sstable.Component;
 import org.apache.cassandra.io.sstable.Descriptor;
+import org.apache.cassandra.io.sstable.IOOptions;
 import org.apache.cassandra.io.sstable.SSTableMultiWriter;
-import org.apache.cassandra.io.sstable.format.SSTableFormat;
-import org.apache.cassandra.io.sstable.format.big.BigTableZeroCopyWriter;
+import org.apache.cassandra.io.sstable.SSTableZeroCopyWriter;
 import org.apache.cassandra.io.sstable.metadata.StatsMetadata;
 import org.apache.cassandra.io.util.DataInputPlus;
 import org.apache.cassandra.io.util.File;
+import org.apache.cassandra.io.util.SequentialWriterOption;
 import org.apache.cassandra.schema.TableId;
 import org.apache.cassandra.streaming.ProgressInfo;
 import org.apache.cassandra.streaming.StreamReceiver;
@@ -60,9 +63,6 @@ public class CassandraEntireSSTableStreamReader implements IStreamReader
 
     public CassandraEntireSSTableStreamReader(StreamMessageHeader messageHeader, CassandraStreamHeader streamHeader, StreamSession session)
     {
-        if (streamHeader.format != SSTableFormat.Type.BIG)
-            throw new AssertionError("Unsupported SSTable format " + streamHeader.format);
-
         if (session.getPendingRepair() != null)
         {
             // we should only ever be streaming pending repair sstables if the session has a pending repair id
@@ -82,9 +82,8 @@ public class CassandraEntireSSTableStreamReader implements IStreamReader
      * @return SSTable transferred
      * @throws IOException if reading the remote sstable fails. Will throw an RTE if local write fails.
      */
-    @SuppressWarnings("resource") // input needs to remain open, streams on top of it can't be closed
     @Override
-    public SSTableMultiWriter read(DataInputPlus in) throws Throwable
+    public SSTableMultiWriter read(DataInputPlus in) throws IOException
     {
         ColumnFamilyStore cfs = ColumnFamilyStore.getIfExists(tableId);
         if (cfs == null)
@@ -103,7 +102,7 @@ public class CassandraEntireSSTableStreamReader implements IStreamReader
                      prettyPrintMemory(totalSize),
                      cfs.metadata());
 
-        BigTableZeroCopyWriter writer = null;
+        SSTableZeroCopyWriter writer = null;
 
         try
         {
@@ -121,8 +120,8 @@ public class CassandraEntireSSTableStreamReader implements IStreamReader
                              prettyPrintMemory(bytesRead),
                              prettyPrintMemory(totalSize));
 
-                writer.writeComponent(component.type, in, length);
-                session.progress(writer.descriptor.filenameFor(component), ProgressInfo.Direction.IN, length, length, length);
+                writer.writeComponent(component, in, length);
+                session.progress(writer.descriptor.fileFor(component).toString(), ProgressInfo.Direction.IN, length, length, length);
                 bytesRead += length;
 
                 logger.debug("[Stream #{}] Finished receiving {} component from {}, componentSize = {}, readBytes = {}, totalSize = {}",
@@ -145,7 +144,11 @@ public class CassandraEntireSSTableStreamReader implements IStreamReader
         {
             logger.error("[Stream {}] Error while reading sstable from stream for table = {}", session.planId(), cfs.metadata(), e);
             if (writer != null)
-                e = writer.abort(e);
+            {
+                Throwable e2 = writer.abort(null);
+                if (e2 != null)
+                    e.addSuppressed(e2);
+            }
             throw e;
         }
     }
@@ -164,8 +167,7 @@ public class CassandraEntireSSTableStreamReader implements IStreamReader
         return dir;
     }
 
-    @SuppressWarnings("resource")
-    protected BigTableZeroCopyWriter createWriter(ColumnFamilyStore cfs, long totalSize, Collection<Component> components) throws IOException
+    protected SSTableZeroCopyWriter createWriter(ColumnFamilyStore cfs, long totalSize, Collection<Component> components) throws IOException
     {
         File dataDir = getDataDir(cfs, totalSize);
 
@@ -174,10 +176,26 @@ public class CassandraEntireSSTableStreamReader implements IStreamReader
 
         LifecycleNewTracker lifecycleNewTracker = CassandraStreamReceiver.fromReceiver(session.getAggregator(tableId)).createLifecycleNewTracker();
 
-        Descriptor desc = cfs.newSSTableDescriptor(dataDir, header.version, header.format);
+        Descriptor desc = cfs.newSSTableDescriptor(dataDir, header.version);
 
-        logger.debug("[Table #{}] {} Components to write: {}", cfs.metadata(), desc.filenameFor(Component.DATA), components);
+        IOOptions ioOptions = new IOOptions(DatabaseDescriptor.getDiskOptimizationStrategy(),
+                                            DatabaseDescriptor.getDiskAccessMode(),
+                                            DatabaseDescriptor.getIndexAccessMode(),
+                                            DatabaseDescriptor.getDiskOptimizationEstimatePercentile(),
+                                            SequentialWriterOption.newBuilder()
+                                                                  .trickleFsync(false)
+                                                                  .bufferSize(2 << 20)
+                                                                  .bufferType(BufferType.OFF_HEAP)
+                                                                  .build(),
+                                            DatabaseDescriptor.getFlushCompression());
 
-        return new BigTableZeroCopyWriter(desc, cfs.metadata, lifecycleNewTracker, components);
+        logger.debug("[Table #{}] {} Components to write: {}", cfs.metadata(), desc, components);
+        return desc.getFormat()
+                   .getWriterFactory()
+                   .builder(desc)
+                   .setComponents(components)
+                   .setTableMetadataRef(cfs.metadata)
+                   .setIOOptions(ioOptions)
+                   .createZeroCopyWriter(lifecycleNewTracker, cfs);
     }
 }
