@@ -18,30 +18,33 @@
 
 package org.apache.cassandra.db.streaming;
 
+import java.io.IOException;
+import java.util.ArrayList;
+import java.util.Iterator;
+import java.util.LinkedHashMap;
+import java.util.List;
+import java.util.Map;
+import java.util.Set;
+
 import com.google.common.annotations.VisibleForTesting;
-import com.google.common.collect.ImmutableList;
+import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.Iterators;
+
+import org.apache.cassandra.config.DatabaseDescriptor;
 import org.apache.cassandra.db.TypeSizes;
 import org.apache.cassandra.io.IVersionedSerializer;
 import org.apache.cassandra.io.sstable.Component;
-import org.apache.cassandra.io.sstable.Descriptor;
+import org.apache.cassandra.io.sstable.SSTable;
+import org.apache.cassandra.io.sstable.format.SSTableFormat;
 import org.apache.cassandra.io.util.DataInputPlus;
 import org.apache.cassandra.io.util.DataOutputPlus;
-
-import java.io.IOException;
-import java.util.*;
+import org.apache.cassandra.io.util.File;
 
 /**
  * SSTable components and their sizes to be tranfered via entire-sstable-streaming
  */
-import org.apache.cassandra.io.util.File;
-
 public final class ComponentManifest implements Iterable<Component>
 {
-    private static final List<Component> STREAM_COMPONENTS = ImmutableList.of(Component.DATA, Component.PRIMARY_INDEX, Component.STATS,
-                                                                             Component.COMPRESSION_INFO, Component.FILTER, Component.SUMMARY,
-                                                                             Component.DIGEST, Component.CRC);
-
     private final LinkedHashMap<Component, Long> components;
 
     public ComponentManifest(Map<Component, Long> components)
@@ -50,13 +53,14 @@ public final class ComponentManifest implements Iterable<Component>
     }
 
     @VisibleForTesting
-    public static ComponentManifest create(Descriptor descriptor)
+    public static ComponentManifest create(SSTable sstable)
     {
-        LinkedHashMap<Component, Long> components = new LinkedHashMap<>(STREAM_COMPONENTS.size());
+        Set<Component> streamingComponents = sstable.getStreamingComponents();
+        LinkedHashMap<Component, Long> components = new LinkedHashMap<>(streamingComponents.size());
 
-        for (Component component : STREAM_COMPONENTS)
+        for (Component component : streamingComponents)
         {
-            File file = new File(descriptor.filenameFor(component));
+            File file = sstable.descriptor.fileFor(component);
             if (!file.exists())
                 continue;
 
@@ -114,45 +118,58 @@ public final class ComponentManifest implements Iterable<Component>
                '}';
     }
 
-    public static final IVersionedSerializer<ComponentManifest> serializer = new IVersionedSerializer<ComponentManifest>()
+    public static final Map<String, IVersionedSerializer<ComponentManifest>> serializers;
+
+    static
     {
-        public void serialize(ComponentManifest manifest, DataOutputPlus out, int version) throws IOException
+        ImmutableMap.Builder<String, IVersionedSerializer<ComponentManifest>> b = ImmutableMap.builder();
+        for (SSTableFormat<?, ?> format : DatabaseDescriptor.getSSTableFormats().values())
         {
-            out.writeUnsignedVInt(manifest.components.size());
-            for (Map.Entry<Component, Long> entry : manifest.components.entrySet())
+            IVersionedSerializer<ComponentManifest> serializer = new IVersionedSerializer<ComponentManifest>()
             {
-                out.writeUTF(entry.getKey().name);
-                out.writeUnsignedVInt(entry.getValue());
-            }
+
+                public void serialize(ComponentManifest manifest, DataOutputPlus out, int version) throws IOException
+                {
+                    out.writeUnsignedVInt32(manifest.components.size());
+                    for (Map.Entry<Component, Long> entry : manifest.components.entrySet())
+                    {
+                        out.writeUTF(entry.getKey().name);
+                        out.writeUnsignedVInt(entry.getValue());
+                    }
+                }
+
+                public ComponentManifest deserialize(DataInputPlus in, int version) throws IOException
+                {
+                    int size = in.readUnsignedVInt32();
+
+                    LinkedHashMap<Component, Long> components = new LinkedHashMap<>(size);
+
+                    for (int i = 0; i < size; i++)
+                    {
+                        Component component = Component.parse(in.readUTF(), format);
+                        long length = in.readUnsignedVInt();
+                        components.put(component, length);
+                    }
+
+                    return new ComponentManifest(components);
+                }
+
+                public long serializedSize(ComponentManifest manifest, int version)
+                {
+                    long size = TypeSizes.sizeofUnsignedVInt(manifest.components.size());
+                    for (Map.Entry<Component, Long> entry : manifest.components.entrySet())
+                    {
+                        size += TypeSizes.sizeof(entry.getKey().name);
+                        size += TypeSizes.sizeofUnsignedVInt(entry.getValue());
+                    }
+                    return size;
+                }
+            };
+
+            b.put(format.name(), serializer);
         }
-
-        public ComponentManifest deserialize(DataInputPlus in, int version) throws IOException
-        {
-            int size = (int) in.readUnsignedVInt();
-
-            LinkedHashMap<Component, Long> components = new LinkedHashMap<>(size);
-
-            for (int i = 0; i < size; i++)
-            {
-                Component component = Component.parse(in.readUTF());
-                long length = in.readUnsignedVInt();
-                components.put(component, length);
-            }
-
-            return new ComponentManifest(components);
-        }
-
-        public long serializedSize(ComponentManifest manifest, int version)
-        {
-            long size = TypeSizes.sizeofUnsignedVInt(manifest.components.size());
-            for (Map.Entry<Component, Long> entry : manifest.components.entrySet())
-            {
-                size += TypeSizes.sizeof(entry.getKey().name);
-                size += TypeSizes.sizeofUnsignedVInt(entry.getValue());
-            }
-            return size;
-        }
-    };
+        serializers = b.build();
+    }
 
     @Override
     public Iterator<Component> iterator()
