@@ -20,8 +20,6 @@
  */
 package org.apache.cassandra.db.transform;
 
-import java.util.Collections;
-
 import org.apache.cassandra.db.partitions.BasePartitionIterator;
 import org.apache.cassandra.db.rows.BaseRowIterator;
 import org.apache.cassandra.utils.Throwables;
@@ -77,7 +75,6 @@ implements BasePartitionIterator<R>
         return fail;
     }
 
-    @SuppressWarnings("resource")
     public final boolean hasNext()
     {
         BaseRowIterator<?> next = null;
@@ -112,7 +109,7 @@ implements BasePartitionIterator<R>
         catch (Throwable t)
         {
             if (next != null)
-                Throwables.close(t, Collections.singleton(next));
+                Throwables.close(t, next);
             throw t;
         }
     }
