@@ -40,6 +40,7 @@ final class ClientsTable extends AbstractVirtualTable
     private static final String SSL_ENABLED = "ssl_enabled";
     private static final String SSL_PROTOCOL = "ssl_protocol";
     private static final String SSL_CIPHER_SUITE = "ssl_cipher_suite";
+    private static final String KEYSPACE_NAME = "keyspace_name";
 
     ClientsTable(String keyspace)
     {
@@ -60,6 +61,7 @@ final class ClientsTable extends AbstractVirtualTable
                            .addRegularColumn(SSL_ENABLED, BooleanType.instance)
                            .addRegularColumn(SSL_PROTOCOL, UTF8Type.instance)
                            .addRegularColumn(SSL_CIPHER_SUITE, UTF8Type.instance)
+                           .addRegularColumn(KEYSPACE_NAME, UTF8Type.instance)
                            .build());
     }
 
@@ -83,7 +85,8 @@ final class ClientsTable extends AbstractVirtualTable
                   .column(REQUEST_COUNT, client.requestCount())
                   .column(SSL_ENABLED, client.sslEnabled())
                   .column(SSL_PROTOCOL, client.sslProtocol().orElse(null))
-                  .column(SSL_CIPHER_SUITE, client.sslCipherSuite().orElse(null));
+                  .column(SSL_CIPHER_SUITE, client.sslCipherSuite().orElse(null))
+                  .column(KEYSPACE_NAME, client.keyspace().orElse(null));
         }
 
         return result;
