@@ -27,6 +27,7 @@ import java.util.UUID;
 import java.util.stream.Collectors;
 
 import com.google.common.annotations.VisibleForTesting;
+import com.google.common.collect.ImmutableSet;
 
 import org.apache.cassandra.cql3.statements.schema.CreateTableStatement;
 import org.apache.cassandra.db.DecoratedKey;
@@ -115,7 +116,7 @@ public class LocalRepairTables
         public DataSet data()
         {
             SimpleDataSet result = new SimpleDataSet(metadata());
-            ActiveRepairService.instance.coordinators().forEach(s -> updateDataset(result, s));
+            ActiveRepairService.instance().coordinators().forEach(s -> updateDataset(result, s));
             return result;
         }
 
@@ -123,7 +124,7 @@ public class LocalRepairTables
         {
             TimeUUID id = TimeUUIDType.instance.compose(partitionKey.getKey());
             SimpleDataSet result = new SimpleDataSet(metadata());
-            CoordinatorState state = ActiveRepairService.instance.coordinator(id);
+            CoordinatorState state = ActiveRepairService.instance().coordinator(id);
             if (state != null)
                 updateDataset(result, state);
             return result;
@@ -212,7 +213,7 @@ public class LocalRepairTables
         public DataSet data()
         {
             SimpleDataSet result = new SimpleDataSet(metadata());
-            ActiveRepairService.instance.coordinators().stream()
+            ActiveRepairService.instance().coordinators().stream()
                                                 .flatMap(s -> s.getSessions().stream())
                                                 .forEach(s -> updateDataset(result, s));
             return result;
@@ -251,7 +252,7 @@ public class LocalRepairTables
         public DataSet data()
         {
             SimpleDataSet result = new SimpleDataSet(metadata());
-            ActiveRepairService.instance.coordinators().stream()
+            ActiveRepairService.instance().coordinators().stream()
                                                 .flatMap(s -> s.getSessions().stream())
                                                 .flatMap(s -> s.getJobs().stream())
                                                 .forEach(s -> updateDataset(result, s));
@@ -291,7 +292,7 @@ public class LocalRepairTables
         public DataSet data()
         {
             SimpleDataSet result = new SimpleDataSet(metadata());
-            ActiveRepairService.instance.participates().stream()
+            ActiveRepairService.instance().participates().stream()
                                         .forEach(s -> updateDataset(result, s));
             return result;
         }
@@ -301,7 +302,7 @@ public class LocalRepairTables
         {
             TimeUUID id = TimeUUIDType.instance.compose(partitionKey.getKey());
             SimpleDataSet result = new SimpleDataSet(metadata());
-            ParticipateState state = ActiveRepairService.instance.participate(id);
+            ParticipateState state = ActiveRepairService.instance().participate(id);
             if (state != null)
                 updateDataset(result, state);
             return result;
@@ -322,7 +323,7 @@ public class LocalRepairTables
             result.column("preview_kind", state.previewKind.name());
             if (state.repairedAt != 0)
                 result.column("repaired_at", new Date(state.repairedAt));
-            result.column("validations", state.validationIds());
+            result.column("validations", ImmutableSet.copyOf(state.validationIds()));
             result.column("ranges", toStringList(state.ranges));
         }
     }
@@ -352,7 +353,7 @@ public class LocalRepairTables
         public DataSet data()
         {
             SimpleDataSet result = new SimpleDataSet(metadata());
-            ActiveRepairService.instance.validations().stream()
+            ActiveRepairService.instance().validations().stream()
                                                      .forEach(s -> updateDataset(result, s));
             return result;
         }
@@ -362,7 +363,7 @@ public class LocalRepairTables
         {
             UUID id = UUIDType.instance.compose(partitionKey.getKey());
             SimpleDataSet result = new SimpleDataSet(metadata());
-            ValidationState state = ActiveRepairService.instance.validation(id);
+            ValidationState state = ActiveRepairService.instance().validation(id);
             if (state != null)
                 updateDataset(result, state);
             return result;
