@@ -19,8 +19,6 @@ package org.apache.cassandra.exceptions;
 
 import java.io.IOException;
 
-import com.google.common.primitives.Ints;
-
 import org.apache.cassandra.db.filter.TombstoneOverwhelmingException;
 import org.apache.cassandra.io.IVersionedSerializer;
 import org.apache.cassandra.io.util.DataInputPlus;
@@ -35,7 +33,11 @@ public enum RequestFailureReason
     UNKNOWN                  (0),
     READ_TOO_MANY_TOMBSTONES (1),
     TIMEOUT                  (2),
-    INCOMPATIBLE_SCHEMA      (3);
+    INCOMPATIBLE_SCHEMA      (3),
+    READ_SIZE                (4),
+    NODE_DOWN                (5),
+    INDEX_NOT_AVAILABLE      (6),
+    READ_TOO_MANY_INDEXES    (7);
 
     public static final Serializer serializer = new Serializer();
 
@@ -96,20 +98,20 @@ public enum RequestFailureReason
 
         public void serialize(RequestFailureReason reason, DataOutputPlus out, int version) throws IOException
         {
-            if (version < VERSION_40)
-                out.writeShort(reason.code);
-            else
-                out.writeUnsignedVInt(reason.code);
+            assert version >= VERSION_40;
+            out.writeUnsignedVInt32(reason.code);
         }
 
         public RequestFailureReason deserialize(DataInputPlus in, int version) throws IOException
         {
-            return fromCode(version < VERSION_40 ? in.readUnsignedShort() : Ints.checkedCast(in.readUnsignedVInt()));
+            assert version >= VERSION_40;
+            return fromCode(in.readUnsignedVInt32());
         }
 
         public long serializedSize(RequestFailureReason reason, int version)
         {
-            return version < VERSION_40 ? 2 : VIntCoding.computeVIntSize(reason.code);
+            assert version >= VERSION_40;
+            return VIntCoding.computeVIntSize(reason.code);
         }
     }
 }
