@@ -18,7 +18,6 @@
 
 package org.apache.cassandra.hints;
 
-import java.io.File;
 import java.io.IOException;
 import java.nio.ByteBuffer;
 import java.nio.channels.FileChannel;
@@ -28,6 +27,8 @@ import com.google.common.annotations.VisibleForTesting;
 
 import org.apache.cassandra.io.compress.ICompressor;
 
+import org.apache.cassandra.io.util.File;
+
 public class CompressedHintsWriter extends HintsWriter
 {
     // compressed and uncompressed size is stored at the beginning of each compressed block
