@@ -23,12 +23,11 @@ import java.util.concurrent.TimeUnit;
 
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Throwables;
-
-import javax.annotation.Nullable;
-
 import com.google.common.primitives.Ints;
 
-import org.apache.cassandra.db.*;
+import javax.annotation.Nullable;
+import org.apache.cassandra.db.Mutation;
+import org.apache.cassandra.db.SystemKeyspace;
 import org.apache.cassandra.io.IVersionedSerializer;
 import org.apache.cassandra.io.util.DataInputBuffer;
 import org.apache.cassandra.io.util.DataInputPlus;
@@ -38,6 +37,7 @@ import org.apache.cassandra.utils.concurrent.Future;
 import org.apache.cassandra.utils.concurrent.ImmediateFuture;
 import org.apache.cassandra.utils.vint.VIntCoding;
 
+import static org.apache.cassandra.config.CassandraRelevantProperties.CASSANDRA_MAX_HINT_TTL;
 import static org.apache.cassandra.db.TypeSizes.sizeof;
 import static org.apache.cassandra.db.TypeSizes.sizeofUnsignedVInt;
 import static org.apache.cassandra.utils.Clock.Global.currentTimeMillis;
@@ -60,7 +60,7 @@ import static org.apache.cassandra.utils.Clock.Global.currentTimeMillis;
 public final class Hint
 {
     public static final Serializer serializer = new Serializer();
-    public static volatile int maxHintTTL = Integer.getInteger("cassandra.maxHintTTL", Integer.MAX_VALUE);
+    public static volatile int maxHintTTL = CASSANDRA_MAX_HINT_TTL.getInt();
 
     final Mutation mutation;
     final long creationTime;  // time of hint creation (in milliseconds)
@@ -120,7 +120,8 @@ public final class Hint
         }
         catch (Exception e)
         {
-            throw Throwables.propagate(e.getCause());
+            Throwables.throwIfUnchecked(e.getCause());
+            throw new RuntimeException(e.getCause());
         }
     }
 
@@ -170,14 +171,14 @@ public final class Hint
         public void serialize(Hint hint, DataOutputPlus out, int version) throws IOException
         {
             out.writeLong(hint.creationTime);
-            out.writeUnsignedVInt(hint.gcgs);
+            out.writeUnsignedVInt32(hint.gcgs);
             Mutation.serializer.serialize(hint.mutation, out, version);
         }
 
         public Hint deserialize(DataInputPlus in, int version) throws IOException
         {
             long creationTime = in.readLong();
-            int gcgs = (int) in.readUnsignedVInt();
+            int gcgs = in.readUnsignedVInt32();
             return new Hint(Mutation.serializer.deserialize(in, version), creationTime, gcgs);
         }
 
@@ -198,7 +199,7 @@ public final class Hint
         Hint deserializeIfLive(DataInputPlus in, long now, long size, int version) throws IOException
         {
             long creationTime = in.readLong();
-            int gcgs = (int) in.readUnsignedVInt();
+            int gcgs = in.readUnsignedVInt32();
             int bytesRead = sizeof(creationTime) + sizeofUnsignedVInt(gcgs);
 
             if (isLive(creationTime, now, gcgs))
@@ -226,7 +227,7 @@ public final class Hint
             try (DataInputBuffer input = new DataInputBuffer(header))
             {
                 long creationTime = input.readLong();
-                int gcgs = (int) input.readUnsignedVInt();
+                int gcgs = input.readUnsignedVInt32();
 
                 if (!isLive(creationTime, now, gcgs))
                 {
