@@ -36,7 +36,6 @@ import com.google.common.collect.ImmutableMap;
 import org.apache.cassandra.db.Keyspace;
 import org.apache.cassandra.io.util.File;
 import org.apache.cassandra.locator.ReplicaLayout;
-import org.apache.cassandra.utils.Clock;
 import org.apache.cassandra.utils.concurrent.Future;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -248,7 +247,7 @@ public final class HintsService implements HintsServiceMBean
 
     /**
      * Get the total size in bytes of all the hints files associating with the host on disk.
-     * @param hostId, belonging host
+     * @param hostId belonging host
      * @return total file size, in bytes
      */
     public long getTotalHintsSize(UUID hostId)
@@ -399,8 +398,6 @@ public final class HintsService implements HintsServiceMBean
 
         // delete all the hints files and remove the HintsStore instance from the map in the catalog
         catalog.exciseStore(hostId);
-
-        bufferPool.clearEarliestHintsForHostId(hostId);
     }
 
     /**
@@ -447,17 +444,15 @@ public final class HintsService implements HintsServiceMBean
     }
 
     /**
-     * Get the earliest hint written for a particular node,
-     * @param hostId UUID of the node to check it's hints.
-     * @return earliest hint as per unix time or Long.MIN_VALUE if hostID is null
+     * Find the oldest hint written for a particular node by looking into descriptors
+     * and current open writer, if any.
+     *
+     * @param hostId UUID of the node to check its hints.
+     * @return the oldest hint of the given host id or Long.MAX_VALUE when not found
      */
-    public long getEarliestHintForHost(UUID hostId)
+    public long findOldestHintTimestamp(UUID hostId)
     {
-        // Need to check only the first descriptor + all buffers.
-        HintsStore store = catalog.get(hostId);
-        HintsDescriptor desc = store.getFirstDescriptor();
-        long timestamp = desc == null ? Clock.Global.currentTimeMillis() : desc.timestamp;
-        return Math.min(timestamp, bufferPool.getEarliestHintForHost(hostId));
+        return catalog.get(hostId).findOldestHintTimestamp();
     }
 
     HintsCatalog getCatalog()
@@ -478,9 +473,4 @@ public final class HintsService implements HintsServiceMBean
     {
         return isDispatchPaused.get();
     }
-
-    HintsBufferPool getHintsBufferPool()
-    {
-        return bufferPool;
-    }
 }
