@@ -40,7 +40,6 @@ import com.google.common.collect.*;
 
 import org.apache.cassandra.utils.FBUtilities;
 
-@SuppressWarnings("resource")
 public class Operation extends RangeIterator<Long, Token>
 {
     public enum OperationType
@@ -204,7 +203,7 @@ public class Operation extends RangeIterator<Long, Token>
         if (currentCluster == null || !currentCluster.isRow())
             return false;
 
-        final int now = FBUtilities.nowInSeconds();
+        final long now = FBUtilities.nowInSeconds();
         boolean result = false;
         int idx = 0;
 
