@@ -17,14 +17,11 @@
  */
 package org.apache.cassandra.io.compress;
 
-import java.nio.file.NoSuchFileException;
-import java.io.DataInput;
 import java.io.DataOutput;
 import java.io.EOFException;
-
-import org.apache.cassandra.io.util.*;
 import java.io.FileNotFoundException;
 import java.io.IOException;
+import java.nio.file.NoSuchFileException;
 import java.util.Collection;
 import java.util.HashMap;
 import java.util.Map;
@@ -32,7 +29,6 @@ import java.util.SortedSet;
 import java.util.TreeSet;
 
 import com.google.common.annotations.VisibleForTesting;
-import com.google.common.base.Throwables;
 import com.google.common.primitives.Longs;
 
 import org.apache.cassandra.db.TypeSizes;
@@ -40,18 +36,25 @@ import org.apache.cassandra.exceptions.ConfigurationException;
 import org.apache.cassandra.io.FSReadError;
 import org.apache.cassandra.io.FSWriteError;
 import org.apache.cassandra.io.IVersionedSerializer;
-import org.apache.cassandra.io.sstable.Component;
 import org.apache.cassandra.io.sstable.CorruptSSTableException;
-import org.apache.cassandra.io.sstable.Descriptor;
 import org.apache.cassandra.io.sstable.format.SSTableReader;
+import org.apache.cassandra.io.util.DataInputPlus;
+import org.apache.cassandra.io.util.DataOutputPlus;
+import org.apache.cassandra.io.util.File;
+import org.apache.cassandra.io.util.FileInputStreamPlus;
+import org.apache.cassandra.io.util.FileOutputStreamPlus;
+import org.apache.cassandra.io.util.Memory;
+import org.apache.cassandra.io.util.SafeMemory;
 import org.apache.cassandra.schema.CompressionParams;
-import org.apache.cassandra.utils.concurrent.Transactional;
 import org.apache.cassandra.utils.concurrent.Ref;
+import org.apache.cassandra.utils.concurrent.Transactional;
+import org.apache.cassandra.utils.concurrent.WrappedSharedCloseable;
 
 /**
  * Holds metadata about compressed file
+ * TODO extract interface ICompressionMetadata which will just provide non-resource properties
  */
-public class CompressionMetadata
+public class CompressionMetadata extends WrappedSharedCloseable
 {
     // dataLength can represent either the true length of the file
     // or some shorter value, in the case we want to impose a shorter limit on readers
@@ -60,42 +63,17 @@ public class CompressionMetadata
     public final long compressedFileLength;
     private final Memory chunkOffsets;
     private final long chunkOffsetsSize;
-    public final String indexFilePath;
+    public final File chunksIndexFile;
     public final CompressionParams parameters;
 
-    /**
-     * Create metadata about given compressed file including uncompressed data length, chunk size
-     * and list of the chunk offsets of the compressed data.
-     *
-     * This is an expensive operation! Don't create more than one for each
-     * sstable.
-     *
-     * @param dataFilePath Path to the compressed file
-     *
-     * @return metadata about given compressed file.
-     */
-    public static CompressionMetadata create(String dataFilePath)
-    {
-        return createWithLength(dataFilePath, new File(dataFilePath).length());
-    }
-
-    public static CompressionMetadata createWithLength(String dataFilePath, long compressedLength)
-    {
-        return new CompressionMetadata(Descriptor.fromFilename(dataFilePath), compressedLength);
-    }
-
     @VisibleForTesting
-    public CompressionMetadata(Descriptor desc, long compressedLength)
+    public static CompressionMetadata open(File chunksIndexFile, long compressedLength, boolean hasMaxCompressedSize)
     {
-        this(desc.filenameFor(Component.COMPRESSION_INFO), compressedLength, desc.version.hasMaxCompressedLength());
-    }
+        CompressionParams parameters;
+        long dataLength;
+        Memory chunkOffsets;
 
-    @VisibleForTesting
-    public CompressionMetadata(String indexFilePath, long compressedLength, boolean hasMaxCompressedSize)
-    {
-        this.indexFilePath = indexFilePath;
-
-        try (FileInputStreamPlus stream = new File(indexFilePath).newInputStream())
+        try (FileInputStreamPlus stream = chunksIndexFile.newInputStream())
         {
             String compressorName = stream.readUTF();
             int optionCount = stream.readInt();
@@ -120,7 +98,6 @@ public class CompressionMetadata
             }
 
             dataLength = stream.readLong();
-            compressedFileLength = compressedLength;
             chunkOffsets = readChunkOffsets(stream);
         }
         catch (FileNotFoundException | NoSuchFileException e)
@@ -129,22 +106,39 @@ public class CompressionMetadata
         }
         catch (IOException e)
         {
-            throw new CorruptSSTableException(e, indexFilePath);
+            throw new CorruptSSTableException(e, chunksIndexFile);
         }
 
-        this.chunkOffsetsSize = chunkOffsets.size();
+        return new CompressionMetadata(chunksIndexFile, parameters, chunkOffsets, chunkOffsets.size(), dataLength, compressedLength);
     }
 
     // do not call this constructor directly, unless used in testing
     @VisibleForTesting
-    public CompressionMetadata(String filePath, CompressionParams parameters, Memory offsets, long offsetsSize, long dataLength, long compressedLength)
+    public CompressionMetadata(File chunksIndexFile,
+                               CompressionParams parameters,
+                               Memory chunkOffsets,
+                               long chunkOffsetsSize,
+                               long dataLength,
+                               long compressedFileLength)
     {
-        this.indexFilePath = filePath;
+        super(chunkOffsets);
+        this.chunksIndexFile = chunksIndexFile;
         this.parameters = parameters;
         this.dataLength = dataLength;
-        this.compressedFileLength = compressedLength;
-        this.chunkOffsets = offsets;
-        this.chunkOffsetsSize = offsetsSize;
+        this.compressedFileLength = compressedFileLength;
+        this.chunkOffsets = chunkOffsets;
+        this.chunkOffsetsSize = chunkOffsetsSize;
+    }
+
+    private CompressionMetadata(CompressionMetadata copy)
+    {
+        super(copy);
+        this.chunksIndexFile = copy.chunksIndexFile;
+        this.parameters = copy.parameters;
+        this.dataLength = copy.dataLength;
+        this.compressedFileLength = copy.compressedFileLength;
+        this.chunkOffsets = copy.chunkOffsets;
+        this.chunkOffsetsSize = copy.chunkOffsetsSize;
     }
 
     public ICompressor compressor()
@@ -171,11 +165,19 @@ public class CompressionMetadata
         return chunkOffsets.size();
     }
 
+    @Override
     public void addTo(Ref.IdentityCollection identities)
     {
+        super.addTo(identities);
         identities.add(chunkOffsets);
     }
 
+    @Override
+    public CompressionMetadata sharedCopy()
+    {
+        return new CompressionMetadata(this);
+    }
+
     /**
      * Read offsets of the individual chunks from the given input.
      *
@@ -183,7 +185,7 @@ public class CompressionMetadata
      *
      * @return collection of the chunk offsets.
      */
-    private Memory readChunkOffsets(DataInput input)
+    private static Memory readChunkOffsets(FileInputStreamPlus input)
     {
         final int chunkCount;
         try
@@ -194,10 +196,9 @@ public class CompressionMetadata
         }
         catch (IOException e)
         {
-            throw new FSReadError(e, indexFilePath);
+            throw new FSReadError(e, input.file);
         }
 
-        @SuppressWarnings("resource")
         Memory offsets = Memory.allocate(chunkCount * 8L);
         int i = 0;
         try
@@ -218,10 +219,10 @@ public class CompressionMetadata
             if (e instanceof EOFException)
             {
                 String msg = String.format("Corrupted Index File %s: read %d but expected %d chunks.",
-                                           indexFilePath, i, chunkCount);
-                throw new CorruptSSTableException(new IOException(msg, e), indexFilePath);
+                                           input.file.path(), i, chunkCount);
+                throw new CorruptSSTableException(new IOException(msg, e), input.file);
             }
-            throw new FSReadError(e, indexFilePath);
+            throw new FSReadError(e, input.file);
         }
     }
 
@@ -237,11 +238,11 @@ public class CompressionMetadata
         long idx = 8 * (position / parameters.chunkLength());
 
         if (idx >= chunkOffsetsSize)
-            throw new CorruptSSTableException(new EOFException(), indexFilePath);
+            throw new CorruptSSTableException(new EOFException(), chunksIndexFile);
 
         if (idx < 0)
             throw new CorruptSSTableException(new IllegalArgumentException(String.format("Invalid negative chunk index %d with position %d", idx, position)),
-                                              indexFilePath);
+                                              chunksIndexFile);
 
         long chunkOffset = chunkOffsets.getLong(idx);
         long nextChunkOffset = (idx + 8 == chunkOffsetsSize)
@@ -251,6 +252,28 @@ public class CompressionMetadata
         return new Chunk(chunkOffset, (int) (nextChunkOffset - chunkOffset - 4)); // "4" bytes reserved for checksum
     }
 
+    public long getDataOffsetForChunkOffset(long chunkOffset)
+    {
+        long l = 0;
+        long h = (chunkOffsetsSize >> 3) - 1;
+        long idx, offset;
+
+        while (l <= h)
+        {
+            idx = (l + h) >>> 1;
+            offset = chunkOffsets.getLong(idx << 3);
+
+            if (offset < chunkOffset)
+                l = idx + 1;
+            else if (offset > chunkOffset)
+                h = idx - 1;
+            else
+                return idx * parameters.chunkLength();
+        }
+
+        throw new IllegalArgumentException("No chunk with offset " + chunkOffset);
+    }
+
     /**
      * @param sections Collection of sections in uncompressed file. Should not contain sections that overlap each other.
      * @return Total chunk size in bytes for given sections including checksum.
@@ -315,16 +338,11 @@ public class CompressionMetadata
         return offsets.toArray(new Chunk[offsets.size()]);
     }
 
-    public void close()
-    {
-        chunkOffsets.close();
-    }
-
     public static class Writer extends Transactional.AbstractTransactional implements Transactional
     {
         // path to the file
         private final CompressionParams parameters;
-        private final String filePath;
+        private final File file;
         private int maxCount = 100;
         private SafeMemory offsets = new SafeMemory(maxCount * 8L);
         private int count = 0;
@@ -332,15 +350,15 @@ public class CompressionMetadata
         // provided by user when setDescriptor
         private long dataLength, chunkCount;
 
-        private Writer(CompressionParams parameters, String path)
+        private Writer(CompressionParams parameters, File file)
         {
             this.parameters = parameters;
-            filePath = path;
+            this.file = file;
         }
 
-        public static Writer open(CompressionParams parameters, String path)
+        public static Writer open(CompressionParams parameters, File file)
         {
-            return new Writer(parameters, path);
+            return new Writer(parameters, file);
         }
 
         public void addOffset(long offset)
@@ -375,7 +393,7 @@ public class CompressionMetadata
             }
             catch (IOException e)
             {
-                throw new FSWriteError(e, filePath);
+                throw new FSWriteError(e, file);
             }
         }
 
@@ -387,6 +405,7 @@ public class CompressionMetadata
             return this;
         }
 
+        @Override
         public void doPrepare()
         {
             assert chunkCount == count;
@@ -401,7 +420,7 @@ public class CompressionMetadata
             }
 
             // flush the data to disk
-            try (FileOutputStreamPlus out = new FileOutputStreamPlus(filePath))
+            try (FileOutputStreamPlus out = file.newOutputStream(File.WriteMode.OVERWRITE))
             {
                 writeHeader(out, dataLength, count);
                 for (int i = 0; i < count; i++)
@@ -412,15 +431,14 @@ public class CompressionMetadata
             }
             catch (FileNotFoundException | NoSuchFileException fnfe)
             {
-                throw Throwables.propagate(fnfe);
+                throw new RuntimeException(fnfe);
             }
             catch (IOException e)
             {
-                throw new FSWriteError(e, filePath);
+                throw new FSWriteError(e, file);
             }
         }
 
-        @SuppressWarnings("resource")
         public CompressionMetadata open(long dataLength, long compressedLength)
         {
             SafeMemory tOffsets = this.offsets.sharedCopy();
@@ -435,7 +453,7 @@ public class CompressionMetadata
             if (tCount < this.count)
                 compressedLength = tOffsets.getLong(tCount * 8L);
 
-            return new CompressionMetadata(filePath, parameters, tOffsets, tCount * 8L, dataLength, compressedLength);
+            return new CompressionMetadata(file, parameters, tOffsets, tCount * 8L, dataLength, compressedLength);
         }
 
         /**
@@ -461,16 +479,19 @@ public class CompressionMetadata
             count = chunkIndex;
         }
 
+        @Override
         protected Throwable doPostCleanup(Throwable failed)
         {
             return offsets.close(failed);
         }
 
+        @Override
         protected Throwable doCommit(Throwable accumulate)
         {
             return accumulate;
         }
 
+        @Override
         protected Throwable doAbort(Throwable accumulate)
         {
             return accumulate;
@@ -495,6 +516,7 @@ public class CompressionMetadata
             this.length = length;
         }
 
+        @Override
         public boolean equals(Object o)
         {
             if (this == o) return true;
@@ -504,6 +526,7 @@ public class CompressionMetadata
             return length == chunk.length && offset == chunk.offset;
         }
 
+        @Override
         public int hashCode()
         {
             int result = (int) (offset ^ (offset >>> 32));
@@ -511,25 +534,37 @@ public class CompressionMetadata
             return result;
         }
 
+        @Override
         public String toString()
         {
             return String.format("Chunk<offset: %d, length: %d>", offset, length);
         }
+
+        /**
+         * @return the end of the chunk in the file, including the checksum
+         */
+        public long chunkEnd()
+        {
+            return offset + length + 4;
+        }
     }
 
     static class ChunkSerializer implements IVersionedSerializer<Chunk>
     {
+        @Override
         public void serialize(Chunk chunk, DataOutputPlus out, int version) throws IOException
         {
             out.writeLong(chunk.offset);
             out.writeInt(chunk.length);
         }
 
+        @Override
         public Chunk deserialize(DataInputPlus in, int version) throws IOException
         {
             return new Chunk(in.readLong(), in.readInt());
         }
 
+        @Override
         public long serializedSize(Chunk chunk, int version)
         {
             long size = TypeSizes.sizeof(chunk.offset);
