@@ -17,7 +17,8 @@
  */
 package org.apache.cassandra.io.sstable;
 
-import java.io.File;
+
+import org.apache.cassandra.io.util.File;
 
 public class CorruptSSTableException extends RuntimeException
 {
