@@ -64,14 +64,14 @@ public class Descriptor
     // to the SSTable naming.
     static final Pattern SSTABLE_DIR_PATTERN = Pattern.compile(".*/(?<keyspace>\\w+)/" +
                                                                "(?<tableName>\\w+)-(?<tableId>[0-9a-f]{32})/" +
-                                                               "(backups/|snapshots/(?<tag>[\\w-]+)/)?" +
+                                                               "(backups/|snapshots/(?<tag>[^/]+)/)?" +
                                                                "(\\.(?<indexName>[\\w-]+)/)?" +
                                                                "(?<component>[\\w-\\+]+)\\.(?<ext>[\\w]+)$");
 
     // Pre 2.1 SSTable directory format is {keyspace}/{tableName}-{tableId}[/backups|/snapshots/{tag}][/.{indexName}]/{component}.db
     static final Pattern LEGACY_SSTABLE_DIR_PATTERN = Pattern.compile(".*/(?<keyspace>\\w+)/" +
                                                                       "(?<tableName>\\w+)/" +
-                                                                      "(backups/|snapshots/(?<tag>[\\w-]+)/)?" +
+                                                                      "(backups/|snapshots/(?<tag>[^/]+)/)?" +
                                                                       "(\\.(?<indexName>[\\w-]+)/)?" +
                                                                       "(?<component>[\\w-]+)\\.(?<ext>[\\w]+)$");
 
