@@ -17,15 +17,18 @@
  */
 package org.apache.cassandra.io.sstable;
 
+import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Iterator;
 
 import org.apache.cassandra.db.DecoratedKey;
 import org.apache.cassandra.io.sstable.format.SSTableReader;
+import org.apache.cassandra.io.util.FileUtils;
 import org.apache.cassandra.utils.CloseableIterator;
 import org.apache.cassandra.utils.IMergeIterator;
 import org.apache.cassandra.utils.MergeIterator;
+import org.apache.cassandra.utils.Throwables;
 
 /**
  * Caller must acquire and release references to the sstables used here.
@@ -39,7 +42,17 @@ public class ReducingKeyIterator implements CloseableIterator<DecoratedKey>
     {
         iters = new ArrayList<>(sstables.size());
         for (SSTableReader sstable : sstables)
-            iters.add(new KeyIterator(sstable.descriptor, sstable.metadata()));
+        {
+            try
+            {
+                iters.add(sstable.keyIterator());
+            }
+            catch (IOException ex)
+            {
+                iters.forEach(FileUtils::closeQuietly);
+                throw new RuntimeException("Failed to create a key iterator for sstable " + sstable.getFilename());
+            }
+        }
     }
 
     private void maybeInit()
@@ -78,7 +91,17 @@ public class ReducingKeyIterator implements CloseableIterator<DecoratedKey>
     public void close()
     {
         if (mi != null)
+        {
             mi.close();
+        }
+        else
+        {
+            // if merging iterator was not initialized before this reducing iterator is closed, we need to close the
+            // underlying iterators manually
+            Throwable err = Throwables.close(null, iters);
+            if (err != null)
+                throw Throwables.unchecked(err);
+        }
     }
 
     public long getTotalBytes()
@@ -121,4 +144,4 @@ public class ReducingKeyIterator implements CloseableIterator<DecoratedKey>
     {
         throw new UnsupportedOperationException();
     }
-}
+}
\ No newline at end of file
