@@ -23,8 +23,14 @@ import java.util.Iterator;
 
 import com.google.common.annotations.VisibleForTesting;
 
-import org.apache.cassandra.db.*;
-import org.apache.cassandra.db.rows.*;
+import org.apache.cassandra.db.DeletionTime;
+import org.apache.cassandra.db.SerializationHeader;
+import org.apache.cassandra.db.rows.BTreeRow;
+import org.apache.cassandra.db.rows.DeserializationHelper;
+import org.apache.cassandra.db.rows.Row;
+import org.apache.cassandra.db.rows.Rows;
+import org.apache.cassandra.db.rows.Unfiltered;
+import org.apache.cassandra.db.rows.UnfilteredSerializer;
 import org.apache.cassandra.io.util.DataInputPlus;
 import org.apache.cassandra.io.util.FileDataInput;
 import org.apache.cassandra.schema.TableMetadata;
@@ -32,7 +38,7 @@ import org.apache.cassandra.utils.AbstractIterator;
 
 /**
  * Utility class to handle deserializing atom from sstables.
- *
+ * <p>
  * Note that this is not a full fledged UnfilteredRowIterator. It's also not closeable, it is always
  * the job of the user to close the underlying ressources.
  */
@@ -136,7 +142,7 @@ public abstract class SSTableSimpleIterator extends AbstractIterator<Unfiltered>
             super(metadata, null, null);
         }
 
-        public Row readStaticRow() throws IOException
+        public Row readStaticRow()
         {
             return Rows.EMPTY_STATIC_ROW;
         }
