@@ -282,6 +282,12 @@ public abstract class SSTableWriter extends SSTable implements Transactional
         txnProxy.prepareToCommit();
     }
 
+    // notify sstable flush observer about sstable writer switched
+    public final void onSSTableWriterSwitched()
+    {
+        observers.forEach(SSTableFlushObserver::onSSTableWriterSwitched);
+    }
+
     public final Throwable commit(Throwable accumulate)
     {
         try
@@ -381,7 +387,7 @@ public abstract class SSTableWriter extends SSTable implements Transactional
             new StatsComponent(finalizeMetadata()).save(descriptor);
 
             // save the table of components
-            TOCComponent.appendTOC(descriptor, components);
+            TOCComponent.updateTOC(descriptor, components);
 
             if (openResult)
                 finalReader = openFinal(SSTableReader.OpenReason.NORMAL);
