@@ -26,9 +26,10 @@ import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Preconditions;
 
 import net.nicoulaj.compilecommand.annotations.DontInline;
-import org.apache.cassandra.config.Config;
 import org.apache.cassandra.utils.FastByteOperations;
 
+import static org.apache.cassandra.config.CassandraRelevantProperties.NIO_DATA_OUTPUT_STREAM_PLUS_BUFFER_SIZE;
+
 /**
  * An implementation of the DataOutputStreamPlus interface using a ByteBuffer to stage writes
  * before flushing them to an underlying channel.
@@ -37,7 +38,7 @@ import org.apache.cassandra.utils.FastByteOperations;
  */
 public class BufferedDataOutputStreamPlus extends DataOutputStreamPlus
 {
-    private static final int DEFAULT_BUFFER_SIZE = Integer.getInteger(Config.PROPERTY_PREFIX + "nio_data_output_stream_plus_buffer_size", 1024 * 32);
+    private static final int DEFAULT_BUFFER_SIZE = NIO_DATA_OUTPUT_STREAM_PLUS_BUFFER_SIZE.getInt();
 
     protected ByteBuffer buffer;
 
@@ -66,6 +67,16 @@ public class BufferedDataOutputStreamPlus extends DataOutputStreamPlus
         this.buffer = buffer;
     }
 
+    protected BufferedDataOutputStreamPlus(int size)
+    {
+        this.buffer = allocate(size);
+    }
+
+    protected ByteBuffer allocate(int size)
+    {
+        return ByteBuffer.allocate(size);
+    }
+
     @Override
     public void write(byte[] b) throws IOException
     {
@@ -75,6 +86,7 @@ public class BufferedDataOutputStreamPlus extends DataOutputStreamPlus
     @Override
     public void write(byte[] b, int off, int len) throws IOException
     {
+        assert buffer != null : "Attempt to use a closed data output";
         if (b == null)
             throw new NullPointerException();
 
@@ -111,6 +123,7 @@ public class BufferedDataOutputStreamPlus extends DataOutputStreamPlus
     @Override
     public void write(ByteBuffer src) throws IOException
     {
+        assert buffer != null : "Attempt to use a closed data output";
         int srcPos = src.position();
         int srcCount;
         int trgAvailable;
@@ -128,6 +141,7 @@ public class BufferedDataOutputStreamPlus extends DataOutputStreamPlus
     @Override
     public void write(int b) throws IOException
     {
+        assert buffer != null : "Attempt to use a closed data output";
         if (!buffer.hasRemaining())
             doFlush(1);
         buffer.put((byte) (b & 0xFF));
@@ -136,6 +150,7 @@ public class BufferedDataOutputStreamPlus extends DataOutputStreamPlus
     @Override
     public void writeBoolean(boolean v) throws IOException
     {
+        assert buffer != null : "Attempt to use a closed data output";
         if (!buffer.hasRemaining())
             doFlush(1);
         buffer.put(v ? (byte)1 : (byte)0);
@@ -148,11 +163,12 @@ public class BufferedDataOutputStreamPlus extends DataOutputStreamPlus
     }
 
     @Override
-    public void writeBytes(long register, int bytes) throws IOException
+    public void writeMostSignificantBytes(long register, int bytes) throws IOException
     {
+        assert buffer != null : "Attempt to use a closed data output";
         if (buffer.remaining() < Long.BYTES)
         {
-            super.writeBytes(register, bytes);
+            super.writeMostSignificantBytes(register, bytes);
         }
         else
         {
@@ -171,6 +187,7 @@ public class BufferedDataOutputStreamPlus extends DataOutputStreamPlus
     @Override
     public void writeChar(int v) throws IOException
     {
+        assert buffer != null : "Attempt to use a closed data output";
         if (buffer.remaining() < 2)
             writeSlow(v, 2);
         else
@@ -180,6 +197,7 @@ public class BufferedDataOutputStreamPlus extends DataOutputStreamPlus
     @Override
     public void writeInt(int v) throws IOException
     {
+        assert buffer != null : "Attempt to use a closed data output";
         if (buffer.remaining() < 4)
             writeSlow(v, 4);
         else
@@ -189,6 +207,7 @@ public class BufferedDataOutputStreamPlus extends DataOutputStreamPlus
     @Override
     public void writeLong(long v) throws IOException
     {
+        assert buffer != null : "Attempt to use a closed data output";
         if (buffer.remaining() < 8)
             writeSlow(v, 8);
         else
@@ -210,6 +229,7 @@ public class BufferedDataOutputStreamPlus extends DataOutputStreamPlus
     @DontInline
     private void writeSlow(long bytes, int count) throws IOException
     {
+        assert buffer != null : "Attempt to use a closed data output";
         int origCount = count;
         if (ByteOrder.BIG_ENDIAN == buffer.order())
             while (count > 0) writeByte((int) (bytes >>> (8 * --count)));
@@ -234,6 +254,7 @@ public class BufferedDataOutputStreamPlus extends DataOutputStreamPlus
     @Override
     public void writeUTF(String s) throws IOException
     {
+        assert buffer != null : "Attempt to use a closed data output";
         UnbufferedDataOutputStreamPlus.writeUTF(s, this);
     }
 
@@ -243,6 +264,7 @@ public class BufferedDataOutputStreamPlus extends DataOutputStreamPlus
     @DontInline
     protected void doFlush(int count) throws IOException
     {
+        assert buffer != null : "Attempt to use a closed data output";
         buffer.flip();
 
         while (buffer.hasRemaining())
@@ -271,7 +293,8 @@ public class BufferedDataOutputStreamPlus extends DataOutputStreamPlus
 
     public BufferedDataOutputStreamPlus order(ByteOrder order)
     {
+        assert buffer != null : "Attempt to use a closed data output";
         this.buffer.order(order);
         return this;
     }
-}
+}
\ No newline at end of file
