@@ -40,6 +40,7 @@ import org.apache.cassandra.utils.concurrent.SharedCloseableImpl;
  */
 public final class ChannelProxy extends SharedCloseableImpl
 {
+    private final File file;
     private final String filePath;
     private final FileChannel channel;
 
@@ -62,14 +63,15 @@ public final class ChannelProxy extends SharedCloseableImpl
 
     public ChannelProxy(File file)
     {
-        this(file.path(), openChannel(file));
+        this(file, openChannel(file));
     }
 
-    public ChannelProxy(String filePath, FileChannel channel)
+    public ChannelProxy(File file, FileChannel channel)
     {
-        super(new Cleanup(filePath, channel));
+        super(new Cleanup(file.path(), channel));
 
-        this.filePath = filePath;
+        this.file = file;
+        this.filePath = file.path();
         this.channel = channel;
     }
 
@@ -77,6 +79,7 @@ public final class ChannelProxy extends SharedCloseableImpl
     {
         super(copy);
 
+        this.file = copy.file;
         this.filePath = copy.filePath;
         this.channel = copy.channel;
     }
@@ -130,6 +133,11 @@ public final class ChannelProxy extends SharedCloseableImpl
         return filePath;
     }
 
+    public File file()
+    {
+        return file;
+    }
+
     public int read(ByteBuffer buffer, long position)
     {
         try
