@@ -17,10 +17,14 @@
  */
 package org.apache.cassandra.io.util;
 
-import java.io.*;
+import java.io.DataInput;
+import java.io.EOFException;
+import java.io.IOException;
+import java.io.InputStream;
 
 import org.apache.cassandra.utils.Shared;
 import org.apache.cassandra.utils.vint.VIntCoding;
+import org.apache.cassandra.utils.vint.VIntCoding.VIntOutOfRangeException;
 
 import static org.apache.cassandra.utils.Shared.Scope.SIMULATION;
 
@@ -30,11 +34,30 @@ import static org.apache.cassandra.utils.Shared.Scope.SIMULATION;
 @Shared(scope = SIMULATION)
 public interface DataInputPlus extends DataInput
 {
+    /**
+     * Read a 64-bit integer back.
+     *
+     * This method assumes it was originally written using
+     * {@link DataOutputPlus#writeVInt(long)} or similar that zigzag encodes the vint.
+     */
     default long readVInt() throws IOException
     {
         return VIntCoding.readVInt(this);
     }
 
+    /**
+     * Read up to a 32-bit integer back.
+     *
+     * This method assumes the integer was originally written using
+     * {@link DataOutputPlus#writeVInt32(int)} or similar that zigzag encodes the vint.
+     *
+     * @throws VIntOutOfRangeException If the vint doesn't fit into a 32-bit integer
+     */
+    default int readVInt32() throws IOException
+    {
+        return VIntCoding.readVInt32(this);
+    }
+
     /**
      * Think hard before opting for an unsigned encoding. Is this going to bite someone because some day
      * they might need to pass in a sentinel value using negative numbers? Is the risk worth it
@@ -47,6 +70,19 @@ public interface DataInputPlus extends DataInput
         return VIntCoding.readUnsignedVInt(this);
     }
 
+    /**
+     * Read up to a 32-bit integer back.
+     *
+     * This method assumes the original integer was written using {@link DataOutputPlus#writeUnsignedVInt32(int)}
+     * or similar that doesn't zigzag encodes the vint.
+     *
+     * @throws VIntOutOfRangeException If the vint doesn't fit into a 32-bit integer
+     */
+    default int readUnsignedVInt32() throws IOException
+    {
+        return VIntCoding.readUnsignedVInt32(this);
+    }
+
     /**
      * Always skips the requested number of bytes, unless EOF is reached
      *
@@ -64,14 +100,8 @@ public interface DataInputPlus extends DataInput
 
     /**
      * Wrapper around an InputStream that provides no buffering but can decode varints
-     *
-     * TODO: probably shouldn't use DataInputStream as a parent
      */
-    public class DataInputStreamPlus extends DataInputStream implements DataInputPlus
+    abstract class DataInputStreamPlus extends InputStream implements DataInputPlus
     {
-        public DataInputStreamPlus(InputStream is)
-        {
-            super(is);
-        }
     }
 }
