@@ -19,6 +19,8 @@
 package org.apache.cassandra.io.util;
 
 import java.nio.ByteBuffer;
+import java.nio.FloatBuffer;
+import java.nio.IntBuffer;
 
 /**
  * Rebufferer for reading data by a RandomAccessReader.
@@ -47,6 +49,16 @@ public interface Rebufferer extends ReaderFileProxy
          */
         ByteBuffer buffer();
 
+        default FloatBuffer floatBuffer()
+        {
+            throw new UnsupportedOperationException();
+        }
+
+        default IntBuffer intBuffer()
+        {
+            throw new UnsupportedOperationException();
+        }
+
         /**
          * Position in the file of the start of the buffer.
          */
