@@ -21,13 +21,13 @@ import java.io.Closeable;
 import java.io.DataInputStream;
 import java.io.EOFException;
 import java.io.IOException;
-import java.io.InputStream;
 import java.nio.ByteBuffer;
 import java.nio.ByteOrder;
 
 import com.google.common.base.Preconditions;
 
 import net.nicoulaj.compilecommand.annotations.DontInline;
+import org.apache.cassandra.io.util.DataInputPlus.DataInputStreamPlus;
 import org.apache.cassandra.utils.FastByteOperations;
 import org.apache.cassandra.utils.vint.VIntCoding;
 
@@ -36,10 +36,10 @@ import static java.lang.Math.min;
 /**
  * Rough equivalent of BufferedInputStream and DataInputStream wrapping a ByteBuffer that can be refilled
  * via rebuffer. Implementations provide this buffer from various channels (socket, file, memory, etc).
- *
+ * <p>
  * RebufferingInputStream is not thread safe.
  */
-public abstract class RebufferingInputStream extends InputStream implements DataInputPlus, Closeable
+public abstract class RebufferingInputStream extends DataInputStreamPlus implements DataInputPlus, Closeable
 {
     protected ByteBuffer buffer;
 
@@ -231,11 +231,19 @@ public abstract class RebufferingInputStream extends InputStream implements Data
             return readPrimitiveSlowly(8);
     }
 
+    @Override
     public long readVInt() throws IOException
     {
         return VIntCoding.decodeZigZag64(readUnsignedVInt());
     }
 
+    @Override
+    public int readVInt32() throws IOException
+    {
+        return VIntCoding.checkedCast(VIntCoding.decodeZigZag64(readUnsignedVInt()));
+    }
+
+    @Override
     public long readUnsignedVInt() throws IOException
     {
         //If 9 bytes aren't available use the slow path in VIntCoding
@@ -267,6 +275,12 @@ public abstract class RebufferingInputStream extends InputStream implements Data
         return retval;
     }
 
+    @Override
+    public int readUnsignedVInt32() throws IOException
+    {
+        return VIntCoding.checkedCast(readUnsignedVInt());
+    }
+
     @Override
     public float readFloat() throws IOException
     {
