@@ -55,9 +55,12 @@ class SimpleChunkReader extends AbstractReaderFileProxy implements ChunkReader
     }
 
     @Override
-    public Rebufferer instantiateRebufferer()
+    public Rebufferer instantiateRebufferer(boolean forScan)
     {
-        return new BufferManagingRebufferer.Unaligned(this);
+        if (Integer.bitCount(bufferSize) == 1)
+            return new BufferManagingRebufferer.Aligned(this);
+        else
+            return new BufferManagingRebufferer.Unaligned(this);
     }
 
     @Override
@@ -69,4 +72,4 @@ class SimpleChunkReader extends AbstractReaderFileProxy implements ChunkReader
                              bufferSize,
                              fileLength());
     }
-}
+}
\ No newline at end of file
