@@ -26,6 +26,8 @@ import java.util.concurrent.ScheduledFuture;
 import java.util.concurrent.TimeUnit;
 import java.util.stream.Collectors;
 
+import com.google.common.annotations.VisibleForTesting;
+
 import com.codahale.metrics.ExponentiallyDecayingReservoir;
 import com.codahale.metrics.Snapshot;
 
@@ -41,13 +43,14 @@ import org.apache.cassandra.service.StorageService;
 import org.apache.cassandra.utils.FBUtilities;
 import org.apache.cassandra.utils.MBeanWrapper;
 
+import static org.apache.cassandra.config.CassandraRelevantProperties.IGNORE_DYNAMIC_SNITCH_SEVERITY;
 
 /**
  * A dynamic snitch that sorts endpoints by latency with an adapted phi failure detector
  */
 public class DynamicEndpointSnitch extends AbstractEndpointSnitch implements LatencySubscribers.Subscriber, DynamicEndpointSnitchMBean
 {
-    private static final boolean USE_SEVERITY = !Boolean.getBoolean("cassandra.ignore_dynamic_snitch_severity");
+    private static final boolean USE_SEVERITY = !IGNORE_DYNAMIC_SNITCH_SEVERITY.getBoolean();
 
     private static final double ALPHA = 0.75; // set to 0.75 to make EDS more biased to towards the newer values
     private static final int WINDOW_SIZE = 100;
@@ -201,7 +204,7 @@ public class DynamicEndpointSnitch extends AbstractEndpointSnitch implements Lat
         {
             Double score = scores.get(replica.endpoint());
             if (score == null)
-                score = 0.0;
+                score = defaultStore(replica.endpoint());
             subsnitchOrderedScores.add(score);
         }
 
@@ -225,6 +228,11 @@ public class DynamicEndpointSnitch extends AbstractEndpointSnitch implements Lat
         return replicas;
     }
 
+    private static double defaultStore(InetAddressAndPort target)
+    {
+        return USE_SEVERITY ? getSeverity(target) : 0.0;
+    }
+
     // Compare endpoints given an immutable snapshot of the scores
     private int compareEndpoints(InetAddressAndPort target, Replica a1, Replica a2, Map<InetAddressAndPort, Double> scores)
     {
@@ -233,12 +241,12 @@ public class DynamicEndpointSnitch extends AbstractEndpointSnitch implements Lat
         
         if (scored1 == null)
         {
-            scored1 = 0.0;
+            scored1 = defaultStore(a1.endpoint());
         }
 
         if (scored2 == null)
         {
-            scored2 = 0.0;
+            scored2 = defaultStore(a2.endpoint());
         }
 
         if (scored1.equals(scored2))
@@ -270,9 +278,10 @@ public class DynamicEndpointSnitch extends AbstractEndpointSnitch implements Lat
         sample.update(unit.toMillis(latency));
     }
 
+    @VisibleForTesting
     public void updateScores() // this is expensive
     {
-        if (!StorageService.instance.isGossipActive())
+        if (!StorageService.instance.isInitialized())
             return;
         if (!registered)
         {
@@ -321,7 +330,7 @@ public class DynamicEndpointSnitch extends AbstractEndpointSnitch implements Lat
 
     public Map<InetAddress, Double> getScores()
     {
-        return scores.entrySet().stream().collect(Collectors.toMap(address -> address.getKey().address, Map.Entry::getValue));
+        return scores.entrySet().stream().collect(Collectors.toMap(address -> address.getKey().getAddress(), Map.Entry::getValue));
     }
 
     public Map<String, Double> getScoresWithPort()
@@ -360,12 +369,19 @@ public class DynamicEndpointSnitch extends AbstractEndpointSnitch implements Lat
         return timings;
     }
 
+    @Override
     public void setSeverity(double severity)
+    {
+        addSeverity(severity);
+    }
+
+    public static void addSeverity(double severity)
     {
         Gossiper.instance.addLocalApplicationState(ApplicationState.SEVERITY, StorageService.instance.valueFactory.severity(severity));
     }
 
-    private double getSeverity(InetAddressAndPort endpoint)
+    @VisibleForTesting
+    public static double getSeverity(InetAddressAndPort endpoint)
     {
         EndpointState state = Gossiper.instance.getEndpointStateForEndpoint(endpoint);
         if (state == null)
