@@ -164,6 +164,26 @@ public interface ReplicaPlan<E extends Endpoints<E>, P extends ReplicaPlan<E, P>
         }
     }
 
+    public static class ForFullRangeRead extends ForRangeRead
+    {
+        public ForFullRangeRead(Keyspace keyspace,
+                                AbstractReplicationStrategy replicationStrategy,
+                                ConsistencyLevel consistencyLevel,
+                                AbstractBounds<PartitionPosition> range,
+                                EndpointsForRange candidates,
+                                EndpointsForRange contact,
+                                int vnodeCount)
+        {
+            super(keyspace, replicationStrategy, consistencyLevel, range, candidates, contact, vnodeCount);
+        }
+
+        @Override
+        public int readQuorum()
+        {
+            return candidates.size();
+        }
+    }
+
     public static class ForWrite extends AbstractReplicaPlan<EndpointsForToken, ForWrite>
     {
         // TODO: this is only needed because of poor isolation of concerns elsewhere - we can remove it soon, and will do so in a follow-up patch
