@@ -92,6 +92,7 @@ public final class ClientMetrics
 
     public void markRequestDiscarded() { requestDiscarded.mark(); }
     public void markRequestDispatched() { requestDispatched.mark(); }
+    public void markTimedOutBeforeProcessing() { timedOutBeforeProcessing.mark(); }
 
     public List<ConnectedClient> allConnectedClients()
     {
@@ -103,8 +104,6 @@ public final class ClientMetrics
         return clients;
     }
 
-    public void markTimedOutBeforeProcessing() { timedOutBeforeProcessing.mark(); }
-
     public void markProtocolException()
     {
         protocolException.mark();
@@ -151,9 +150,9 @@ public final class ClientMetrics
         timedOutBeforeProcessing = registerMeter("TimedOutBeforeProcessing");
         protocolException = registerMeter("ProtocolException");
         unknownException = registerMeter("UnknownException");
-        queueTime = registerTimer("Queued");
 
         initialized = true;
+        queueTime = registerTimer("Queued");
     }
 
     private int countConnectedClients()
@@ -219,7 +218,7 @@ public final class ClientMetrics
         return Metrics.meter(factory.createMetricName(name));
     }
 
-    public com.codahale.metrics.Timer registerTimer(String name)
+    public Timer registerTimer(String name)
     {
         return Metrics.timer(factory.createMetricName(name));
     }
