@@ -130,6 +130,12 @@ public class KeyspaceMetrics
     public final Histogram bytesValidated;
     /** histogram over the number of partitions we have validated */
     public final Histogram partitionsValidated;
+    /** Lifetime count of reads for keys outside the node's owned token ranges for this keyspace **/
+    public final Counter outOfRangeTokenReads;
+    /** Lifetime count of writes for keys outside the node's owned token ranges for this keyspace **/
+    public final Counter outOfRangeTokenWrites;
+    /** Lifetime count of paxos requests for keys outside the node's owned token ranges for this keyspace **/
+    public final Counter outOfRangeTokenPaxosRequests;
 
     /*
      * Metrics for inconsistencies detected between repaired data sets across replicas. These
@@ -282,6 +288,10 @@ public class KeyspaceMetrics
         tooManySSTableIndexesReadAborts = createKeyspaceMeter("TooManySSTableIndexesReadAborts");
 
         formatSpecificGauges = createFormatSpecificGauges(keyspace);
+
+        outOfRangeTokenReads = createKeyspaceCounter("ReadOutOfRangeToken");
+        outOfRangeTokenWrites = createKeyspaceCounter("WriteOutOfRangeToken");
+        outOfRangeTokenPaxosRequests = createKeyspaceCounter("PaxosOutOfRangeToken");
     }
 
     /**
