@@ -31,10 +31,13 @@ public class ReadRepairMetrics
     public static final Meter repairedBlocking = Metrics.meter(factory.createMetricName("RepairedBlocking"));
     public static final Meter reconcileRead = Metrics.meter(factory.createMetricName("ReconcileRead"));
 
-    @Deprecated
+    /** @deprecated See CASSANDRA-13910 */
+    @Deprecated(since = "4.0")
     public static final Meter repairedBackground = Metrics.meter(factory.createMetricName("RepairedBackground"));
-    @Deprecated
+    /** @deprecated See CASSANDRA-13910 */
+    @Deprecated(since = "4.0")
     public static final Meter attempted = Metrics.meter(factory.createMetricName("Attempted"));
+    public static final Meter timedOut = Metrics.meter(factory.createMetricName("RepairTimedOut"));
 
     // Incremented when additional requests were sent during blocking read repair due to unavailable or slow nodes
     public static final Meter speculatedRead = Metrics.meter(factory.createMetricName("SpeculatedRead"));
