@@ -22,21 +22,17 @@ import java.io.IOException;
 import java.nio.ByteBuffer;
 import java.util.Objects;
 
-import com.google.common.annotations.VisibleForTesting;
-
 import io.netty.buffer.ByteBuf;
 import io.netty.buffer.ByteBufAllocator;
 import org.apache.cassandra.config.DatabaseDescriptor;
 import org.apache.cassandra.io.compress.BufferType;
 import org.apache.cassandra.io.util.DataInputBuffer;
-import org.apache.cassandra.io.util.DataInputPlus;
 import org.apache.cassandra.io.util.DataOutputBufferFixed;
 import org.apache.cassandra.locator.InetAddressAndPort;
 import org.apache.cassandra.utils.memory.BufferPools;
 
 import static java.util.concurrent.TimeUnit.MILLISECONDS;
 import static org.apache.cassandra.locator.InetAddressAndPort.Serializer.inetAddressAndPortSerializer;
-import static org.apache.cassandra.net.MessagingService.VERSION_30;
 import static org.apache.cassandra.net.MessagingService.VERSION_40;
 import static org.apache.cassandra.net.Message.validateLegacyProtocolMagic;
 import static org.apache.cassandra.net.Crc.*;
@@ -52,8 +48,7 @@ import static org.apache.cassandra.net.OutboundConnectionSettings.*;
  * it will simply disconnect and reconnect with a more appropriate version. But if the version is acceptable, the connection
  * initiator sends the third message of the protocol, after which it considers the connection ready.
  */
-@VisibleForTesting
-public class HandshakeProtocol
+class HandshakeProtocol
 {
     static final long TIMEOUT_MILLIS = 3 * DatabaseDescriptor.getRpcTimeout(MILLISECONDS);
 
@@ -94,8 +89,6 @@ public class HandshakeProtocol
         private static final int MIN_LENGTH = 8;
         private static final int MAX_LENGTH = 12 + InetAddressAndPort.Serializer.MAXIMUM_SIZE;
 
-        @Deprecated // this is ignored by post40 nodes, i.e. if maxMessagingVersion is set
-        final int requestMessagingVersion;
         // the messagingVersion bounds the sender will accept to initiate a connection;
         // if the remote peer supports any, the newest supported version will be selected; otherwise the nearest supported version
         final AcceptVersions acceptVersions;
@@ -103,17 +96,15 @@ public class HandshakeProtocol
         final Framing framing;
         final InetAddressAndPort from;
 
-        Initiate(int requestMessagingVersion, AcceptVersions acceptVersions, ConnectionType type, Framing framing, InetAddressAndPort from)
+        Initiate(AcceptVersions acceptVersions, ConnectionType type, Framing framing, InetAddressAndPort from)
         {
-            this.requestMessagingVersion = requestMessagingVersion;
             this.acceptVersions = acceptVersions;
             this.type = type;
             this.framing = framing;
             this.from = from;
         }
 
-        @VisibleForTesting
-        int encodeFlags()
+        private int encodeFlags()
         {
             int flags = 0;
             if (type.isMessaging())
@@ -123,11 +114,7 @@ public class HandshakeProtocol
 
             // framing id is split over 2nd and 4th bits, for backwards compatibility
             flags |= ((framing.id & 1) << 2) | ((framing.id & 2) << 3);
-            flags |= (requestMessagingVersion << 8);
-
-            if (requestMessagingVersion < VERSION_40 || acceptVersions.max < VERSION_40)
-                return flags; // for testing, permit serializing as though we are pre40
-
+            flags |= (acceptVersions.min << 8); // legacy (pre40)
             flags |= (acceptVersions.min << 16);
             flags |= (acceptVersions.max << 24);
             return flags;
@@ -140,12 +127,8 @@ public class HandshakeProtocol
             {
                 out.writeInt(Message.PROTOCOL_MAGIC);
                 out.writeInt(encodeFlags());
-
-                if (requestMessagingVersion >= VERSION_40 && acceptVersions.max >= VERSION_40)
-                {
-                    inetAddressAndPortSerializer.serialize(from, out, requestMessagingVersion);
-                    out.writeInt(computeCrc32(buffer, 0, buffer.position()));
-                }
+                inetAddressAndPortSerializer.serialize(from, out, acceptVersions.min);
+                out.writeInt(computeCrc32(buffer, 0, buffer.position()));
                 buffer.flip();
                 return GlobalBufferPoolAllocator.wrap(buffer);
             }
@@ -167,9 +150,17 @@ public class HandshakeProtocol
                 validateLegacyProtocolMagic(in.readInt());
                 int flags = in.readInt();
 
-                int requestedMessagingVersion = getBits(flags, 8, 8);
+                // legacy pre40 messagingVersion flag
+                if (getBits(flags, 8, 8) < VERSION_40)
+                    return null;
+
                 int minMessagingVersion = getBits(flags, 16, 8);
                 int maxMessagingVersion = getBits(flags, 24, 8);
+
+                // 5.0+ does not support pre40
+                if (maxMessagingVersion < MessagingService.VERSION_40)
+                    return null;
+
                 int framingBits = getBits(flags, 2, 1) | (getBits(flags, 4, 1) << 1);
                 Framing framing = Framing.forId(framingBits);
 
@@ -179,23 +170,15 @@ public class HandshakeProtocol
                                     ? ConnectionType.STREAMING
                                     : ConnectionType.fromId(getBits(flags, 0, 2));
 
-                InetAddressAndPort from = null;
+                InetAddressAndPort from = inetAddressAndPortSerializer.deserialize(in, minMessagingVersion);
 
-                if (requestedMessagingVersion >= VERSION_40 && maxMessagingVersion >= MessagingService.VERSION_40)
-                {
-                    from = inetAddressAndPortSerializer.deserialize(in, requestedMessagingVersion);
-
-                    int computed = computeCrc32(nio, start, nio.position());
-                    int read = in.readInt();
-                    if (read != computed)
-                        throw new InvalidCrc(read, computed);
-                }
+                int computed = computeCrc32(nio, start, nio.position());
+                int read = in.readInt();
+                if (read != computed)
+                    throw new InvalidCrc(read, computed);
 
                 buf.skipBytes(nio.position() - start);
-                return new Initiate(requestedMessagingVersion,
-                                    minMessagingVersion == 0 && maxMessagingVersion == 0
-                                        ? null : new AcceptVersions(minMessagingVersion, maxMessagingVersion),
-                                    type, framing, from);
+                return new Initiate(new AcceptVersions(minMessagingVersion, maxMessagingVersion), type, framing, from);
 
             }
             catch (EOFException e)
@@ -204,7 +187,6 @@ public class HandshakeProtocol
             }
         }
 
-        @VisibleForTesting
         @Override
         public boolean equals(Object other)
         {
@@ -214,17 +196,15 @@ public class HandshakeProtocol
             Initiate that = (Initiate)other;
             return    this.type == that.type
                    && this.framing == that.framing
-                   && this.requestMessagingVersion == that.requestMessagingVersion
                    && Objects.equals(this.acceptVersions, that.acceptVersions);
         }
 
         @Override
         public String toString()
         {
-            return String.format("Initiate(request: %d, min: %d, max: %d, type: %s, framing: %b, from: %s)",
-                                 requestMessagingVersion,
-                                 acceptVersions == null ? requestMessagingVersion : acceptVersions.min,
-                                 acceptVersions == null ? requestMessagingVersion : acceptVersions.max,
+            return String.format("Initiate(min: %d, max: %d, type: %s, framing: %b, from: %s)",
+                                 acceptVersions.min,
+                                 acceptVersions.max,
                                  type, framing, from);
         }
     }
@@ -238,9 +218,8 @@ public class HandshakeProtocol
      *   1) the messaging version of the peer sending this message
      *   2) the negotiated messaging version if one could be accepted by both peers,
      *      or if not the closest version that this peer could support to the ones requested
-     *   3) a CRC protectingn the integrity of the message
+     *   3) a CRC protecting the integrity of the message
      *
-     * Note that the pre40 equivalent of this message contains ONLY the messaging version of the peer.
      */
     static class Accept
     {
@@ -266,18 +245,7 @@ public class HandshakeProtocol
             return buffer;
         }
 
-        /**
-         * Respond to pre40 nodes only with our current messagingVersion
-         */
-        static ByteBuf respondPre40(int messagingVersion, ByteBufAllocator allocator)
-        {
-            ByteBuf buffer = allocator.directBuffer(4);
-            buffer.clear();
-            buffer.writeInt(messagingVersion);
-            return buffer;
-        }
-
-        static Accept maybeDecode(ByteBuf in, int handshakeMessagingVersion) throws InvalidCrc
+        static Accept maybeDecode(ByteBuf in) throws InvalidCrc
         {
             int readerIndex = in.readerIndex();
             if (in.readableBytes() < 4)
@@ -285,9 +253,9 @@ public class HandshakeProtocol
             int maxMessagingVersion = in.readInt();
             int useMessagingVersion = 0;
 
-            // if the other node is pre-4.0, it will respond only with its maxMessagingVersion
-            if (maxMessagingVersion < VERSION_40 || handshakeMessagingVersion < VERSION_40)
-                return new Accept(useMessagingVersion, maxMessagingVersion);
+            // pre-4.0 not supported, close the connection
+            if (maxMessagingVersion < VERSION_40)
+                return null;
 
             if (in.readableBytes() < 8)
             {
@@ -305,7 +273,6 @@ public class HandshakeProtocol
             return new Accept(useMessagingVersion, maxMessagingVersion);
         }
 
-        @VisibleForTesting
         @Override
         public boolean equals(Object other)
         {
@@ -321,91 +288,6 @@ public class HandshakeProtocol
         }
     }
 
-    /**
-     * The third message of the handshake, sent by pre40 nodes on reception of {@link Accept}.
-     * This message contains:
-     *   1) The connection initiator's {@link org.apache.cassandra.net.MessagingService#current_version} (4 bytes).
-     *      This indicates the max messaging version supported by this node.
-     *   2) The connection initiator's broadcast address as encoded by {@link InetAddressAndPort.Serializer}.
-     *      This can be either 7 bytes for an IPv4 address, or 19 bytes for an IPv6 one, post40.
-     *      This can be either 5 bytes for an IPv4 address, or 17 bytes for an IPv6 one, pre40.
-     * <p>
-     * This message concludes the legacy handshake protocol.
-     */
-    static class ConfirmOutboundPre40
-    {
-        private static final int MAX_LENGTH = 4 + InetAddressAndPort.Serializer.MAXIMUM_SIZE;
-
-        final int maxMessagingVersion;
-        final InetAddressAndPort from;
-
-        ConfirmOutboundPre40(int maxMessagingVersion, InetAddressAndPort from)
-        {
-            this.maxMessagingVersion = maxMessagingVersion;
-            this.from = from;
-        }
-
-        ByteBuf encode()
-        {
-            ByteBuffer buffer = BufferPools.forNetworking().get(MAX_LENGTH, BufferType.OFF_HEAP);
-            try (DataOutputBufferFixed out = new DataOutputBufferFixed(buffer))
-            {
-                out.writeInt(maxMessagingVersion);
-                // pre-4.0 nodes should only receive the address, never port, and it's ok to hardcode VERSION_30
-                inetAddressAndPortSerializer.serialize(from, out, VERSION_30);
-                buffer.flip();
-                return GlobalBufferPoolAllocator.wrap(buffer);
-            }
-            catch (IOException e)
-            {
-                throw new IllegalStateException(e);
-            }
-        }
-
-        @SuppressWarnings("resource")
-        static ConfirmOutboundPre40 maybeDecode(ByteBuf in)
-        {
-            ByteBuffer nio = in.nioBuffer();
-            int start = nio.position();
-            DataInputPlus input = new DataInputBuffer(nio, false);
-            try
-            {
-                int version = input.readInt();
-                InetAddressAndPort address = inetAddressAndPortSerializer.deserialize(input, version);
-                in.skipBytes(nio.position() - start);
-                return new ConfirmOutboundPre40(version, address);
-            }
-            catch (EOFException e)
-            {
-                // makes the assumption we didn't have enough bytes to deserialize an IPv6 address,
-                // as we only check the MIN_LENGTH of the buf.
-                return null;
-            }
-            catch (IOException e)
-            {
-                throw new IllegalStateException(e);
-            }
-        }
-
-        @VisibleForTesting
-        @Override
-        public boolean equals(Object other)
-        {
-            if (!(other instanceof ConfirmOutboundPre40))
-                return false;
-
-            ConfirmOutboundPre40 that = (ConfirmOutboundPre40) other;
-            return this.maxMessagingVersion == that.maxMessagingVersion
-                   && Objects.equals(this.from, that.from);
-        }
-
-        @Override
-        public String toString()
-        {
-            return String.format("ConfirmOutboundPre40(maxMessagingVersion: %d; address: %s)", maxMessagingVersion, from);
-        }
-    }
-
     private static int getBits(int packed, int start, int count)
     {
         return (packed >>> start) & ~(-1 << count);
