@@ -18,7 +18,6 @@
 
 package org.apache.cassandra.net;
 
-import java.net.InetAddress;
 import java.util.function.Function;
 
 import com.google.common.base.Preconditions;
@@ -71,16 +70,6 @@ public class InboundConnectionSettings
         this(null, null, null, null, null, null, null, null, null);
     }
 
-    public boolean authenticate(InetAddressAndPort endpoint)
-    {
-        return authenticator.authenticate(endpoint.getAddress(), endpoint.getPort());
-    }
-
-    public boolean authenticate(InetAddress address, int port)
-    {
-        return authenticator.authenticate(address, port);
-    }
-
     public String toString()
     {
         return format("address: (%s), nic: %s, encryption: %s",
