@@ -27,14 +27,12 @@ import java.util.Map;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicInteger;
 
-import javax.annotation.Nullable;
-
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.primitives.Ints;
-
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import javax.annotation.Nullable;
 import org.apache.cassandra.config.DatabaseDescriptor;
 import org.apache.cassandra.exceptions.RequestFailureReason;
 import org.apache.cassandra.io.IVersionedAsymmetricSerializer;
@@ -52,19 +50,13 @@ import org.apache.cassandra.utils.TimeUUID;
 
 import static java.util.concurrent.TimeUnit.MINUTES;
 import static java.util.concurrent.TimeUnit.NANOSECONDS;
-
 import static org.apache.cassandra.db.TypeSizes.sizeof;
 import static org.apache.cassandra.db.TypeSizes.sizeofUnsignedVInt;
-import static org.apache.cassandra.locator.InetAddressAndPort.Serializer.inetAddressAndPortSerializer;
-import static org.apache.cassandra.net.MessagingService.VERSION_3014;
-import static org.apache.cassandra.net.MessagingService.VERSION_30;
 import static org.apache.cassandra.net.MessagingService.VERSION_40;
-import static org.apache.cassandra.net.MessagingService.instance;
+import static org.apache.cassandra.net.MessagingService.VERSION_50;
 import static org.apache.cassandra.utils.FBUtilities.getBroadcastAddressAndPort;
 import static org.apache.cassandra.utils.MonotonicClock.Global.approxTime;
-import static org.apache.cassandra.utils.vint.VIntCoding.computeUnsignedVIntSize;
-import static org.apache.cassandra.utils.vint.VIntCoding.getUnsignedVInt;
-import static org.apache.cassandra.utils.vint.VIntCoding.skipUnsignedVInt;
+import static org.apache.cassandra.utils.vint.VIntCoding.*;
 
 /**
  * Immutable main unit of internode communication - what used to be {@code MessageIn} and {@code MessageOut} fused
@@ -112,7 +104,7 @@ public class Message<T>
         return header.verb;
     }
 
-    boolean isFailureResponse()
+    public boolean isFailureResponse()
     {
         return verb() == Verb.FAILURE_RSP;
     }
@@ -328,6 +320,11 @@ public class Message<T>
         return new Message<>(header.withParam(ParamType.FORWARD_TO, peers), payload);
     }
 
+    public Message<T> withFrom(InetAddressAndPort from)
+    {
+        return new Message<>(header.withFrom(from), payload);
+    }
+
     public Message<T> withFlag(MessageFlag flag)
     {
         return new Message<>(header.withFlag(flag), payload);
@@ -461,6 +458,11 @@ public class Message<T>
             this.params = params;
         }
 
+        Header withFrom(InetAddressAndPort from)
+        {
+            return new Header(id, verb, from, createdAtNanos, expiresAtNanos, flags, params);
+        }
+
         Header withFlag(MessageFlag flag)
         {
             return new Header(id, verb, from, createdAtNanos, expiresAtNanos, flag.addTo(flags), params);
@@ -734,15 +736,17 @@ public class Message<T>
 
         public <T> void serialize(Message<T> message, DataOutputPlus out, int version) throws IOException
         {
-            if (version >= VERSION_40)
-                serializePost40(message, out, version);
-            else
-                serializePre40(message, out, version);
+            serializeHeader(message.header, out, version);
+            out.writeUnsignedVInt32(message.payloadSize(version));
+            message.verb().serializer().serialize(message.payload, out, version);
         }
 
         public <T> Message<T> deserialize(DataInputPlus in, InetAddressAndPort peer, int version) throws IOException
         {
-            return version >= VERSION_40 ? deserializePost40(in, peer, version) : deserializePre40(in, version);
+            Header header = deserializeHeader(in, peer, version);
+            skipUnsignedVInt(in); // payload size, not needed by payload deserializer
+            T payload = (T) header.verb.serializer().deserialize(in, version);
+            return new Message<>(header, payload);
         }
 
         /**
@@ -752,158 +756,25 @@ public class Message<T>
          */
         public <T> Message<T> deserialize(DataInputPlus in, Header header, int version) throws IOException
         {
-            return version >= VERSION_40 ? deserializePost40(in, header, version) : deserializePre40(in, header, version);
+            skipHeader(in);
+            skipUnsignedVInt(in); // payload size, not needed by payload deserializer
+            T payload = (T) header.verb.serializer().deserialize(in, version);
+            return new Message<>(header, payload);
         }
 
         private <T> int serializedSize(Message<T> message, int version)
-        {
-            return version >= VERSION_40 ? serializedSizePost40(message, version) : serializedSizePre40(message, version);
-        }
-
-        /**
-         * Size of the next message in the stream. Returns -1 if there aren't sufficient bytes read yet to determine size.
-         */
-        int inferMessageSize(ByteBuffer buf, int index, int limit, int version) throws InvalidLegacyProtocolMagic
-        {
-            int size = version >= VERSION_40 ? inferMessageSizePost40(buf, index, limit) : inferMessageSizePre40(buf, index, limit);
-            if (size > DatabaseDescriptor.getInternodeMaxMessageSizeInBytes())
-                throw new OversizedMessageException(size);
-            return size;
-        }
-
-        /**
-         * Partially deserialize the message - by only extracting the header and leaving the payload alone.
-         *
-         * To get the rest of the message without repeating the work done here, use {@link #deserialize(DataInputPlus, Header, int)}
-         * method.
-         *
-         * It's assumed that the provided buffer contains all the bytes necessary to deserialize the header fully.
-         */
-        Header extractHeader(ByteBuffer buf, InetAddressAndPort from, long currentTimeNanos, int version) throws IOException
-        {
-            return version >= VERSION_40
-                 ? extractHeaderPost40(buf, from, currentTimeNanos, version)
-                 : extractHeaderPre40(buf, currentTimeNanos, version);
-        }
-
-        private static long getExpiresAtNanos(long createdAtNanos, long expirationPeriodNanos)
-        {
-            return createdAtNanos + expirationPeriodNanos;
-        }
-
-        /*
-         * 4.0 ser/deser
-         */
-
-        private void serializeHeaderPost40(Header header, DataOutputPlus out, int version) throws IOException
-        {
-            out.writeUnsignedVInt(header.id);
-            // int cast cuts off the high-order half of the timestamp, which we can assume remains
-            // the same between now and when the recipient reconstructs it.
-            out.writeInt((int) approxTime.translate().toMillisSinceEpoch(header.createdAtNanos));
-            out.writeUnsignedVInt(NANOSECONDS.toMillis(header.expiresAtNanos - header.createdAtNanos));
-            out.writeUnsignedVInt(header.verb.id);
-            out.writeUnsignedVInt(header.flags);
-            serializeParams(header.params, out, version);
-        }
-
-        private Header deserializeHeaderPost40(DataInputPlus in, InetAddressAndPort peer, int version) throws IOException
-        {
-            long id = in.readUnsignedVInt();
-            long currentTimeNanos = approxTime.now();
-            MonotonicClockTranslation timeSnapshot = approxTime.translate();
-            long creationTimeNanos = calculateCreationTimeNanos(in.readInt(), timeSnapshot, currentTimeNanos);
-            long expiresAtNanos = getExpiresAtNanos(creationTimeNanos, TimeUnit.MILLISECONDS.toNanos(in.readUnsignedVInt()));
-            Verb verb = Verb.fromId(Ints.checkedCast(in.readUnsignedVInt()));
-            int flags = Ints.checkedCast(in.readUnsignedVInt());
-            Map<ParamType, Object> params = deserializeParams(in, version);
-            return new Header(id, verb, peer, creationTimeNanos, expiresAtNanos, flags, params);
-        }
-
-        private void skipHeaderPost40(DataInputPlus in) throws IOException
-        {
-            skipUnsignedVInt(in); // id
-            in.skipBytesFully(4); // createdAt
-            skipUnsignedVInt(in); // expiresIn
-            skipUnsignedVInt(in); // verb
-            skipUnsignedVInt(in); // flags
-            skipParamsPost40(in); // params
-        }
-
-        private int serializedHeaderSizePost40(Header header, int version)
         {
             long size = 0;
-            size += sizeofUnsignedVInt(header.id);
-            size += CREATION_TIME_SIZE;
-            size += sizeofUnsignedVInt(NANOSECONDS.toMillis(header.expiresAtNanos - header.createdAtNanos));
-            size += sizeofUnsignedVInt(header.verb.id);
-            size += sizeofUnsignedVInt(header.flags);
-            size += serializedParamsSize(header.params, version);
-            return Ints.checkedCast(size);
-        }
-
-        private Header extractHeaderPost40(ByteBuffer buf, InetAddressAndPort from, long currentTimeNanos, int version) throws IOException
-        {
-            MonotonicClockTranslation timeSnapshot = approxTime.translate();
-
-            int index = buf.position();
-
-            long id = getUnsignedVInt(buf, index);
-            index += computeUnsignedVIntSize(id);
-
-            int createdAtMillis = buf.getInt(index);
-            index += sizeof(createdAtMillis);
-
-            long expiresInMillis = getUnsignedVInt(buf, index);
-            index += computeUnsignedVIntSize(expiresInMillis);
-
-            Verb verb = Verb.fromId(Ints.checkedCast(getUnsignedVInt(buf, index)));
-            index += computeUnsignedVIntSize(verb.id);
-
-            int flags = Ints.checkedCast(getUnsignedVInt(buf, index));
-            index += computeUnsignedVIntSize(flags);
-
-            Map<ParamType, Object> params = extractParams(buf, index, version);
-
-            long createdAtNanos = calculateCreationTimeNanos(createdAtMillis, timeSnapshot, currentTimeNanos);
-            long expiresAtNanos = getExpiresAtNanos(createdAtNanos, TimeUnit.MILLISECONDS.toNanos(expiresInMillis));
-
-            return new Header(id, verb, from, createdAtNanos, expiresAtNanos, flags, params);
-        }
-
-        private <T> void serializePost40(Message<T> message, DataOutputPlus out, int version) throws IOException
-        {
-            serializeHeaderPost40(message.header, out, version);
-            out.writeUnsignedVInt(message.payloadSize(version));
-            message.verb().serializer().serialize(message.payload, out, version);
-        }
-
-        private <T> Message<T> deserializePost40(DataInputPlus in, InetAddressAndPort peer, int version) throws IOException
-        {
-            Header header = deserializeHeaderPost40(in, peer, version);
-            skipUnsignedVInt(in); // payload size, not needed by payload deserializer
-            T payload = (T) header.verb.serializer().deserialize(in, version);
-            return new Message<>(header, payload);
-        }
-
-        private <T> Message<T> deserializePost40(DataInputPlus in, Header header, int version) throws IOException
-        {
-            skipHeaderPost40(in);
-            skipUnsignedVInt(in); // payload size, not needed by payload deserializer
-            T payload = (T) header.verb.serializer().deserialize(in, version);
-            return new Message<>(header, payload);
-        }
-
-        private <T> int serializedSizePost40(Message<T> message, int version)
-        {
-            long size = 0;
-            size += serializedHeaderSizePost40(message.header, version);
+            size += serializedHeader(message.header, version);
             int payloadSize = message.payloadSize(version);
             size += sizeofUnsignedVInt(payloadSize) + payloadSize;
             return Ints.checkedCast(size);
         }
 
-        private int inferMessageSizePost40(ByteBuffer buf, int readerIndex, int readerLimit)
+        /**
+         * Size of the next message in the stream. Returns -1 if there aren't sufficient bytes read yet to determine size.
+         */
+        int inferMessageSize(ByteBuffer buf, int readerIndex, int readerLimit)
         {
             int index = readerIndex;
 
@@ -931,7 +802,7 @@ public class Message<T>
                 return -1;
             index += flagsSize;
 
-            int paramsSize = extractParamsSizePost40(buf, index, readerLimit);
+            int paramsSize = extractParamsSize(buf, index, readerLimit);
             if (paramsSize < 0)
                 return -1;
             index += paramsSize;
@@ -941,222 +812,102 @@ public class Message<T>
                 return -1;
             index += computeUnsignedVIntSize(payloadSize) + payloadSize;
 
-            return index - readerIndex;
+            int size = index - readerIndex;
+
+            if (size > DatabaseDescriptor.getInternodeMaxMessageSizeInBytes())
+                throw new OversizedMessageException(size);
+            return size;
         }
 
-        /*
-         * legacy ser/deser
+        /**
+         * Partially deserialize the message - by only extracting the header and leaving the payload alone.
+         *
+         * To get the rest of the message without repeating the work done here, use {@link #deserialize(DataInputPlus, Header, int)}
+         * method.
+         *
+         * It's assumed that the provided buffer contains all the bytes necessary to deserialize the header fully.
          */
-
-        private void serializeHeaderPre40(Header header, DataOutputPlus out, int version) throws IOException
-        {
-            out.writeInt(PROTOCOL_MAGIC);
-            out.writeInt(Ints.checkedCast(header.id));
-            // int cast cuts off the high-order half of the timestamp, which we can assume remains
-            // the same between now and when the recipient reconstructs it.
-            out.writeInt((int) approxTime.translate().toMillisSinceEpoch(header.createdAtNanos));
-            inetAddressAndPortSerializer.serialize(header.from, out, version);
-            out.writeInt(header.verb.toPre40Verb().id);
-            serializeParams(addFlagsToLegacyParams(header.params, header.flags), out, version);
-        }
-
-        private Header deserializeHeaderPre40(DataInputPlus in, int version) throws IOException
-        {
-            validateLegacyProtocolMagic(in.readInt());
-            int id = in.readInt();
-            long currentTimeNanos = approxTime.now();
-            MonotonicClockTranslation timeSnapshot = approxTime.translate();
-            long creationTimeNanos = calculateCreationTimeNanos(in.readInt(), timeSnapshot, currentTimeNanos);
-            InetAddressAndPort from = inetAddressAndPortSerializer.deserialize(in, version);
-            Verb verb = Verb.fromId(in.readInt());
-            Map<ParamType, Object> params = deserializeParams(in, version);
-            int flags = removeFlagsFromLegacyParams(params);
-            return new Header(id, verb, from, creationTimeNanos, verb.expiresAtNanos(creationTimeNanos), flags, params);
-        }
-
-        private static final int PRE_40_MESSAGE_PREFIX_SIZE = 12; // protocol magic + id + createdAt
-
-        private void skipHeaderPre40(DataInputPlus in) throws IOException
-        {
-            in.skipBytesFully(PRE_40_MESSAGE_PREFIX_SIZE); // magic, id, createdAt
-            in.skipBytesFully(in.readByte());              // from
-            in.skipBytesFully(4);                          // verb
-            skipParamsPre40(in);                           // params
-        }
-
-        private int serializedHeaderSizePre40(Header header, int version)
-        {
-            long size = 0;
-            size += PRE_40_MESSAGE_PREFIX_SIZE;
-            size += inetAddressAndPortSerializer.serializedSize(header.from, version);
-            size += sizeof(header.verb.id);
-            size += serializedParamsSize(addFlagsToLegacyParams(header.params, header.flags), version);
-            return Ints.checkedCast(size);
-        }
-
-        private Header extractHeaderPre40(ByteBuffer buf, long currentTimeNanos, int version) throws IOException
+        Header extractHeader(ByteBuffer buf, InetAddressAndPort from, long currentTimeNanos, int version) throws IOException
         {
             MonotonicClockTranslation timeSnapshot = approxTime.translate();
 
             int index = buf.position();
 
-            index += 4; // protocol magic
-
-            long id = buf.getInt(index);
-            index += 4;
+            long id = getUnsignedVInt(buf, index);
+            index += computeUnsignedVIntSize(id);
 
             int createdAtMillis = buf.getInt(index);
-            index += 4;
+            index += sizeof(createdAtMillis);
 
-            InetAddressAndPort from = inetAddressAndPortSerializer.extract(buf, index);
-            index += 1 + buf.get(index);
+            long expiresInMillis = getUnsignedVInt(buf, index);
+            index += computeUnsignedVIntSize(expiresInMillis);
 
-            Verb verb = Verb.fromId(buf.getInt(index));
-            index += 4;
+            Verb verb = Verb.fromId(getUnsignedVInt32(buf, index));
+            index += computeUnsignedVIntSize(verb.id);
+
+            int flags = getUnsignedVInt32(buf, index);
+            index += computeUnsignedVIntSize(flags);
 
             Map<ParamType, Object> params = extractParams(buf, index, version);
-            int flags = removeFlagsFromLegacyParams(params);
 
             long createdAtNanos = calculateCreationTimeNanos(createdAtMillis, timeSnapshot, currentTimeNanos);
-            long expiresAtNanos = verb.expiresAtNanos(createdAtNanos);
+            long expiresAtNanos = getExpiresAtNanos(createdAtNanos, TimeUnit.MILLISECONDS.toNanos(expiresInMillis));
 
             return new Header(id, verb, from, createdAtNanos, expiresAtNanos, flags, params);
         }
 
-        private <T> void serializePre40(Message<T> message, DataOutputPlus out, int version) throws IOException
+        private static long getExpiresAtNanos(long createdAtNanos, long expirationPeriodNanos)
         {
-            if (message.isFailureResponse())
-                message = toPre40FailureResponse(message);
-
-            serializeHeaderPre40(message.header, out, version);
-
-            if (message.payload != null && message.payload != NoPayload.noPayload)
-            {
-                int payloadSize = message.payloadSize(version);
-                out.writeInt(payloadSize);
-                message.getPayloadSerializer().serialize(message.payload, out, version);
-            }
-            else
-            {
-                out.writeInt(0);
-            }
+            return createdAtNanos + expirationPeriodNanos;
         }
 
-        private <T> Message<T> deserializePre40(DataInputPlus in, int version) throws IOException
+        private void serializeHeader(Header header, DataOutputPlus out, int version) throws IOException
         {
-            Header header = deserializeHeaderPre40(in, version);
-            return deserializePre40(in, header, false, version);
+            out.writeUnsignedVInt(header.id);
+            // int cast cuts off the high-order half of the timestamp, which we can assume remains
+            // the same between now and when the recipient reconstructs it.
+            out.writeInt((int) approxTime.translate().toMillisSinceEpoch(header.createdAtNanos));
+            out.writeUnsignedVInt(NANOSECONDS.toMillis(header.expiresAtNanos - header.createdAtNanos));
+            out.writeUnsignedVInt32(header.verb.id);
+            out.writeUnsignedVInt32(header.flags);
+            serializeParams(header.params, out, version);
         }
 
-        private <T> Message<T> deserializePre40(DataInputPlus in, Header header, int version) throws IOException
+        private Header deserializeHeader(DataInputPlus in, InetAddressAndPort peer, int version) throws IOException
         {
-            return deserializePre40(in, header, true, version);
+            long id = in.readUnsignedVInt();
+            long currentTimeNanos = approxTime.now();
+            MonotonicClockTranslation timeSnapshot = approxTime.translate();
+            long creationTimeNanos = calculateCreationTimeNanos(in.readInt(), timeSnapshot, currentTimeNanos);
+            long expiresAtNanos = getExpiresAtNanos(creationTimeNanos, TimeUnit.MILLISECONDS.toNanos(in.readUnsignedVInt()));
+            Verb verb = Verb.fromId(in.readUnsignedVInt32());
+            int flags = in.readUnsignedVInt32();
+            Map<ParamType, Object> params = deserializeParams(in, version);
+            return new Header(id, verb, peer, creationTimeNanos, expiresAtNanos, flags, params);
         }
 
-        private <T> Message<T> deserializePre40(DataInputPlus in, Header header, boolean skipHeader, int version) throws IOException
+        private void skipHeader(DataInputPlus in) throws IOException
         {
-            if (skipHeader)
-                skipHeaderPre40(in);
-
-            int payloadSize = in.readInt();
-            T payload = deserializePayloadPre40(in, version, getPayloadSerializer(header.verb, header.id, header.from), payloadSize);
-
-            Message<T> message = new Message<>(header, payload);
-
-            return header.params.containsKey(ParamType.FAILURE_RESPONSE)
-                 ? (Message<T>) toPost40FailureResponse(message)
-                 : message;
+            skipUnsignedVInt(in); // id
+            in.skipBytesFully(4); // createdAt
+            skipUnsignedVInt(in); // expiresIn
+            skipUnsignedVInt(in); // verb
+            skipUnsignedVInt(in); // flags
+            skipParams(in); // params
         }
 
-        private <T> T deserializePayloadPre40(DataInputPlus in, int version, IVersionedAsymmetricSerializer<?, T> serializer, int payloadSize) throws IOException
+        private int serializedHeader(Header header, int version)
         {
-            if (payloadSize == 0 || serializer == null)
-            {
-                // if there's no deserializer for the verb, skip the payload bytes to leave
-                // the stream in a clean state (for the next message)
-                in.skipBytesFully(payloadSize);
-                return null;
-            }
-
-            return serializer.deserialize(in, version);
-        }
-
-        private <T> int serializedSizePre40(Message<T> message, int version)
-        {
-            if (message.isFailureResponse())
-                message = toPre40FailureResponse(message);
-
             long size = 0;
-            size += serializedHeaderSizePre40(message.header, version);
-            int payloadSize = message.payloadSize(version);
-            size += sizeof(payloadSize);
-            size += payloadSize;
+            size += sizeofUnsignedVInt(header.id);
+            size += CREATION_TIME_SIZE;
+            size += sizeofUnsignedVInt(NANOSECONDS.toMillis(header.expiresAtNanos - header.createdAtNanos));
+            size += sizeofUnsignedVInt(header.verb.id);
+            size += sizeofUnsignedVInt(header.flags);
+            size += serializedParamsSize(header.params, version);
             return Ints.checkedCast(size);
         }
 
-        private int inferMessageSizePre40(ByteBuffer buf, int readerIndex, int readerLimit) throws InvalidLegacyProtocolMagic
-        {
-            int index = readerIndex;
-            // protocol magic
-            index += 4;
-            if (index > readerLimit)
-                return -1;
-            validateLegacyProtocolMagic(buf.getInt(index - 4));
-
-            // rest of prefix
-            index += PRE_40_MESSAGE_PREFIX_SIZE - 4;
-            // ip address
-            index += 1;
-            if (index > readerLimit)
-                return -1;
-            index += buf.get(index - 1);
-            // verb
-            index += 4;
-            if (index > readerLimit)
-                return -1;
-
-            int paramsSize = extractParamsSizePre40(buf, index, readerLimit);
-            if (paramsSize < 0)
-                return -1;
-            index += paramsSize;
-
-            // payload
-            index += 4;
-
-            if (index > readerLimit)
-                return -1;
-            index += buf.getInt(index - 4);
-
-            return index - readerIndex;
-        }
-
-        private Message toPre40FailureResponse(Message post40)
-        {
-            Map<ParamType, Object> params = new EnumMap<>(ParamType.class);
-            params.putAll(post40.header.params);
-
-            params.put(ParamType.FAILURE_RESPONSE, LegacyFlag.instance);
-            params.put(ParamType.FAILURE_REASON, post40.payload);
-
-            Header header = new Header(post40.id(), post40.verb().toPre40Verb(), post40.from(), post40.createdAtNanos(), post40.expiresAtNanos(), 0, params);
-            return new Message<>(header, NoPayload.noPayload);
-        }
-
-        private Message<RequestFailureReason> toPost40FailureResponse(Message<?> pre40)
-        {
-            Map<ParamType, Object> params = new EnumMap<>(ParamType.class);
-            params.putAll(pre40.header.params);
-
-            params.remove(ParamType.FAILURE_RESPONSE);
-
-            RequestFailureReason reason = (RequestFailureReason) params.remove(ParamType.FAILURE_REASON);
-            if (null == reason)
-                reason = RequestFailureReason.UNKNOWN;
-
-            Header header = new Header(pre40.id(), Verb.FAILURE_RSP, pre40.from(), pre40.createdAtNanos(), pre40.expiresAtNanos(), pre40.header.flags, params);
-            return new Message<>(header, reason);
-        }
-
         /*
          * created at + cross-node
          */
@@ -1209,59 +960,20 @@ public class Message<T>
          * param ser/deser
          */
 
-        private Map<ParamType, Object> addFlagsToLegacyParams(Map<ParamType, Object> params, int flags)
-        {
-            if (flags == 0)
-                return params;
-
-            Map<ParamType, Object> extended = new EnumMap<>(ParamType.class);
-            extended.putAll(params);
-
-            if (MessageFlag.CALL_BACK_ON_FAILURE.isIn(flags))
-                extended.put(ParamType.FAILURE_CALLBACK, LegacyFlag.instance);
-
-            if (MessageFlag.TRACK_REPAIRED_DATA.isIn(flags))
-                extended.put(ParamType.TRACK_REPAIRED_DATA, LegacyFlag.instance);
-
-            return extended;
-        }
-
-        private int removeFlagsFromLegacyParams(Map<ParamType, Object> params)
-        {
-            int flags = 0;
-
-            if (null != params.remove(ParamType.FAILURE_CALLBACK))
-                flags = MessageFlag.CALL_BACK_ON_FAILURE.addTo(flags);
-
-            if (null != params.remove(ParamType.TRACK_REPAIRED_DATA))
-                flags = MessageFlag.TRACK_REPAIRED_DATA.addTo(flags);
-
-            return flags;
-        }
-
         private void serializeParams(Map<ParamType, Object> params, DataOutputPlus out, int version) throws IOException
         {
-            if (version >= VERSION_40)
-                out.writeUnsignedVInt(params.size());
-            else
-                out.writeInt(params.size());
+            out.writeUnsignedVInt32(params.size());
 
             for (Map.Entry<ParamType, Object> kv : params.entrySet())
             {
                 ParamType type = kv.getKey();
-                if (version >= VERSION_40)
-                    out.writeUnsignedVInt(type.id);
-                else
-                    out.writeUTF(type.legacyAlias);
+                out.writeUnsignedVInt32(type.id);
 
                 IVersionedSerializer serializer = type.serializer;
                 Object value = kv.getValue();
 
                 int length = Ints.checkedCast(serializer.serializedSize(value, version));
-                if (version >= VERSION_40)
-                    out.writeUnsignedVInt(length);
-                else
-                    out.writeInt(length);
+                out.writeUnsignedVInt32(length);
 
                 serializer.serialize(value, out, version);
             }
@@ -1269,7 +981,7 @@ public class Message<T>
 
         private Map<ParamType, Object> deserializeParams(DataInputPlus in, int version) throws IOException
         {
-            int count = version >= VERSION_40 ? Ints.checkedCast(in.readUnsignedVInt()) : in.readInt();
+            int count = in.readUnsignedVInt32();
 
             if (count == 0)
                 return NO_PARAMS;
@@ -1278,25 +990,13 @@ public class Message<T>
 
             for (int i = 0; i < count; i++)
             {
-                ParamType type = version >= VERSION_40
-                    ? ParamType.lookUpById(Ints.checkedCast(in.readUnsignedVInt()))
-                    : ParamType.lookUpByAlias(in.readUTF());
+                ParamType type = ParamType.lookUpById(in.readUnsignedVInt32());
 
-                int length = version >= VERSION_40
-                    ? Ints.checkedCast(in.readUnsignedVInt())
-                    : in.readInt();
+                int length = in.readUnsignedVInt32();
 
                 if (null != type)
                 {
-                    // Have to special case deserializer as pre-4.0 needs length to decode correctly
-                    if (version < VERSION_40 && type == ParamType.RESPOND_TO)
-                    {
-                        params.put(type, InetAddressAndPort.FwdFrmSerializer.fwdFrmSerializer.pre40DeserializeWithLength(in, version, length));
-                    }
-                    else
-                    {
-                        params.put(type, type.serializer.deserialize(in, version));
-                    }
+                    params.put(type, type.serializer.deserialize(in, version));
                 }
                 else
                 {
@@ -1307,12 +1007,9 @@ public class Message<T>
             return params;
         }
 
-        /*
-         * Extract post-4.0 params map from a ByteBuffer without modifying it.
-         */
         private Map<ParamType, Object> extractParams(ByteBuffer buf, int readerIndex, int version) throws IOException
         {
-            long count = version >= VERSION_40 ? getUnsignedVInt(buf, readerIndex) : buf.getInt(readerIndex);
+            long count = getUnsignedVInt(buf, readerIndex);
 
             if (count == 0)
                 return NO_PARAMS;
@@ -1330,33 +1027,20 @@ public class Message<T>
             }
         }
 
-        private void skipParamsPost40(DataInputPlus in) throws IOException
+        private void skipParams(DataInputPlus in) throws IOException
         {
-            int count = Ints.checkedCast(in.readUnsignedVInt());
+            int count = in.readUnsignedVInt32();
 
             for (int i = 0; i < count; i++)
             {
                 skipUnsignedVInt(in);
-                in.skipBytesFully(Ints.checkedCast(in.readUnsignedVInt()));
-            }
-        }
-
-        private void skipParamsPre40(DataInputPlus in) throws IOException
-        {
-            int count = in.readInt();
-
-            for (int i = 0; i < count; i++)
-            {
-                in.skipBytesFully(in.readShort());
-                in.skipBytesFully(in.readInt());
+                in.skipBytesFully(in.readUnsignedVInt32());
             }
         }
 
         private long serializedParamsSize(Map<ParamType, Object> params, int version)
         {
-            long size = version >= VERSION_40
-                      ? computeUnsignedVIntSize(params.size())
-                      : sizeof(params.size());
+            long size = computeUnsignedVIntSize(params.size());
 
             for (Map.Entry<ParamType, Object> kv : params.entrySet())
             {
@@ -1365,10 +1049,7 @@ public class Message<T>
 
                 long valueLength = type.serializer.serializedSize(value, version);
 
-                if (version >= VERSION_40)
-                    size += sizeofUnsignedVInt(type.id) + sizeofUnsignedVInt(valueLength);
-                else
-                    size += sizeof(type.legacyAlias) + 4;
+                size += sizeofUnsignedVInt(type.id) + sizeofUnsignedVInt(valueLength);
 
                 size += valueLength;
             }
@@ -1376,7 +1057,7 @@ public class Message<T>
             return size;
         }
 
-        private int extractParamsSizePost40(ByteBuffer buf, int readerIndex, int readerLimit)
+        private int extractParamsSize(ByteBuffer buf, int readerIndex, int readerLimit)
         {
             int index = readerIndex;
 
@@ -1401,33 +1082,6 @@ public class Message<T>
             return index - readerIndex;
         }
 
-        private int extractParamsSizePre40(ByteBuffer buf, int readerIndex, int readerLimit)
-        {
-            int index = readerIndex;
-
-            index += 4;
-            if (index > readerLimit)
-                return -1;
-            int paramsCount = buf.getInt(index - 4);
-
-            for (int i = 0; i < paramsCount; i++)
-            {
-                // try to read length and skip to the end of the param name
-                index += 2;
-
-                if (index > readerLimit)
-                    return -1;
-                index += buf.getShort(index - 2);
-                // try to read length and skip to the end of the param value
-                index += 4;
-                if (index > readerLimit)
-                    return -1;
-                index += buf.getInt(index - 4);
-            }
-
-            return index - readerIndex;
-        }
-
         private <T> int payloadSize(Message<T> message, int version)
         {
             long payloadSize = message.payload != null && message.payload != NoPayload.noPayload
@@ -1439,21 +1093,11 @@ public class Message<T>
 
     private IVersionedAsymmetricSerializer<T, ?> getPayloadSerializer()
     {
-        return getPayloadSerializer(verb(), id(), from());
+        return verb().serializer();
     }
 
-    // Verb#serializer() is null for legacy response messages. Once all Verbs with null handlers
-    // are removed in a future major, this method can be replaced with a call to verb.serializer.
-    private static <In,Out> IVersionedAsymmetricSerializer<In, Out> getPayloadSerializer(Verb verb, long id, InetAddressAndPort from)
-    {
-        return null != verb.serializer()
-             ? verb.serializer()
-             : instance().callbacks.responseSerializer(id, from);
-    }
-
-    private int serializedSize30;
-    private int serializedSize3014;
     private int serializedSize40;
+    private int serializedSize50;
 
     /**
      * Serialized size of the entire message, for the provided messaging version. Caches the calculated value.
@@ -1462,45 +1106,37 @@ public class Message<T>
     {
         switch (version)
         {
-            case VERSION_30:
-                if (serializedSize30 == 0)
-                    serializedSize30 = serializer.serializedSize(this, VERSION_30);
-                return serializedSize30;
-            case VERSION_3014:
-                if (serializedSize3014 == 0)
-                    serializedSize3014 = serializer.serializedSize(this, VERSION_3014);
-                return serializedSize3014;
             case VERSION_40:
                 if (serializedSize40 == 0)
                     serializedSize40 = serializer.serializedSize(this, VERSION_40);
                 return serializedSize40;
+            case VERSION_50:
+                if (serializedSize50 == 0)
+                    serializedSize50 = serializer.serializedSize(this, VERSION_50);
+                return serializedSize50;
             default:
-                throw new IllegalStateException();
+                throw new IllegalStateException("Unkown serialization version " + version);
         }
     }
 
-    private int payloadSize30   = -1;
-    private int payloadSize3014 = -1;
     private int payloadSize40   = -1;
+    private int payloadSize50   = -1;
 
     private int payloadSize(int version)
     {
         switch (version)
         {
-            case VERSION_30:
-                if (payloadSize30 < 0)
-                    payloadSize30 = serializer.payloadSize(this, VERSION_30);
-                return payloadSize30;
-            case VERSION_3014:
-                if (payloadSize3014 < 0)
-                    payloadSize3014 = serializer.payloadSize(this, VERSION_3014);
-                return payloadSize3014;
             case VERSION_40:
                 if (payloadSize40 < 0)
                     payloadSize40 = serializer.payloadSize(this, VERSION_40);
                 return payloadSize40;
+            case VERSION_50:
+                if (payloadSize50 < 0)
+                    payloadSize50 = serializer.payloadSize(this, VERSION_50);
+                return payloadSize50;
+
             default:
-                throw new IllegalStateException();
+                throw new IllegalStateException("Unkown serialization version " + version);
         }
     }
 
