@@ -20,15 +20,17 @@ package org.apache.cassandra.net;
 import java.io.IOException;
 import java.nio.channels.ClosedChannelException;
 import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collections;
 import java.util.List;
+import java.util.Map;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.TimeoutException;
+import java.util.stream.Collectors;
 
 import com.google.common.annotations.VisibleForTesting;
-
-import org.apache.cassandra.utils.concurrent.AsyncPromise;
-import org.apache.cassandra.utils.concurrent.FutureCombiner;
+import com.google.common.collect.Lists;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -44,6 +46,8 @@ import org.apache.cassandra.metrics.MessagingMetrics;
 import org.apache.cassandra.service.AbstractWriteResponseHandler;
 import org.apache.cassandra.utils.ExecutorUtils;
 import org.apache.cassandra.utils.FBUtilities;
+import org.apache.cassandra.utils.concurrent.AsyncPromise;
+import org.apache.cassandra.utils.concurrent.FutureCombiner;
 
 import static java.util.Collections.synchronizedList;
 import static java.util.concurrent.TimeUnit.MINUTES;
@@ -200,26 +204,22 @@ import static org.apache.cassandra.utils.Throwables.maybeFail;
  * implemented in {@link org.apache.cassandra.db.virtual.InternodeInboundTable} and
  * {@link org.apache.cassandra.db.virtual.InternodeOutboundTable} respectively.
  */
-public class MessagingService extends MessagingServiceMBeanImpl
+public class MessagingService extends MessagingServiceMBeanImpl implements MessageDelivery
 {
     private static final Logger logger = LoggerFactory.getLogger(MessagingService.class);
 
     // 8 bits version, so don't waste versions
-    public static final int VERSION_30 = 10;
-    public static final int VERSION_3014 = 11;
-    public static final int VERSION_40 = 12;
-    public static final int VERSION_41 = 13;
-    public static final int minimum_version = VERSION_30;
-    public static final int current_version = VERSION_40;
-    static AcceptVersions accept_messaging = new AcceptVersions(minimum_version, current_version);
-    static AcceptVersions accept_streaming = new AcceptVersions(current_version, current_version);
-    public static String SSL_FACTORY_CONTEXT_DESCRIPTION = "server_encryption_options";
-
     public enum Version
     {
+        /** @deprecated See CASSANDRA-18314 */
+        @Deprecated(since = "5.0")
         VERSION_30(10),
+        /** @deprecated See CASSANDRA-18314 */
+        @Deprecated(since = "5.0")
         VERSION_3014(11),
-        VERSION_40(12);
+        VERSION_40(12),
+        // c14227 TTL overflow, 'uint' timestamps
+        VERSION_50(13);
 
         public final int value;
 
@@ -227,6 +227,65 @@ public class MessagingService extends MessagingServiceMBeanImpl
         {
             this.value = value;
         }
+
+        public static List<Version> supportedVersions()
+        {
+            List<Version> versions = Lists.newArrayList();
+            for (Version version : values())
+                if (minimum_version <= version.value)
+                    versions.add(version);
+
+            return Collections.unmodifiableList(versions);
+        }
+    }
+    // Maintance Note:
+    // Try to keep Version enum in-sync for testing.  By having the versions in the enum tests can get access without forcing this class
+    // to load, which adds a lot of costs to each test
+    /** @deprecated See CASSANDRA-18816 */
+    @Deprecated(since = "5.0")
+    public static final int VERSION_30 = 10;
+    /** @deprecated See CASSANDRA-18816 */
+    @Deprecated(since = "5.0")
+    public static final int VERSION_3014 = 11;
+    public static final int VERSION_40 = 12;
+    public static final int VERSION_50 = 13; // c14227 TTL overflow, 'uint' timestamps
+    public static final int minimum_version = VERSION_40;
+    public static final int maximum_version = VERSION_50;
+    // we want to use a modified behavior for the tools and clients - that is, since they are not running a server, they
+    // should not need to run in a compatibility mode. They should be able to connect to the server regardless whether
+    // it uses messaving version 4 or 5
+    public static final int current_version = DatabaseDescriptor.getStorageCompatibilityMode().isBefore(5) ? VERSION_40 : VERSION_50;
+    static AcceptVersions accept_messaging;
+    static AcceptVersions accept_streaming;
+    static
+    {
+        if (DatabaseDescriptor.isClientInitialized())
+        {
+            accept_messaging = new AcceptVersions(minimum_version, maximum_version);
+            accept_streaming = new AcceptVersions(minimum_version, maximum_version);
+        }
+        else
+        {
+            accept_messaging = new AcceptVersions(minimum_version, current_version);
+            accept_streaming = new AcceptVersions(current_version, current_version);
+        }
+    }
+    static Map<Integer, Integer> versionOrdinalMap = Arrays.stream(Version.values()).collect(Collectors.toMap(v -> v.value, v -> v.ordinal()));
+
+    /**
+     * This is an optimisation to speed up the translation of the serialization
+     * version to the {@link Version} enum ordinal.
+     *
+     * @param version the serialization version
+     * @return a {@link Version} ordinal value
+     */
+    public static int getVersionOrdinal(int version)
+    {
+        Integer ordinal = versionOrdinalMap.get(version);
+        if (ordinal == null)
+            throw new IllegalStateException("Unkown serialization version: " + version);
+
+        return ordinal;
     }
 
     private static class MSHandle
@@ -276,13 +335,14 @@ public class MessagingService extends MessagingServiceMBeanImpl
         OutboundConnections.scheduleUnusedConnectionMonitoring(this, ScheduledExecutors.scheduledTasks, 1L, TimeUnit.HOURS);
     }
 
-    public <T> org.apache.cassandra.utils.concurrent.Future<Message<T>> sendWithResult(Message message, InetAddressAndPort to)
+    @Override
+    public <REQ, RSP> org.apache.cassandra.utils.concurrent.Future<Message<RSP>> sendWithResult(Message<REQ> message, InetAddressAndPort to)
     {
-        AsyncPromise<Message<T>> promise = new AsyncPromise<>();
-        MessagingService.instance().sendWithCallback(message, to, new RequestCallback<T>()
+        AsyncPromise<Message<RSP>> promise = new AsyncPromise<>();
+        sendWithCallback(message, to, new RequestCallback<RSP>()
         {
             @Override
-            public void onResponse(Message<T> msg)
+            public void onResponse(Message<RSP> msg)
             {
                 promise.trySuccess(msg);
             }
@@ -389,11 +449,6 @@ public class MessagingService extends MessagingServiceMBeanImpl
         send(message.responseWith(response), message.respondTo());
     }
 
-    public <V> void respondWithFailure(RequestFailureReason reason, Message<?> message)
-    {
-        send(Message.failureResponse(message.id(), message.expiresAtNanos(), reason), message.respondTo());
-    }
-
     public void send(Message message, InetAddressAndPort to, ConnectionType specifyConnection)
     {
         if (logger.isTraceEnabled())
@@ -477,7 +532,10 @@ public class MessagingService extends MessagingServiceMBeanImpl
     {
         OutboundConnections pool = channelManagers.get(to);
         if (pool != null)
+        {
             pool.interrupt();
+            logger.info("Interrupted outbound connections to {}", to);
+        }
     }
 
     /**
