@@ -17,6 +17,8 @@
  */
 package org.apache.cassandra.net;
 
+import java.util.Map;
+
 import org.apache.cassandra.exceptions.RequestFailureReason;
 import org.apache.cassandra.locator.InetAddressAndPort;
 
@@ -63,4 +65,19 @@ public interface RequestCallback<T>
         return false;
     }
 
+    static boolean isTimeout(Map<InetAddressAndPort, RequestFailureReason> failureReasonByEndpoint)
+    {
+        // The reason that all must be timeout to be called a timeout is as follows
+        // Assume RF=6, QUORUM, and failureReasonByEndpoint.size() == 3
+        // R1 -> TIMEOUT
+        // R2 -> TIMEOUT
+        // R3 -> READ_TOO_MANY_TOMBSTONES
+        // Since we got a reply back, and that was a failure, we should return a failure letting the user know.
+        // When all failures are a timeout, then this is a race condition with
+        // org.apache.cassandra.utils.concurrent.Awaitable.await(long, java.util.concurrent.TimeUnit)
+        // The race is that the message expire path runs and expires all messages, this then casues the condition
+        // to signal telling the caller "got all replies!".
+        return failureReasonByEndpoint.values().stream().allMatch(RequestFailureReason.TIMEOUT::equals);
+    }
+
 }
