@@ -27,20 +27,21 @@ import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ConcurrentMap;
-import java.util.concurrent.TimeUnit;
+import java.util.concurrent.Executor;
+import java.util.concurrent.RejectedExecutionException;
 import java.util.concurrent.atomic.AtomicBoolean;
 
+import javax.annotation.Nullable;
+
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.collect.Lists;
 import com.google.common.util.concurrent.*;
-import org.apache.cassandra.concurrent.ExecutorFactory;
-import org.apache.cassandra.concurrent.ExecutorPlus;
-import org.apache.cassandra.utils.TimeUUID;
-import org.apache.cassandra.repair.state.SessionState;
-import org.apache.cassandra.utils.concurrent.AsyncFuture;
+
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import org.apache.cassandra.concurrent.ExecutorPlus;
+import org.apache.cassandra.concurrent.Stage;
 import org.apache.cassandra.config.DatabaseDescriptor;
 import org.apache.cassandra.db.ColumnFamilyStore;
 import org.apache.cassandra.db.Keyspace;
@@ -49,19 +50,23 @@ import org.apache.cassandra.dht.Token;
 import org.apache.cassandra.exceptions.RepairException;
 import org.apache.cassandra.gms.*;
 import org.apache.cassandra.locator.InetAddressAndPort;
+import org.apache.cassandra.net.Message;
 import org.apache.cassandra.repair.consistent.ConsistentSession;
 import org.apache.cassandra.repair.consistent.LocalSession;
 import org.apache.cassandra.repair.consistent.LocalSessions;
+import org.apache.cassandra.repair.messages.SyncResponse;
+import org.apache.cassandra.repair.messages.ValidationResponse;
+import org.apache.cassandra.repair.state.SessionState;
 import org.apache.cassandra.schema.SystemDistributedKeyspace;
 import org.apache.cassandra.schema.TableId;
 import org.apache.cassandra.streaming.PreviewKind;
-import org.apache.cassandra.streaming.SessionSummary;
 import org.apache.cassandra.tracing.Tracing;
 import org.apache.cassandra.utils.FBUtilities;
 import org.apache.cassandra.utils.MerkleTrees;
 import org.apache.cassandra.utils.Pair;
 import org.apache.cassandra.utils.Throwables;
-import org.apache.cassandra.utils.concurrent.Future;
+import org.apache.cassandra.utils.TimeUUID;
+import org.apache.cassandra.utils.concurrent.AsyncFuture;
 
 /**
  * Coordinates the (active) repair of a list of non overlapping token ranges.
@@ -125,9 +130,11 @@ public class RepairSession extends AsyncFuture<RepairSessionResult> implements I
     private final ConcurrentMap<Pair<RepairJobDesc, SyncNodePair>, CompletableRemoteSyncTask> syncingTasks = new ConcurrentHashMap<>();
 
     // Tasks(snapshot, validate request, differencing, ...) are run on taskExecutor
-    public final ExecutorPlus taskExecutor;
+    public final SafeExecutor taskExecutor;
     public final boolean optimiseStreams;
+    public final SharedContext ctx;
     public final Scheduler validationScheduler;
+    private volatile List<RepairJob> jobs = Collections.emptyList();
 
     private volatile boolean terminated = false;
 
@@ -142,8 +149,9 @@ public class RepairSession extends AsyncFuture<RepairSessionResult> implements I
      * @param paxosOnly true if we should only complete paxos operations, not run a normal repair
      * @param cfnames names of columnfamilies
      */
-    public RepairSession(TimeUUID parentRepairSession,
+    public RepairSession(SharedContext ctx,
                          Scheduler validationScheduler,
+                         TimeUUID parentRepairSession,
                          CommonRange commonRange,
                          String keyspace,
                          RepairParallelism parallelismDegree,
@@ -155,22 +163,24 @@ public class RepairSession extends AsyncFuture<RepairSessionResult> implements I
                          boolean paxosOnly,
                          String... cfnames)
     {
+        this.ctx = ctx;
         this.validationScheduler = validationScheduler;
         this.repairPaxos = repairPaxos;
         this.paxosOnly = paxosOnly;
         assert cfnames.length > 0 : "Repairing no column families seems pointless, doesn't it";
-        this.state = new SessionState(parentRepairSession, keyspace, cfnames, commonRange);
+        this.state = new SessionState(ctx.clock(), parentRepairSession, keyspace, cfnames, commonRange);
         this.parallelismDegree = parallelismDegree;
         this.isIncremental = isIncremental;
         this.previewKind = previewKind;
         this.pullRepair = pullRepair;
         this.optimiseStreams = optimiseStreams;
-        this.taskExecutor = createExecutor();
+        this.taskExecutor = new SafeExecutor(createExecutor(ctx));
     }
 
-    protected ExecutorPlus createExecutor()
+    @VisibleForTesting
+    protected ExecutorPlus createExecutor(SharedContext ctx)
     {
-        return ExecutorFactory.Global.executorFactory().pooled("RepairJobTask", Integer.MAX_VALUE);
+        return ctx.executorFactory().pooled("RepairJobTask", Integer.MAX_VALUE);
     }
 
     public TimeUUID getId()
@@ -188,13 +198,20 @@ public class RepairSession extends AsyncFuture<RepairSessionResult> implements I
         return state.commonRange.endpoints;
     }
 
-    public void trackValidationCompletion(Pair<RepairJobDesc, InetAddressAndPort> key, ValidationTask task)
+    public synchronized void trackValidationCompletion(Pair<RepairJobDesc, InetAddressAndPort> key, ValidationTask task)
     {
+        if (terminated)
+        {
+            task.abort(new RuntimeException("Session terminated"));
+            return;
+        }
         validating.put(key, task);
     }
 
-    public void trackSyncCompletion(Pair<RepairJobDesc, SyncNodePair> key, CompletableRemoteSyncTask task)
+    public synchronized void trackSyncCompletion(Pair<RepairJobDesc, SyncNodePair> key, CompletableRemoteSyncTask task)
     {
+        if (terminated)
+            return;
         syncingTasks.put(key, task);
     }
 
@@ -202,26 +219,28 @@ public class RepairSession extends AsyncFuture<RepairSessionResult> implements I
      * Receive merkle tree response or failed response from {@code endpoint} for current repair job.
      *
      * @param desc repair job description
-     * @param endpoint endpoint that sent merkle tree
-     * @param trees calculated merkle trees, or null if validation failed
+     * @param message containing the merkle trees or an error
      */
-    public void validationComplete(RepairJobDesc desc, InetAddressAndPort endpoint, MerkleTrees trees)
+    public void validationComplete(RepairJobDesc desc, Message<ValidationResponse> message)
     {
+        InetAddressAndPort endpoint = message.from();
+        MerkleTrees trees = message.payload.trees;
         ValidationTask task = validating.remove(Pair.create(desc, endpoint));
+        // replies without a callback get dropped, so if in mixed mode this should be ignored
+        ctx.messaging().send(message.emptyResponse(), message.from());
         if (task == null)
         {
-            assert terminated : "The repair session should be terminated if the validation we're completing no longer exists.";
-            
             // The trees may be off-heap, and will therefore need to be released.
             if (trees != null)
                 trees.release();
-            
+
+            // either the session completed so the validation is no longer needed, or this is a retry; in both cases there is nothing to do
             return;
         }
 
-        String message = String.format("Received merkle tree for %s from %s", desc.columnFamily, endpoint);
-        logger.info("{} {}", previewKind.logPrefix(getId()), message);
-        Tracing.traceRepair(message);
+        String msg = String.format("Received merkle tree for %s from %s", desc.columnFamily, endpoint);
+        logger.info("{} {}", previewKind.logPrefix(getId()), msg);
+        Tracing.traceRepair(msg);
         task.treesReceived(trees);
     }
 
@@ -229,21 +248,20 @@ public class RepairSession extends AsyncFuture<RepairSessionResult> implements I
      * Notify this session that sync completed/failed with given {@code SyncNodePair}.
      *
      * @param desc synced repair job
-     * @param nodes nodes that completed sync
-     * @param success true if sync succeeded
+     * @param message nodes that completed sync and if they were successful
      */
-    public void syncComplete(RepairJobDesc desc, SyncNodePair nodes, boolean success, List<SessionSummary> summaries)
+    public void syncComplete(RepairJobDesc desc, Message<SyncResponse> message)
     {
+        SyncNodePair nodes = message.payload.nodes;
         CompletableRemoteSyncTask task = syncingTasks.remove(Pair.create(desc, nodes));
+        // replies without a callback get dropped, so if in mixed mode this should be ignored
+        ctx.messaging().send(message.emptyResponse(), message.from());
         if (task == null)
-        {
-            assert terminated : "The repair session should be terminated if the sync task we're completing no longer exists.";
             return;
-        }
 
         if (logger.isDebugEnabled())
             logger.debug("{} Repair completed between {} and {} on {}", previewKind.logPrefix(getId()), nodes.coordinator, nodes.peer, desc.columnFamily);
-        task.syncComplete(success, summaries);
+        task.syncComplete(message.payload.success, message.payload.summaries);
     }
 
     @VisibleForTesting
@@ -300,7 +318,7 @@ public class RepairSession extends AsyncFuture<RepairSessionResult> implements I
         // Checking all nodes are live
         for (InetAddressAndPort endpoint : state.commonRange.endpoints)
         {
-            if (!FailureDetector.instance.isAlive(endpoint) && !state.commonRange.hasSkippedReplicas)
+            if (!ctx.failureDetector().isAlive(endpoint) && !state.commonRange.hasSkippedReplicas)
             {
                 message = String.format("Cannot proceed on repair because a neighbor (%s) is dead: session failed", endpoint);
                 state.phase.fail(message);
@@ -317,7 +335,7 @@ public class RepairSession extends AsyncFuture<RepairSessionResult> implements I
 
         // Create and submit RepairJob for each ColumnFamily
         state.phase.jobsSubmitted();
-        List<Future<RepairResult>> jobs = new ArrayList<>(state.cfnames.length);
+        List<RepairJob> jobs = new ArrayList<>(state.cfnames.length);
         for (String cfname : state.cfnames)
         {
             RepairJob job = new RepairJob(this, cfname);
@@ -325,6 +343,7 @@ public class RepairSession extends AsyncFuture<RepairSessionResult> implements I
             executor.execute(job);
             jobs.add(job);
         }
+        this.jobs = jobs;
 
         // When all RepairJobs are done without error, cleanup and set the final result
         FBUtilities.allOf(jobs).addCallback(new FutureCallback<List<RepairResult>>()
@@ -337,10 +356,9 @@ public class RepairSession extends AsyncFuture<RepairSessionResult> implements I
                 Tracing.traceRepair("Completed sync of range {}", state.commonRange);
                 trySuccess(new RepairSessionResult(state.id, state.keyspace, state.commonRange.ranges, results, state.commonRange.hasSkippedReplicas));
 
-                taskExecutor.shutdown();
                 // mark this session as terminated
-                terminate();
-                awaitTaskExecutorTermination();
+                terminate(null);
+                taskExecutor.shutdown();
             }
 
             public void onFailure(Throwable t)
@@ -354,12 +372,19 @@ public class RepairSession extends AsyncFuture<RepairSessionResult> implements I
                 Tracing.traceRepair("Session completed with the following error: {}", t);
                 forceShutdown(t);
             }
-        }, executor);
+        }, taskExecutor);
     }
 
-    public void terminate()
+    public synchronized void terminate(@Nullable Throwable reason)
     {
         terminated = true;
+        List<RepairJob> jobs = this.jobs;
+        if (jobs != null)
+        {
+            for (RepairJob job : jobs)
+                job.abort(reason);
+        }
+        this.jobs = null;
         validating.clear();
         syncingTasks.clear();
     }
@@ -372,26 +397,9 @@ public class RepairSession extends AsyncFuture<RepairSessionResult> implements I
     public void forceShutdown(Throwable reason)
     {
         tryFailure(reason);
+        terminate(reason);
         taskExecutor.shutdown();
-        terminate();
-        awaitTaskExecutorTermination();
     }
-
-    private void awaitTaskExecutorTermination()
-    {
-        try
-        {
-            if (taskExecutor.awaitTermination(30, TimeUnit.SECONDS))
-                logger.debug("{} session task executor shut down gracefully", previewKind.logPrefix(getId()));
-            else
-                logger.warn("{} session task executor unable to shut down gracefully", previewKind.logPrefix(getId()));
-        }
-        catch (InterruptedException e)
-        {
-            Thread.currentThread().interrupt();
-        }
-    }
-
     public void onRemove(InetAddressAndPort endpoint)
     {
         convict(endpoint, Double.MAX_VALUE);
@@ -455,4 +463,33 @@ public class RepairSession extends AsyncFuture<RepairSessionResult> implements I
         }
         return false;
     }
+
+    private static class SafeExecutor implements Executor
+    {
+        private final ExecutorPlus delegate;
+
+        private SafeExecutor(ExecutorPlus delegate)
+        {
+            this.delegate = delegate;
+        }
+
+        @Override
+        public void execute(Runnable command)
+        {
+            try
+            {
+                delegate.execute(command);
+            }
+            catch (RejectedExecutionException e)
+            {
+                // task executor was shutdown, so fall back to a known good executor to finish callbacks
+                Stage.INTERNAL_RESPONSE.execute(command);
+            }
+        }
+
+        public void shutdown()
+        {
+            delegate.shutdown();
+        }
+    }
 }
