@@ -40,6 +40,12 @@ public class CleanupMessage extends RepairMessage
         this.parentRepairSession = parentRepairSession;
     }
 
+    @Override
+    public TimeUUID parentRepairSession()
+    {
+        return parentRepairSession;
+    }
+
     @Override
     public boolean equals(Object o)
     {
