@@ -20,17 +20,16 @@ package org.apache.cassandra.repair.messages;
 import java.io.IOException;
 import java.util.Objects;
 
+import org.apache.cassandra.io.IVersionedSerializer;
 import org.apache.cassandra.io.util.DataInputPlus;
 import org.apache.cassandra.io.util.DataOutputPlus;
 import org.apache.cassandra.repair.RepairJobDesc;
 
 public class SnapshotMessage extends RepairMessage
 {
-    public final static MessageSerializer serializer = new SnapshotMessageSerializer();
-
     public SnapshotMessage(RepairJobDesc desc)
     {
-        super(Type.SNAPSHOT, desc);
+        super(desc);
     }
 
     @Override
@@ -39,16 +38,16 @@ public class SnapshotMessage extends RepairMessage
         if (!(o instanceof SnapshotMessage))
             return false;
         SnapshotMessage other = (SnapshotMessage) o;
-        return messageType == other.messageType;
+        return desc.equals(other.desc);
     }
 
     @Override
     public int hashCode()
     {
-        return Objects.hash(messageType);
+        return Objects.hash(desc);
     }
 
-    public static class SnapshotMessageSerializer implements MessageSerializer<SnapshotMessage>
+    public static final IVersionedSerializer<SnapshotMessage> serializer = new IVersionedSerializer<SnapshotMessage>()
     {
         public void serialize(SnapshotMessage message, DataOutputPlus out, int version) throws IOException
         {
@@ -65,5 +64,5 @@ public class SnapshotMessage extends RepairMessage
         {
             return RepairJobDesc.serializer.serializedSize(message.desc, version);
         }
-    }
+    };
 }
