@@ -42,6 +42,7 @@ import org.slf4j.LoggerFactory;
 import org.apache.cassandra.config.CassandraRelevantProperties;
 import org.apache.cassandra.cql3.functions.Function;
 import org.apache.cassandra.cql3.functions.FunctionName;
+import org.apache.cassandra.cql3.functions.UserFunction;
 import org.apache.cassandra.db.ColumnFamilyStore;
 import org.apache.cassandra.db.Keyspace;
 import org.apache.cassandra.db.KeyspaceNotDefinedException;
@@ -220,6 +221,12 @@ public class Schema implements SchemaProvider
         return keyspaceInstances.getIfReady(keyspaceName);
     }
 
+    /**
+     * Returns {@link ColumnFamilyStore} by the table identifier. Note that though, if called for {@link TableMetadata#id},
+     * when metadata points to a secondary index table, the {@link TableMetadata#id} denotes the identifier of the main
+     * table, not the index table. Thus, this method will return CFS of the main table rather than, probably expected,
+     * CFS for the index backing table.
+     */
     public ColumnFamilyStore getColumnFamilyStoreInstance(TableId id)
     {
         TableMetadata metadata = getTableMetadata(id);
@@ -443,13 +450,13 @@ public class Schema implements SchemaProvider
     /* Function helpers */
 
     /**
-     * Get all function overloads with the specified name
+     * Get all user-defined function overloads with the specified name.
      *
      * @param name fully qualified function name
      * @return an empty list if the keyspace or the function name are not found;
-     *         a non-empty collection of {@link Function} otherwise
+     *         a non-empty collection of {@link UserFunction} otherwise
      */
-    public Collection<Function> getFunctions(FunctionName name)
+    public Collection<UserFunction> getUserFunctions(FunctionName name)
     {
         if (!name.hasKeyspace())
             throw new IllegalArgumentException(String.format("Function name must be fully qualified: got %s", name));
@@ -457,26 +464,24 @@ public class Schema implements SchemaProvider
         KeyspaceMetadata ksm = getKeyspaceMetadata(name.keyspace);
         return ksm == null
                ? Collections.emptyList()
-               : ksm.functions.get(name);
+               : ksm.userFunctions.get(name);
     }
 
     /**
-     * Find the function with the specified name
+     * Find the function with the specified name and arguments.
      *
      * @param name     fully qualified function name
      * @param argTypes function argument types
      * @return an empty {@link Optional} if the keyspace or the function name are not found;
      *         a non-empty optional of {@link Function} otherwise
      */
-    public Optional<Function> findFunction(FunctionName name, List<AbstractType<?>> argTypes)
+    public Optional<UserFunction> findUserFunction(FunctionName name, List<AbstractType<?>> argTypes)
     {
         if (!name.hasKeyspace())
             throw new IllegalArgumentException(String.format("Function name must be fully quallified: got %s", name));
 
-        KeyspaceMetadata ksm = getKeyspaceMetadata(name.keyspace);
-        return ksm == null
-               ? Optional.empty()
-               : ksm.functions.find(name, argTypes);
+        return Optional.ofNullable(getKeyspaceMetadata(name.keyspace))
+                       .flatMap(ksm -> ksm.userFunctions.find(name, argTypes));
     }
 
     /* Version control */
@@ -780,4 +785,4 @@ public class Schema implements SchemaProvider
                : Collections.emptyMap();
     }
 
-}
+}
\ No newline at end of file
