@@ -36,11 +36,13 @@ import org.apache.cassandra.utils.ByteBufferUtil;
 
 import static java.lang.String.format;
 import static java.util.stream.Collectors.toMap;
+import static org.apache.cassandra.schema.TableParams.Option.*;
 
 public final class TableParams
 {
     public enum Option
     {
+        ALLOW_AUTO_SNAPSHOT,
         BLOOM_FILTER_FP_CHANCE,
         CACHING,
         COMMENT,
@@ -50,6 +52,7 @@ public final class TableParams
         DEFAULT_TIME_TO_LIVE,
         EXTENSIONS,
         GC_GRACE_SECONDS,
+        INCREMENTAL_BACKUPS,
         MAX_INDEX_INTERVAL,
         MEMTABLE_FLUSH_PERIOD_IN_MS,
         MIN_INDEX_INTERVAL,
@@ -67,9 +70,11 @@ public final class TableParams
     }
 
     public final String comment;
+    public final boolean allowAutoSnapshot;
     public final double bloomFilterFpChance;
     public final double crcCheckChance;
     public final int gcGraceSeconds;
+    public final boolean incrementalBackups;
     public final int defaultTimeToLive;
     public final int memtableFlushPeriodInMs;
     public final int minIndexInterval;
@@ -87,11 +92,13 @@ public final class TableParams
     private TableParams(Builder builder)
     {
         comment = builder.comment;
-        bloomFilterFpChance = builder.bloomFilterFpChance == null
+        allowAutoSnapshot = builder.allowAutoSnapshot;
+        bloomFilterFpChance = builder.bloomFilterFpChance == -1
                             ? builder.compaction.defaultBloomFilterFbChance()
                             : builder.bloomFilterFpChance;
         crcCheckChance = builder.crcCheckChance;
         gcGraceSeconds = builder.gcGraceSeconds;
+        incrementalBackups = builder.incrementalBackups;
         defaultTimeToLive = builder.defaultTimeToLive;
         memtableFlushPeriodInMs = builder.memtableFlushPeriodInMs;
         minIndexInterval = builder.minIndexInterval;
@@ -114,7 +121,8 @@ public final class TableParams
 
     public static Builder builder(TableParams params)
     {
-        return new Builder().bloomFilterFpChance(params.bloomFilterFpChance)
+        return new Builder().allowAutoSnapshot(params.allowAutoSnapshot)
+                            .bloomFilterFpChance(params.bloomFilterFpChance)
                             .caching(params.caching)
                             .comment(params.comment)
                             .compaction(params.compaction)
@@ -123,6 +131,7 @@ public final class TableParams
                             .crcCheckChance(params.crcCheckChance)
                             .defaultTimeToLive(params.defaultTimeToLive)
                             .gcGraceSeconds(params.gcGraceSeconds)
+                            .incrementalBackups(params.incrementalBackups)
                             .maxIndexInterval(params.maxIndexInterval)
                             .memtableFlushPeriodInMs(params.memtableFlushPeriodInMs)
                             .minIndexInterval(params.minIndexInterval)
@@ -147,7 +156,7 @@ public final class TableParams
         if (bloomFilterFpChance <=  minBloomFilterFpChanceValue || bloomFilterFpChance > 1)
         {
             fail("%s must be larger than %s and less than or equal to 1.0 (got %s)",
-                 Option.BLOOM_FILTER_FP_CHANCE,
+                 BLOOM_FILTER_FP_CHANCE,
                  minBloomFilterFpChanceValue,
                  bloomFilterFpChance);
         }
@@ -155,33 +164,33 @@ public final class TableParams
         if (crcCheckChance < 0 || crcCheckChance > 1.0)
         {
             fail("%s must be larger than or equal to 0 and smaller than or equal to 1.0 (got %s)",
-                 Option.CRC_CHECK_CHANCE,
+                 CRC_CHECK_CHANCE,
                  crcCheckChance);
         }
 
         if (defaultTimeToLive < 0)
-            fail("%s must be greater than or equal to 0 (got %s)", Option.DEFAULT_TIME_TO_LIVE, defaultTimeToLive);
+            fail("%s must be greater than or equal to 0 (got %s)", DEFAULT_TIME_TO_LIVE, defaultTimeToLive);
 
         if (defaultTimeToLive > Attributes.MAX_TTL)
-            fail("%s must be less than or equal to %d (got %s)", Option.DEFAULT_TIME_TO_LIVE, Attributes.MAX_TTL, defaultTimeToLive);
+            fail("%s must be less than or equal to %d (got %s)", DEFAULT_TIME_TO_LIVE, Attributes.MAX_TTL, defaultTimeToLive);
 
         if (gcGraceSeconds < 0)
-            fail("%s must be greater than or equal to 0 (got %s)", Option.GC_GRACE_SECONDS, gcGraceSeconds);
+            fail("%s must be greater than or equal to 0 (got %s)", GC_GRACE_SECONDS, gcGraceSeconds);
 
         if (minIndexInterval < 1)
-            fail("%s must be greater than or equal to 1 (got %s)", Option.MIN_INDEX_INTERVAL, minIndexInterval);
+            fail("%s must be greater than or equal to 1 (got %s)", MIN_INDEX_INTERVAL, minIndexInterval);
 
         if (maxIndexInterval < minIndexInterval)
         {
             fail("%s must be greater than or equal to %s (%s) (got %s)",
-                 Option.MAX_INDEX_INTERVAL,
-                 Option.MIN_INDEX_INTERVAL,
+                 MAX_INDEX_INTERVAL,
+                 MIN_INDEX_INTERVAL,
                  minIndexInterval,
                  maxIndexInterval);
         }
 
         if (memtableFlushPeriodInMs < 0)
-            fail("%s must be greater than or equal to 0 (got %s)", Option.MEMTABLE_FLUSH_PERIOD_IN_MS, memtableFlushPeriodInMs);
+            fail("%s must be greater than or equal to 0 (got %s)", MEMTABLE_FLUSH_PERIOD_IN_MS, memtableFlushPeriodInMs);
 
         if (cdc && memtable.factory().writesShouldSkipCommitLog())
             fail("CDC cannot work if writes skip the commit log. Check your memtable configuration.");
@@ -204,9 +213,12 @@ public final class TableParams
         TableParams p = (TableParams) o;
 
         return comment.equals(p.comment)
+            && additionalWritePolicy.equals(p.additionalWritePolicy)
+            && allowAutoSnapshot == p.allowAutoSnapshot
             && bloomFilterFpChance == p.bloomFilterFpChance
             && crcCheckChance == p.crcCheckChance
-            && gcGraceSeconds == p.gcGraceSeconds
+            && gcGraceSeconds == p.gcGraceSeconds 
+            && incrementalBackups == p.incrementalBackups
             && defaultTimeToLive == p.defaultTimeToLive
             && memtableFlushPeriodInMs == p.memtableFlushPeriodInMs
             && minIndexInterval == p.minIndexInterval
@@ -225,9 +237,12 @@ public final class TableParams
     public int hashCode()
     {
         return Objects.hashCode(comment,
+                                additionalWritePolicy,
+                                allowAutoSnapshot,
                                 bloomFilterFpChance,
                                 crcCheckChance,
                                 gcGraceSeconds,
+                                incrementalBackups,
                                 defaultTimeToLive,
                                 memtableFlushPeriodInMs,
                                 minIndexInterval,
@@ -246,22 +261,25 @@ public final class TableParams
     public String toString()
     {
         return MoreObjects.toStringHelper(this)
-                          .add(Option.COMMENT.toString(), comment)
-                          .add(Option.BLOOM_FILTER_FP_CHANCE.toString(), bloomFilterFpChance)
-                          .add(Option.CRC_CHECK_CHANCE.toString(), crcCheckChance)
-                          .add(Option.GC_GRACE_SECONDS.toString(), gcGraceSeconds)
-                          .add(Option.DEFAULT_TIME_TO_LIVE.toString(), defaultTimeToLive)
-                          .add(Option.MEMTABLE_FLUSH_PERIOD_IN_MS.toString(), memtableFlushPeriodInMs)
-                          .add(Option.MIN_INDEX_INTERVAL.toString(), minIndexInterval)
-                          .add(Option.MAX_INDEX_INTERVAL.toString(), maxIndexInterval)
-                          .add(Option.SPECULATIVE_RETRY.toString(), speculativeRetry)
-                          .add(Option.CACHING.toString(), caching)
-                          .add(Option.COMPACTION.toString(), compaction)
-                          .add(Option.COMPRESSION.toString(), compression)
-                          .add(Option.MEMTABLE.toString(), memtable)
-                          .add(Option.EXTENSIONS.toString(), extensions)
-                          .add(Option.CDC.toString(), cdc)
-                          .add(Option.READ_REPAIR.toString(), readRepair)
+                          .add(COMMENT.toString(), comment)
+                          .add(ADDITIONAL_WRITE_POLICY.toString(), additionalWritePolicy)
+                          .add(ALLOW_AUTO_SNAPSHOT.toString(), allowAutoSnapshot)
+                          .add(BLOOM_FILTER_FP_CHANCE.toString(), bloomFilterFpChance)
+                          .add(CRC_CHECK_CHANCE.toString(), crcCheckChance)
+                          .add(GC_GRACE_SECONDS.toString(), gcGraceSeconds)
+                          .add(DEFAULT_TIME_TO_LIVE.toString(), defaultTimeToLive)
+                          .add(INCREMENTAL_BACKUPS.toString(), incrementalBackups)
+                          .add(MEMTABLE_FLUSH_PERIOD_IN_MS.toString(), memtableFlushPeriodInMs)
+                          .add(MIN_INDEX_INTERVAL.toString(), minIndexInterval)
+                          .add(MAX_INDEX_INTERVAL.toString(), maxIndexInterval)
+                          .add(SPECULATIVE_RETRY.toString(), speculativeRetry)
+                          .add(CACHING.toString(), caching)
+                          .add(COMPACTION.toString(), compaction)
+                          .add(COMPRESSION.toString(), compression)
+                          .add(MEMTABLE.toString(), memtable)
+                          .add(EXTENSIONS.toString(), extensions)
+                          .add(CDC.toString(), cdc)
+                          .add(READ_REPAIR.toString(), readRepair)
                           .toString();
     }
 
@@ -269,6 +287,8 @@ public final class TableParams
     {
         // option names should be in alphabetical order
         builder.append("additional_write_policy = ").appendWithSingleQuotes(additionalWritePolicy.toString())
+               .newLine()
+               .append("AND allow_auto_snapshot = ").append(allowAutoSnapshot)
                .newLine()
                .append("AND bloom_filter_fp_chance = ").append(bloomFilterFpChance)
                .newLine()
@@ -301,6 +321,8 @@ public final class TableParams
                .newLine()
                .append("AND gc_grace_seconds = ").append(gcGraceSeconds)
                .newLine()
+               .append("AND incremental_backups = ").append(incrementalBackups)
+               .newLine()
                .append("AND max_index_interval = ").append(maxIndexInterval)
                .newLine()
                .append("AND memtable_flush_period_in_ms = ").append(memtableFlushPeriodInMs)
@@ -315,9 +337,11 @@ public final class TableParams
     public static final class Builder
     {
         private String comment = "";
-        private Double bloomFilterFpChance;
+        private boolean allowAutoSnapshot = true;
+        private double bloomFilterFpChance = -1;
         private double crcCheckChance = 1.0;
         private int gcGraceSeconds = 864000; // 10 days
+        private boolean incrementalBackups = true;
         private int defaultTimeToLive = 0;
         private int memtableFlushPeriodInMs = 0;
         private int minIndexInterval = 128;
@@ -347,6 +371,12 @@ public final class TableParams
             return this;
         }
 
+        public Builder allowAutoSnapshot(boolean val)
+        {
+            allowAutoSnapshot = val;
+            return this;
+        }
+
         public Builder bloomFilterFpChance(double val)
         {
             bloomFilterFpChance = val;
@@ -365,6 +395,12 @@ public final class TableParams
             return this;
         }
 
+        public Builder incrementalBackups(boolean val)
+        {
+            incrementalBackups = val;
+            return this;
+        }
+
         public Builder defaultTimeToLive(int val)
         {
             defaultTimeToLive = val;
