@@ -18,14 +18,17 @@
 
 package org.apache.cassandra.serializers;
 
+import java.nio.BufferUnderflowException;
 import java.nio.ByteBuffer;
 import java.util.Collection;
 import java.util.List;
+import java.util.function.Consumer;
+
+import com.google.common.collect.Range;
 
 import org.apache.cassandra.db.TypeSizes;
 import org.apache.cassandra.db.marshal.ByteBufferAccessor;
 import org.apache.cassandra.db.marshal.ValueAccessor;
-import org.apache.cassandra.transport.ProtocolVersion;
 import org.apache.cassandra.db.marshal.AbstractType;
 import org.apache.cassandra.utils.ByteBufferUtil;
 
@@ -34,79 +37,49 @@ public abstract class CollectionSerializer<T> extends TypeSerializer<T>
     protected abstract List<ByteBuffer> serializeValues(T value);
     protected abstract int getElementCount(T value);
 
-    public abstract <V> T deserializeForNativeProtocol(V value, ValueAccessor<V> accessor, ProtocolVersion version);
-
-    public T deserializeForNativeProtocol(ByteBuffer value, ProtocolVersion version)
-    {
-        return deserializeForNativeProtocol(value, ByteBufferAccessor.instance, version);
-    }
-
-    public abstract <V> void validateForNativeProtocol(V value, ValueAccessor<V> accessor, ProtocolVersion version);
-
+    @Override
     public ByteBuffer serialize(T input)
     {
         List<ByteBuffer> values = serializeValues(input);
-        // See deserialize() for why using the protocol v3 variant is the right thing to do.
-        return pack(values, ByteBufferAccessor.instance, getElementCount(input), ProtocolVersion.V3);
+        return pack(values, ByteBufferAccessor.instance, getElementCount(input));
     }
 
-    public <V> T deserialize(V value, ValueAccessor<V> accessor)
+    public static ByteBuffer pack(Collection<ByteBuffer> values, int elements)
     {
-        // The only cases we serialize/deserialize collections internally (i.e. not for the protocol sake),
-        // is:
-        //  1) when collections are frozen
-        //  2) for internal calls.
-        // In both case, using the protocol 3 version variant is the right thing to do.
-        return deserializeForNativeProtocol(value, accessor, ProtocolVersion.V3);
+        return pack(values, ByteBufferAccessor.instance, elements);
     }
 
-    public <T1> void validate(T1 value, ValueAccessor<T1> accessor) throws MarshalException
-    {
-        // Same thing as above
-        validateForNativeProtocol(value, accessor, ProtocolVersion.V3);
-    }
-
-    public static ByteBuffer pack(Collection<ByteBuffer> values, int elements, ProtocolVersion version)
-    {
-        return pack(values, ByteBufferAccessor.instance, elements, version);
-    }
-
-    public static <V> V pack(Collection<V> values, ValueAccessor<V> accessor, int elements, ProtocolVersion version)
+    public static <V> V pack(Collection<V> values, ValueAccessor<V> accessor, int elements)
     {
         int size = 0;
         for (V value : values)
-            size += sizeOfValue(value, accessor, version);
+            size += sizeOfValue(value, accessor);
 
-        ByteBuffer result = ByteBuffer.allocate(sizeOfCollectionSize(elements, version) + size);
-        writeCollectionSize(result, elements, version);
+        ByteBuffer result = ByteBuffer.allocate(sizeOfCollectionSize() + size);
+        writeCollectionSize(result, elements);
         for (V value : values)
         {
-            writeValue(result, value, accessor, version);
+            writeValue(result, value, accessor);
         }
         return accessor.valueOf((ByteBuffer) result.flip());
     }
 
-    protected static void writeCollectionSize(ByteBuffer output, int elements, ProtocolVersion version)
+    protected static void writeCollectionSize(ByteBuffer output, int elements)
     {
         output.putInt(elements);
     }
 
-    public static int readCollectionSize(ByteBuffer input, ProtocolVersion version)
-    {
-        return readCollectionSize(input, ByteBufferAccessor.instance, version);
-    }
-
-    public static <V> int readCollectionSize(V value, ValueAccessor<V> accessor, ProtocolVersion version)
+    public static <V> int readCollectionSize(V value, ValueAccessor<V> accessor)
     {
         return accessor.toInt(value);
     }
 
-    public static int sizeOfCollectionSize(int elements, ProtocolVersion version)
+    public static int sizeOfCollectionSize()
     {
         return TypeSizes.INT_SIZE;
     }
 
-    public static <V> void writeValue(ByteBuffer output, V value, ValueAccessor<V> accessor, ProtocolVersion version)
+    public static <V> void writeValue(ByteBuffer output, V value, ValueAccessor<V> accessor)
     {
         if (value == null)
         {
@@ -118,7 +91,7 @@ public abstract class CollectionSerializer<T> extends TypeSerializer<T>
         accessor.write(value, output);
     }
 
-    public static <V> V readValue(V input, ValueAccessor<V> accessor, int offset, ProtocolVersion version)
+    public static <V> V readValue(V input, ValueAccessor<V> accessor, int offset)
     {
         int size = accessor.getInt(input, offset);
         if (size < 0)
@@ -127,27 +100,27 @@ public abstract class CollectionSerializer<T> extends TypeSerializer<T>
         return accessor.slice(input, offset + TypeSizes.INT_SIZE, size);
     }
 
-    public static <V> V readNonNullValue(V input, ValueAccessor<V> accessor, int offset, ProtocolVersion version)
+    public static <V> V readNonNullValue(V input, ValueAccessor<V> accessor, int offset)
     {
-        V value = readValue(input, accessor, offset, version);
+        V value = readValue(input, accessor, offset);
         if (value == null)
             throw new MarshalException("Null value read when not allowed");
         return value;
     }
 
-    protected static void skipValue(ByteBuffer input, ProtocolVersion version)
+    protected static void skipValue(ByteBuffer input)
     {
         int size = input.getInt();
         input.position(input.position() + size);
     }
 
-    public static <V> int skipValue(V input, ValueAccessor<V> accessor, int offset, ProtocolVersion version)
+    public static <V> int skipValue(V input, ValueAccessor<V> accessor, int offset)
     {
         int size = accessor.getInt(input, offset);
         return TypeSizes.sizeof(size) + size;
     }
 
-    public static <V> int sizeOfValue(V value, ValueAccessor<V> accessor, ProtocolVersion version)
+    public static <V> int sizeOfValue(V value, ValueAccessor<V> accessor)
     {
         return value == null ? 4 : 4 + accessor.size(value);
     }
@@ -187,21 +160,73 @@ public abstract class CollectionSerializer<T> extends TypeSerializer<T>
                                                       AbstractType<?> comparator,
                                                       boolean frozen);
 
+    /**
+     * Returns the index of an element in a serialized collection.
+     * <p>
+     * Note that this is only supported by sets and maps, but not by lists.
+     *
+     * @param collection The serialized collection. This cannot be {@code null}.
+     * @param key The key for which the index must be found. This cannot be {@code null} nor
+     * {@link ByteBufferUtil#UNSET_BYTE_BUFFER}).
+     * @param comparator The type to use to compare the {@code key} value to those in the collection.
+     * @return The index of the element associated with {@code key} if one exists, {@code -1} otherwise.
+     */
+    public abstract int getIndexFromSerialized(ByteBuffer collection, ByteBuffer key, AbstractType<?> comparator);
+
+    /**
+     * Returns the range of indexes corresponding to the specified range of elements in the serialized collection.
+     * <p>
+     * Note that this is only supported by sets and maps, but not by lists.
+     *
+     * @param collection The serialized collection. This cannot be {@code null}.
+     * @param from  The left bound of the slice to extract. This cannot be {@code null} but if this is
+     * {@link ByteBufferUtil#UNSET_BYTE_BUFFER}, then the returned slice starts at the beginning of the collection.
+     * @param to The left bound of the slice to extract. This cannot be {@code null} but if this is
+     * {@link ByteBufferUtil#UNSET_BYTE_BUFFER}, then the returned slice ends at the end of the collection.
+     * @param comparator The type to use to compare the {@code from} and {@code to} values to those in the collection.
+     * @return The range of indexes corresponding to specified range of elements.
+     */
+    public abstract Range<Integer> getIndexesRangeFromSerialized(ByteBuffer collection,
+                                                                 ByteBuffer from,
+                                                                 ByteBuffer to,
+                                                                 AbstractType<?> comparator);
+
     /**
      * Creates a new serialized map composed from the data from {@code input} between {@code startPos}
      * (inclusive) and {@code endPos} (exclusive), assuming that data holds {@code count} elements.
      */
-    protected ByteBuffer copyAsNewCollection(ByteBuffer input, int count, int startPos, int endPos, ProtocolVersion version)
+    protected ByteBuffer copyAsNewCollection(ByteBuffer input, int count, int startPos, int endPos)
     {
-        int sizeLen = sizeOfCollectionSize(count, version);
+        int sizeLen = sizeOfCollectionSize();
         if (count == 0)
             return ByteBuffer.allocate(sizeLen);
 
         int bodyLen = endPos - startPos;
         ByteBuffer output = ByteBuffer.allocate(sizeLen + bodyLen);
-        writeCollectionSize(output, count, version);
+        writeCollectionSize(output, count);
         output.position(0);
         ByteBufferUtil.copyBytes(input, startPos, output, sizeLen, bodyLen);
         return output;
     }
+
+    public void forEach(ByteBuffer input, Consumer<ByteBuffer> action)
+    {
+        try
+        {
+            int collectionSize = readCollectionSize(input, ByteBufferAccessor.instance);
+            int offset = sizeOfCollectionSize();
+
+            for (int i = 0; i < collectionSize; i++)
+            {
+                ByteBuffer value = readValue(input, ByteBufferAccessor.instance, offset);
+                offset += sizeOfValue(value, ByteBufferAccessor.instance);
+
+                action.accept(value);
+            }
+        }
+        catch (BufferUnderflowException | IndexOutOfBoundsException e)
+        {
+            throw new MarshalException("Not enough bytes to read a set");
+        }
+    }
 }
