@@ -196,7 +196,7 @@ public abstract class AbstractWriteResponseHandler<T> implements RequestCallback
         }
     }
 
-    public final void expired()
+    protected final void logFailureOrTimeoutToIdealCLDelegate()
     {
         //Tracking ideal CL was not configured
         if (idealCLDelegate == null)
@@ -216,6 +216,11 @@ public abstract class AbstractWriteResponseHandler<T> implements RequestCallback
         }
     }
 
+    public final void expired()
+    {
+        logFailureOrTimeoutToIdealCLDelegate();
+    }
+
     /**
      * @return the minimum number of endpoints that must respond.
      */
@@ -257,6 +262,11 @@ public abstract class AbstractWriteResponseHandler<T> implements RequestCallback
      */
     protected abstract int ackCount();
 
+    public Dispatcher.RequestTime getRequestTime()
+    {
+        return requestTime;
+    }
+
     /**
      * null message means "response from local write"
      */
@@ -266,9 +276,13 @@ public abstract class AbstractWriteResponseHandler<T> implements RequestCallback
     {
         //The ideal CL should only count as a strike if the requested CL was achieved.
         //If the requested CL is not achieved it's fine for the ideal CL to also not be achieved.
-        if (idealCLDelegate != null)
+        if (idealCLDelegate != null && blockFor() + failures <= candidateReplicaCount())
         {
             idealCLDelegate.requestedCLAchieved = true;
+            if (idealCLDelegate == this)
+            {
+                replicaPlan.keyspace().metric.idealCLWriteLatency.addNano(nanoTime() - requestTime.startedAtNanos());
+            }
         }
 
         condition.signalAll();
@@ -287,6 +301,8 @@ public abstract class AbstractWriteResponseHandler<T> implements RequestCallback
 
         failureReasonByEndpoint.put(from, failureReason);
 
+        logFailureOrTimeoutToIdealCLDelegate();
+
         if (blockFor() + n > candidateReplicaCount())
             signal();
 
@@ -316,10 +332,6 @@ public abstract class AbstractWriteResponseHandler<T> implements RequestCallback
             {
                 replicaPlan.keyspace().metric.writeFailedIdealCL.inc();
             }
-            else
-            {
-                replicaPlan.keyspace().metric.idealCLWriteLatency.addNano(nanoTime() - requestTime.startedAtNanos());
-            }
         }
     }
 
