@@ -26,8 +26,8 @@ import java.util.concurrent.ScheduledFuture;
 import java.util.concurrent.ThreadPoolExecutor;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.TimeoutException;
-import java.util.concurrent.atomic.AtomicBoolean;
 import javax.management.openmbean.CompositeData;
+import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.function.Predicate;
 import java.util.stream.Collectors;
@@ -44,8 +44,9 @@ import com.google.common.collect.Multimap;
 import org.apache.cassandra.concurrent.ExecutorPlus;
 import org.apache.cassandra.config.Config;
 import org.apache.cassandra.config.DurationSpec;
-import org.apache.cassandra.db.compaction.CompactionManager;
 import org.apache.cassandra.repair.Scheduler;
+import org.apache.cassandra.repair.SharedContext;
+import org.apache.cassandra.exceptions.ConfigurationException;
 import org.apache.cassandra.locator.AbstractReplicationStrategy;
 import org.apache.cassandra.locator.EndpointsByRange;
 import org.apache.cassandra.locator.EndpointsForRange;
@@ -59,11 +60,10 @@ import org.apache.cassandra.schema.Schema;
 import org.apache.cassandra.schema.TableMetadata;
 import org.apache.cassandra.streaming.PreviewKind;
 import org.apache.cassandra.utils.TimeUUID;
-import org.apache.cassandra.utils.concurrent.CountDownLatch;
+import org.apache.cassandra.utils.concurrent.AsyncPromise;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import org.apache.cassandra.concurrent.ScheduledExecutors;
 import org.apache.cassandra.config.DatabaseDescriptor;
 import org.apache.cassandra.db.ColumnFamilyStore;
 import org.apache.cassandra.db.Keyspace;
@@ -73,10 +73,8 @@ import org.apache.cassandra.exceptions.RequestFailureReason;
 import org.apache.cassandra.gms.ApplicationState;
 import org.apache.cassandra.gms.EndpointState;
 import org.apache.cassandra.gms.FailureDetector;
-import org.apache.cassandra.gms.Gossiper;
 import org.apache.cassandra.gms.IEndpointStateChangeSubscriber;
 import org.apache.cassandra.gms.IFailureDetectionEventListener;
-import org.apache.cassandra.gms.IFailureDetector;
 import org.apache.cassandra.gms.VersionedValue;
 import org.apache.cassandra.locator.InetAddressAndPort;
 import org.apache.cassandra.locator.TokenMetadata;
@@ -84,7 +82,6 @@ import org.apache.cassandra.metrics.RepairMetrics;
 import org.apache.cassandra.net.RequestCallback;
 import org.apache.cassandra.net.Verb;
 import org.apache.cassandra.net.Message;
-import org.apache.cassandra.net.MessagingService;
 import org.apache.cassandra.repair.CommonRange;
 import org.apache.cassandra.repair.NoSuchRepairSessionException;
 import org.apache.cassandra.service.paxos.PaxosRepair;
@@ -106,8 +103,6 @@ import org.apache.cassandra.repair.messages.RepairOption;
 import org.apache.cassandra.repair.messages.SyncResponse;
 import org.apache.cassandra.repair.messages.ValidationResponse;
 import org.apache.cassandra.schema.TableId;
-import org.apache.cassandra.utils.FBUtilities;
-import org.apache.cassandra.utils.MBeanWrapper;
 import org.apache.cassandra.utils.MerkleTrees;
 import org.apache.cassandra.utils.Pair;
 import org.apache.cassandra.utils.concurrent.Future;
@@ -119,14 +114,16 @@ import static com.google.common.collect.Iterables.transform;
 import static java.util.Collections.synchronizedSet;
 import static java.util.concurrent.TimeUnit.MILLISECONDS;
 import static org.apache.cassandra.concurrent.ExecutorFactory.Global.executorFactory;
+import static org.apache.cassandra.config.CassandraRelevantProperties.PARENT_REPAIR_STATUS_CACHE_SIZE;
+import static org.apache.cassandra.config.CassandraRelevantProperties.PARENT_REPAIR_STATUS_EXPIRY_SECONDS;
+import static org.apache.cassandra.config.CassandraRelevantProperties.PAXOS_REPAIR_ALLOW_MULTIPLE_PENDING_UNSAFE;
+import static org.apache.cassandra.config.CassandraRelevantProperties.SKIP_PAXOS_REPAIR_ON_TOPOLOGY_CHANGE;
+import static org.apache.cassandra.config.CassandraRelevantProperties.SKIP_PAXOS_REPAIR_ON_TOPOLOGY_CHANGE_KEYSPACES;
 import static org.apache.cassandra.config.Config.RepairCommandPoolFullStrategy.reject;
 import static org.apache.cassandra.config.DatabaseDescriptor.*;
-import static org.apache.cassandra.net.Message.out;
 import static org.apache.cassandra.net.Verb.PREPARE_MSG;
-import static org.apache.cassandra.utils.Clock.Global.currentTimeMillis;
+import static org.apache.cassandra.repair.messages.RepairMessage.notDone;
 import static org.apache.cassandra.utils.Simulate.With.MONITORS;
-import static org.apache.cassandra.utils.Clock.Global.nanoTime;
-import static org.apache.cassandra.utils.concurrent.CountDownLatch.newCountDownLatch;
 
 /**
  * ActiveRepairService is the starting point for manual "active" repairs.
@@ -145,6 +142,7 @@ import static org.apache.cassandra.utils.concurrent.CountDownLatch.newCountDownL
 @Simulate(with = MONITORS)
 public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFailureDetectionEventListener, ActiveRepairServiceMBean
 {
+
     public enum ParentRepairStatus
     {
         IN_PROGRESS, COMPLETED, FAILED
@@ -152,21 +150,35 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
 
     public static class ConsistentSessions
     {
-        public final LocalSessions local = new LocalSessions();
-        public final CoordinatorSessions coordinated = new CoordinatorSessions();
+        public final LocalSessions local;
+        public final CoordinatorSessions coordinated;
+
+        public ConsistentSessions(SharedContext ctx)
+        {
+            local = new LocalSessions(ctx);
+            coordinated = new CoordinatorSessions(ctx);
+        }
     }
 
-    public final ConsistentSessions consistent = new ConsistentSessions();
+    public final ConsistentSessions consistent;
 
     private boolean registeredForEndpointChanges = false;
 
     private static final Logger logger = LoggerFactory.getLogger(ActiveRepairService.class);
-    // singleton enforcement
-    public static final ActiveRepairService instance = new ActiveRepairService(FailureDetector.instance, Gossiper.instance);
 
     public static final long UNREPAIRED_SSTABLE = 0;
     public static final TimeUUID NO_PENDING_REPAIR = null;
 
+    public static ActiveRepairService instance()
+    {
+        return Holder.instance;
+    }
+
+    private static class Holder
+    {
+        private static final ActiveRepairService instance = new ActiveRepairService();
+    }
+
     /**
      * A map of active coordinator session.
      */
@@ -177,6 +189,7 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
     private final Cache<TimeUUID, CoordinatorState> repairs;
     // map of top level repair id (parent repair id) -> participate state
     private final Cache<TimeUUID, ParticipateState> participates;
+    public final SharedContext ctx;
 
     private volatile ScheduledFuture<?> irCleanup;
 
@@ -208,26 +221,28 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
         return RepairCommandExecutorHandle.repairCommandExecutor;
     }
 
-    private final IFailureDetector failureDetector;
-    private final Gossiper gossiper;
     private final Cache<Integer, Pair<ParentRepairStatus, List<String>>> repairStatusByCmd;
+    public final ExecutorPlus snapshotExecutor;
 
-    public final ExecutorPlus snapshotExecutor = executorFactory().configurePooled("RepairSnapshotExecutor", 1)
-                                                                  .withKeepAlive(1, TimeUnit.HOURS)
-                                                                  .build();
-
-    public ActiveRepairService(IFailureDetector failureDetector, Gossiper gossiper)
+    public ActiveRepairService()
     {
-        this.failureDetector = failureDetector;
-        this.gossiper = gossiper;
+        this(SharedContext.Global.instance);
+    }
+
+    @VisibleForTesting
+    public ActiveRepairService(SharedContext ctx)
+    {
+        this.ctx = ctx;
+        consistent = new ConsistentSessions(ctx);
+        this.snapshotExecutor = ctx.executorFactory().configurePooled("RepairSnapshotExecutor", 1)
+                                   .withKeepAlive(1, TimeUnit.HOURS)
+                                   .build();
         this.repairStatusByCmd = CacheBuilder.newBuilder()
-                                             .expireAfterWrite(
-                                             Long.getLong("cassandra.parent_repair_status_expiry_seconds",
-                                                          TimeUnit.SECONDS.convert(1, TimeUnit.DAYS)), TimeUnit.SECONDS)
+                                             .expireAfterWrite(PARENT_REPAIR_STATUS_EXPIRY_SECONDS.getLong(), TimeUnit.SECONDS)
                                              // using weight wouldn't work so well, since it doesn't reflect mutation of cached data
                                              // see https://github.com/google/guava/wiki/CachesExplained
                                              // We assume each entry is unlikely to be much more than 100 bytes, so bounding the size should be sufficient.
-                                             .maximumSize(Long.getLong("cassandra.parent_repair_status_cache_size", 100_000))
+                                             .maximumSize(PARENT_REPAIR_STATUS_CACHE_SIZE.getLong())
                                              .build();
 
         DurationSpec.LongNanosecondsBound duration = getRepairStateExpires();
@@ -242,15 +257,15 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
                                    .maximumSize(numElements)
                                    .build();
 
-        MBeanWrapper.instance.registerMBean(this, MBEAN_NAME);
+        ctx.mbean().registerMBean(this, MBEAN_NAME);
     }
 
     public void start()
     {
         consistent.local.start();
-        this.irCleanup = ScheduledExecutors.optionalTasks.scheduleAtFixedRate(consistent.local::cleanup, 0,
-                                                                              LocalSessions.CLEANUP_INTERVAL,
-                                                                              TimeUnit.SECONDS);
+        this.irCleanup = ctx.optionalTasks().scheduleAtFixedRate(consistent.local::cleanup, 0,
+                                                                 LocalSessions.CLEANUP_INTERVAL,
+                                                                 TimeUnit.SECONDS);
     }
 
     @VisibleForTesting
@@ -283,30 +298,60 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
         consistent.local.cancelSession(sessionID, force);
     }
 
-    @Deprecated
+    /** @deprecated See CASSANDRA-15234 */
+    @Deprecated(since = "4.1")
     public void setRepairSessionSpaceInMegabytes(int sizeInMegabytes)
     {
         DatabaseDescriptor.setRepairSessionSpaceInMiB(sizeInMegabytes);
     }
 
-    @Deprecated
+    /** @deprecated See CASSANDRA-15234 */
+    @Deprecated(since = "4.1")
     public int getRepairSessionSpaceInMegabytes()
     {
         return DatabaseDescriptor.getRepairSessionSpaceInMiB();
     }
 
+    /** @deprecated See CASSANDRA-17668 */
+    @Deprecated(since = "4.1")
     @Override
     public void setRepairSessionSpaceInMebibytes(int sizeInMebibytes)
     {
         DatabaseDescriptor.setRepairSessionSpaceInMiB(sizeInMebibytes);
     }
 
+    /** @deprecated See CASSANDRA-17668 */
+    @Deprecated(since = "4.1")
     @Override
     public int getRepairSessionSpaceInMebibytes()
     {
         return DatabaseDescriptor.getRepairSessionSpaceInMiB();
     }
 
+    @Override
+    public void setRepairSessionSpaceInMiB(int sizeInMebibytes)
+    {
+        try
+        {
+            DatabaseDescriptor.setRepairSessionSpaceInMiB(sizeInMebibytes);
+        }
+        catch (ConfigurationException e)
+        {
+            throw new IllegalArgumentException(e.getMessage());
+        }
+    }
+
+    /*
+     * In CASSANDRA-17668, JMX setters that did not throw standard exceptions were deprecated in favor of ones that do.
+     * For consistency purposes, the respective getter "getRepairSessionSpaceInMebibytes" was also deprecated and
+     * replaced by this method.
+     */
+    @Override
+    public int getRepairSessionSpaceInMiB()
+    {
+        return DatabaseDescriptor.getRepairSessionSpaceInMiB();
+    }
+
     @Override
     public int getConcurrentMerkleTreeRequests()
     {
@@ -329,7 +374,7 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
 
         for (ColumnFamilyStore cfs : SchemaArgsParser.parse(schemaArgs))
         {
-            String keyspace = cfs.keyspace.getName();
+            String keyspace = cfs.getKeyspaceName();
             Collection<Range<Token>> ranges = userRanges != null
                                               ? userRanges
                                               : StorageService.instance.getLocalReplicas(keyspace).ranges();
@@ -349,7 +394,7 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
                                               : null;
         for (ColumnFamilyStore cfs : SchemaArgsParser.parse(schemaArgs))
         {
-            String keyspace = cfs.keyspace.getName();
+            String keyspace = cfs.getKeyspaceName();
             Collection<Range<Token>> ranges = userRanges != null
                                               ? userRanges
                                               : StorageService.instance.getLocalReplicas(keyspace).ranges();
@@ -369,7 +414,7 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
                                               : null;
         for (ColumnFamilyStore cfs : SchemaArgsParser.parse(schemaArgs))
         {
-            String keyspace = cfs.keyspace.getName();
+            String keyspace = cfs.getKeyspaceName();
             Collection<Range<Token>> ranges = userRanges != null
                                               ? userRanges
                                               : StorageService.instance.getLocalReplicas(keyspace).ranges();
@@ -413,7 +458,7 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
         if (cfnames.length == 0)
             return null;
 
-        final RepairSession session = new RepairSession(parentRepairSession, validationScheduler, range, keyspace,
+        final RepairSession session = new RepairSession(ctx, validationScheduler, parentRepairSession, range, keyspace,
                                                         parallelismDegree, isIncremental, pullRepair,
                                                         previewKind, optimiseStreams, repairPaxos, paxosOnly, cfnames);
         repairs.getIfPresent(parentRepairSession).register(session.state);
@@ -448,8 +493,8 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
                IEndpointStateChangeSubscriber &
                IFailureDetectionEventListener> void registerOnFdAndGossip(final T task)
     {
-        gossiper.register(task);
-        failureDetector.registerFailureDetectionEventListener(task);
+        ctx.gossiper().register(task);
+        ctx.failureDetector().registerFailureDetectionEventListener(task);
 
         // unregister listeners at completion
         task.addListener(new Runnable()
@@ -459,8 +504,8 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
              */
             public void run()
             {
-                failureDetector.unregisterFailureDetectionEventListener(task);
-                gossiper.unregister(task);
+                ctx.failureDetector().unregisterFailureDetectionEventListener(task);
+                ctx.gossiper().unregister(task);
             }
         });
     }
@@ -496,9 +541,9 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
      * @param dataCenters         the data centers to involve in the repair
      * @return neighbors with whom we share the provided range
      */
-    public static EndpointsForRange getNeighbors(String keyspaceName, Iterable<Range<Token>> keyspaceLocalRanges,
-                                                 Range<Token> toRepair, Collection<String> dataCenters,
-                                                 Collection<String> hosts)
+    public EndpointsForRange getNeighbors(String keyspaceName, Iterable<Range<Token>> keyspaceLocalRanges,
+                                          Range<Token> toRepair, Collection<String> dataCenters,
+                                          Collection<String> hosts)
     {
         StorageService ss = StorageService.instance;
         EndpointsByRange replicaSets = ss.getRangeToAddressMap(keyspaceName);
@@ -521,7 +566,8 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
         if (rangeSuperSet == null || !replicaSets.containsKey(rangeSuperSet))
             return EndpointsForRange.empty(toRepair);
 
-        EndpointsForRange neighbors = replicaSets.get(rangeSuperSet).withoutSelf();
+        // same as withoutSelf(), but done this way for testing
+        EndpointsForRange neighbors = replicaSets.get(rangeSuperSet).filter(r -> !ctx.broadcastAddressAndPort().equals(r.endpoint()));
 
         if (dataCenters != null && !dataCenters.isEmpty())
         {
@@ -538,7 +584,7 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
                 try
                 {
                     final InetAddressAndPort endpoint = InetAddressAndPort.getByName(host.trim());
-                    if (endpoint.equals(FBUtilities.getBroadcastAddressAndPort()) || neighbors.endpoints().contains(endpoint))
+                    if (endpoint.equals(ctx.broadcastAddressAndPort()) || neighbors.endpoints().contains(endpoint))
                         specifiedHost.add(endpoint);
                 }
                 catch (UnknownHostException e)
@@ -547,7 +593,7 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
                 }
             }
 
-            if (!specifiedHost.contains(FBUtilities.getBroadcastAddressAndPort()))
+            if (!specifiedHost.contains(ctx.broadcastAddressAndPort()))
                 throw new IllegalArgumentException("The current host must be part of the repair");
 
             if (specifiedHost.size() <= 1)
@@ -558,7 +604,7 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
                 throw new IllegalArgumentException(String.format(msg, hosts, toRepair, neighbors));
             }
 
-            specifiedHost.remove(FBUtilities.getBroadcastAddressAndPort());
+            specifiedHost.remove(ctx.broadcastAddressAndPort());
             return neighbors.keep(specifiedHost);
         }
 
@@ -570,14 +616,14 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
      * incremental repairs, forced incremental repairs, and full repairs, the UNREPAIRED_SSTABLE value will prevent
      * sstables from being promoted to repaired or preserve the repairedAt/pendingRepair values, respectively.
      */
-    static long getRepairedAt(RepairOption options, boolean force)
+    long getRepairedAt(RepairOption options, boolean force)
     {
         // we only want to set repairedAt for incremental repairs including all replicas for a token range. For non-global incremental repairs, full repairs, the UNREPAIRED_SSTABLE value will prevent
         // sstables from being promoted to repaired or preserve the repairedAt/pendingRepair values, respectively. For forced repairs, repairedAt time is only set to UNREPAIRED_SSTABLE if we actually
         // end up skipping replicas
         if (options.isIncremental() && options.isGlobal() && !force)
         {
-            return currentTimeMillis();
+            return ctx.clock().currentTimeMillis();
         }
         else
         {
@@ -585,11 +631,11 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
         }
     }
 
-    public static boolean verifyCompactionsPendingThreshold(TimeUUID parentRepairSession, PreviewKind previewKind)
+    public boolean verifyCompactionsPendingThreshold(TimeUUID parentRepairSession, PreviewKind previewKind)
     {
         // Snapshot values so failure message is consistent with decision
-        int pendingCompactions = CompactionManager.instance.getPendingTasks();
-        int pendingThreshold = ActiveRepairService.instance.getRepairPendingCompactionRejectThreshold();
+        int pendingCompactions = ctx.compactionManager().getPendingTasks();
+        int pendingThreshold = getRepairPendingCompactionRejectThreshold();
         if (pendingCompactions > pendingThreshold)
         {
             logger.error("[{}] Rejecting incoming repair, pending compactions ({}) above threshold ({})",
@@ -599,54 +645,28 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
         return true;
     }
 
-    public TimeUUID prepareForRepair(TimeUUID parentRepairSession, InetAddressAndPort coordinator, Set<InetAddressAndPort> endpoints, RepairOption options, boolean isForcedRepair, List<ColumnFamilyStore> columnFamilyStores)
+    public Future<?> prepareForRepair(TimeUUID parentRepairSession, InetAddressAndPort coordinator, Set<InetAddressAndPort> endpoints, RepairOption options, boolean isForcedRepair, List<ColumnFamilyStore> columnFamilyStores)
     {
         if (!verifyCompactionsPendingThreshold(parentRepairSession, options.getPreviewKind()))
             failRepair(parentRepairSession, "Rejecting incoming repair, pending compactions above threshold"); // failRepair throws exception
 
         long repairedAt = getRepairedAt(options, isForcedRepair);
         registerParentRepairSession(parentRepairSession, coordinator, columnFamilyStores, options.getRanges(), options.isIncremental(), repairedAt, options.isGlobal(), options.getPreviewKind());
-        final CountDownLatch prepareLatch = newCountDownLatch(endpoints.size());
-        final AtomicBoolean status = new AtomicBoolean(true);
-        final Set<String> failedNodes = synchronizedSet(new HashSet<String>());
-        final AtomicInteger timeouts = new AtomicInteger(0);
-        RequestCallback callback = new RequestCallback()
-        {
-            @Override
-            public void onResponse(Message msg)
-            {
-                prepareLatch.decrement();
-            }
-
-            @Override
-            public void onFailure(InetAddressAndPort from, RequestFailureReason failureReason)
-            {
-                status.set(false);
-                failedNodes.add(from.toString());
-                if (failureReason == RequestFailureReason.TIMEOUT)
-                    timeouts.incrementAndGet();
-                prepareLatch.decrement();
-            }
-
-            @Override
-            public boolean invokeOnFailure()
-            {
-                return true;
-            }
-        };
+        AtomicInteger pending = new AtomicInteger(endpoints.size());
+        Set<String> failedNodes = synchronizedSet(new HashSet<>());
+        AsyncPromise<Void> promise = new AsyncPromise<>();
 
         List<TableId> tableIds = new ArrayList<>(columnFamilyStores.size());
         for (ColumnFamilyStore cfs : columnFamilyStores)
             tableIds.add(cfs.metadata.id);
 
         PrepareMessage message = new PrepareMessage(parentRepairSession, tableIds, options.getRanges(), options.isIncremental(), repairedAt, options.isGlobal(), options.getPreviewKind());
-        register(new ParticipateState(FBUtilities.getBroadcastAddressAndPort(), message));
+        register(new ParticipateState(ctx.clock(), ctx.broadcastAddressAndPort(), message));
         for (InetAddressAndPort neighbour : endpoints)
         {
-            if (FailureDetector.instance.isAlive(neighbour))
+            if (ctx.failureDetector().isAlive(neighbour))
             {
-                Message<RepairMessage> msg = out(PREPARE_MSG, message);
-                MessagingService.instance().sendWithCallback(msg, neighbour, callback);
+                sendPrepareWithRetries(parentRepairSession, pending, failedNodes, promise, neighbour, message);
             }
             else
             {
@@ -654,7 +674,7 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
                 // remaining ones go down, we still want to fail so we don't create repair sessions that can't complete
                 if (isForcedRepair && !options.isIncremental())
                 {
-                    prepareLatch.decrement();
+                    pending.decrementAndGet();
                 }
                 else
                 {
@@ -663,22 +683,66 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
                 }
             }
         }
-        try
-        {
-            if (!prepareLatch.await(getRpcTimeout(MILLISECONDS), MILLISECONDS) || timeouts.get() > 0)
-                failRepair(parentRepairSession, "Did not get replies from all endpoints.");
-        }
-        catch (InterruptedException e)
-        {
-            failRepair(parentRepairSession, "Interrupted while waiting for prepare repair response.");
-        }
+        // implement timeout to bound the runtime of the future
+        long timeoutMillis = getRepairRetrySpec().isEnabled() ? getRepairRpcTimeout(MILLISECONDS)
+                                                              : getRpcTimeout(MILLISECONDS);
+        ctx.optionalTasks().schedule(() -> {
+            if (promise.isDone())
+                return;
+            String errorMsg = "Did not get replies from all endpoints.";
+            if (promise.tryFailure(new RuntimeException(errorMsg)))
+                participateFailed(parentRepairSession, errorMsg);
+        }, timeoutMillis, MILLISECONDS);
 
-        if (!status.get())
+        return promise;
+    }
+
+    private void sendPrepareWithRetries(TimeUUID parentRepairSession,
+                                        AtomicInteger pending,
+                                        Set<String> failedNodes,
+                                        AsyncPromise<Void> promise,
+                                        InetAddressAndPort to,
+                                        RepairMessage msg)
+    {
+        RepairMessage.sendMessageWithRetries(ctx, notDone(promise), msg, PREPARE_MSG, to, new RequestCallback<>()
         {
-            failRepair(parentRepairSession, "Got negative replies from endpoints " + failedNodes);
-        }
+            @Override
+            public void onResponse(Message<Object> msg)
+            {
+                ack();
+            }
+
+            @Override
+            public void onFailure(InetAddressAndPort from, RequestFailureReason failureReason)
+            {
+                failedNodes.add(from.toString());
+                if (failureReason == RequestFailureReason.TIMEOUT)
+                {
+                    pending.set(-1);
+                    promise.setFailure(failRepairException(parentRepairSession, "Did not get replies from all endpoints."));
+                }
+                else
+                {
+                    ack();
+                }
+            }
+
+            private void ack()
+            {
+                if (pending.decrementAndGet() == 0)
+                {
+                    if (failedNodes.isEmpty())
+                    {
+                        promise.setSuccess(null);
+                    }
+                    else
+                    {
+                        promise.setFailure(failRepairException(parentRepairSession, "Got negative replies from endpoints " + failedNodes));
+                    }
+                }
+            }
+        });
 
-        return parentRepairSession;
     }
 
     /**
@@ -692,10 +756,9 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
         {
             try
             {
-                if (FailureDetector.instance.isAlive(endpoint))
+                if (ctx.failureDetector().isAlive(endpoint))
                 {
                     CleanupMessage message = new CleanupMessage(parentRepairSession);
-                    Message<CleanupMessage> msg = Message.out(Verb.CLEANUP_MSG, message);
 
                     RequestCallback loggingCallback = new RequestCallback()
                     {
@@ -713,8 +776,7 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
                                          "of messages like this.", parentRepairSession, endpoint);
                         }
                     };
-
-                    MessagingService.instance().sendWithCallback(msg, endpoint, loggingCallback);
+                    RepairMessage.sendMessageWithRetries(ctx, message, Verb.CLEANUP_MSG, endpoint, loggingCallback);
                 }
             }
             catch (Exception exc)
@@ -728,12 +790,22 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
     }
 
     private void failRepair(TimeUUID parentRepairSession, String errorMsg)
+    {
+        throw failRepairException(parentRepairSession, errorMsg);
+    }
+
+    private RuntimeException failRepairException(TimeUUID parentRepairSession, String errorMsg)
+    {
+        participateFailed(parentRepairSession, errorMsg);
+        removeParentRepairSession(parentRepairSession);
+        return new RuntimeException(errorMsg);
+    }
+
+    private void participateFailed(TimeUUID parentRepairSession, String errorMsg)
     {
         ParticipateState state = participate(parentRepairSession);
         if (state != null)
             state.phase.fail(errorMsg);
-        removeParentRepairSession(parentRepairSession);
-        throw new RuntimeException(errorMsg);
     }
 
     public synchronized void registerParentRepairSession(TimeUUID parentRepairSession, InetAddressAndPort coordinator, List<ColumnFamilyStore> columnFamilyStores, Collection<Range<Token>> ranges, boolean isIncremental, long repairedAt, boolean isGlobal, PreviewKind previewKind)
@@ -741,8 +813,8 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
         assert isIncremental || repairedAt == ActiveRepairService.UNREPAIRED_SSTABLE;
         if (!registeredForEndpointChanges)
         {
-            Gossiper.instance.register(this);
-            FailureDetector.instance.registerFailureDetectionEventListener(this);
+            ctx.gossiper().register(this);
+            ctx.failureDetector().registerFailureDetectionEventListener(this);
             registeredForEndpointChanges = true;
         }
 
@@ -781,25 +853,25 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
      */
     public synchronized ParentRepairSession removeParentRepairSession(TimeUUID parentSessionId)
     {
-        String snapshotName = parentSessionId.toString();
         ParentRepairSession session = parentRepairSessions.remove(parentSessionId);
         if (session == null)
             return null;
 
-        if (session.hasSnapshots)
+        String snapshotName = parentSessionId.toString();
+        if (session.hasSnapshots.get())
         {
             snapshotExecutor.submit(() -> {
                 logger.info("[repair #{}] Clearing snapshots for {}", parentSessionId,
                             session.columnFamilyStores.values()
                                                       .stream()
                                                       .map(cfs -> cfs.metadata().toString()).collect(Collectors.joining(", ")));
-                long startNanos = nanoTime();
+                long startNanos = ctx.clock().nanoTime();
                 for (ColumnFamilyStore cfs : session.columnFamilyStores.values())
                 {
                     if (cfs.snapshotExists(snapshotName))
                         cfs.clearSnapshot(snapshotName);
                 }
-                logger.info("[repair #{}] Cleared snapshots in {}ms", parentSessionId, TimeUnit.NANOSECONDS.toMillis(nanoTime() - startNanos));
+                logger.info("[repair #{}] Cleared snapshots in {}ms", parentSessionId, TimeUnit.NANOSECONDS.toMillis(ctx.clock().nanoTime() - startNanos));
             });
         }
         return session;
@@ -813,6 +885,13 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
 
         if (session == null)
         {
+            switch (message.verb())
+            {
+                case VALIDATION_RSP:
+                case SYNC_RSP:
+                    ctx.messaging().send(message.emptyResponse(), message.from());
+                    break;
+            }
             if (payload instanceof ValidationResponse)
             {
                 // The trees may be off-heap, and will therefore need to be released.
@@ -830,13 +909,10 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
         switch (message.verb())
         {
             case VALIDATION_RSP:
-                ValidationResponse validation = (ValidationResponse) payload;
-                session.validationComplete(desc, message.from(), validation.trees);
+                session.validationComplete(desc, (Message<ValidationResponse>) message);
                 break;
             case SYNC_RSP:
-                // one of replica is synced.
-                SyncResponse sync = (SyncResponse) payload;
-                session.syncComplete(desc, sync.nodes, sync.success, sync.summaries);
+                session.syncComplete(desc, (Message<SyncResponse>) message);
                 break;
             default:
                 break;
@@ -858,7 +934,7 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
         public final long repairedAt;
         public final InetAddressAndPort coordinator;
         public final PreviewKind previewKind;
-        public volatile boolean hasSnapshots = false;
+        public final AtomicBoolean hasSnapshots = new AtomicBoolean(false);
 
         public ParentRepairSession(InetAddressAndPort coordinator, List<ColumnFamilyStore> columnFamilyStores, Collection<Range<Token>> ranges, boolean isIncremental, long repairedAt, boolean isGlobal, PreviewKind previewKind)
         {
@@ -915,9 +991,9 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
                    '}';
         }
 
-        public void setHasSnapshots()
+        public boolean setHasSnapshots()
         {
-            hasSnapshots = true;
+            return hasSnapshots.compareAndSet(false, true);
         }
     }
 
@@ -1043,24 +1119,24 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
                     throw new RuntimeException(String.format("Insufficient live nodes to repair paxos for %s in %s for %s.\n" +
                                                              "There must be enough live nodes to satisfy EACH_QUORUM, but the following nodes are down: %s\n" +
                                                              "This check can be skipped by setting either the yaml property skip_paxos_repair_on_topology_change or " +
-                                                             "the system property cassandra.skip_paxos_repair_on_topology_change to false. The jmx property " +
+                                                             "the system property %s to false. The jmx property " +
                                                              "StorageService.SkipPaxosRepairOnTopologyChange can also be set to false to temporarily disable without " +
                                                              "restarting the node\n" +
                                                              "Individual keyspaces can be skipped with the yaml property skip_paxos_repair_on_topology_change_keyspaces, the" +
-                                                             "system property cassandra.skip_paxos_repair_on_topology_change_keyspaces, or temporarily with the jmx" +
+                                                             "system property %s, or temporarily with the jmx" +
                                                              "property StorageService.SkipPaxosRepairOnTopologyChangeKeyspaces\n" +
                                                              "Skipping this check can lead to paxos correctness issues",
-                                                             range, ksName, reason, downEndpoints));
+                                                             range, ksName, reason, downEndpoints, SKIP_PAXOS_REPAIR_ON_TOPOLOGY_CHANGE.getKey(), SKIP_PAXOS_REPAIR_ON_TOPOLOGY_CHANGE_KEYSPACES.getKey()));
                 }
                 EndpointsForToken pending = StorageService.instance.getTokenMetadata().pendingEndpointsForToken(range.right, ksName);
-                if (pending.size() > 1 && !Boolean.getBoolean("cassandra.paxos_repair_allow_multiple_pending_unsafe"))
+                if (pending.size() > 1 && !PAXOS_REPAIR_ALLOW_MULTIPLE_PENDING_UNSAFE.getBoolean())
                 {
                     throw new RuntimeException(String.format("Cannot begin paxos auto repair for %s in %s.%s, multiple pending endpoints exist for range (%s). " +
-                                                             "Set -Dcassandra.paxos_repair_allow_multiple_pending_unsafe=true to skip this check",
-                                                             range, table.keyspace, table.name, pending));
+                                                             "Set -D%s=true to skip this check",
+                                                             range, table.keyspace, table.name, pending, PAXOS_REPAIR_ALLOW_MULTIPLE_PENDING_UNSAFE.getKey()));
 
                 }
-                Future<Void> future = PaxosCleanup.cleanup(endpoints, table, Collections.singleton(range), false, repairCommandExecutor());
+                Future<Void> future = PaxosCleanup.cleanup(ctx, endpoints, table, Collections.singleton(range), false, repairCommandExecutor());
                 futures.add(future);
             }
         }
