@@ -56,6 +56,9 @@ public interface StorageProxyMBean
     public void setNativeTransportMaxConcurrentConnections(Long nativeTransportMaxConcurrentConnections);
     public Long getNativeTransportMaxConcurrentConnections();
 
+    public void setNativeTransportMaxConcurrentConnectionsPerIp(Long nativeTransportMaxConcurrentConnections);
+    public Long getNativeTransportMaxConcurrentConnectionsPerIp();
+
     public void reloadTriggerClasses();
 
     public long getReadRepairAttempted();
