@@ -139,4 +139,10 @@ public class PartitionRangeQueryPager extends AbstractQueryPager<PartitionRangeR
              ? new IncludingExcludingBounds<>(lastReturnedKey, bounds.right)
              : new ExcludingBounds<>(lastReturnedKey, bounds.right);
     }
+
+    @Override
+    public boolean isTopK()
+    {
+        return query.isTopK();
+    }
 }
