@@ -29,6 +29,7 @@ import com.google.common.collect.ImmutableList;
 
 import org.apache.cassandra.config.DatabaseDescriptor;
 import org.apache.cassandra.db.TypeSizes;
+import org.apache.cassandra.db.marshal.ByteBufferAccessor;
 import org.apache.cassandra.db.marshal.BytesType;
 import org.apache.cassandra.db.marshal.TupleType;
 import org.apache.cassandra.dht.Range;
@@ -180,7 +181,7 @@ public class PaxosRepairHistory
         Ballot[] ballotLowBounds = new Ballot[tuples.size()];
         for (int i = 0 ; i < tuples.size() ; ++i)
         {
-            ByteBuffer[] split = TYPE.split(tuples.get(i));
+            ByteBuffer[] split = TYPE.split(ByteBufferAccessor.instance, tuples.get(i));
             if (i < tokenInclusiveUpperBounds.length)
                 tokenInclusiveUpperBounds[i] = TOKEN_FACTORY.fromByteArray(split[0]);
             ballotLowBounds[i] = Ballot.deserialize(split[1]);
@@ -314,7 +315,7 @@ public class PaxosRepairHistory
     {
         public void serialize(PaxosRepairHistory history, DataOutputPlus out, int version) throws IOException
         {
-            out.writeUnsignedVInt(history.size());
+            out.writeUnsignedVInt32(history.size());
             for (int i = 0; i < history.size() ; ++i)
             {
                 Token.serializer.serialize(history.tokenInclusiveUpperBound[i], out, version);
@@ -325,7 +326,7 @@ public class PaxosRepairHistory
 
         public PaxosRepairHistory deserialize(DataInputPlus in, int version) throws IOException
         {
-            int size = (int) in.readUnsignedVInt();
+            int size = in.readUnsignedVInt32();
             Token[] tokenInclusiveUpperBounds = new Token[size];
             Ballot[] ballotLowBounds = new Ballot[size + 1];
             for (int i = 0; i < size; i++)
