@@ -184,7 +184,7 @@ public class UncommittedTableData
             if (table == null)
                 return Range.normalize(FULL_RANGE);
 
-            String ksName = table.keyspace.getName();
+            String ksName = table.getKeyspaceName();
             List<Range<Token>> ranges = StorageService.instance.getLocalAndPendingRanges(ksName);
 
             // don't filter anything if we're not aware of any locally replicated ranges
@@ -245,7 +245,6 @@ public class UncommittedTableData
         }
     }
 
-    @SuppressWarnings("resource")
     private static CloseableIterator<PaxosKeyState> merge(Collection<UncommittedDataFile> files, Collection<Range<Token>> ranges)
     {
         List<CloseableIterator<PaxosKeyState>> iterators = new ArrayList<>(files.size());
