@@ -29,7 +29,6 @@ import javax.annotation.Nullable;
 import com.google.common.base.Joiner;
 
 import org.apache.cassandra.config.DatabaseDescriptor;
-import org.apache.cassandra.db.ColumnFamilyStore;
 import org.apache.cassandra.db.DecoratedKey;
 import org.apache.cassandra.db.DeletionTime;
 import org.apache.cassandra.db.ReadCommand;
@@ -51,7 +50,6 @@ import org.apache.cassandra.index.Index;
 import org.apache.cassandra.locator.Endpoints;
 import org.apache.cassandra.locator.ReplicaPlan;
 import org.apache.cassandra.net.Message;
-import org.apache.cassandra.schema.IndexMetadata;
 import org.apache.cassandra.schema.TableMetadata;
 import org.apache.cassandra.service.reads.repair.NoopReadRepair;
 import org.apache.cassandra.service.reads.repair.ReadRepair;
@@ -121,39 +119,29 @@ public class DataResolver<E extends Endpoints<E>, P extends ReplicaPlan.ForRead<
             });
         }
 
-        if (!needsReplicaFilteringProtection())
-        {
-            ResolveContext context = new ResolveContext(replicas);
-            return resolveWithReadRepair(context,
-                                         i -> shortReadProtectedResponse(i, context, runOnShortRead),
-                                         UnaryOperator.identity(),
-                                         repairedDataTracker);
-        }
+        if (usesReplicaFilteringProtection())
+            return resolveWithReplicaFilteringProtection(replicas, repairedDataTracker);
 
-        return resolveWithReplicaFilteringProtection(replicas, repairedDataTracker);
+        ResolveContext context = new ResolveContext(replicas, true);
+        return resolveWithReadRepair(context,
+                                     i -> shortReadProtectedResponse(i, context, runOnShortRead),
+                                     UnaryOperator.identity(),
+                                     repairedDataTracker);
     }
 
-    private boolean needsReplicaFilteringProtection()
+    private boolean usesReplicaFilteringProtection()
     {
         if (command.rowFilter().isEmpty())
             return false;
 
-        IndexMetadata indexMetadata = command.indexMetadata();
+        if (command.isTopK())
+            return false;
 
-        if (indexMetadata == null || !indexMetadata.isCustom())
-        {
+        Index.QueryPlan queryPlan = command.indexQueryPlan();
+        if (queryPlan == null)
             return true;
-        }
 
-        ColumnFamilyStore cfs = ColumnFamilyStore.getIfExists(command.metadata().id);
-
-        assert cfs != null;
-
-        Index index = command.getIndex(cfs);
-
-        assert index != null;
-
-        return index.supportsReplicaFilteringProtection(command.rowFilter());
+        return queryPlan.supportsReplicaFilteringProtection(command.rowFilter());
     }
 
     private class ResolveContext
@@ -161,23 +149,43 @@ public class DataResolver<E extends Endpoints<E>, P extends ReplicaPlan.ForRead<
         private final E replicas;
         private final DataLimits.Counter mergedResultCounter;
 
-        private ResolveContext(E replicas)
+        /**
+         * @param replicas the collection of {@link Endpoints} involved in the query
+         * @param enforceLimits whether or not to enforce counter limits in this context
+         */
+        private ResolveContext(E replicas, boolean enforceLimits)
         {
             this.replicas = replicas;
             this.mergedResultCounter = command.limits().newCounter(command.nowInSec(),
                                                                    true,
                                                                    command.selectsFullPartition(),
                                                                    enforceStrictLiveness);
+
+            // In case of top-k query, do not trim reconciled rows here because QueryPlan#postProcessor()
+            // needs to compare all rows. Also avoid enforcing the limit if explicitly requested.
+            if (command.isTopK() || !enforceLimits)
+                this.mergedResultCounter.onlyCount();
         }
 
         private boolean needsReadRepair()
         {
+            // Each replica may return different estimated top-K rows, it doesn't mean data is not replicated.
+            // Even though top-K queries are limited to CL ONE & LOCAL-ONE, they use the ScanAllRangesCommandIterator
+            // that combines the separate replica plans of each data range into a single replica plan. This is an
+            // optimisation but can result in the number of replicas being > 1.
+            if (command.isTopK())
+                return false;
+
             return replicas.size() > 1;
         }
 
         private boolean needShortReadProtection()
         {
-            // If we have only one result, there is no read repair to do and we can't get short reads
+            // SRP doesn't make sense for top-k which needs to re-query replica with larger limit instead of fetching more partitions
+            if (command.isTopK())
+                return false;
+
+            // If we have only one result, there is no read repair to do, and we can't get short reads
             // Also, so-called "short reads" stems from nodes returning only a subset of the results they have for a
             // partition due to the limit, but that subset not being enough post-reconciliation. So if we don't have limit,
             // don't bother protecting against short reads.
@@ -221,56 +229,69 @@ public class DataResolver<E extends Endpoints<E>, P extends ReplicaPlan.ForRead<
         return resolveInternal(context, listener, responseProvider, preCountFilter);
     }
 
-    @SuppressWarnings("resource")
     private PartitionIterator resolveWithReplicaFilteringProtection(E replicas, RepairedDataTracker repairedDataTracker)
     {
         // Protecting against inconsistent replica filtering (some replica returning a row that is outdated but that
         // wouldn't be removed by normal reconciliation because up-to-date replica have filtered the up-to-date version
         // of that row) involves 3 main elements:
-        //   1) We combine short-read protection and a merge listener that identifies potentially "out-of-date"
-        //      rows to create an iterator that is guaranteed to produce enough valid row results to satisfy the query
-        //      limit if enough actually exist. A row is considered out-of-date if its merged from is non-empty and we
-        //      receive not response from at least one replica. In this case, it is possible that filtering at the
-        //      "silent" replica has produced a more up-to-date result.
+        //   1) We combine an unlimited, short-read-protected iterator of unfiltered partitions with a merge listener 
+        //      that identifies potentially "out-of-date" rows. A row is considered out-of-date if its merged form is 
+        //      non-empty, and we receive no response from some replica. It is also potentially out-of-date if there is
+        //      disagreement around the value of a single column in a non-empty row, and some replica provides no value
+        //      for that column. In either case, it is possible that filtering at the "silent" replica has produced a 
+        //      more up-to-date result.
         //   2) This iterator is passed to the standard resolution process with read-repair, but is first wrapped in a
         //      response provider that lazily "completes" potentially out-of-date rows by directly querying them on the
         //      replicas that were previously silent. As this iterator is consumed, it caches valid data for potentially
-        //      out-of-date rows, and this cached data is merged with the fetched data as rows are requested. If there
-        //      is no replica divergence, only rows in the partition being evalutated will be cached (then released
-        //      when the partition is consumed).
+        //      out-of-date rows, and this cached data is merged with the fetched data as rows are requested. Only rows 
+        //      in the partition being evalutated will be cached (then released when the partition is consumed).
         //   3) After a "complete" row is materialized, it must pass the row filter supplied by the original query
-        //      before it counts against the limit.
+        //      before it counts against the limit. If this "pre-count" filter causes a short read, additional rows
+        //      will be fetched from the first-phase iterator.
 
-        // We need separate contexts, as each context has his own counter
-        ResolveContext firstPhaseContext = new ResolveContext(replicas);
-        ResolveContext secondPhaseContext = new ResolveContext(replicas);
         ReplicaFilteringProtection<E> rfp = new ReplicaFilteringProtection<>(replicaPlan().keyspace(),
                                                                              command,
                                                                              replicaPlan().consistencyLevel(),
                                                                              requestTime,
-                                                                             firstPhaseContext.replicas,
+                                                                             replicas,
                                                                              DatabaseDescriptor.getCachedReplicaRowsWarnThreshold(),
                                                                              DatabaseDescriptor.getCachedReplicaRowsFailThreshold());
 
+        ResolveContext firstPhaseContext = new ResolveContext(replicas, false);
         PartitionIterator firstPhasePartitions = resolveInternal(firstPhaseContext,
                                                                  rfp.mergeController(),
                                                                  i -> shortReadProtectedResponse(i, firstPhaseContext, null),
-                                                                 UnaryOperator.identity());
+                                                                 null);
 
+        ResolveContext secondPhaseContext = new ResolveContext(replicas, true);
         PartitionIterator completedPartitions = resolveWithReadRepair(secondPhaseContext,
                                                                       i -> rfp.queryProtectedPartitions(firstPhasePartitions, i),
-                                                                      results -> command.rowFilter().filter(results, command.metadata(), command.nowInSec()),
+                                                                      preCountFilterForReplicaFilteringProtection(),
                                                                       repairedDataTracker);
 
         // Ensure that the RFP instance has a chance to record metrics when the iterator closes.
         return PartitionIterators.doOnClose(completedPartitions, firstPhasePartitions::close);
     }
 
-    @SuppressWarnings("resource")
+    private  UnaryOperator<PartitionIterator> preCountFilterForReplicaFilteringProtection()
+    {
+        // Key columns are immutable and should never need to participate in replica filtering
+        if (!command.rowFilter().hasNonKeyExpression())
+            return results -> results;
+
+        return results -> {
+            Index.Searcher searcher = command.indexSearcher();
+            // in case of "ALLOW FILTERING" without index
+            if (searcher == null)
+                return command.rowFilter().filter(results, command.metadata(), command.nowInSec());
+            return searcher.filterReplicaFilteringProtection(results);
+        };
+    }
+
     private PartitionIterator resolveInternal(ResolveContext context,
                                               UnfilteredPartitionIterators.MergeListener mergeListener,
                                               ResponseProvider responseProvider,
-                                              UnaryOperator<PartitionIterator> preCountFilter)
+                                              @Nullable UnaryOperator<PartitionIterator> preCountFilter)
     {
         int count = context.replicas.size();
         List<UnfilteredPartitionIterator> results = new ArrayList<>(count);
@@ -294,7 +315,10 @@ public class DataResolver<E extends Endpoints<E>, P extends ReplicaPlan.ForRead<
         UnfilteredPartitionIterator merged = UnfilteredPartitionIterators.merge(results, mergeListener);
         Filter filter = new Filter(command.nowInSec(), command.metadata().enforceStrictLiveness());
         FilteredPartitions filtered = FilteredPartitions.filter(merged, filter);
-        PartitionIterator counted = Transformation.apply(preCountFilter.apply(filtered), context.mergedResultCounter);
+
+        PartitionIterator counted = preCountFilter == null
+                                    ? filtered
+                                    : Transformation.apply(preCountFilter.apply(filtered), context.mergedResultCounter);
 
         return Transformation.apply(counted, new EmptyPartitionsDiscarder());
     }
@@ -364,11 +388,11 @@ public class DataResolver<E extends Endpoints<E>, P extends ReplicaPlan.ForRead<
                         }
                     }
 
-                    public Row onMergedRows(Row merged, Row[] versions)
+                    public void onMergedRows(Row merged, Row[] versions)
                     {
                         try
                         {
-                            return rowListener.onMergedRows(merged, versions);
+                            rowListener.onMergedRows(merged, versions);
                         }
                         catch (AssertionError e)
                         {
@@ -389,7 +413,7 @@ public class DataResolver<E extends Endpoints<E>, P extends ReplicaPlan.ForRead<
                     {
                         try
                         {
-                            // The code for merging range tombstones is a tad complex and we had the assertions there triggered
+                            // The code for merging range tombstones is a tad complex, and we had the assertions there triggered
                             // unexpectedly in a few occasions (CASSANDRA-13237, CASSANDRA-13719). It's hard to get insights
                             // when that happen without more context that what the assertion errors give us however, hence the
                             // catch here that basically gather as much as context as reasonable.
