@@ -54,17 +54,19 @@ public class ProgressInfo implements Serializable
     public final String fileName;
     public final Direction direction;
     public final long currentBytes;
+    public final long deltaBytes; // change from previous ProgressInfo
     public final long totalBytes;
 
-    public ProgressInfo(InetAddressAndPort peer, int sessionIndex, String fileName, Direction direction, long currentBytes, long totalBytes)
+    public ProgressInfo(InetAddressAndPort peer, int sessionIndex, String fileName, Direction direction,
+                        long currentBytes,  long deltaBytes, long totalBytes)
     {
-        assert totalBytes > 0;
 
         this.peer = peer;
         this.sessionIndex = sessionIndex;
         this.fileName = fileName;
         this.direction = direction;
         this.currentBytes = currentBytes;
+        this.deltaBytes = deltaBytes;
         this.totalBytes = totalBytes;
     }
 
@@ -76,6 +78,11 @@ public class ProgressInfo implements Serializable
         return currentBytes >= totalBytes;
     }
 
+    public int progressPercentage()
+    {
+        return totalBytes == 0 ? 100 : (int) ((100 * currentBytes) / totalBytes);
+    }
+
     /**
      * ProgressInfo is considered to be equal only when all attributes except currentBytes are equal.
      */
@@ -111,10 +118,10 @@ public class ProgressInfo implements Serializable
         StringBuilder sb = new StringBuilder(fileName);
         sb.append(" ").append(currentBytes);
         sb.append("/").append(totalBytes).append(" bytes ");
-        sb.append("(").append(currentBytes*100/totalBytes).append("%) ");
+        sb.append("(").append(progressPercentage()).append("%) ");
         sb.append(direction == Direction.OUT ? "sent to " : "received from ");
         sb.append("idx:").append(sessionIndex);
         sb.append(peer.toString(withPorts));
         return sb.toString();
     }
-}
+}
\ No newline at end of file
