@@ -22,6 +22,8 @@ import org.apache.cassandra.schema.TableId;
 import org.apache.cassandra.streaming.messages.OutgoingStreamMessage;
 import org.apache.cassandra.utils.FBUtilities;
 
+import static org.apache.cassandra.config.CassandraRelevantProperties.STREAM_HOOK;
+
 public interface StreamHook
 {
     public static final StreamHook instance = createHook();
@@ -32,7 +34,7 @@ public interface StreamHook
 
     static StreamHook createHook()
     {
-        String className =  System.getProperty("cassandra.stream_hook");
+        String className = STREAM_HOOK.getString();
         if (className != null)
         {
             return FBUtilities.construct(className, StreamHook.class.getSimpleName());
