@@ -31,6 +31,7 @@ import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Function;
 import com.google.common.cache.Cache;
 import com.google.common.cache.CacheBuilder;
+import com.google.common.cache.Weigher;
 import com.google.common.collect.Iterables;
 import com.google.common.collect.Sets;
 import com.google.common.util.concurrent.RateLimiter;
@@ -230,6 +231,8 @@ public class StreamManager implements StreamManagerMBean
         @Override
         public void onRegister(StreamResultFuture result)
         {
+            if (!DatabaseDescriptor.getStreamingStatsEnabled())
+                return;
             // reason for synchronized rather than states.get is to detect duplicates
             // streaming shouldn't be producing duplicates as that would imply a planId collision
             synchronized (states)
@@ -250,18 +253,35 @@ public class StreamManager implements StreamManagerMBean
         }
     };
 
+    protected void addStreamingStateAgain(StreamingState state)
+    {
+        if (!DatabaseDescriptor.getStreamingStatsEnabled())
+            return;
+        states.put(state.id(), state);
+    }
+
     public StreamManager()
     {
         DurationSpec.LongNanosecondsBound duration = DatabaseDescriptor.getStreamingStateExpires();
         long sizeBytes = DatabaseDescriptor.getStreamingStateSize().toBytes();
-        long numElements = sizeBytes / StreamingState.ELEMENT_SIZE;
-        logger.info("Storing streaming state for {} or for {} elements", duration, numElements);
+        logger.info("Storing streaming state for {} or for size {}", duration, sizeBytes);
         states = CacheBuilder.newBuilder()
                              .expireAfterWrite(duration.quantity(), duration.unit())
-                             .maximumSize(numElements)
+                             .maximumWeight(sizeBytes)
+                             .weigher(new StreamingStateWeigher())
                              .build();
     }
 
+    private static class StreamingStateWeigher implements Weigher<TimeUUID,StreamingState>
+    {
+        @Override
+        public int weigh(TimeUUID key, StreamingState val)
+        {
+            long costOfStreamingState = val.unsharedHeapSize() + TimeUUID.TIMEUUID_SIZE;
+            return Math.toIntExact(costOfStreamingState);
+        }
+    }
+
     public void start()
     {
         addListener(listener);
@@ -312,6 +332,30 @@ public class StreamManager implements StreamManagerMBean
         }));
     }
 
+    @Override
+    public boolean getStreamingStatsEnabled()
+    {
+        return DatabaseDescriptor.getStreamingStatsEnabled();
+    }
+
+    @Override
+    public void setStreamingStatsEnabled(boolean streamingStatsEnabled)
+    {
+        DatabaseDescriptor.setStreamingStatsEnabled(streamingStatsEnabled);
+    }
+
+    @Override
+    public String getStreamingSlowEventsLogTimeout()
+    {
+        return DatabaseDescriptor.getStreamingSlowEventsLogTimeout().toString();
+    }
+
+    @Override
+    public void setStreamingSlowEventsLogTimeout(String value)
+    {
+        DatabaseDescriptor.setStreamingSlowEventsLogTimeout(value);
+    }
+
     public void registerInitiator(final StreamResultFuture result)
     {
         result.addEventListener(notifier);
@@ -421,6 +465,17 @@ public class StreamManager implements StreamManagerMBean
         return streamResultFuture.getSession(peer, sessionIndex);
     }
 
+    public long getTotalRemainingOngoingBytes()
+    {
+        long total = 0;
+        for (StreamResultFuture fut : Iterables.concat(initiatorStreams.values(), followerStreams.values()))
+        {
+            for (SessionInfo sessionInfo : fut.getCurrentState().sessions)
+                total += sessionInfo.getTotalSizeToReceive() - sessionInfo.getTotalSizeReceived();
+        }
+        return total;
+    }
+
     public interface StreamListener
     {
         default void onRegister(StreamResultFuture result) {}
