@@ -143,8 +143,10 @@ public class StreamTransferTask extends StreamTask
             }
         }
         streams.clear();
-        if (fail != null)
-            Throwables.propagate(fail);
+        if (fail != null) {
+            Throwables.throwIfUnchecked(fail);
+            throw new RuntimeException(fail);
+        }
     }
 
     public synchronized int getTotalNumberOfFiles()
