@@ -29,7 +29,6 @@ import java.net.UnknownHostException;
 import java.util.HashSet;
 import java.util.Set;
 
-import com.google.common.base.Throwables;
 import com.google.common.net.HostAndPort;
 import org.apache.commons.cli.CommandLine;
 import org.apache.commons.cli.CommandLineParser;
@@ -63,7 +62,8 @@ public class LoaderOptions
     public static final String NOPROGRESS_OPTION = "no-progress";
     public static final String NATIVE_PORT_OPTION = "port";
     public static final String STORAGE_PORT_OPTION = "storage-port";
-    @Deprecated
+    /** @deprecated See CASSANDRA-17602 */
+    @Deprecated(since = "5.0")
     public static final String SSL_STORAGE_PORT_OPTION = "ssl-storage-port";
     public static final String USER_OPTION = "username";
     public static final String PASSWD_OPTION = "password";
@@ -78,21 +78,23 @@ public class LoaderOptions
      * provide options instead of using these constant fields.
      * @deprecated Use {@code throttle-mib} instead
      */
-    @Deprecated
+    /** @deprecated See CASSANDRA-17677 */
+    @Deprecated(since = "5.0")
     public static final String THROTTLE_MBITS = "throttle";
     public static final String THROTTLE_MEBIBYTES = "throttle-mib";
     /**
      * Inter-datacenter throttle defined in megabits per second. CASSANDRA-10637 introduced a builder and is the
      * preferred way to provide options instead of using these constant fields.
-     * @deprecated Use {@code inter-dc-throttle-mib} instead
+     * @deprecated Use {@code inter-dc-throttle-mib} instead. See CASSANDRA-17677
      */
-    @Deprecated
+    @Deprecated(since = "5.0")
     public static final String INTER_DC_THROTTLE_MBITS = "inter-dc-throttle";
     public static final String INTER_DC_THROTTLE_MEBIBYTES = "inter-dc-throttle-mib";
     public static final String ENTIRE_SSTABLE_THROTTLE_MEBIBYTES = "entire-sstable-throttle-mib";
     public static final String ENTIRE_SSTABLE_INTER_DC_THROTTLE_MEBIBYTES = "entire-sstable-inter-dc-throttle-mib";
     public static final String TOOL_NAME = "sstableloader";
     public static final String TARGET_KEYSPACE = "target-keyspace";
+    public static final String TARGET_TABLE = "target-table";
 
     /* client encryption options */
     public static final String SSL_TRUSTSTORE = "truststore";
@@ -124,6 +126,7 @@ public class LoaderOptions
     public final Set<InetSocketAddress> hosts;
     public final Set<InetAddressAndPort> ignores;
     public final String targetKeyspace;
+    public final String targetTable;
 
     LoaderOptions(Builder builder)
     {
@@ -147,6 +150,7 @@ public class LoaderOptions
         hosts = builder.hosts;
         ignores = builder.ignores;
         targetKeyspace = builder.targetKeyspace;
+        targetTable = builder.targetTable;
     }
 
     static class Builder
@@ -175,6 +179,7 @@ public class LoaderOptions
         Set<InetSocketAddress> hosts = new HashSet<>();
         Set<InetAddressAndPort> ignores = new HashSet<>();
         String targetKeyspace;
+        String targetTable;
 
         Builder()
         {
@@ -198,7 +203,7 @@ public class LoaderOptions
             }
             catch (UnknownHostException e)
             {
-                Throwables.propagate(e);
+                throw new RuntimeException(e);
             }
 
             return new LoaderOptions(this);
@@ -258,7 +263,8 @@ public class LoaderOptions
             return this;
         }
 
-        @Deprecated
+        /** @deprecated See CASSANDRA-17677 */
+        @Deprecated(since = "5.0")
         public Builder throttle(int throttleMegabits)
         {
             this.throttleBytes = (long) DataRateSpec.LongBytesPerSecondBound.megabitsPerSecondInBytesPerSecond(throttleMegabits).toBytesPerSecond();
@@ -277,7 +283,8 @@ public class LoaderOptions
             return this;
         }
 
-        @Deprecated
+        /** @deprecated See CASSANDRA-17677 */
+        @Deprecated(since = "5.0")
         public Builder interDcThrottle(int interDcThrottle)
         {
             return interDcThrottleMegabits(interDcThrottle);
@@ -289,7 +296,8 @@ public class LoaderOptions
             return this;
         }
 
-        @Deprecated
+        /** @deprecated See CASSANDRA-17677 */
+        @Deprecated(since = "5.0")
         public Builder entireSSTableThrottle(int entireSSTableThrottle)
         {
             this.entireSSTableThrottleMebibytes = entireSSTableThrottle;
@@ -302,7 +310,8 @@ public class LoaderOptions
             return this;
         }
 
-        @Deprecated
+        /** @deprecated See CASSANDRA-17677 */
+        @Deprecated(since = "5.0")
         public Builder entireSSTableInterDcThrottle(int entireSSTableInterDcThrottle)
         {
             this.entireSSTableInterDcThrottleMebibytes = entireSSTableInterDcThrottle;
@@ -315,7 +324,8 @@ public class LoaderOptions
             return this;
         }
 
-        @Deprecated
+        /** @deprecated See CASSANDRA-17602 */
+        @Deprecated(since = "5.0")
         public Builder sslStoragePort(int sslStoragePort)
         {
             this.sslStoragePort = storagePort;
@@ -340,7 +350,8 @@ public class LoaderOptions
             return this;
         }
 
-        @Deprecated
+        /** @deprecated See CASSANDRA-7544 */
+        @Deprecated(since = "4.0")
         public Builder hosts(Set<InetAddress> hosts)
         {
             this.hostsArg.addAll(hosts);
@@ -389,6 +400,18 @@ public class LoaderOptions
             return this;
         }
 
+        public Builder targetKeyspace(String keyspace)
+        {
+            this.targetKeyspace = keyspace;
+            return this;
+        }
+
+        public Builder targetTable(String table)
+        {
+            this.targetKeyspace = table;
+            return this;
+        }
+
         public Builder parseArgs(String cmdArgs[])
         {
             CommandLineParser parser = new GnuParser();
@@ -657,10 +680,16 @@ public class LoaderOptions
                 {
                     targetKeyspace = cmd.getOptionValue(TARGET_KEYSPACE);
                     if (StringUtils.isBlank(targetKeyspace))
-                    {
                         errorMsg("Empty keyspace is not supported.", options);
-                    }
                 }
+
+                if (cmd.hasOption(TARGET_TABLE))
+                {
+                    targetTable = cmd.getOptionValue(TARGET_TABLE);
+                    if (StringUtils.isBlank(targetTable))
+                        errorMsg("Empty table is not supported.", options);
+                }
+
                 return this;
             }
             catch (ParseException | ConfigurationException | MalformedURLException e)
@@ -771,6 +800,7 @@ public class LoaderOptions
         options.addOption("ciphers", SSL_CIPHER_SUITES, "CIPHER-SUITES", "Client SSL: comma-separated list of encryption suites to use");
         options.addOption("f", CONFIG_PATH, "path to config file", "cassandra.yaml file path for streaming throughput and client/server SSL.");
         options.addOption("k", TARGET_KEYSPACE, "target keyspace name", "target keyspace name");
+        options.addOption("tb", TARGET_TABLE, "target table name", "target table name");
         return options;
     }
 
