@@ -52,6 +52,8 @@ import com.google.common.base.Throwables;
 
 import org.apache.cassandra.locator.EndpointSnitchInfoMBean;
 import org.apache.cassandra.tools.nodetool.*;
+import org.apache.cassandra.tools.nodetool.GuardrailsConfigCommand.GetGuardrailsConfig;
+import org.apache.cassandra.tools.nodetool.GuardrailsConfigCommand.SetGuardrailsConfig;
 import org.apache.cassandra.utils.FBUtilities;
 
 import com.google.common.collect.Maps;
@@ -97,6 +99,7 @@ public class NodeTool
                 Assassinate.class,
                 CassHelp.class,
                 CIDRFilteringStats.class,
+                CheckTokenMetadata.class,
                 Cleanup.class,
                 ClearSnapshot.class,
                 ClientStats.class,
@@ -144,6 +147,7 @@ public class NodeTool
                 GetDefaultKeyspaceRF.class,
                 GetEndpoints.class,
                 GetFullQueryLog.class,
+                GetGuardrailsConfig.class,
                 GetInterDCStreamThroughput.class,
                 GetLoggingLevels.class,
                 GetMaxHintWindow.class,
@@ -205,6 +209,7 @@ public class NodeTool
                 SetConcurrentCompactors.class,
                 SetConcurrentViewBuilders.class,
                 SetDefaultKeyspaceRF.class,
+                SetGuardrailsConfig.class,
                 SetHintedHandoffThrottleInKB.class,
                 SetInterDCStreamThroughput.class,
                 SetLoggingLevel.class,
