@@ -21,15 +21,16 @@ import java.io.PrintStream;
 import java.util.Map;
 import java.util.Set;
 
-import org.apache.cassandra.config.Schema;
-import org.apache.cassandra.db.lifecycle.LifecycleTransaction;
 import org.apache.cassandra.db.ColumnFamilyStore;
 import org.apache.cassandra.db.Directories;
 import org.apache.cassandra.db.Keyspace;
+import org.apache.cassandra.db.lifecycle.LifecycleTransaction;
 import org.apache.cassandra.io.sstable.Component;
 import org.apache.cassandra.io.sstable.Descriptor;
-import org.apache.cassandra.io.sstable.metadata.MetadataType;
+import org.apache.cassandra.io.sstable.format.SSTableFormat.Components;
+import org.apache.cassandra.io.sstable.format.StatsComponent;
 import org.apache.cassandra.io.sstable.metadata.StatsMetadata;
+import org.apache.cassandra.schema.Schema;
 import org.apache.cassandra.utils.JVMStabilityInspector;
 
 /**
@@ -65,12 +66,12 @@ public class SSTableLevelResetter
         try
         {
             // load keyspace descriptions.
-            Schema.instance.loadFromDisk(false);
+            Schema.instance.loadFromDisk();
 
             String keyspaceName = args[1];
             String columnfamily = args[2];
             // validate columnfamily
-            if (Schema.instance.getCFMetaData(keyspaceName, columnfamily) == null)
+            if (Schema.instance.getTableMetadataRef(keyspaceName, columnfamily) == null)
             {
                 System.err.println("ColumnFamily not found: " + keyspaceName + "/" + columnfamily);
                 System.exit(1);
@@ -90,19 +91,19 @@ public class SSTableLevelResetter
             boolean foundSSTable = false;
             for (Map.Entry<Descriptor, Set<Component>> sstable : lister.list().entrySet())
             {
-                if (sstable.getValue().contains(Component.STATS))
+                if (sstable.getValue().contains(Components.STATS))
                 {
                     foundSSTable = true;
                     Descriptor descriptor = sstable.getKey();
-                    StatsMetadata metadata = (StatsMetadata) descriptor.getMetadataSerializer().deserialize(descriptor, MetadataType.STATS);
+                    StatsMetadata metadata = StatsComponent.load(descriptor).statsMetadata();
                     if (metadata.sstableLevel > 0)
                     {
-                        out.println("Changing level from " + metadata.sstableLevel + " to 0 on " + descriptor.filenameFor(Component.DATA));
+                        out.println("Changing level from " + metadata.sstableLevel + " to 0 on " + descriptor.fileFor(Components.DATA));
                         descriptor.getMetadataSerializer().mutateLevel(descriptor, 0);
                     }
                     else
                     {
-                        out.println("Skipped " + descriptor.filenameFor(Component.DATA) + " since it is already on level 0");
+                        out.println("Skipped " + descriptor.fileFor(Components.DATA) + " since it is already on level 0");
                     }
                 }
             }
