@@ -26,20 +26,8 @@ import java.nio.file.attribute.FileTime;
 import java.util.Arrays;
 import java.util.List;
 
-import org.apache.cassandra.io.sstable.Component;
 import org.apache.cassandra.io.sstable.Descriptor;
-
-/**
- * Set repairedAt status on a given set of sstables.
- *
- * If you pass --is-repaired, it will set the repairedAt time to the last modified time.
- *
- * If you know you ran repair 2 weeks ago, you can do something like
- *
- * {@code
- * sstablerepairset --is-repaired -f <(find /var/lib/cassandra/data/.../ -iname "*Data.db*" -mtime +14)
- * }
- */
+import org.apache.cassandra.io.sstable.format.SSTableFormat.Components;
 import org.apache.cassandra.io.util.File;
 
 public class SSTableRepairedAtSetter
@@ -81,7 +69,7 @@ public class SSTableRepairedAtSetter
 
         for (String fname: fileNames)
         {
-            Descriptor descriptor = Descriptor.fromFilename(fname);
+            Descriptor descriptor = Descriptor.fromFileWithComponent(new File(fname), false).left;
             if (!descriptor.version.isCompatible())
             {
                 System.err.println("SSTable " + fname + " is in a old and unsupported format");
@@ -90,7 +78,7 @@ public class SSTableRepairedAtSetter
 
             if (setIsRepaired)
             {
-                FileTime f = Files.getLastModifiedTime(new File(descriptor.filenameFor(Component.DATA)).toPath());
+                FileTime f = Files.getLastModifiedTime(descriptor.fileFor(Components.DATA).toPath());
                 descriptor.getMetadataSerializer().mutateRepairMetadata(descriptor, f.toMillis(), null, false);
             }
             else
