@@ -39,17 +39,19 @@ import org.apache.cassandra.db.ColumnFamilyStore;
 import org.apache.cassandra.db.Directories;
 import org.apache.cassandra.db.Keyspace;
 import org.apache.cassandra.db.compaction.CompactionManager;
-import org.apache.cassandra.db.compaction.Verifier;
 import org.apache.cassandra.dht.Murmur3Partitioner;
 import org.apache.cassandra.dht.Range;
 import org.apache.cassandra.dht.Token;
 import org.apache.cassandra.io.sstable.Component;
 import org.apache.cassandra.io.sstable.Descriptor;
+import org.apache.cassandra.io.sstable.IVerifier;
 import org.apache.cassandra.io.sstable.format.SSTableReader;
 import org.apache.cassandra.schema.Schema;
 import org.apache.cassandra.utils.JVMStabilityInspector;
 import org.apache.cassandra.utils.OutputHandler;
+import org.apache.cassandra.utils.Throwables;
 
+import static org.apache.cassandra.config.CassandraRelevantProperties.TEST_UTIL_ALLOW_TOOL_REINIT_FOR_TEST;
 import static org.apache.cassandra.tools.BulkLoader.CmdLineOptions;
 
 public class StandaloneVerifier
@@ -62,19 +64,28 @@ public class StandaloneVerifier
     private static final String CHECK_VERSION = "check_version";
     private static final String MUTATE_REPAIR_STATUS = "mutate_repair_status";
     private static final String QUICK = "quick";
+    private static final String FORCE = "force";
     private static final String TOKEN_RANGE = "token_range";
 
     public static void main(String args[])
     {
         Options options = Options.parseArgs(args);
+        if (!options.force)
+        {
+            System.err.println("verify will not run without -f or --force. See CASSANDRA-17017 for details.");
+            Options.printUsage(Options.getCmdLineOptions());
+            System.exit(1);
+        }
         initDatabaseDescriptorForTool();
 
         System.out.println("sstableverify using the following options: " + options);
 
+        List<SSTableReader> sstables = new ArrayList<>();
+        int exitCode = 0;
         try
         {
             // load keyspace descriptions.
-            Schema.instance.loadFromDisk(false);
+            Schema.instance.loadFromDisk();
 
             boolean hasFailed = false;
 
@@ -90,13 +101,11 @@ public class StandaloneVerifier
             OutputHandler handler = new OutputHandler.SystemOutput(options.verbose, options.debug);
             Directories.SSTableLister lister = cfs.getDirectories().sstableLister(Directories.OnTxnErr.THROW).skipTemporary(true);
 
-            List<SSTableReader> sstables = new ArrayList<>();
-
             // Verify sstables
             for (Map.Entry<Descriptor, Set<Component>> entry : lister.list().entrySet())
             {
                 Set<Component> components = entry.getValue();
-                if (!components.contains(Component.DATA) || !components.contains(Component.PRIMARY_INDEX))
+                if (!components.containsAll(entry.getKey().getFormat().primaryComponents()))
                     continue;
 
                 try
@@ -110,45 +119,56 @@ public class StandaloneVerifier
                     System.err.println(String.format("Error Loading %s: %s", entry.getKey(), e.getMessage()));
                     if (options.debug)
                         e.printStackTrace(System.err);
-                    System.exit(1);
+                    exitCode = 1;
+                    return;
                 }
             }
-            Verifier.Options verifyOptions = Verifier.options().invokeDiskFailurePolicy(false)
-                                                               .extendedVerification(options.extended)
-                                                               .checkVersion(options.checkVersion)
-                                                               .mutateRepairStatus(options.mutateRepairStatus)
-                                                               .checkOwnsTokens(!options.tokens.isEmpty())
-                                                               .tokenLookup(ignore -> options.tokens)
-                                                               .build();
+            IVerifier.Options verifyOptions = IVerifier.options().invokeDiskFailurePolicy(false)
+                                                       .extendedVerification(options.extended)
+                                                       .checkVersion(options.checkVersion)
+                                                       .mutateRepairStatus(options.mutateRepairStatus)
+                                                       .checkOwnsTokens(!options.tokens.isEmpty())
+                                                       .tokenLookup(ignore -> options.tokens)
+                                                       .build();
             handler.output("Running verifier with the following options: " + verifyOptions);
             for (SSTableReader sstable : sstables)
             {
-                try (Verifier verifier = new Verifier(cfs, sstable, handler, true, verifyOptions))
+                try (IVerifier verifier = sstable.getVerifier(cfs, handler, true, verifyOptions))
                 {
                     verifier.verify();
                 }
                 catch (Exception e)
                 {
-                    handler.warn(String.format("Error verifying %s: %s", sstable, e.getMessage()), e);
+                    handler.warn(e, String.format("Error verifying %s: %s", sstable, e.getMessage()));
                     hasFailed = true;
                 }
             }
 
             CompactionManager.instance.finishCompactionsAndShutdown(5, TimeUnit.MINUTES);
 
-            System.exit( hasFailed ? 1 : 0 ); // We need that to stop non daemonized threads
+            for (SSTableReader reader : sstables)
+                Throwables.perform((Throwable) null, () -> reader.selfRef().close());
+
+            exitCode = hasFailed ? 1 : 0; // We need that to stop non daemonized threads
         }
         catch (Exception e)
         {
             System.err.println(e.getMessage());
             if (options.debug)
                 e.printStackTrace(System.err);
-            System.exit(1);
+            exitCode = 1;
+        }
+        finally
+        {
+            for (SSTableReader reader : sstables)
+                Throwables.perform((Throwable) null, () -> reader.selfRef().close());
+
+            System.exit(exitCode);
         }
     }
 
     private static void initDatabaseDescriptorForTool() {
-        if (Boolean.getBoolean(Util.ALLOW_TOOL_REINIT_FOR_TEST))
+        if (TEST_UTIL_ALLOW_TOOL_REINIT_FOR_TEST.getBoolean())
             DatabaseDescriptor.toolInitialization(false); //Necessary for testing
         else
             Util.initDatabaseDescriptor();
@@ -165,6 +185,7 @@ public class StandaloneVerifier
         public boolean checkVersion;
         public boolean mutateRepairStatus;
         public boolean quick;
+        public boolean force;
         public Collection<Range<Token>> tokens;
 
         private Options(String keyspaceName, String cfName)
@@ -190,8 +211,8 @@ public class StandaloneVerifier
                 String[] args = cmd.getArgs();
                 if (args.length != 2)
                 {
-                    String msg = args.length < 2 ? "Missing arguments" : "Too many arguments";
-                    System.err.println(msg);
+                    String prefix = args.length < 2 ? "Missing" : "Too many";
+                    System.err.println(prefix + " arguments");
                     printUsage(options);
                     System.exit(1);
                 }
@@ -207,6 +228,7 @@ public class StandaloneVerifier
                 opts.checkVersion = cmd.hasOption(CHECK_VERSION);
                 opts.mutateRepairStatus = cmd.hasOption(MUTATE_REPAIR_STATUS);
                 opts.quick = cmd.hasOption(QUICK);
+                opts.force = cmd.hasOption(FORCE);
 
                 if (cmd.hasOption(TOKEN_RANGE))
                 {
@@ -260,17 +282,24 @@ public class StandaloneVerifier
             options.addOption("c",  CHECK_VERSION,         "make sure sstables are the latest version");
             options.addOption("r",  MUTATE_REPAIR_STATUS,  "don't mutate repair status");
             options.addOption("q",  QUICK,                 "do a quick check, don't read all data");
+            options.addOption("f",  FORCE,                 "force verify - see CASSANDRA-17017");
             options.addOptionList("t", TOKEN_RANGE, "range", "long token range of the format left,right. This may be provided multiple times to define multiple different ranges");
             return options;
         }
 
         public static void printUsage(CmdLineOptions options)
         {
-            String usage = String.format("%s [options] <keyspace> <column_family>", TOOL_NAME);
+            String usage = String.format("%s [options] <keyspace> <column_family> force", TOOL_NAME);
             StringBuilder header = new StringBuilder();
             header.append("--\n");
             header.append("Verify the sstable for the provided table." );
             header.append("\n--\n");
+            header.append("NOTE: There are significant risks associated with using this tool; it likely doesn't do what " +
+                          "you expect and there are known edge cases. You must provide a -f or --force argument in " +
+                          "order to allow usage of the tool -> see CASSANDRA-9947 and CASSANDRA-17017 for known risks.\n");
+            header.append("https://issues.apache.org/jira/browse/CASSANDRA-9947\n");
+            header.append("https://issues.apache.org/jira/browse/CASSANDRA-17017");
+            header.append("\n--\n");
             header.append("Options are:");
             new HelpFormatter().printHelp(usage, header.toString(), options, "");
         }
