@@ -22,17 +22,27 @@ import io.airlift.airline.Command;
 import java.io.IOError;
 import java.io.IOException;
 
+import io.airlift.airline.Option;
 import org.apache.cassandra.tools.NodeProbe;
 import org.apache.cassandra.tools.NodeTool.NodeToolCmd;
 
+import static org.apache.cassandra.config.CassandraRelevantProperties.RESET_BOOTSTRAP_PROGRESS;
+
 @Command(name = "resume", description = "Resume bootstrap streaming")
 public class BootstrapResume extends NodeToolCmd
 {
+    @Option(title = "force",
+            name = { "-f", "--force"},
+            description = "Use --force to resume bootstrap regardless of cassandra.reset_bootstrap_progress environment variable. WARNING: This is potentially dangerous, see CASSANDRA-17679")
+    boolean force = false;
+
     @Override
     protected void execute(NodeProbe probe)
     {
         try
         {
+            if ((!RESET_BOOTSTRAP_PROGRESS.isPresent() || RESET_BOOTSTRAP_PROGRESS.getBoolean()) && !force)
+                throw new RuntimeException("'nodetool bootstrap resume' is disabled.");
             probe.resumeBootstrap(probe.output().out);
         }
         catch (IOException e)
