@@ -23,6 +23,8 @@ import io.airlift.airline.Command;
 
 import java.util.ArrayList;
 import java.util.List;
+import java.util.Map;
+import java.util.Set;
 
 import io.airlift.airline.Option;
 import org.apache.cassandra.tools.NodeProbe;
@@ -36,6 +38,9 @@ public class GetSSTables extends NodeToolCmd
            description = "Specify the key in hexadecimal string format")
     private boolean hexFormat = false;
 
+    @Option(name={"-l", "--show-levels"}, description="If the table is using leveled compaction the level of each sstable will be included in the output (Default: false)")
+    private boolean showLevels = false;
+
     @Arguments(usage = "<keyspace> <cfname> <key>", description = "The keyspace, the column family, and the key")
     private List<String> args = new ArrayList<>();
 
@@ -47,10 +52,19 @@ public class GetSSTables extends NodeToolCmd
         String cf = args.get(1);
         String key = args.get(2);
 
-        List<String> sstables = probe.getSSTables(ks, cf, key, hexFormat);
-        for (String sstable : sstables)
+        if (showLevels && probe.isLeveledCompaction(ks, cf))
         {
-            probe.output().out.println(sstable);
+            Map<Integer, Set<String>> sstables = probe.getSSTablesWithLevel(ks, cf, key, hexFormat);
+            for (Integer level : sstables.keySet())
+                for (String sstable : sstables.get(level))
+                    probe.output().out.println(level + ": " + sstable);
+        } else
+        {
+            List<String> sstables = probe.getSSTables(ks, cf, key, hexFormat);
+            for (String sstable : sstables)
+            {
+                probe.output().out.println(sstable);
+            }
         }
     }
 }
