@@ -23,7 +23,6 @@ import java.lang.reflect.Method;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
-import java.util.HashMap;
 import java.util.HashSet;
 import java.util.LinkedHashMap;
 import java.util.LinkedHashSet;
@@ -129,12 +128,12 @@ public abstract class GuardrailsConfigCommand extends NodeTool.NodeToolCmd
             allGetters.putAll(thresholds);
 
             return allGetters.entrySet()
-                                   .stream()
-                                   .sorted(Map.Entry.comparingByKey())
-                                   .collect(Collectors.toMap(Map.Entry::getKey,
-                                                             Map.Entry::getValue,
-                                                             (e1, e2) -> e1,
-                                                             LinkedHashMap::new));
+                             .stream()
+                             .sorted(Map.Entry.comparingByKey())
+                             .collect(Collectors.toMap(Map.Entry::getKey,
+                                                       Map.Entry::getValue,
+                                                       (e1, e2) -> e1,
+                                                       LinkedHashMap::new));
         }
 
         @Override
@@ -192,9 +191,9 @@ public abstract class GuardrailsConfigCommand extends NodeTool.NodeToolCmd
             String snakeCaseName = args.get(0);
 
             Method setter = getAllSetters(probe).entrySet().stream()
-                                                    .findFirst()
-                                                    .map(o -> o.getValue().get(0))
-                                                    .orElseThrow(() -> new IllegalStateException(format("Guardrail %s not found.", snakeCaseName)));
+                                                .findFirst()
+                                                .map(o -> o.getValue().get(0))
+                                                .orElseThrow(() -> new IllegalStateException(format("Guardrail %s not found.", snakeCaseName)));
 
             sanitizeArguments(setter, args);
             validateArguments(setter, snakeCaseName, args);
@@ -349,12 +348,17 @@ public abstract class GuardrailsConfigCommand extends NodeTool.NodeToolCmd
      * Special map for methods which do not adhere to camel-case convention precisely.
      * These will be translated manually.
      */
-    private static final Map<String, String> toSnakeCaseTranslationMap = new HashMap<String, String>()
-    {{
-        put("FieldsPerUDTFailThreshold", "fields_per_udt_fail_threshold");
-        put("FieldsPerUDTWarnThreshold", "fields_per_udt_warn_threshold");
-        put("FieldsPerUDTThreshold", "fields_per_udt_threshold");
-    }};
+    private static final Map<String, String> toSnakeCaseTranslationMap = Map.of("ZeroTTLOnTWCSEnabled", "zero_ttl_on_twcs_enabled",
+                                                                                "ZeroTTLOnTWCSWarned", "zero_ttl_on_twcs_warned",
+                                                                                "FieldsPerUDTFailThreshold", "fields_per_udt_fail_threshold",
+                                                                                "FieldsPerUDTWarnThreshold", "fields_per_udt_warn_threshold",
+                                                                                "FieldsPerUDTThreshold", "fields_per_udt_threshold",
+                                                                                "SimpleStrategyEnabled", "simplestrategy_enabled",
+                                                                                "NonPartitionRestrictedQueryEnabled", "non_partition_restricted_index_query_enabled");
+    /**
+     * Set of guardrails which are flags, even though their suffix would suggest they are part of "values" which have warned, ignored, and disallowed sub-categories
+     */
+    private static final Set<String> specialFlags = Set.of("intersect_filtering_query_warned", "zero_ttl_on_twcs_warned");
 
     @VisibleForTesting
     public enum GuardrailCategory
@@ -419,9 +423,15 @@ public abstract class GuardrailsConfigCommand extends NodeTool.NodeToolCmd
                     continue;
                 }
                 else if (key.endsWith("_disallowed") ||
-                         key.endsWith("_ignored") ||
-                         key.endsWith("_warned"))
+                         key.endsWith("_ignored"))
                     bucket = values;
+                else if (key.endsWith("_warned"))
+                {
+                    if (specialFlags.contains(key))
+                        bucket = flags;
+                    else
+                        bucket = values;
+                }
                 else
                     bucket = others;
 
