@@ -40,6 +40,11 @@ public class ListSnapshots extends NodeToolCmd
     description = "Skip snapshots with TTL")
     private boolean noTTL = false;
 
+    @Option(title = "ephemeral",
+    name = { "-e", "--ephemeral" },
+    description = "Include ephememeral snapshots")
+    private boolean includeEphemeral = false;
+
     @Override
     public void execute(NodeProbe probe)
     {
@@ -50,6 +55,7 @@ public class ListSnapshots extends NodeToolCmd
 
             Map<String, String> options = new HashMap<>();
             options.put("no_ttl", Boolean.toString(noTTL));
+            options.put("include_ephemeral", Boolean.toString(includeEphemeral));
 
             final Map<String, TabularData> snapshotDetails = probe.getSnapshotDetails(options);
             if (snapshotDetails.isEmpty())
@@ -62,7 +68,11 @@ public class ListSnapshots extends NodeToolCmd
             TableBuilder table = new TableBuilder();
             // display column names only once
             final List<String> indexNames = snapshotDetails.entrySet().iterator().next().getValue().getTabularType().getIndexNames();
-            table.add(indexNames.toArray(new String[indexNames.size()]));
+
+            if (includeEphemeral)
+                table.add(indexNames.toArray(new String[indexNames.size()]));
+            else
+                table.add(indexNames.subList(0, indexNames.size() - 1).toArray(new String[indexNames.size() - 1]));
 
             for (final Map.Entry<String, TabularData> snapshotDetail : snapshotDetails.entrySet())
             {
@@ -70,12 +80,15 @@ public class ListSnapshots extends NodeToolCmd
                 for (Object eachValue : values)
                 {
                     final List<?> value = (List<?>) eachValue;
-                    table.add(value.toArray(new String[value.size()]));
+                    if (includeEphemeral)
+                        table.add(value.toArray(new String[value.size()]));
+                    else
+                        table.add(value.subList(0, value.size() - 1).toArray(new String[value.size() - 1]));
                 }
             }
             table.printTo(out);
 
-            out.println("\nTotal TrueDiskSpaceUsed: " + FileUtils.stringifyFileSize(trueSnapshotsSize) + "\n");
+            out.println("\nTotal TrueDiskSpaceUsed: " + FileUtils.stringifyFileSize(trueSnapshotsSize) + '\n');
         }
         catch (Exception e)
         {
