@@ -20,7 +20,7 @@ package org.apache.cassandra.tools.nodetool;
 import java.io.PrintStream;
 
 import static java.lang.String.format;
-import io.airlift.command.Command;
+import io.airlift.airline.Command;
 
 import org.apache.cassandra.tools.NodeProbe;
 import org.apache.cassandra.tools.NodeTool.NodeToolCmd;
@@ -33,12 +33,12 @@ public class ProxyHistograms extends NodeToolCmd
     {
         PrintStream out = probe.output().out;
         String[] percentiles = {"50%", "75%", "95%", "98%", "99%", "Min", "Max"};
-        double[] readLatency = probe.metricPercentilesAsArray(probe.getProxyMetric("Read"));
-        double[] writeLatency = probe.metricPercentilesAsArray(probe.getProxyMetric("Write"));
-        double[] rangeLatency = probe.metricPercentilesAsArray(probe.getProxyMetric("RangeSlice"));
-        double[] casReadLatency = probe.metricPercentilesAsArray(probe.getProxyMetric("CASRead"));
-        double[] casWriteLatency = probe.metricPercentilesAsArray(probe.getProxyMetric("CASWrite"));
-        double[] viewWriteLatency = probe.metricPercentilesAsArray(probe.getProxyMetric("ViewWrite"));
+        Double[] readLatency = probe.metricPercentilesAsArray(probe.getProxyMetric("Read"));
+        Double[] writeLatency = probe.metricPercentilesAsArray(probe.getProxyMetric("Write"));
+        Double[] rangeLatency = probe.metricPercentilesAsArray(probe.getProxyMetric("RangeSlice"));
+        Double[] casReadLatency = probe.metricPercentilesAsArray(probe.getProxyMetric("CASRead"));
+        Double[] casWriteLatency = probe.metricPercentilesAsArray(probe.getProxyMetric("CASWrite"));
+        Double[] viewWriteLatency = probe.metricPercentilesAsArray(probe.getProxyMetric("ViewWrite"));
 
         out.println("proxy histograms");
         out.println(format("%-10s%19s%19s%19s%19s%19s%19s",
@@ -48,13 +48,13 @@ public class ProxyHistograms extends NodeToolCmd
         for (int i = 0; i < percentiles.length; i++)
         {
             out.println(format("%-10s%19.2f%19.2f%19.2f%19.2f%19.2f%19.2f",
-                               percentiles[i],
-                               readLatency[i],
-                               writeLatency[i],
-                               rangeLatency[i],
-                               casReadLatency[i],
-                               casWriteLatency[i],
-                               viewWriteLatency[i]));
+                    percentiles[i],
+                    readLatency[i],
+                    writeLatency[i],
+                    rangeLatency[i],
+                    casReadLatency[i],
+                    casWriteLatency[i],
+                    viewWriteLatency[i]));
         }
         out.println();
     }
