@@ -18,10 +18,10 @@
 
 package org.apache.cassandra.tools.nodetool.stats;
 
-import java.io.IOException;
 import java.io.PrintStream;
+import java.util.Map;
 
-import com.fasterxml.jackson.databind.ObjectMapper;
+import org.apache.cassandra.utils.JsonUtils;
 import org.yaml.snakeyaml.DumperOptions;
 import org.yaml.snakeyaml.Yaml;
 
@@ -40,17 +40,17 @@ public interface StatsPrinter<T extends StatsHolder>
         @Override
         public void print(T data, PrintStream out)
         {
-            ObjectMapper mapper = new ObjectMapper();
-            try
-            {
-                String json = mapper.writerWithDefaultPrettyPrinter()
-                                    .writeValueAsString(data.convert2Map());
-                out.println(json);
-            }
-            catch (IOException e)
-            {
-                out.println(e);
-            }
+            // First need to get a Map representation of stats
+            final Map<String, Object> stats = data.convert2Map();
+            // but then also need slight massaging to coerce NaN values into nulls
+            for (Object statEntry : stats.values())
+                if (statEntry instanceof Map<?,?>)
+                    for (Map.Entry<String, Object> entry : ((Map<String, Object>) statEntry).entrySet())
+                        if (entry.getValue() instanceof Double && !Double.isFinite((Double) entry.getValue()))
+                            entry.setValue(null);
+
+            // and then we can serialize
+            out.println(JsonUtils.writeAsPrettyJsonString(stats));
         }
     }
 
