@@ -25,15 +25,21 @@ import java.util.*;
 import com.google.common.collect.ArrayListMultimap;
 
 import javax.management.InstanceNotFoundException;
+
+import org.apache.commons.lang3.time.DurationFormatUtils;
+
 import org.apache.cassandra.db.*;
+import org.apache.cassandra.db.compaction.TimeWindowCompactionStrategy;
+import org.apache.cassandra.db.compaction.TimeWindowCompactionStrategyOptions;
 import org.apache.cassandra.io.util.*;
 import org.apache.cassandra.metrics.*;
+import org.apache.cassandra.schema.SchemaConstants;
 import org.apache.cassandra.tools.*;
 
 public class TableStatsHolder implements StatsHolder
 {
     public final List<StatsKeyspace> keyspaces;
-    public final int numberOfTables;
+    public int numberOfTables = 0;
     public final boolean humanReadable;
     public final String sortKey;
     public final int top;
@@ -48,14 +54,7 @@ public class TableStatsHolder implements StatsHolder
         this.locationCheck = locationCheck;
 
         if (!this.isTestTableStatsHolder())
-        {
-            this.numberOfTables = probe.getNumberOfTables();
             this.initializeKeyspaces(probe, ignore, tableNames);
-        }
-        else
-        {
-            this.numberOfTables = 0;
-        }
     }
 
     @Override
@@ -68,7 +67,7 @@ public class TableStatsHolder implements StatsHolder
     }
 
     /**
-     * @returns Map<String, Object> a nested HashMap of keyspaces, their tables, and the tables' statistics.
+     * @return {@code Map<String, Object>} a nested HashMap of keyspaces, their tables, and the tables' statistics.
      */
     private Map<String, Object> convertAllToMap()
     {
@@ -100,7 +99,7 @@ public class TableStatsHolder implements StatsHolder
     }
 
     /**
-     * @returns Map<String, Object> a nested HashMap of the sorted and filtered table names and the HashMaps of their statistics.
+     * @return {@code Map<String, Object>} a nested HashMap of the sorted and filtered table names and the HashMaps of their statistics.
      */
     private Map<String, Object> convertSortedFilteredSubsetToMap()
     {
@@ -123,6 +122,8 @@ public class TableStatsHolder implements StatsHolder
         mpTable.put("old_sstable_count", table.oldSSTableCount);
         mpTable.put("sstables_in_each_level", table.sstablesInEachLevel);
         mpTable.put("sstable_bytes_in_each_level", table.sstableBytesInEachLevel);
+        mpTable.put("max_sstable_size", table.maxSSTableSize);
+        mpTable.put("twcs", table.twcs);
         mpTable.put("space_used_live", table.spaceUsedLive);
         mpTable.put("space_used_total", table.spaceUsedTotal);
         mpTable.put("space_used_by_snapshots_total", table.spaceUsedBySnapshotsTotal);
@@ -140,6 +141,7 @@ public class TableStatsHolder implements StatsHolder
         mpTable.put("local_read_latency_ms", String.format("%01.3f", table.localReadLatencyMs));
         mpTable.put("local_write_count", table.localWriteCount);
         mpTable.put("local_write_latency_ms", String.format("%01.3f", table.localWriteLatencyMs));
+        mpTable.put("local_read_write_ratio", String.format("%01.5f", table.localReadWriteRatio));
         mpTable.put("pending_flushes", table.pendingFlushes);
         mpTable.put("percent_repaired", table.percentRepaired);
         mpTable.put("bytes_repaired", table.bytesRepaired);
@@ -166,13 +168,22 @@ public class TableStatsHolder implements StatsHolder
                     table.averageTombstonesPerSliceLastFiveMinutes);
         mpTable.put("maximum_tombstones_per_slice_last_five_minutes",
                     table.maximumTombstonesPerSliceLastFiveMinutes);
-        mpTable.put("dropped_mutations", table.droppedMutations);
         mpTable.put("droppable_tombstone_ratio",
                     String.format("%01.5f", table.droppableTombstoneRatio));
         mpTable.put("top_size_partitions", table.topSizePartitions);
         mpTable.put("top_tombstone_partitions", table.topTombstonePartitions);
         if (locationCheck)
             mpTable.put("sstables_in_correct_location", table.isInCorrectLocation);
+
+        mpTable.put("sai_local_query_latency_ms",String.format("%01.3f", table.saiQueryLatencyMs));
+        mpTable.put("sai_post_filtering_read_latency",String.format("%01.3f", table.saiPostFilteringReadLatencyMs));
+        mpTable.put("sai_disk_used_bytes",table.saiDiskUsedBytes);
+        mpTable.put("sai_sstable_indexes_hit",table.saiSSTableIndexesHit);
+        mpTable.put("sai_index_segments_hit",table.saiIndexSegmentsHit);
+        mpTable.put("sai_rows_filtered",table.saiRowsFiltered);
+        mpTable.put("sai_total_query_timeouts",table.saiTotalQueryTimeouts);
+        mpTable.put("sai_total_queryable_index_ratio", table.saiTotalQueryableIndexRatio);
+
         return mpTable;
     }
 
@@ -206,6 +217,8 @@ public class TableStatsHolder implements StatsHolder
             }
         }
 
+        numberOfTables = selectedTableMbeans.size();
+
         // make sure all specified keyspace and tables exist
         filter.verifyKeyspaces(probe.getKeyspaces());
         filter.verifyTables();
@@ -228,6 +241,8 @@ public class TableStatsHolder implements StatsHolder
                 statsTable.isIndex = tableName.contains(".");
                 statsTable.sstableCount = probe.getColumnFamilyMetric(keyspaceName, tableName, "LiveSSTableCount");
                 statsTable.oldSSTableCount = probe.getColumnFamilyMetric(keyspaceName, tableName, "OldVersionSSTableCount");
+                Long sstableSize = (Long) probe.getColumnFamilyMetric(keyspaceName, tableName, "MaxSSTableSize");
+                statsTable.maxSSTableSize = sstableSize == null ? 0 : sstableSize;
 
                 int[] leveledSStables = table.getSSTableCountPerLevel();
                 if (leveledSStables != null)
@@ -244,6 +259,7 @@ public class TableStatsHolder implements StatsHolder
                         statsTable.sstablesInEachLevel.add(count + ((count > maxCount) ? "/" + maxCount : ""));
                     }
                 }
+                statsTable.sstableCountPerTWCSBucket = table.getSSTableCountPerTWCSBucket();
 
                 long[] leveledSSTablesBytes = table.getPerLevelSizeBytes();
                 if (leveledSSTablesBytes != null)
@@ -291,6 +307,9 @@ public class TableStatsHolder implements StatsHolder
                 statsTable.spaceUsedLive = format((Long) probe.getColumnFamilyMetric(keyspaceName, tableName, "LiveDiskSpaceUsed"), humanReadable);
                 statsTable.spaceUsedTotal = format((Long) probe.getColumnFamilyMetric(keyspaceName, tableName, "TotalDiskSpaceUsed"), humanReadable);
                 statsTable.spaceUsedBySnapshotsTotal = format((Long) probe.getColumnFamilyMetric(keyspaceName, tableName, "SnapshotsSize"), humanReadable);
+
+                maybeAddTWCSWindowWithMaxDuration(statsTable, probe, keyspaceName, tableName);
+
                 if (offHeapSize != null)
                 {
                     statsTable.offHeapUsed = true;
@@ -332,6 +351,9 @@ public class TableStatsHolder implements StatsHolder
 
                 double localWriteLatency = ((CassandraMetricsRegistry.JmxTimerMBean) probe.getColumnFamilyMetric(keyspaceName, tableName, "WriteLatency")).getMean() / 1000;
                 double localWLatency = localWriteLatency > 0 ? localWriteLatency : Double.NaN;
+
+                statsTable.localReadWriteRatio = statsTable.localWriteCount > 0 ? statsTable.localReadCount / (double) statsTable.localWriteCount : 0;
+
                 statsTable.localWriteLatencyMs = localWLatency;
                 statsTable.pendingFlushes = probe.getColumnFamilyMetric(keyspaceName, tableName, "PendingFlushes");
 
@@ -366,7 +388,6 @@ public class TableStatsHolder implements StatsHolder
                 histogram = (CassandraMetricsRegistry.JmxHistogramMBean) probe.getColumnFamilyMetric(keyspaceName, tableName, "TombstoneScannedHistogram");
                 statsTable.averageTombstonesPerSliceLastFiveMinutes = histogram.getMean();
                 statsTable.maximumTombstonesPerSliceLastFiveMinutes = histogram.getMax();
-                statsTable.droppedMutations = format((Long) probe.getColumnFamilyMetric(keyspaceName, tableName, "DroppedMutations"), humanReadable);
                 statsTable.droppableTombstoneRatio = probe.getDroppableTombstoneRatio(keyspaceName, tableName);
                 statsTable.topSizePartitions = format(table.getTopSizePartitions(), humanReadable);
                 if (table.getTopSizePartitionsLastUpdate() != null)
@@ -375,12 +396,81 @@ public class TableStatsHolder implements StatsHolder
                 if (table.getTopTombstonePartitionsLastUpdate() != null)
                     statsTable.topTombstonePartitionsLastUpdate = millisToDateString(table.getTopTombstonePartitionsLastUpdate());
 
+                if (!SchemaConstants.isSystemKeyspace(keyspaceName))
+                {
+                    Object totalIndexCount = probe.getSaiMetric(keyspaceName, tableName, "TotalIndexCount");
+                    statsTable.saiTotalIndexCount = (totalIndexCount != null) ? (int) totalIndexCount : 0;
+
+                    if (statsTable.saiTotalIndexCount > 0)
+                    {
+                        Object queryLatencyMetric = probe.getSaiMetric(keyspaceName, tableName, "QueryLatency");
+                        double queryLatency = getMetricMean(queryLatencyMetric);
+                        statsTable.saiQueryLatencyMs = queryLatency > 0 ? queryLatency : Double.NaN;
+
+                        Object PostFilteringReadLatency = probe.getSaiMetric(keyspaceName, tableName, "PostFilteringReadLatency");
+                        double postfilteringreadlatency = getMetricMean(PostFilteringReadLatency);
+                        statsTable.saiPostFilteringReadLatencyMs = postfilteringreadlatency > 0 ? postfilteringreadlatency : Double.NaN;
+
+                        Object diskUsedBytes = probe.getSaiMetric(keyspaceName, tableName, "DiskUsedBytes");
+                        long saidiskusedbytes = (diskUsedBytes != null) ? (long) diskUsedBytes : 0L;
+                        statsTable.saiDiskUsedBytes = format(saidiskusedbytes, humanReadable);
+
+                        Object SSTableIndexesHit = probe.getSaiMetric(keyspaceName, tableName, "SSTableIndexesHit");
+                        statsTable.saiSSTableIndexesHit = getMetricMean(SSTableIndexesHit);
+
+                        Object IndexSegmentsHit = probe.getSaiMetric(keyspaceName, tableName, "IndexSegmentsHit");
+                        statsTable.saiIndexSegmentsHit = getMetricMean(IndexSegmentsHit);
+
+                        Object RowsFiltered = probe.getSaiMetric(keyspaceName, tableName, "RowsFiltered");
+                        statsTable.saiRowsFiltered = getMetricMean(RowsFiltered);
+
+                        Object totalQueryTimeouts = probe.getSaiMetric(keyspaceName, tableName, "TotalQueryTimeouts");
+                        statsTable.saiTotalQueryTimeouts = (totalQueryTimeouts != null) ? (Long) totalQueryTimeouts : 0L;
+
+                        Object totalQueryableIndexCount = probe.getSaiMetric(keyspaceName, tableName, "TotalQueryableIndexCount");
+                        int saiTotalQueryableIndexCount = (totalQueryableIndexCount != null) ? (int) totalQueryableIndexCount : 0;
+
+                        statsTable.saiTotalQueryableIndexRatio = String.format("%d/%d", saiTotalQueryableIndexCount, statsTable.saiTotalIndexCount);
+                    }
+                }
+
                 statsKeyspace.tables.add(statsTable);
             }
             keyspaces.add(statsKeyspace);
         }
     }
 
+    private double getMetricMean(Object metricObject) {
+        if (metricObject instanceof CassandraMetricsRegistry.JmxTimerMBean) {
+            return ((CassandraMetricsRegistry.JmxTimerMBean) metricObject).getMean() / 1000;
+        }
+        if (metricObject instanceof CassandraMetricsRegistry.JmxHistogramMBean) {
+            return Math.round(((CassandraMetricsRegistry.JmxHistogramMBean) metricObject).getMean() * 100.0) / 100.0;
+        }
+        throw new IllegalArgumentException("Unsupported metric object type: " + metricObject.getClass().getName());
+    }
+
+    private void maybeAddTWCSWindowWithMaxDuration(StatsTable statsTable, NodeProbe probe, String keyspaceName, String tableName)
+    {
+        Map<String, String> compactionParameters = probe.getCfsProxy(statsTable.keyspaceName, statsTable.tableName)
+                                                        .getCompactionParameters();
+
+        if (compactionParameters == null)
+            return;
+
+        String compactor = compactionParameters.get("class");
+
+        if (compactor == null || !compactor.endsWith(TimeWindowCompactionStrategy.class.getSimpleName()))
+            return;
+
+        String unit = compactionParameters.get(TimeWindowCompactionStrategyOptions.COMPACTION_WINDOW_UNIT_KEY);
+        String size = compactionParameters.get(TimeWindowCompactionStrategyOptions.COMPACTION_WINDOW_SIZE_KEY);
+
+        statsTable.twcsDurationInMillis = (Long) probe.getColumnFamilyMetric(keyspaceName, tableName, "MaxSSTableDuration");
+        String maxDuration = millisToDuration(statsTable.twcsDurationInMillis);
+        statsTable.twcs = String.format("%s %s, max duration: %s", size, unit, maxDuration);
+    }
+
     private String format(long bytes, boolean humanReadable)
     {
         return humanReadable ? FileUtils.stringifyFileSize(bytes) : Long.toString(bytes);
@@ -402,6 +492,11 @@ public class TableStatsHolder implements StatsHolder
         return df.format(new Date(millis));
     }
 
+    private String millisToDuration(long millis)
+    {
+        return DurationFormatUtils.formatDurationWords(millis, true, true);
+    }
+
     /**
      * Sort and filter this TableStatHolder's tables as specified by its sortKey and top attributes.
      */
