@@ -22,7 +22,10 @@ import java.net.InetSocketAddress;
 import java.net.UnknownHostException;
 import java.util.*;
 import java.util.concurrent.ConcurrentHashMap;
+import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
+import java.util.concurrent.locks.LockSupport;
+import java.util.function.BooleanSupplier;
 
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -54,6 +57,8 @@ import org.apache.cassandra.service.*;
 import org.apache.cassandra.transport.messages.EventMessage;
 import org.apache.cassandra.utils.FBUtilities;
 
+import static org.apache.cassandra.utils.Clock.Global.nanoTime;
+
 public class Server implements CassandraDaemon.Server
 {
     static
@@ -64,7 +69,7 @@ public class Server implements CassandraDaemon.Server
     private static final Logger logger = LoggerFactory.getLogger(Server.class);
     private static final boolean useEpoll = NativeTransportService.useEpoll();
 
-    private final ConnectionTracker connectionTracker = new ConnectionTracker();
+    private final ConnectionTracker connectionTracker;
 
     private final Connection.Factory connectionFactory = new Connection.Factory()
     {
@@ -79,7 +84,7 @@ public class Server implements CassandraDaemon.Server
     private final AtomicBoolean isRunning = new AtomicBoolean(false);
     private final PipelineConfigurator pipelineConfigurator;
     private final EventLoopGroup workerGroup;
-
+    private final Dispatcher dispatcher;
     private Server (Builder builder)
     {
         this.socket = builder.getSocket();
@@ -96,14 +101,16 @@ public class Server implements CassandraDaemon.Server
                 workerGroup = new NioEventLoopGroup();
         }
 
+        dispatcher = new Dispatcher(DatabaseDescriptor.useNativeTransportLegacyFlusher());
         pipelineConfigurator = builder.pipelineConfigurator != null
                                ? builder.pipelineConfigurator
                                : new PipelineConfigurator(useEpoll,
                                                           DatabaseDescriptor.getRpcKeepAlive(),
-                                                          DatabaseDescriptor.useNativeTransportLegacyFlusher(),
-                                                          builder.tlsEncryptionPolicy);
+                                                          builder.tlsEncryptionPolicy,
+                                                          dispatcher);
 
         EventNotifier notifier = builder.eventNotifier != null ? builder.eventNotifier : new EventNotifier();
+        connectionTracker = new ConnectionTracker(isRunning::get);
         notifier.registerConnectionTracker(connectionTracker);
         StorageService.instance.register(notifier);
         Schema.instance.registerListener(notifier);
@@ -111,8 +118,13 @@ public class Server implements CassandraDaemon.Server
 
     public void stop()
     {
-        if (isRunning.compareAndSet(true, false))
-            close();
+        stop(false);
+    }
+
+    public void stop(boolean force)
+    {
+         if (isRunning.compareAndSet(true, false))
+             close(force);
     }
 
     public boolean isRunning()
@@ -168,8 +180,23 @@ public class Server implements CassandraDaemon.Server
         connectionTracker.protocolVersionTracker.clear();
     }
 
-    private void close()
+
+    private void close(boolean force)
     {
+        if (!force)
+        {
+            long deadline = nanoTime() + DatabaseDescriptor.getNativeTransportTimeout(TimeUnit.NANOSECONDS);
+            while (!dispatcher.isDone())
+            {
+                if (nanoTime() > deadline)
+                {
+                    logger.warn("Some connections took longer than the native transport timeout to complete");
+                    break;
+                }
+                LockSupport.parkNanos(TimeUnit.MILLISECONDS.toNanos(100));
+            }
+        }
+
         // Close opened connections
         connectionTracker.closeAll();
 
@@ -257,11 +284,13 @@ public class Server implements CassandraDaemon.Server
         public final ChannelGroup allChannels = new DefaultChannelGroup(GlobalEventExecutor.INSTANCE);
         private final EnumMap<Event.Type, ChannelGroup> groups = new EnumMap<>(Event.Type.class);
         private final ProtocolVersionTracker protocolVersionTracker = new ProtocolVersionTracker();
+        private final BooleanSupplier isRunning;
 
-        public ConnectionTracker()
+        private ConnectionTracker(BooleanSupplier isRunning)
         {
             for (Event.Type type : Event.Type.values())
                 groups.put(type, new DefaultChannelGroup(type.toString(), GlobalEventExecutor.INSTANCE));
+            this.isRunning = isRunning;
         }
 
         public void addConnection(Channel ch, Connection connection)
@@ -272,6 +301,11 @@ public class Server implements CassandraDaemon.Server
                 protocolVersionTracker.addConnection(((InetSocketAddress) ch.remoteAddress()).getAddress(), connection.getVersion());
         }
 
+        public boolean isRunning()
+        {
+            return isRunning.getAsBoolean();
+        }
+
         public void register(Event.Type type, Channel ch)
         {
             groups.get(type).add(ch);
@@ -293,7 +327,7 @@ public class Server implements CassandraDaemon.Server
 
         void closeAll()
         {
-            allChannels.close().awaitUninterruptibly();
+            allChannels.flush().close().awaitUninterruptibly();
         }
 
         int countConnectedClients()
