@@ -87,6 +87,11 @@ public interface Clock
             INITIALIZE_MESSAGE = null;
         }
 
+        public static Clock clock()
+        {
+            return instance;
+        }
+
         /**
          * Semantically equivalent to {@link System#nanoTime()}
          */
@@ -133,6 +138,11 @@ public interface Clock
      */
     public long currentTimeMillis();
 
+    public default long nowInSeconds()
+    {
+        return currentTimeMillis() / 1000L;
+    }
+
     @Intercept
     public static void waitUntil(long deadlineNanos) throws InterruptedException
     {
