@@ -17,19 +17,37 @@
  */
 package org.apache.cassandra.utils;
 
-import java.nio.ByteBuffer;
+import java.io.IOException;
 
+import org.apache.cassandra.io.util.DataOutputStreamPlus;
 import org.apache.cassandra.utils.concurrent.SharedCloseable;
 
 public interface IFilter extends SharedCloseable
 {
-    void add(ByteBuffer key);
+    interface FilterKey
+    {
+        /**
+         * Places the murmur3 hash of the key in the given long array of size at least two.
+         */
+        void filterHash(long[] dest);
 
-    boolean isPresent(ByteBuffer key);
+        default short filterHashLowerBits()
+        {
+            long[] dest = new long[2];
+            filterHash(dest);
+            return (short) dest[1];
+        }
+    }
+
+    void add(FilterKey key);
+
+    boolean isPresent(FilterKey key);
 
     void clear();
 
-    long serializedSize();
+    long serializedSize(boolean oldSerializationFormat);
+
+    void serialize(DataOutputStreamPlus out, boolean oldSerializationFormat) throws IOException;
 
     void close();
 
@@ -37,7 +55,10 @@ public interface IFilter extends SharedCloseable
 
     /**
      * Returns the amount of memory in bytes used off heap.
+     *
      * @return the amount of memory in bytes used off heap
      */
     long offHeapSize();
+
+    boolean isInformative();
 }
