@@ -56,7 +56,8 @@ import org.slf4j.LoggerFactory;
 
 import org.apache.cassandra.auth.jmx.AuthenticationProxy;
 
-import static org.apache.cassandra.config.CassandraRelevantProperties.JAVA_VERSION;
+import static org.apache.cassandra.config.CassandraRelevantProperties.CASSANDRA_JMX_AUTHORIZER;
+import static org.apache.cassandra.config.CassandraRelevantProperties.CASSANDRA_JMX_REMOTE_LOGIN_CONFIG;
 import static org.apache.cassandra.config.CassandraRelevantProperties.COM_SUN_MANAGEMENT_JMXREMOTE_ACCESS_FILE;
 import static org.apache.cassandra.config.CassandraRelevantProperties.COM_SUN_MANAGEMENT_JMXREMOTE_AUTHENTICATE;
 import static org.apache.cassandra.config.CassandraRelevantProperties.COM_SUN_MANAGEMENT_JMXREMOTE_PASSWORD_FILE;
@@ -65,7 +66,9 @@ import static org.apache.cassandra.config.CassandraRelevantProperties.COM_SUN_MA
 import static org.apache.cassandra.config.CassandraRelevantProperties.COM_SUN_MANAGEMENT_JMXREMOTE_SSL_ENABLED_CIPHER_SUITES;
 import static org.apache.cassandra.config.CassandraRelevantProperties.COM_SUN_MANAGEMENT_JMXREMOTE_SSL_ENABLED_PROTOCOLS;
 import static org.apache.cassandra.config.CassandraRelevantProperties.COM_SUN_MANAGEMENT_JMXREMOTE_SSL_NEED_CLIENT_AUTH;
-
+import static org.apache.cassandra.config.CassandraRelevantProperties.JAVAX_RMI_SSL_CLIENT_ENABLED_CIPHER_SUITES;
+import static org.apache.cassandra.config.CassandraRelevantProperties.JAVAX_RMI_SSL_CLIENT_ENABLED_PROTOCOLS;
+import static org.apache.cassandra.config.CassandraRelevantProperties.JAVA_RMI_SERVER_HOSTNAME;
 
 public class JMXServerUtils
 {
@@ -75,7 +78,6 @@ public class JMXServerUtils
      * Creates a server programmatically. This allows us to set parameters which normally are
      * inaccessable.
      */
-    @SuppressWarnings("resource")
     @VisibleForTesting
     public static JMXConnectorServer createJMXServer(int port, String hostname, boolean local)
     throws IOException
@@ -86,7 +88,7 @@ public class JMXServerUtils
         if (local)
         {
             serverAddress = InetAddress.getLoopbackAddress();
-            System.setProperty("java.rmi.server.hostname", serverAddress.getHostAddress());
+            JAVA_RMI_SERVER_HOSTNAME.setString(serverAddress.getHostAddress());
         }
 
         // Configure the RMI client & server socket factories, including SSL config.
@@ -149,7 +151,6 @@ public class JMXServerUtils
         return jmxServer;
     }
 
-    @SuppressWarnings("resource")
     public static JMXConnectorServer createJMXServer(int port, boolean local) throws IOException
     {
         return createJMXServer(port, null, local);
@@ -158,13 +159,7 @@ public class JMXServerUtils
     private static Map<String, Object> configureSecureCredentials()
     {
         Map<String, Object> env = new HashMap<>();
-        if (JAVA_VERSION.getString().startsWith("1.8"))
-        {
-            env.put("jmx.remote.rmi.server.credential.types",
-                    new String[] { String[].class.getName(), String.class.getName() });
-        } else {
-            env.put("jmx.remote.rmi.server.credentials.filter.pattern", String.class.getName() + ";!*");
-        }
+        env.put("jmx.remote.rmi.server.credentials.filter.pattern", String.class.getName() + ";!*");
         return env;
     }
 
@@ -185,7 +180,7 @@ public class JMXServerUtils
         // before creating the authenticator. If no password file has been
         // explicitly set, it's read from the default location
         // $JAVA_HOME/lib/management/jmxremote.password
-        String configEntry = System.getProperty("cassandra.jmx.remote.login.config");
+        String configEntry = CASSANDRA_JMX_REMOTE_LOGIN_CONFIG.getString();
         if (configEntry != null)
         {
             env.put(JMXConnectorServer.AUTHENTICATOR, new AuthenticationProxy(configEntry));
@@ -211,7 +206,7 @@ public class JMXServerUtils
         // can be set as the JMXConnectorServer's MBeanServerForwarder.
         // If no custom proxy is supplied, check system properties for the location of the
         // standard access file & stash it in env
-        String authzProxyClass = System.getProperty("cassandra.jmx.authorizer");
+        String authzProxyClass = CASSANDRA_JMX_AUTHORIZER.getString();
         if (authzProxyClass != null)
         {
             final InvocationHandler handler = FBUtilities.construct(authzProxyClass, "JMX authz proxy");
@@ -241,7 +236,7 @@ public class JMXServerUtils
             String protocolList = COM_SUN_MANAGEMENT_JMXREMOTE_SSL_ENABLED_PROTOCOLS.getString();
             if (protocolList != null)
             {
-                System.setProperty("javax.rmi.ssl.client.enabledProtocols", protocolList);
+                JAVAX_RMI_SSL_CLIENT_ENABLED_PROTOCOLS.setString(protocolList);
                 protocols = StringUtils.split(protocolList, ',');
             }
 
@@ -249,7 +244,7 @@ public class JMXServerUtils
             String cipherList = COM_SUN_MANAGEMENT_JMXREMOTE_SSL_ENABLED_CIPHER_SUITES.getString();
             if (cipherList != null)
             {
-                System.setProperty("javax.rmi.ssl.client.enabledCipherSuites", cipherList);
+                JAVAX_RMI_SSL_CLIENT_ENABLED_CIPHER_SUITES.setString(cipherList);
                 ciphers = StringUtils.split(cipherList, ',');
             }
 
