@@ -31,7 +31,6 @@ public abstract class MergeIterator<In,Out> extends AbstractIterator<Out> implem
         this.reducer = reducer;
     }
 
-    @SuppressWarnings("resource")
     public static <In, Out> MergeIterator<In, Out> get(List<? extends Iterator<In>> sources,
                                                        Comparator<? super In> comparator,
                                                        Reducer<In, Out> reducer)
@@ -395,10 +394,11 @@ public abstract class MergeIterator<In,Out> extends AbstractIterator<Out> implem
 
         private boolean isLowerBound()
         {
+            assert item != null;
             return item == lowerBound;
         }
 
-        public void consume(Reducer reducer)
+        public <Out> void consume(Reducer<In, Out> reducer)
         {
             if (isLowerBound())
             {
@@ -488,4 +488,4 @@ public abstract class MergeIterator<In,Out> extends AbstractIterator<Out> implem
             return (Out) source.next();
         }
     }
-}
+}
\ No newline at end of file
