@@ -21,74 +21,35 @@
 package org.apache.cassandra.utils;
 
 import java.io.*;
-import java.lang.reflect.Field;
 import java.nio.MappedByteBuffer;
 import java.nio.channels.ClosedChannelException;
 import java.nio.channels.FileChannel;
-import java.util.concurrent.atomic.AtomicInteger;
-
-import org.apache.cassandra.config.Config;
 
 import com.google.common.base.Preconditions;
+
+import org.apache.cassandra.config.CassandraRelevantEnv;
+import org.apache.cassandra.config.CassandraRelevantProperties;
 import org.apache.cassandra.io.util.File;
+import org.apache.cassandra.utils.memory.MemoryUtil;
+
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 /*
- * A wrapper around various mechanisms for syncing files that makes it possible it intercept
+ * A wrapper around various mechanisms for syncing files that makes it possible to intercept
  * and skip syncing. Useful for unit tests in certain environments where syncs can have outliers
- * bad enough to causes tests to run 10s of seconds longer.
+ * bad enough to cause tests to run 10s of seconds longer.
  */
 public class SyncUtil
 {
     public static final boolean SKIP_SYNC;
-
-    private static final Field mbbFDField;
-    private static final Field fdClosedField;
-    private static final Field fdUseCountField;
-
     private static final Logger logger = LoggerFactory.getLogger(SyncUtil.class);
 
     static
     {
-        Field mbbFDFieldTemp = null;
-        try
-        {
-            mbbFDFieldTemp = MappedByteBuffer.class.getDeclaredField("fd");
-            mbbFDFieldTemp.setAccessible(true);
-        }
-        catch (NoSuchFieldException e)
-        {
-        }
-        mbbFDField = mbbFDFieldTemp;
-
-        //Java 8
-        Field fdClosedFieldTemp = null;
-        try
-        {
-            fdClosedFieldTemp = FileDescriptor.class.getDeclaredField("closed");
-            fdClosedFieldTemp.setAccessible(true);
-        }
-        catch (NoSuchFieldException e)
-        {
-        }
-        fdClosedField = fdClosedFieldTemp;
-
-        //Java 7
-        Field fdUseCountTemp = null;
-        try
-        {
-            fdUseCountTemp = FileDescriptor.class.getDeclaredField("useCount");
-            fdUseCountTemp.setAccessible(true);
-        }
-        catch (NoSuchFieldException e)
-        {
-        }
-        fdUseCountField = fdUseCountTemp;
-
         //If skipping syncing is requested by any means then skip them.
-        boolean skipSyncProperty = Boolean.getBoolean(Config.PROPERTY_PREFIX + "skip_sync");
-        boolean skipSyncEnv = Boolean.valueOf(System.getenv().getOrDefault("CASSANDRA_SKIP_SYNC", "false"));
+        boolean skipSyncProperty = CassandraRelevantProperties.TEST_CASSANDRA_SKIP_SYNC.getBoolean();
+        boolean skipSyncEnv = CassandraRelevantEnv.CASSANDRA_SKIP_SYNC.getBoolean();
         SKIP_SYNC = skipSyncProperty || skipSyncEnv;
         if (SKIP_SYNC)
         {
@@ -99,23 +60,14 @@ public class SyncUtil
     public static MappedByteBuffer force(MappedByteBuffer buf)
     {
         Preconditions.checkNotNull(buf);
+        Object attachment = MemoryUtil.getAttachment(buf);
+        if (attachment instanceof Runnable)
+        {
+            ((Runnable) attachment).run();
+            return buf;
+        }
         if (SKIP_SYNC)
         {
-            Object fd = null;
-            try
-            {
-                if (mbbFDField != null)
-                {
-                    fd = mbbFDField.get(buf);
-                }
-            }
-            catch (Exception e)
-            {
-                throw new RuntimeException(e);
-            }
-            //This is what MappedByteBuffer.force() throws if a you call force() on an umapped buffer
-            if (mbbFDField != null && fd == null)
-                throw new UnsupportedOperationException();
             return buf;
         }
         else
@@ -123,40 +75,12 @@ public class SyncUtil
             return buf.force();
         }
     }
-
+    
     public static void sync(FileDescriptor fd) throws SyncFailedException
     {
         Preconditions.checkNotNull(fd);
-        if (SKIP_SYNC)
-        {
-            boolean closed = false;
-            try
-            {
-                if (fdClosedField != null)
-                    closed = fdClosedField.getBoolean(fd);
-            }
-            catch (Exception e)
-            {
-                throw new RuntimeException(e);
-            }
-
-            int useCount = 1;
-            try
-            {
-                if (fdUseCountField != null)
-                    useCount = ((AtomicInteger)fdUseCountField.get(fd)).get();
-            }
-            catch (Exception e)
-            {
-                throw new RuntimeException(e);
-            }
-            if (closed || !fd.valid() || useCount < 0)
-                throw new SyncFailedException("Closed " + closed + " valid " + fd.valid() + " useCount " + useCount);
-        }
-        else
-        {
+        if (!SKIP_SYNC)
             fd.sync();
-        }
     }
 
     public static void force(FileChannel fc, boolean metaData) throws IOException
@@ -183,8 +107,8 @@ public class SyncUtil
     {
         if (SKIP_SYNC)
             return;
-        else
-            NativeLibrary.trySync(fd);
+
+        NativeLibrary.trySync(fd);
     }
 
     public static void trySyncDir(File dir)
