@@ -35,6 +35,7 @@ import org.apache.cassandra.io.util.File;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import net.openhft.chronicle.core.io.BackgroundResourceReleaser;
 import net.openhft.chronicle.queue.ChronicleQueue;
 import net.openhft.chronicle.queue.impl.single.SingleChronicleQueue;
 import net.openhft.chronicle.queue.impl.single.SingleChronicleQueueBuilder;
@@ -42,6 +43,7 @@ import net.openhft.chronicle.queue.ExcerptAppender;
 import net.openhft.chronicle.queue.RollCycles;
 import net.openhft.chronicle.wire.WireOut;
 import net.openhft.chronicle.wire.WriteMarshallable;
+import net.openhft.posix.PosixAPI;
 import org.apache.cassandra.concurrent.NamedThreadFactory;
 import org.apache.cassandra.io.FSError;
 import org.apache.cassandra.utils.JVMStabilityInspector;
@@ -51,6 +53,7 @@ import org.apache.cassandra.utils.concurrent.UncheckedInterruptedException;
 import org.apache.cassandra.utils.concurrent.WeightedQueue;
 
 import static java.lang.String.format;
+import static org.apache.cassandra.config.CassandraRelevantProperties.CHRONICLE_ANNOUNCER_DISABLE;
 
 /**
  * Bin log is a is quick and dirty binary log that is kind of a NIH version of binary logging with a traditional logging
@@ -75,6 +78,13 @@ public class BinLog implements Runnable
     public static final String VERSION = "version";
     public static final String TYPE = "type";
 
+    static
+    {
+        // Avoid the chronicle announcement which is commercial advertisement, and debug info we already print at startup
+        // https://github.com/OpenHFT/Chronicle-Core/blob/chronicle-core-2.23.36/src/main/java/net/openhft/chronicle/core/announcer/Announcer.java#L32-L33
+        CHRONICLE_ANNOUNCER_DISABLE.setBoolean(true);
+    }
+
     private ChronicleQueue queue;
     private ExcerptAppender appender;
     @VisibleForTesting
@@ -122,6 +132,7 @@ public class BinLog implements Runnable
 
     private BinLog(Path path, BinLogOptions options, BinLogArchiver archiver)
     {
+        Preconditions.checkNotNull(PosixAPI.posix(), "Cannot initialize OpenHFT Posix");
         Preconditions.checkNotNull(path, "path was null");
         Preconditions.checkNotNull(options.roll_cycle, "roll_cycle was null");
         Preconditions.checkArgument(options.max_queue_weight > 0, "max_queue_weight must be > 0");
@@ -135,7 +146,6 @@ public class BinLog implements Runnable
         appender = queue.acquireAppender();
         this.blocking = options.block;
         this.path = path;
-
         this.options = options;
     }
 
@@ -170,6 +180,7 @@ public class BinLog implements Runnable
 
         shouldContinue = false;
         sampleQueue.put(NO_OP);
+        BackgroundResourceReleaser.stop();
         binLogThread.join();
         appender.close();
         appender = null;
