@@ -17,10 +17,7 @@
  */
 package org.apache.cassandra.utils.obs;
 
-import java.io.DataInput;
-import java.io.DataInputStream;
-import java.io.DataOutput;
-import java.io.IOException;
+import java.io.*;
 
 import com.google.common.annotations.VisibleForTesting;
 
@@ -144,8 +141,7 @@ public class OffHeapBitSet implements IBitSet
         return TypeSizes.sizeof((int) bytes.size()) + bytes.size();
     }
 
-    @SuppressWarnings("resource")
-    public static OffHeapBitSet deserialize(DataInputStream in, boolean oldBfFormat) throws IOException
+    public static <I extends InputStream & DataInput> OffHeapBitSet deserialize(I in, boolean oldBfFormat) throws IOException
     {
         long byteCount = in.readInt() * 8L;
         Memory memory = Memory.allocate(byteCount);
